/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>

#include <oci.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#if WITH_GTKSOURCEVIEW
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcelanguage.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#endif

#include "appconfig.h"
#include "trace.h"
#include "gutil.h"
#include "syntax.h"
#include "command_tab.h"

extern OCIEnv      *envhp;
extern OCIError    *errhp;
extern OCISvcCtx   *svchp;
extern GtkWidget   *window1;

static GHashTable  *conf_hash = NULL;
static GtkWidget *conf_window = NULL;
static GtkWidget *treeviewExternalEntry=0;
static GtkWidget *textviewExternalEntry=0;
static GtkWidget *htmlExternalEntry=0;
static GtkWidget *treeviewFontButton=0;
static GtkWidget *textviewFontButton=0;
static GtkWidget *treeviewGridlinesCombo=0;
static GtkWidget *treeviewRubberbandingCheck=0;

static GtkWidget *textviewCurrentLineCheck=0;
static GtkWidget *textviewLineNumberCheck=0;
static GtkWidget *textviewLineMarkerCheck=0;
static GtkWidget *textviewMarginCheck=0;
static GtkWidget *textviewUndoSpin=0;
static GtkWidget *textviewTabWidthSpin=0;

static gchar text_font[128];
static gchar tree_font[128];



const gchar *config_get(const gchar *key)
{
	return(g_hash_table_lookup(conf_hash, (gpointer) key));
}


void config_set(const gchar *key,
		const gchar *value)
{
  
#if DEBUG
	fprintf(stderr, "%s:%d config_set('%s', '%s')\n", 
		__FILE__, __LINE__,
		key, value);
#endif
	g_hash_table_insert(conf_hash, 
			    (gpointer) key, 
			    (gpointer) g_strdup(value));
}


gint config_get_integer(const gchar *key)
{
	gchar *value=0;
	gint intval=0;

	value = g_hash_table_lookup(conf_hash, (gpointer) key);

	if(!value) {
		return 0;
	}

	intval = atoi(value);
	return intval;
}


void config_set_integer(const gchar *key,
			gint intval)
{
	gchar valbuf[16];
  
	g_snprintf(valbuf, sizeof(valbuf), "%d", intval);
	g_hash_table_insert(conf_hash, (gpointer) key, (gpointer) g_strdup(valbuf));
}


void config_read(void) 
{
	gchar *config_file=0, *font=0;
	GtkStyle *style=0;
	const gchar *oracle_sid = g_getenv("ORACLE_SID");
	gchar *trace_default = "execute count,recursive calls,db block gets,db block changes,consistent gets,consistent changes,session logical reads,physical reads,physical writes,redo size,enqueue requests,enqueue waits,bytes sent via SQL*Net to client,bytes received via SQL*Net from client,SQL*Net roundtrips to/from client,sorts (memory),sorts (disk),sorts (rows),CPU used by this session";

	conf_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
					  NULL, g_free);

	style = gtk_widget_get_style(lw("ddlText"));
	font = pango_font_description_to_string(style->font_desc);
	config_set("textfont", g_strdup(font) );
	g_snprintf(text_font, sizeof(text_font), "%s", font);
	g_free(font);

	style = gtk_widget_get_style(lw("resList"));
	font = pango_font_description_to_string(style->font_desc);
	config_set("listfont", g_strdup(font) );
	g_snprintf(tree_font, sizeof(tree_font), "%s", font);
	g_free(font);

	/* set default values */
	config_set("oracle_user", "");
	config_set("oracle_host", "");
	config_set("oracle_port", "1521");
	config_set("connections", "");
	config_set("trace_list", trace_default);

	if (oracle_sid) {
		config_set("oracle_sid", oracle_sid );
	}
	else {
		config_set("oracle_sid", "");
	}

	config_set("external_editor", "xemacs");
	config_set("external_viewer", "gnumeric");

	config_set("textview_linenumbers", "1");
	config_set("textview_linemarkers", "0");
	config_set("textview_highlight_current", "0");
	config_set("textview_margin", "0");

	if ((config_file = g_build_filename(g_get_home_dir(), ".gsqlr2", NULL)) != NULL) {
    
		if( g_file_test(config_file, G_FILE_TEST_IS_REGULAR)) {
			gchar *new_config_file=0;

			new_config_file = g_build_filename(g_get_home_dir(), ".gtko2", NULL);
			rename(config_file, new_config_file);
			g_free(new_config_file);
		}

		g_free(config_file);
	}

	if ((config_file = g_build_filename(g_get_home_dir(), ".gtko2", NULL)) != NULL) {
 
		if(g_file_test(config_file, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
			GKeyFile *keyfile=0;
			guint x, pagenum;
			gint width, height;
			GError *widtherr=0, *heighterr=0;

			keyfile = g_key_file_new();

			if( g_key_file_load_from_file(keyfile, config_file, 
						      G_KEY_FILE_KEEP_COMMENTS, 
						      (GError **) NULL) == TRUE)
			{
				gchar **kv=0;
				gsize  ksize;

				kv = g_key_file_get_keys(keyfile, "defaults", 
							 &ksize, (GError **) NULL);
				for(x=0; x<ksize; x++) {
					config_set(g_strdup(kv[x]), 
						   g_key_file_get_string(keyfile,
									 "defaults", kv[x],
									 (GError **)NULL));
				}
	
				width  = g_key_file_get_integer(keyfile, "defaults",
								"width", &widtherr );
				height = g_key_file_get_integer(keyfile, "defaults",
								"height", &heighterr );
				if (!widtherr && !heighterr) {
					gtk_window_resize(GTK_WINDOW(window1), width, height);
				}

				g_clear_error(&widtherr);
				g_clear_error(&heighterr);

				width = g_key_file_get_integer(keyfile, "defaults",
							       "buffer_width", &widtherr );
				if(!widtherr)
					gtk_paned_set_position(GTK_PANED(lw("vpaned1")), width);

				g_clear_error(&widtherr);
				width = g_key_file_get_integer(keyfile, "defaults",
							       "table_width", &widtherr );
				if (!widtherr) {
					gtk_paned_set_position(GTK_PANED(lw("objectPane")), width);
				}

				pagenum = g_key_file_get_integer(keyfile, "buffers", "buffer_count",
								 (GError **)NULL);
				for(x=0; x<pagenum; x++) {
					gchar *buffer_label=0, *buffer_text=0;
					gchar buffername[128];
					GtkTextView *tview=0;
					GtkTextBuffer *tbuff=0;
					GtkTextIter start;
					const gchar *tfont = config_get("textfont");

					snprintf(buffername, sizeof(buffername), "buffername_%d", x);
					buffer_label = g_key_file_get_string(keyfile, "buffers",
									     buffername, (GError **)NULL);

					if (!buffer_label) {
						buffer_label = g_strdup("SQL");
					}

					tview = GTK_TEXT_VIEW(command_tab_add(buffer_label));
					tbuff = gtk_text_view_get_buffer(tview);
					g_free(buffer_label);

					gtk_widget_set_font(GTK_WIDGET(tview), tfont);

					snprintf(buffername, sizeof(buffername), "buffer_%d", x);
					buffer_text = g_key_file_get_string(keyfile, "buffers",
									    buffername, (GError **)NULL);
					if (buffer_text) {
						gtk_text_buffer_get_iter_at_offset(tbuff, &start, 0);
						gtk_text_buffer_insert(tbuff, &start, g_strdup(buffer_text), -1);
						g_free(buffer_text);
	  
#ifndef WITH_GTKSOURCEVIEW
						buffer_syntax_highlight(tbuff, -1);
#endif
					}
				}

				pagenum = g_key_file_get_integer(keyfile, "buffers",
								 "buffer_active", &widtherr);
				if(!widtherr)
					gtk_notebook_set_current_page(GTK_NOTEBOOK(lw("sqlNotebook")), pagenum);

 
				g_key_file_free(keyfile);
			}
		}
		else {
			command_tab_add("SQL");
		}
		g_free(config_file);
	}
}


void config_write(void) 
{
	GKeyFile *keyfile=0;
	gint file;
	gchar *config_file=0, *data=0, *buffer=0;
	gsize length;
  

	config_file = g_build_filename(g_get_home_dir(), ".gtko2", NULL);
	if (!config_file) {
		return;
	}
  
	unlink(config_file);
	file = open(config_file, O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR);
	if (file != -1) {
    
		keyfile = g_key_file_new();
		if (keyfile) {
			gint p, pages;
			gchar pagename[64];
			gint width, height;
			gchar *config[] = {"oracle_user", 
					   "oracle_sid", 
					   "oracle_host", 
					   "oracle_port", 
					   "oracle_mode", 
					   "connections", 
					   "trace_list", 
					   "textfont", 
					   "listfont", 
					   "external_viewer", 
					   "external_editor",
					   "textview_linenumbers", 
					   "textview_linemarkers",
					   "textview_highlight_current",
					   "textview_margin"};

      
			for (p=0; p<(sizeof(config)/sizeof(config[0])); p++) {
				const gchar *value=0;

				value = config_get(config[p]);
				if (value)
					g_key_file_set_string(keyfile, "defaults", config[p], value);
				else
					g_key_file_set_string(keyfile, "defaults", config[p], "");
			}

			gtk_window_get_size(GTK_WINDOW(window1), &width, &height);
			g_key_file_set_integer(keyfile, "defaults", "width", width);
			g_key_file_set_integer(keyfile, "defaults", "height", height);
      
			width = gtk_paned_get_position(GTK_PANED(lw("vpaned1")));
			g_key_file_set_integer(keyfile, "defaults", "buffer_width", width);

			width = gtk_paned_get_position(GTK_PANED(lw("objectPane")));
			g_key_file_set_integer(keyfile, "defaults", "table_width", width);

			g_key_file_remove_group(keyfile, "buffers", NULL);

			pages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(lw("sqlNotebook")));
			g_key_file_set_integer(keyfile, "buffers", "buffer_count", pages);
			if (pages) {

				g_key_file_set_integer(keyfile, "buffers", "buffer_active", 
						       gtk_notebook_get_current_page(GTK_NOTEBOOK(lw("sqlNotebook"))));
      
				for (p=0; p<pages; p++) {
					GtkWidget *page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw("sqlNotebook")), p);
					const gchar *tablabel = gtk_label_get_text(GTK_LABEL(g_object_get_data(G_OBJECT(page), "labelptr")));
							      
					buffer = getCommandText(p);
					snprintf(pagename, 64, "buffername_%d", p);
					g_key_file_set_string(keyfile, "buffers", pagename, tablabel);
					snprintf(pagename, 64, "buffer_%d", p);
					g_key_file_set_string(keyfile, "buffers", pagename, buffer ? buffer : "");
				}
			}

			data = g_key_file_to_data(keyfile, &length, (GError **)NULL);

#if DEBUG
			/*	fprintf(stderr, "appconfig.c:\n%s\n", data); */
#endif

			write(file, data, length);
			chmod(config_file, S_IRUSR | S_IWUSR );
			close(file);

			g_free(data);
			g_key_file_free(keyfile);
		}
	}

	g_free(config_file);
}





static void on_confOKButton_clicked(GtkButton *button,
				    gpointer user_data)
{
	if (conf_window) {
		const gchar *fontname=0;
		guint x;
		gint pages;

		fontname = gtk_font_button_get_font_name(GTK_FONT_BUTTON(treeviewFontButton));
		g_snprintf(tree_font, sizeof(tree_font), "%s", fontname);
		config_set("listfont", fontname);

		fontname = gtk_font_button_get_font_name(GTK_FONT_BUTTON(textviewFontButton));
		g_snprintf(text_font, sizeof(text_font), "%s", fontname);
		config_set("textfont", fontname);

		pages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(lw("sqlNotebook")) );
		for(x=0; x<pages; x++) {
			GtkWidget *tview=0;
      
			tview = gtk_bin_get_child(GTK_BIN(gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw("sqlNotebook")), x) ) );
			gtk_widget_set_font(tview, fontname);

#if WITH_GTKSOURCEVIEW
			gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(tview),
							      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewLineNumberCheck)));
			gtk_source_view_set_show_line_markers(GTK_SOURCE_VIEW(tview),
							      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewLineMarkerCheck)));
			gtk_source_view_set_show_margin(GTK_SOURCE_VIEW(tview),
							gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewMarginCheck)));
			gtk_source_view_set_highlight_current_line(GTK_SOURCE_VIEW(tview),
								   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewCurrentLineCheck)));
#endif
		}

		gtk_widget_set_font(lw("ddlText"), fontname);

		config_set("external_editor", gtk_entry_get_text(GTK_ENTRY(textviewExternalEntry)) );
		config_set("external_viewer", gtk_entry_get_text(GTK_ENTRY(treeviewExternalEntry)) );

#if GTK_CHECK_VERSION(2,10,0)
		gtk_tree_view_set_grid_lines(GTK_TREE_VIEW(lw("resList")),
					     gtk_combo_box_get_active(GTK_COMBO_BOX(treeviewGridlinesCombo)) );
		gtk_tree_view_set_rubber_banding(GTK_TREE_VIEW(lw("resList")),
						 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(treeviewRubberbandingCheck)));
#endif

#if WITH_GTKSOURCEVIEW
		config_set_integer("textview_linenumbers",
				   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewLineNumberCheck)) );
		config_set_integer("textview_linemarkers",
				   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewLineMarkerCheck)) );
		config_set_integer("textview_margin",
				   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewMarginCheck)) );
		config_set_integer("textview_highlight_current",
				   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(textviewCurrentLineCheck)));
#endif
		gtk_widget_destroy(conf_window);
		conf_window=0;
	}
}



static void on_confCancelButton_clicked(GtkButton *button,
					gpointer user_data)
{
	if (conf_window) {
		gtk_widget_destroy(conf_window);
		conf_window=0;
	} 
}




void on_extviewerChooserButton_selection_changed(GtkFileChooser *filechooser,
						 gpointer user_data)
{
	gchar *filename = gtk_file_chooser_get_filename(filechooser);

	if (filename) {
		gtk_entry_set_text(GTK_ENTRY(treeviewExternalEntry), filename);
		g_free(filename);
	}

}


void on_exteditorChooserButton_selection_changed(GtkFileChooser *filechooser,
						 gpointer user_data)
{
	gchar *filename = gtk_file_chooser_get_filename(filechooser);

	if (filename) {
		gtk_entry_set_text(GTK_ENTRY(textviewExternalEntry), filename);
		g_free(filename);
	}

}



GtkWidget*
create_confWindow (void)
{
  GtkWidget *confWindow=0;
  GtkWidget *vbox1=0;
  GtkWidget *notebook1=0;
  GtkWidget *vbox3=0;
  GtkWidget *frame1=0;
  GtkWidget *alignment3=0;
  GtkWidget *vbox4=0;
  GtkWidget *hbox7=0;
  GtkWidget *label12=0;
  GtkWidget *hbox9=0;
  GtkWidget *label14=0;
  GtkWidget *label10=0;
  GtkWidget *frame2=0;
  GtkWidget *alignment4=0;
  GtkWidget *vbox5=0;
  GtkWidget *hbox8=0;
  GtkWidget *label13=0;
  GtkWidget *hbox10=0;
  GtkWidget *hbox11=0;
  GtkWidget *hbox12=0;
  GtkWidget *label15=0;
  GtkObject *textviewUndoSpin_adj=0;
  GtkWidget *label16=0;
  GtkObject *textviewTabWidthSpin_adj=0;
  GtkWidget *label11=0;
  GtkWidget *label1=0;
  GtkWidget *vbox2=0;
  GtkWidget *hbox4=0;
  GtkWidget *label7=0;
  GtkWidget *hbox5=0;
  GtkWidget *label8=0;
  GtkWidget *hbox6=0;
  GtkWidget *label9=0;
  GtkWidget *label2=0;
  GtkWidget *hbox1=0;
  GtkWidget *confOKButton=0;
  GtkWidget *alignment2=0;
  GtkWidget *hbox3=0;
  GtkWidget *image2=0;
  GtkWidget *label6=0;
  GtkWidget *confCancelButton=0;
  GtkWidget *alignment1=0;
  GtkWidget *hbox2=0;
  GtkWidget *image1=0;
  GtkWidget *label5=0;
  const gchar *extprog=0;

  confWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (confWindow), _("GTKO Preferences"));
  gtk_window_set_position (GTK_WINDOW (confWindow), GTK_WIN_POS_CENTER);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (confWindow), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (confWindow), TRUE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (confWindow), vbox1);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook1, TRUE, TRUE, 0);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook1), GTK_POS_LEFT);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox3);

  frame1 = gtk_frame_new (NULL);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox3), frame1, TRUE, TRUE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 2);

  alignment3 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment3);
  gtk_container_add (GTK_CONTAINER (frame1), alignment3);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment3), 0, 0, 12, 0);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (alignment3), vbox4);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox7, FALSE, FALSE, 0);

  label12 = gtk_label_new (_("Font:"));
  gtk_widget_show (label12);
  gtk_box_pack_start (GTK_BOX (hbox7), label12, FALSE, FALSE, 0);
  gtk_label_set_width_chars (GTK_LABEL (label12), 8);

  treeviewFontButton = gtk_font_button_new ();
  gtk_widget_show (treeviewFontButton);
  gtk_box_pack_start (GTK_BOX (hbox7), treeviewFontButton, FALSE, FALSE, 0);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox9);
  gtk_box_pack_start (GTK_BOX (vbox4), hbox9, FALSE, FALSE, 0);

  treeviewRubberbandingCheck = gtk_check_button_new_with_mnemonic (_("rubberbanding"));
  gtk_widget_show (treeviewRubberbandingCheck);
  gtk_box_pack_start (GTK_BOX (hbox9), treeviewRubberbandingCheck, FALSE, FALSE, 0);

  label14 = gtk_label_new (_("gridlines:"));
  gtk_widget_show (label14);
  gtk_box_pack_start (GTK_BOX (hbox9), label14, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label14), 0.9, 0.5);
  gtk_label_set_width_chars (GTK_LABEL (label14), 10);

  treeviewGridlinesCombo = gtk_combo_box_new_text ();
  gtk_widget_show (treeviewGridlinesCombo);
  gtk_box_pack_start (GTK_BOX (hbox9), treeviewGridlinesCombo, FALSE, FALSE, 0);
  gtk_combo_box_append_text (GTK_COMBO_BOX (treeviewGridlinesCombo), _("None"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (treeviewGridlinesCombo), _("Horizontal"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (treeviewGridlinesCombo), _("Vertical"));
  gtk_combo_box_append_text (GTK_COMBO_BOX (treeviewGridlinesCombo), _("Both"));


  label10 = gtk_label_new (_("<b>GtkTreeView</b>"));
  gtk_widget_show (label10);
  gtk_frame_set_label_widget (GTK_FRAME (frame1), label10);
  gtk_label_set_use_markup (GTK_LABEL (label10), TRUE);

  frame2 = gtk_frame_new (NULL);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox3), frame2, TRUE, TRUE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 2);

  alignment4 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_widget_show (alignment4);
  gtk_container_add (GTK_CONTAINER (frame2), alignment4);
  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment4), 0, 0, 12, 0);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (alignment4), vbox5);

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox8);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox8, FALSE, FALSE, 0);

  label13 = gtk_label_new (_("Font:"));
  gtk_widget_show (label13);
  gtk_box_pack_start (GTK_BOX (hbox8), label13, FALSE, FALSE, 0);
  gtk_label_set_width_chars (GTK_LABEL (label13), 8);

  textviewFontButton = gtk_font_button_new ();
  gtk_widget_show (textviewFontButton);
  gtk_box_pack_start (GTK_BOX (hbox8), textviewFontButton, FALSE, FALSE, 0);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox10, TRUE, TRUE, 0);

  textviewCurrentLineCheck = gtk_check_button_new_with_mnemonic (_("highlight line"));
  gtk_box_pack_start (GTK_BOX (hbox10), textviewCurrentLineCheck, FALSE, FALSE, 10);

  textviewLineNumberCheck = gtk_check_button_new_with_mnemonic (_("line numbers"));
  gtk_box_pack_end (GTK_BOX (hbox10), textviewLineNumberCheck, TRUE, TRUE, 10);

  hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox11);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox11, TRUE, TRUE, 0);

  textviewLineMarkerCheck = gtk_check_button_new_with_mnemonic (_("line markers"));
  gtk_box_pack_start (GTK_BOX (hbox11), textviewLineMarkerCheck, FALSE, FALSE, 10);

  textviewMarginCheck = gtk_check_button_new_with_mnemonic (_("show margin"));
  gtk_box_pack_start (GTK_BOX (hbox11), textviewMarginCheck, FALSE, FALSE, 10);

  hbox12 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox12);
  gtk_box_pack_start (GTK_BOX (vbox5), hbox12, TRUE, TRUE, 0);

  label15 = gtk_label_new (_("undo levels:"));
  gtk_box_pack_start (GTK_BOX (hbox12), label15, FALSE, FALSE, 0);

  textviewUndoSpin_adj = gtk_adjustment_new (100, 1, 1000, 1, 10, 10);
  textviewUndoSpin = gtk_spin_button_new (GTK_ADJUSTMENT (textviewUndoSpin_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (hbox12), textviewUndoSpin, FALSE, FALSE, 0);

  label16 = gtk_label_new (_("tab width:"));
  gtk_box_pack_start (GTK_BOX (hbox12), label16, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label16), 0.9, 0.5);
  gtk_label_set_width_chars (GTK_LABEL (label16), 10);

  textviewTabWidthSpin_adj = gtk_adjustment_new (2, 1, 20, 1, 10, 10);
  textviewTabWidthSpin = gtk_spin_button_new (GTK_ADJUSTMENT (textviewTabWidthSpin_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (hbox12), textviewTabWidthSpin, FALSE, FALSE, 0);

#if WITH_GTKSOURCEVIEW
  gtk_widget_show (textviewCurrentLineCheck);
  gtk_widget_show (textviewLineNumberCheck);
  gtk_widget_show (textviewLineMarkerCheck);
  gtk_widget_show (textviewMarginCheck);
  gtk_widget_show (label15);
  gtk_widget_show (textviewUndoSpin);
  gtk_widget_show (label16);
  gtk_widget_show (textviewTabWidthSpin);
  label11 = gtk_label_new (_("<b>GtkSourceView</b>"));
#else
  label11 = gtk_label_new (_("<b>GtkTextView</b>"));
#endif

  gtk_widget_show (label11);
  gtk_frame_set_label_widget (GTK_FRAME (frame2), label11);
  gtk_label_set_use_markup (GTK_LABEL (label11), TRUE);

  label1 = gtk_label_new (_("Interface"));
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox2);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox4), 2);

  label7 = gtk_label_new (_("External Editor:"));
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (hbox4), label7, FALSE, FALSE, 0);
  gtk_label_set_width_chars (GTK_LABEL (label7), 14);

  textviewExternalEntry = gtk_entry_new ();
  gtk_widget_show (textviewExternalEntry);
  gtk_box_pack_start (GTK_BOX (hbox4), textviewExternalEntry, TRUE, TRUE, 0);
 
  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox5, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox5), 2);

  label8 = gtk_label_new (_("External Viewer:"));
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox5), label8, FALSE, FALSE, 0);
  gtk_label_set_width_chars (GTK_LABEL (label8), 14);

  treeviewExternalEntry = gtk_entry_new ();
  gtk_widget_show (treeviewExternalEntry);
  gtk_box_pack_start (GTK_BOX (hbox5), treeviewExternalEntry, TRUE, TRUE, 0);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox6);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox6, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbox6), 2);

  label9 = gtk_label_new (_("HTML Viewer:"));
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (hbox6), label9, FALSE, FALSE, 0);
  gtk_label_set_width_chars (GTK_LABEL (label9), 14);

  htmlExternalEntry = gtk_entry_new ();
  gtk_widget_show (htmlExternalEntry);
  gtk_box_pack_start (GTK_BOX (hbox6), htmlExternalEntry, TRUE, TRUE, 0);

  label2 = gtk_label_new (_("Tools"));
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

  confOKButton = gtk_button_new ();
  gtk_widget_show (confOKButton);
  gtk_box_pack_end (GTK_BOX (hbox1), confOKButton, FALSE, FALSE, 8);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (confOKButton), alignment2);

  hbox3 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox3);

  image2 = gtk_image_new_from_stock ("gtk-ok", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_box_pack_start (GTK_BOX (hbox3), image2, FALSE, FALSE, 0);

  label6 = gtk_label_new_with_mnemonic (_("OK"));
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox3), label6, FALSE, FALSE, 0);
  gtk_label_set_width_chars (GTK_LABEL (label6), 6);

  confCancelButton = gtk_button_new ();
  gtk_widget_show (confCancelButton);
  gtk_box_pack_end (GTK_BOX (hbox1), confCancelButton, FALSE, FALSE, 8);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (confCancelButton), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image1 = gtk_image_new_from_stock ("gtk-cancel", GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

  label5 = gtk_label_new_with_mnemonic (_("Cancel"));
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox2), label5, FALSE, FALSE, 0);
  gtk_label_set_width_chars (GTK_LABEL (label5), 6);

  g_signal_connect ((gpointer) confOKButton, "clicked",
                    G_CALLBACK (on_confOKButton_clicked),
                    NULL);
  g_signal_connect ((gpointer) confCancelButton, "clicked",
                    G_CALLBACK (on_confCancelButton_clicked),
                    NULL);


  gtk_font_button_set_font_name(GTK_FONT_BUTTON(treeviewFontButton), config_get("listfont") );
  gtk_font_button_set_font_name(GTK_FONT_BUTTON(textviewFontButton), config_get("textfont") );
  gtk_combo_box_set_active(GTK_COMBO_BOX(treeviewGridlinesCombo), 0);

  if( (extprog = config_get("external_editor")) != NULL) 
    gtk_entry_set_text(GTK_ENTRY(textviewExternalEntry), extprog);
  else
    gtk_entry_set_text(GTK_ENTRY(textviewExternalEntry), "xemacs");

 if( (extprog = config_get("external_viewer")) != NULL) 
    gtk_entry_set_text(GTK_ENTRY(treeviewExternalEntry), extprog);
  else
    gtk_entry_set_text(GTK_ENTRY(treeviewExternalEntry), "gnumeric");

 if( (extprog = config_get("external_html")) != NULL) 
    gtk_entry_set_text(GTK_ENTRY(htmlExternalEntry), extprog);
  else
    gtk_entry_set_text(GTK_ENTRY(htmlExternalEntry), "firefox");


#if WITH_GTKSOURCEVIEW
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(textviewLineNumberCheck), config_get_integer("textview_linenumbers") );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(textviewLineMarkerCheck), config_get_integer("textview_linemarkers") );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(textviewMarginCheck), config_get_integer("textview_margin") );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(textviewCurrentLineCheck), config_get_integer("textview_highlight_current") );
#endif


  return(confWindow);
}



void config_window_show() 
{
	if (!conf_window) {
		conf_window = create_confWindow();
		gtk_widget_show(conf_window);
	}
}
