/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <oci.h>

#include "bind_variable.h"
#include "gutil.h"
#include "appconfig.h"
#include "query.h"
#include "orautil.h"


static void bind_set_combo_from_define(GtkTextBuffer *buffer,
				       const gchar *bindname,
				       GtkComboBox *combo)
{
	GtkTextIter iter1;
	GtkTextIter iter2;
	gchar *line=0, *bindname_amp=0;
	GList *qvalues=0;


#if DEBUG
	fprintf(stderr, "bind_variable.c: checking for define for %s\n",
		bindname);
#endif

	if (!bindname || !buffer || !combo) {
		return;
	}

	bindname_amp = g_strdup(&bindname[1]);

	for (gtk_text_buffer_get_start_iter(buffer, &iter1);
	     !gtk_text_iter_is_end(&iter1);
	     gtk_text_iter_forward_line(&iter1) ) {

		GArray *toks=0;
		guint numwords=0;

		iter2 = iter1;
		gtk_text_iter_forward_to_line_end(&iter2);
		line = gtk_text_buffer_get_text(buffer, &iter1,
						&iter2, FALSE);
    
		toks = g_strsplit_garray(line, " ", -1);
		g_free(line);

		numwords = toks->len;

		if (!g_ascii_strncasecmp(g_array_index(toks, gchar *, 0), "define", 6)
		    && !g_ascii_strncasecmp(g_array_index(toks, gchar *, 1), bindname_amp, strlen(bindname_amp)) 
		    && !g_ascii_strncasecmp(g_array_index(toks, gchar *, 2), "=", 1)
		    && !g_ascii_strncasecmp(g_array_index(toks, gchar *, 3), "select", 5)
			) {
			
			GString *qry_str = g_string_new("select ");
			guint x=4;

			for(x=4; x < numwords; x++) {
				qry_str = g_string_append(qry_str, g_array_index(toks, gchar *, x));
				qry_str = g_string_append(qry_str, " ");
			}

#if DEBUG
			fprintf(stderr, "bind_variable.c: query for bind variable %s:\n\t'%s'\n",
				bindname, qry_str->str );
#endif
			qvalues = execute_sql_to_glist(qry_str->str, NULL);
			g_string_free(qry_str, TRUE);

			for( ; qvalues; qvalues = g_list_next(qvalues)) {
				gtk_combo_box_append_text(combo, (gchar *) qvalues->data);
				g_free(qvalues->data);
			}

			g_list_free(qvalues);
			gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(combo))),
					   "Pick...");
		}
		g_strsplit_garray_free(toks);
	}

	g_free(bindname_amp);
}


void create_bind_entries() 
{
	GtkTextView *view=0;
	GtkTextIter iter;
	GtkTextIter wsiter;
	GtkTextBuffer *buffer=0;
	GtkWidget *entry=0;
	GtkTextMark *entry_mark=0;
	gchar c;
	gchar tipbuf[128], *bindname=0;
	GSList *blist=0;
	GdkColor color;
	const gchar *font=0;


	buffer = getCurrentSqlBuffer(-1);
	view = getCurrentTextView();

	gdk_color_parse("#faf8e4", &color);

	blist = g_object_get_data(G_OBJECT(view), "bindlist");
	if (blist) {
		GSList *tmplist=0;

		fprintf(stderr, "clearing previous bind_list of length %d\n", 
			g_slist_length(blist) );

		for (tmplist=blist; tmplist; tmplist = g_slist_next(tmplist) ) {
			GtkTextMark *mark=0;

			mark = g_object_get_data(G_OBJECT(tmplist->data),
						 "entry_mark");

			if (mark) {
				GtkTextIter miter;

				gtk_text_buffer_get_iter_at_mark(buffer, 
								 &miter, mark);
				gtk_text_iter_forward_char(&miter);
				gtk_text_buffer_backspace(buffer, &miter,
							  TRUE, TRUE);
			}
		}

		blist = NULL;
	}

	gtk_text_buffer_get_start_iter(buffer, &iter);
	while (gtk_text_iter_forward_char(&iter)) {
		GtkTextChildAnchor *child_anchor=0;
		guint combo_set=0;

		c = gtk_text_iter_get_char(&iter);
		if (c == '\'' || c == '"') {

			gtk_text_iter_forward_char(&iter); 

			while (gtk_text_iter_get_char(&iter) != c 
			       && !gtk_text_iter_is_end(&iter) )
				gtk_text_iter_forward_char(&iter); 

		}

		if (gtk_text_iter_is_end(&iter)) {
			break;
		}
  
		if (gtk_text_iter_get_char(&iter) == ':' 
		    || gtk_text_iter_get_char(&iter) == '&') {
			gint offset=0;

			if (gtk_text_iter_get_char(&iter) == '&') {
				combo_set = 1;
			}

			wsiter = iter;
			offset = gtk_text_iter_get_offset(&wsiter);

			gtk_text_iter_forward_char(&iter);
			c = gtk_text_iter_get_char(&iter);

			if (!g_ascii_isalpha(c)) {
				break;
			}

			for ( ; g_ascii_isalnum(c) || c == '_'; 
			      c = gtk_text_iter_get_char(&iter)) {
				gtk_text_iter_forward_char(&iter);
			}

			bindname = gtk_text_buffer_get_text(buffer, &wsiter,
							    &iter, FALSE);

#if DEBUG
			fprintf(stderr, "bind_variable.c: bindname = '%s'\n", 
				bindname);
#endif

			snprintf(tipbuf, 128, "Bind value for %s", bindname);
			font = config_get("textfont");

			if (combo_set) {
				GtkWidget *entry_child=0;
  
				entry = gtk_combo_box_entry_new_text();
				entry_child = gtk_bin_get_child(GTK_BIN(entry));
				gtk_widget_modify_base(entry_child, GTK_STATE_NORMAL,
						       &color);

				if (font) {
					gtk_widget_set_font(entry_child, font);
				}

				bind_set_combo_from_define(buffer, bindname,
							   GTK_COMBO_BOX(entry) );
			}
			else {

				entry = gtk_entry_new();
				gtk_widget_modify_base(entry, GTK_STATE_NORMAL,
						       &color);

				if(font) {
					gtk_widget_set_font(entry, font);
				}

				gtk_tooltips_set_tip (gtk_tooltips_new(), entry,
						      tipbuf, NULL);
			}

			gtk_widget_set_size_request(entry, -1, 20);
			g_object_set_data(G_OBJECT(entry), "offset", GINT_TO_POINTER(offset) );

			blist = g_slist_append(blist, (gpointer *) entry);
			g_free(bindname);

			child_anchor = gtk_text_buffer_create_child_anchor(buffer, &iter);
			gtk_text_view_add_child_at_anchor(view, entry, child_anchor);

			gtk_text_iter_backward_char(&iter);
			entry_mark = gtk_text_buffer_create_mark(buffer, (gchar *)NULL,
								 &iter, FALSE);
			g_object_set_data(G_OBJECT(entry), "entry_mark", entry_mark);
			gtk_text_iter_forward_char(&iter);

			gtk_widget_show_all(entry);
		}
	}

	g_object_set_data(G_OBJECT(view), "bindlist", blist);
}



void bind_variable_set(OCIStmt *sth,
		       OCIError *errhp,
		       gint offset)
{
	ub4 bindno = 1;
	GSList *blist=0;
	GtkEntry *bind_entry=0;
	GtkComboBox *bind_combo=0;
	gint bind_offset=0;
	const gchar *bind_value=0;

	gint bufnum = gtk_notebook_get_current_page(GTK_NOTEBOOK(lw("sqlNotebook")));
	GtkWidget *scrwin = gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw("sqlNotebook")), bufnum);
	GtkTextView *view = GTK_TEXT_VIEW(gtk_bin_get_child(GTK_BIN(scrwin)));


	if ( !(blist = (GSList *) g_object_get_data(G_OBJECT(view), "bindlist")) ) {
		return;
	}

#if DEBUG
	fprintf(stderr, "blist len: %d\n", g_slist_length(blist) );
#endif	

	for( ;
	     blist; 
	     blist = g_slist_next(blist) ) {

		if (blist->data) {
			if (G_IS_OBJECT(G_OBJECT(blist->data))) {

				if (GTK_IS_ENTRY(blist->data)) {
					bind_entry = GTK_ENTRY(blist->data);
					bind_value = gtk_entry_get_text(GTK_ENTRY(bind_entry) );
					bind_offset = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(bind_entry),
											  "offset") );
				}
				else {
					bind_combo = GTK_COMBO_BOX(blist->data);
					bind_value = gtk_combo_box_get_active_text(bind_combo);
					bind_offset = GPOINTER_TO_INT( g_object_get_data(G_OBJECT(bind_combo),
											  "offset") );
				}
			}
		}

		if (bind_offset > offset) {

#if DEBUG
			fprintf(stderr, "query_async.c: binding pos %d : '%s'\n",
				bindno, bind_value);
#endif

			ociBindString(sth, errhp, bindno++, (char *)bind_value);  
		}

	}

}
