/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <gtk/gtk.h>
#include <oci.h>

#include "gutil.h"
#include "appconfig.h"
#include "query.h"
#include "syntax.h"
#include "orautil.h"
#include "table.h"
#include "browser_sql.h"
#include "browser.h"

extern int oramajor;
extern int oraminor;
extern OCIEnv *envhp;
extern OCIError *errhp;
extern OCISvcCtx *svchp;

static GtkTreeView *objectBrowser=0;
static GtkTreeStore *objectStore=0;

gchar fg_normal[32];
gulong comment_signal=0;


browsernode java_nodes[] = {
	{B_JAVA_CLASSES, "Classes", "none", 9, NULL},
	{B_JAVA_METHODS, "Methods", "none", 9, NULL},
	{-1, NULL, NULL, -1, NULL}
};

browsernode pubsyn_nodes[] = {
	{B_PUBLIC_SYNONYM_ALL, "ALL_", "synonym_pixbuf", 9, NULL},
	{B_PUBLIC_SYNONYM_DBA, "DBA_", "synonym_pixbuf", 9, NULL},
	{B_PUBLIC_SYNONYM_USER, "USER_", "synonym_pixbuf", 9, NULL},
	{B_PUBLIC_SYNONYM_VDOLLAR, "V$", "synonym_pixbuf", 9, NULL},
	{B_PUBLIC_SYNONYM_OTHERS, "Others", "synonym_pixbuf", 9, NULL},
	{-1, NULL, NULL, -1, NULL}
};

browsernode plsql_nodes[] = {
	{B_FUNCTIONS, "Functions", "function_pixbuf", 9, NULL},
	{B_PROCEDURES, "Procedures", "procedure_pixbuf", 9, NULL},
	{B_PACKAGES, "Packages", "package_pixbuf", 9, NULL},
	{B_TYPES, "Types", "type_pixbuf", 9, NULL},
	{B_TRIGGERS, "Triggers", "none", 9, NULL},
	{B_JAVA, "Java", "none", 9, java_nodes},
	{-1, NULL, NULL, -1, NULL}
};

browsernode scheduler_nodes[] = {
	{B_SCHEDULER_SCHEDULES, "Schedules", "schedule_pixbuf", 10},
	{B_SCHEDULER_WINDOWS, "Windows", "schedule_pixbuf", 10},
	{B_SCHEDULER_JOBS, "Jobs", "schedule_pixbuf", 10},
	{B_SCHEDULER_PROGRAMS, "Programs", "schedule_pixbuf", 10},
	{-1, NULL, NULL, -1, NULL}
};

browsernode top_nodes[] = {
	{B_TABLES, "Tables", "heap_pixbuf", 9, NULL},
	{B_INDEXES, "Indexes", "iot_pixbuf", 9, NULL},
	{B_VIEWS, "Views", "view_pixbuf", 9, NULL},
	{B_CLUSTERS, "Clusters", "cluster_pixbuf", 9, NULL},
	{B_MVIEWS, "MViews", "mview_pixbuf", 9, NULL},
	{B_MVIEW_LOGS, "Mview Logs", "mview_log_pixbuf", 9, NULL},
	{B_SYNONYMS, "Synonyms", "synonym_pixbuf", 9, NULL},
	{B_PUBLIC_SYNONYMS, "Public Synonyms", "synonym_pixbuf", 9, pubsyn_nodes},
	{B_QUEUES, "Queues", "queue_pixbuf", 9, NULL},
	{B_SEQUENCES, "Sequences", "sequence_pixbuf", 9, NULL},
	{B_PLSQL, "PLSQL Source", "plsql_pixbuf", 9, plsql_nodes},
	{B_LOBS, "LOBs", "lob_pixbuf", 9, NULL},
	{B_LINKS, "DB Links", "dblink_pixbuf", 9, NULL},
	{B_DIRECTORIES, "Directories", "directory_pixbuf", 9, NULL},
	{B_LIBRARIES, "Libraries", "library_pixbuf", 9, NULL},
	{B_SCHEDULER, "Scheduler", "schedule_pixbuf", 10, scheduler_nodes},
	{B_RECYCLEBIN, "Recycle Bin", "recyclebin_pixbuf", 10, NULL},
	{-1, NULL, NULL, -1, NULL}
};

browsernode asm_nodes[] = {
	{B_ASM_DISKGROUP, "Disk Groups", "none", 10, NULL},
	{B_ASM_DISK, "Disks", "none", 10, NULL},
	{B_ASM_FILE, "Files", "none", 10, NULL},
	{-1, NULL, NULL, -1, NULL}
};

browsernode rman_nodes[] = {
	{B_RMAN_BACKUPS, "Backups", "none", 10, NULL},
	{B_RMAN_ALGOS, "Algorithms", "none", 10.2, NULL},
	{-1, NULL, NULL, -1, NULL}
};

browsernode datafile_nodes[] = {
	{B_DATAFILE_10METRIC, "10min Metrics", "none", 10, NULL},
	{B_DATAFILE_1HMETRIC, "Metric Hist", "none", 10, NULL},
	{-1, NULL, NULL, -1, NULL}
};

browsernode file_nodes[] = {
	/* {B_CONTROLFILE, "Contolfile", "none", 9, NULL}, */
	{B_TABLESPACES, "Tablespaces", "none", 9, NULL},
	{B_DATAFILES, "Datafiles", "none", 9, datafile_nodes},
	{B_TEMPFILES, "Tempfiles", "none", 9, NULL},
	{B_REDOLOGS, "Redo Logs", "none", 9, NULL},
	{B_ARCHIVEDLOGS, "Archive Logs", "none", 9, NULL},
	{B_ASM, "ASM", "none", 10, asm_nodes},
	{B_RMAN, "RMAN", "none", 10, rman_nodes},    
	{-1, NULL, NULL, -1, NULL}
};

browsernode security_nodes[] = {
	{B_USERS, "Users", "none", 9, NULL},
	{B_ROLES, "Roles", "none", 9, NULL},
	{B_PROFILES, "Profiles", "none", 9, NULL},
	{-1, NULL, NULL, -1, NULL}
};

browsernode advice_nodes[] = {
	{B_ADVICE_DBCACHE, "DB Cache", "advice_pixbuf", 9},
	{B_ADVICE_PXBUFFER, "PX Buffer", "advice_pixbuf", 10},
	{B_ADVICE_POOL, "Sh Pool", "advice_pixbuf", 9},
	{B_ADVICE_SGA, "SGA Target", "advice_pixbuf", 10.2},
	{B_ADVICE_PGA, "PGA", "advice_pixbuf", 9},
	{B_ADVICE_JAVA, "Java", "advice_pixbuf", 10},
	{B_ADVICE_MTTR, "MTTR", "advice_pixbuf", 9},
	{B_ADVICE_RAC, "RAC", "advice_pixbuf", 11},
	{B_ADVICE_STREAMS, "Streams", "advice_pixbuf", 10.2},
	{B_ADVICE_PARTITION, "Partition", "advice_pixbuf", 11},
	{-1, NULL, NULL, -1, NULL}
};

browsernode instance_nodes[] = {
	{B_SERVICES, "Services", "none", 10, NULL},
	{B_SESSIONS, "All Sessions", "session_pixbuf", 9, NULL},
	{B_SESSIONS_ACTIVE, "Active Sessions", "sessionactive_pixbuf", 9, NULL},
	{B_SESSIONS_LONGOPS, "Session LongOps", "none", 9, NULL},
	{B_ADVICE, "Advice", "advice_pixbuf", 9, advice_nodes},
	{-1, NULL, NULL, -1, NULL}
};

browsernode dba_nodes[] = {
	{B_INSTANCE, "Instances", "instance_pixbuf", 9, NULL},
	{B_SECURITY, "Security", "none", 9, security_nodes},
	{B_DATABASE, "Database", "none", 9, file_nodes},
	{-1, NULL, NULL, -1, NULL}
};


void browser_free_selection(browserSelection *bs)
{
  
	if(!bs) {
		return;
	}

	if(bs->path_arr) {
		g_strfreev(bs->path_arr);
	}

	if(bs->path) {
		gtk_tree_path_free(bs->path);
	}

	if(bs->root_path) {
		gtk_tree_path_free(bs->root_path);
	}
}


gboolean browser_get_selection(GtkTreeView *tv,
			       browserSelection *bs)
{
	gchar *path_ptr=0;


	bs->treeview = tv;
	bs->sel = gtk_tree_view_get_selection(tv);
	if (!bs->sel) {
		return FALSE;
	}

	if (!gtk_tree_selection_get_selected(bs->sel, &bs->model, NULL)) {
		return FALSE;
	}

	if (!gtk_tree_selection_get_selected(bs->sel,
					    &bs->model, 
					    &bs->iter))
	{
		return FALSE;
	}

	bs->path = gtk_tree_model_get_path(bs->model, &bs->iter);
	path_ptr = gtk_tree_path_to_string(bs->path);
	g_snprintf(bs->path_str, sizeof(bs->path_str),
		   "%s", path_ptr);
	g_free(path_ptr);

	if (gtk_tree_model_iter_parent(bs->model,
				       &bs->iter_parent,
				       &bs->iter)) {
		gtk_tree_model_iter_parent(bs->model, 
					   &bs->iter_grandparent,
					   &bs->iter_parent);
	}

	bs->path_arr = g_strsplit(bs->path_str, ":", -1);
	bs->root_path = gtk_tree_path_new_from_string((const gchar *) bs->path_arr[0]);

	if (!gtk_tree_model_get_iter(bs->model, &bs->iter_root, bs->root_path)) {
		return FALSE;
	}

	bs->depth = gtk_tree_path_get_depth(bs->path);

#if DEBUG
	fprintf(stderr, "%s:%d  depth=%d   path=%s  children=%d\n",
		__FILE__, __LINE__,
		bs->depth, bs->path_str,
		gtk_tree_model_iter_has_child(bs->model, &bs->iter));
#endif

	return TRUE;
}




void browser_copy_gtktreeview_columns(GtkTreeView *source_view,
				      GtkTreeView *dest_view,
				      GtkTreeIter *dest_iter,
				      gboolean delete,
				      ...)
{
	GtkTreeModel *source_model=0, *dest_model=0;
	GtkTreePath *dest_path=0;
	va_list ap;
	gint colnum=0;
	gboolean initial_column=TRUE;
	gboolean expanded;

	source_model = gtk_tree_view_get_model(source_view);
	if (!source_model) {
		return;
	}

	dest_model = gtk_tree_view_get_model(dest_view);
	if (!dest_model) {
		return;
	}

	dest_path = gtk_tree_model_get_path(dest_model, dest_iter);
	expanded  = gtk_tree_view_row_expanded(dest_view, dest_path);

	if (delete) {

		if (gtk_tree_model_iter_n_children(dest_model, dest_iter)) {
			GtkTreeIter    child_iter;
    
			gtk_tree_model_iter_children(dest_model, &child_iter, dest_iter);
			while(gtk_tree_store_remove(GTK_TREE_STORE(dest_model), &child_iter));
		}

	}

	va_start(ap, delete);
	while ((colnum = va_arg(ap, gint)) != -1) {
		GtkTreeIter source_iter, dest_child_iter;
		gboolean next;
		GType col_type;
		gint dest_colnum;
		gpointer *value=0;
  
  
		if(gtk_tree_model_get_n_columns(source_model) < colnum+1) {
			continue;
		}

		col_type = gtk_tree_model_get_column_type(source_model, colnum);
		dest_colnum = va_arg(ap, gint);      

		if (!initial_column) {
			gtk_tree_model_iter_children(dest_model, &dest_child_iter, dest_iter);
		}

		next = gtk_tree_model_get_iter_first(source_model, &source_iter);
		while (next) {

			if(initial_column) {
				gtk_tree_store_append(GTK_TREE_STORE(dest_model),
						      &dest_child_iter, dest_iter);
			}

			gtk_tree_model_get(source_model, &source_iter,
					   colnum, &value, -1);
			gtk_tree_store_set(GTK_TREE_STORE(dest_model),
					   &dest_child_iter, dest_colnum,
					   value, -1);
      
			if (col_type != GDK_TYPE_PIXBUF) {
				g_free(value);
			}

			gtk_tree_model_iter_next(dest_model, &dest_child_iter);
			next = gtk_tree_model_iter_next(source_model, &source_iter);      
		}

		initial_column=FALSE;
	}
	va_end(ap);

	if (dest_path) {

		if(expanded) {
			gtk_tree_view_expand_row(dest_view, dest_path, FALSE);
		}
		gtk_tree_path_free(dest_path);
	}

}




static void browser_add_nodes_at_iter(browsernode *node,
				      GtkTreeStore *store,
				      GtkTreeIter *parent_iter,
				      const gchar *schema)
{
	GtkTreeIter new_iter;
	gchar type_str[8];
	guint x;


	if (gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), parent_iter) ) {
		GtkTreeIter    child_iter;
		
		gtk_tree_model_iter_children(GTK_TREE_MODEL(store), &child_iter, parent_iter);
		while (gtk_tree_store_remove(store, &child_iter)) {

#if DEBUG
			fprintf(stderr, "%s:%d removing node\n", __FILE__, __LINE__);
#endif
		}
	}

	for (x=0; node[x].typeid != -1; x++) {

		if (((float)oramajor + ((float)oraminor/(float)10)) >= node[x].min_version ) {

			g_snprintf(type_str, sizeof(type_str), "%d", node[x].typeid );
			gtk_tree_store_append(store, &new_iter, parent_iter);
			gtk_tree_store_set(store, &new_iter, 
					   BC_TYPEID, type_str, 
					   BC_PARTITIONED, "0",
					   BC_OWNER, schema,
					   BC_OBJECT, node[x].title, 
					   BC_PIXBUF, query_get_pixbuf(node[x].pixbuf_name),
					   -1);

			if(node[x].child_node) {
				browser_add_nodes_at_iter(node[x].child_node,
							  store, &new_iter, schema);
			}
		}
	}

}



void object_browser_init(GtkTreeView *view,
			 guint dba_role)
{
	GtkCellRenderer *rend=0;
	GtkTreeViewColumn *col=0;
	GtkTreeIter iter;
	GtkStyle *w_style=0;
	gchar type_str[8];
	const gchar *font = config_get("listfont");


	if (objectBrowser) {
		clear_gtktreeview(objectBrowser);
	}
	else {
		objectBrowser = view;
	}

	/* type id */
	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 0, NULL);
	col  = gtk_tree_view_column_new_with_attributes("Type", rend, 
							"text", BC_TYPEID, NULL);
	gtk_tree_view_append_column(objectBrowser, col);

	/* partitioned */
	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 0, NULL);
	col  = gtk_tree_view_column_new_with_attributes("Partitioned", rend, 
							"text", BC_PARTITIONED, NULL);
	gtk_tree_view_append_column(objectBrowser, col);


	/* owner */
	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 0, NULL);
	col  = gtk_tree_view_column_new_with_attributes("Partitioned", rend,
							"text", BC_OWNER, NULL);
	gtk_tree_view_append_column(objectBrowser, col);


	/*  pixbuf */
	rend = gtk_cell_renderer_pixbuf_new();
	g_object_set(G_OBJECT(rend), "ypad", 0, "xpad", 0, NULL);
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_pack_start(col, rend, FALSE);
	gtk_tree_view_column_add_attribute(col, rend, "pixbuf", BC_PIXBUF);
	g_object_set(G_OBJECT(col), "resizable", TRUE, NULL);

	gtk_tree_view_append_column(objectBrowser, col);
	gtk_tree_view_set_expander_column(objectBrowser, col);

	/* label */
	rend = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, rend, FALSE);
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 0, NULL);
	gtk_tree_view_column_add_attribute(col, rend, "markup", BC_OBJECT);

	/* info label */
	rend = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_end(col, rend, FALSE);
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 0, NULL);
	gtk_tree_view_column_add_attribute(col, rend, "markup", BC_OBJECT_INFO);


	/* junk info columns */
	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 2, NULL);
	col  = gtk_tree_view_column_new_with_attributes(" ", rend, "text", BC_JUNK1, NULL);
	g_object_set(G_OBJECT(col), "resizable", TRUE, NULL);
	gtk_tree_view_append_column(objectBrowser, col);

	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 2, NULL);
	col  = gtk_tree_view_column_new_with_attributes(" ", rend, "text", BC_JUNK2, NULL);
	g_object_set(G_OBJECT(col), "resizable", TRUE, NULL);
	gtk_tree_view_append_column(objectBrowser, col);
  
	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 2, NULL);
	col  = gtk_tree_view_column_new_with_attributes(" ", rend, "text", BC_JUNK3, NULL);
	g_object_set(G_OBJECT(col), "resizable", TRUE, NULL);
	gtk_tree_view_append_column(objectBrowser, col);
  
	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 2, NULL);
	col  = gtk_tree_view_column_new_with_attributes(" ", rend, "text", BC_JUNK4, NULL);
	g_object_set(G_OBJECT(col), "resizable", TRUE, NULL);
	gtk_tree_view_append_column(objectBrowser, col);
  
	rend = gtk_cell_renderer_text_new();
	g_object_set(G_OBJECT(rend), "font", font, "ypad", 0, "xpad", 2, NULL);
	col  = gtk_tree_view_column_new_with_attributes(" ", rend, "text", BC_JUNK5, NULL);
	g_object_set(G_OBJECT(col), "resizable", TRUE, NULL);
	gtk_tree_view_append_column(objectBrowser, col);

	objectStore = gtk_tree_store_new(11,
					 G_TYPE_STRING,
					 G_TYPE_STRING,
					 G_TYPE_STRING,
					 GDK_TYPE_PIXBUF,
					 G_TYPE_STRING,
					 G_TYPE_STRING, 
					 G_TYPE_STRING, 
					 G_TYPE_STRING,
					 G_TYPE_STRING,
					 G_TYPE_STRING,
					 G_TYPE_STRING);
 
	g_snprintf(type_str, sizeof(type_str), "%d", B_SCHEMA);
	gtk_tree_store_append(objectStore, &iter, (GtkTreeIter *) NULL);
	gtk_tree_store_set(objectStore, &iter,
			   BC_TYPEID, type_str, 
			   BC_PARTITIONED, "0", 
			   BC_OWNER, get_schema(), 
			   BC_PIXBUF, query_get_pixbuf("schema_pixbuf"), 
			   BC_OBJECT, get_schema(), -1);
	browser_add_nodes_at_iter(top_nodes, objectStore, &iter, get_schema());

	g_snprintf(type_str, sizeof(type_str), "%d", B_SCHEMAS);
	gtk_tree_store_append(objectStore, &iter, (GtkTreeIter *) NULL);
	gtk_tree_store_set(objectStore, &iter,
			   BC_TYPEID, type_str, 
			   BC_PARTITIONED, "0", 
			   BC_OWNER, "Other Schemas",
			   BC_PIXBUF, query_get_pixbuf("schemas_pixbuf"), 
			   BC_OBJECT, "Other Schemas", -1);


	if (dba_role) {
		GtkTreeIter instance_iter, instances_iter;
		OCIStmt *sth=0;
		OCIDefine *defnp=0;
		gchar inst_id[16];
		gchar instance_name[128];
		gchar inst_type_str[8];
		gchar *inst_sql = "\
                select * from (\
                 select 0, '(All)' from dual \
                  union all \
                 select inst_id, instance_name from gv$instance \
                ) order by 1";

		g_snprintf(type_str, sizeof(type_str), "%d", B_DBA);
		gtk_tree_store_append(objectStore, &iter, (GtkTreeIter *) NULL);
		gtk_tree_store_set(objectStore, &iter,
				   BC_TYPEID, type_str, 
				   BC_PARTITIONED, "0", 
				   BC_OWNER, get_schema(), 
				   BC_PIXBUF, NULL, 
				   BC_OBJECT, "DBA", -1);
		browser_add_nodes_at_iter(dba_nodes, objectStore, &iter, get_schema());

		gtk_tree_model_iter_children(GTK_TREE_MODEL(objectStore),
					     &instances_iter, &iter);

		OCIHandleAlloc((dvoid *)envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT,
			       (size_t)0, (dvoid **)0);
		ociPrepare(sth, errhp, inst_sql);
		OCIDefineByPos(sth, &defnp, errhp, (ub4)1, (dvoid *)inst_id,
			       (sb4)sizeof(inst_id)+1,
			       (ub2)SQLT_STR, 0, (ub2 *)0, (ub2 *)0,
			       (ub4)OCI_DEFAULT);
		OCIDefineByPos(sth, &defnp, errhp, (ub4)2, (dvoid *)instance_name,
			       (sb4)sizeof(instance_name)+1,
			       (ub2)SQLT_STR, 0, (ub2 *)0, (ub2 *)0,
			       (ub4)OCI_DEFAULT);
		OCIStmtExecute(svchp, sth, errhp, (ub4)0, (ub4)0, (OCISnapshot *)0,
			       (OCISnapshot *)0, (ub4)OCI_DEFAULT);
		g_snprintf(inst_type_str, sizeof(inst_type_str), "%d", B_INSTANCE);
		while (OCIStmtFetch2(sth, errhp, (ub4)1, (ub2)OCI_FETCH_NEXT,
				     (sb4)0, (ub4)OCI_DEFAULT) == OCI_SUCCESS) {

			gtk_tree_store_append(objectStore, &instance_iter,
					      (GtkTreeIter *) &instances_iter);
			gtk_tree_store_set(objectStore, &instance_iter,
					   BC_TYPEID, inst_type_str, 
					   BC_PARTITIONED, inst_id, 
					   BC_OWNER, get_schema(), 
					   BC_PIXBUF, query_get_pixbuf("instance_pixbuf"), 
					   BC_OBJECT, instance_name, -1);
			browser_add_nodes_at_iter(instance_nodes, objectStore,
						  &instance_iter, get_schema());
		}

	}

	gtk_tree_view_set_model(objectBrowser, GTK_TREE_MODEL(objectStore) );
	g_object_unref(G_OBJECT(objectStore));

	gtk_tree_model_get_iter_first(gtk_tree_view_get_model(objectBrowser), &iter);
	expand_at_iter(objectBrowser, &iter);

	hide_gtktreeview_column(objectBrowser, BC_TYPEID);
	hide_gtktreeview_column(objectBrowser, BC_PARTITIONED);
	hide_gtktreeview_column(objectBrowser, BC_OWNER);

	comment_signal = g_signal_connect ((gpointer) lw("tableCommentEntry"),
					   "activate", G_CALLBACK (on_tableCommentEntry_activate),
					   NULL);

	w_style = gtk_widget_get_style(GTK_WIDGET(objectBrowser));
	g_snprintf(fg_normal, sizeof(fg_normal), "#%.2x%.2x%.2x",
		   w_style->fg[0].red/256, 
		   w_style->fg[0].green/256,
		   w_style->fg[0].blue/256);
	g_object_unref(w_style);
}




void add_other_schema_nodes(GtkTreeIter *iter,
			    const gchar *other_schema)
{
	browser_add_nodes_at_iter(top_nodes, objectStore, iter, other_schema);
}



void on_detailListActivated(GtkTreeView *treeview,
			    GtkTreePath *path,
			    GtkTreeViewColumn *arg2,
			    gpointer user_data)
{
	browserSelection bs;
	gchar sql[2048];
	gchar *typeid=0;
	gchar *owner=0;
	gchar *object=0;
	gchar *partitioned=0;


	if (!browser_get_selection(treeview, &bs)) {
		return;
	}

	gtk_tree_model_get(bs.model, &bs.iter,
			   0, &typeid,
			   -1);

	switch (atoi(typeid)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:

		gtk_tree_model_get(bs.model, &bs.iter,
				   1, &partitioned,
				   2, &owner,
				   4, &object,
				   -1);

		if(atoi(partitioned)) {

			g_snprintf(sql, sizeof(sql), table_detail_list_sql,
				   B_TABLE_PARTITION);

			execute_sql_at_iter(sql, treeview, &bs.iter,
					    owner, object, NULL);
			gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
					     "<b>Tables</b>");
		}

		g_free(partitioned);
		g_free(owner);
		g_free(object);
		break;


	case B_INDEX:

		gtk_tree_model_get(bs.model, &bs.iter,
				   1, &partitioned,
				   2, &owner,
				   4, &object,
				   -1);

		if(atoi(partitioned)) {

			g_snprintf(sql, sizeof(sql), index_detail_list_sql,
				   B_INDEX_PARTITION);

			execute_sql_at_iter(sql, treeview, &bs.iter,
					    owner, object, NULL);
		}

		g_free(partitioned);
		g_free(owner);
		g_free(object);
		break;

 
	case B_TABLE_COLUMN:

		gtk_tree_model_get(bs.model, &bs.iter, 6, &object, -1);

		if (object) {
			if(object[0] != '\0') {
				gchar     **tp=0;

				tp = g_strsplit(object, ".", -1);

				if (2 == g_strv_length(tp) ) {
					GList     *tablist=0;	  
	  
					tablist = execute_sql_to_glist(column_detail_list_sql,
								       tp[0],
								       tp[1],
								       NULL);

					if( tablist ) {
						show_table_data_window((const gchar *)tablist->data);
						g_free(tablist->data);
						g_list_free(tablist);
					}

				}

				g_strfreev(tp);
			}
			g_free(object);
		}

		break;

	default:
		break;
	}
  
  
	browser_free_selection(&bs);
	g_free(typeid);
}


















