/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <gtk/gtk.h>
#include <oci.h>

#include "gutil.h"
#include "query.h"
#include "orautil.h"
#include "dbms_space.h"
#include "browser.h"
#include "browser_activated_sql.h"
#include "browser_activated.h"


void objectDetailSelected(browserSelection *);

extern int oramajor;



void 
on_objectActivated(GtkTreeView *treeview,
		   GtkTreePath *path,
		   GtkTreeViewColumn *arg2,
		   gpointer user_data)
{
	browserSelection bs;
	GtkTreeView *objectBrowser=0;
	gchar *type=0, *partitioned=0;
	gchar *owner=0, *object=0;
	gchar *parent_object=0;
	gchar sql[16000], mb_buf[16];
	gchar *lob_segment=0, **lob_table=0;
	const gchar *schema = get_schema();
	gdouble mb;
	dspace ds;

	objectBrowser = GTK_TREE_VIEW(lw("objectList"));
	if (!objectBrowser) {
		fprintf(stderr, "%s:%d problem getting pointer to objectList\n",
			__FILE__, __LINE__);
		return;
	}

	if (!browser_get_selection(objectBrowser, &bs)) {
		return;
	}

	gtk_tree_model_get(bs.model, &bs.iter, 
			   BC_TYPEID, &type,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   BC_PARTITIONED, &partitioned,
			   -1);

	switch (atoi(type)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:
	case B_MVIEW:

		g_snprintf(sql, sizeof(sql), table_activated_sql, B_INDEX,
			   B_TABLE_PARTITION);
    
		execute_sql_at_iter(sql, objectBrowser, &bs.iter, owner,
				    object, owner, object, NULL);
		set_gtktreeview_column_names(objectBrowser, "MB", 
					     "Tablespace", NULL);


		if (!atoi(partitioned)) {
 
			get_dbms_space(owner, object, NULL, "TABLE",
				       &ds, TRUE);
			mb = (float)ds.totbytes_i/(float)1048576;

			g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
			gtk_tree_store_set(GTK_TREE_STORE(bs.model), 
					   &bs.iter, BC_JUNK1, mb_buf, -1);
		}

		break;


	case B_TABLE_PARTITION:

		gtk_tree_model_get(bs.model, &bs.iter_parent, BC_OBJECT,
				   &parent_object, -1);
		g_snprintf(sql, sizeof(sql), table_partition_activated_sql,
			   B_TABLE_SUBPARTITION);

		execute_sql_at_iter(sql, objectBrowser, &bs.iter, owner,
				    parent_object, object, NULL);
		set_gtktreeview_column_names(objectBrowser, "MB", 
					     "Tablespace", NULL);

		if( !atoi(partitioned)) {
 
			get_dbms_space(owner, parent_object, object,
				       "TABLE PARTITION", &ds, TRUE);
			mb = (float)ds.totbytes_i/(float)1048576;

			g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
			gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
					   BC_JUNK1, mb_buf, -1);
		}

		g_free(parent_object);
		break;



	case B_TABLE_SUBPARTITION:
    
		gtk_tree_model_get(bs.model, &bs.iter_grandparent,
				   BC_OBJECT, &parent_object, -1);

		get_dbms_space(owner, parent_object, object,
			       "TABLE SUBPARTITION", &ds, TRUE);

		mb = (float)ds.totbytes_i/(float)1048576;
		g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
		gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
				   BC_JUNK1, mb_buf, -1);
 
		g_free(parent_object);
		break;


	case B_INDEX:

		g_snprintf(sql, sizeof(sql), index_activated_sql,
			   B_INDEX_PARTITION);

		execute_sql_at_iter(sql, objectBrowser, &bs.iter,
				    owner, object, NULL);
		set_gtktreeview_column_names(objectBrowser, "MB",
					     "Tablespace", NULL);

		if( !atoi(partitioned)) {
 
			get_dbms_space(owner, object, NULL, "INDEX",
				       &ds, TRUE);
			mb = (float)ds.totbytes_i/(float)1048576;

			g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
			gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
					   BC_JUNK1, mb_buf, -1);
		}

		break;


	case B_INDEX_PARTITION:

		gtk_tree_model_get(bs.model, &bs.iter_parent, BC_OBJECT,
				   &parent_object, -1);
		g_snprintf(sql, sizeof(sql), index_partition_activated_sql,
			   B_INDEX_SUBPARTITION);

		execute_sql_at_iter(sql, objectBrowser, &bs.iter, schema,
				    parent_object, object, NULL);
		set_gtktreeview_column_names(objectBrowser, "MB", "# Rows",
					     "Tablespace", NULL);

		if( !atoi(partitioned)) {
 
			get_dbms_space(owner, parent_object, object, 
				       "INDEX PARTITION", &ds, TRUE);
			mb = (float)ds.totbytes_i/(float)1048576;

			g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
			gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
					   BC_JUNK1, mb_buf, -1);
		}

		g_free(parent_object);
		break;


	case B_INDEX_SUBPARTITION:

		gtk_tree_model_get(bs.model, &bs.iter_grandparent,
				   BC_OBJECT, &parent_object, -1);

		get_dbms_space(owner, parent_object, object, 
			       "INDEX SUBPARTITION", &ds, TRUE);
		mb = (float)ds.totbytes_i/(float)1048576;
		g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
		gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
				   BC_JUNK1, mb_buf, -1);
		set_gtktreeview_column_names(objectBrowser, "MB", 
					     "# Rows", "Tablespace", NULL);
 
		g_free(parent_object);
		break;



	case B_CLUSTER:

		get_dbms_space(owner, object, NULL, "CLUSTER",
			       &ds, TRUE);
		mb = (float)ds.totbytes_i/(float)1048576;
		g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
		gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
				   BC_JUNK1, mb_buf, -1);

		set_gtktreeview_column_names(objectBrowser, "MB", 
					     "Tablespace", " ", NULL);
		break;



	case B_MVIEWS:
   
		g_snprintf(sql, sizeof(sql), mview_activated_sql, B_MVIEW);
		execute_sql_at_iter(sql, objectBrowser, 
				    &bs.iter, schema, NULL);
		set_gtktreeview_column_names(objectBrowser, " ", 
					     " ", " ", NULL);
		break;



	case B_MVIEW_LOG:

		get_dbms_space(owner, object, NULL, "TABLE",
			       &ds, TRUE);
		mb = (float)ds.totbytes_i/(float)1048576;
		g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
		gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
				   BC_JUNK1, mb_buf, -1);

		break;



	case B_LOB:

		gtk_tree_model_get(bs.model, &bs.iter, BC_JUNK2,
				   &lob_segment, -1);
		lob_table = g_strsplit(lob_segment, ".", -1);

		g_snprintf(sql, sizeof(sql), lob_activated_sql,
			   B_LOB_PARTITION);
    
		if (g_strv_length(lob_table) > 1) {
			execute_sql_at_iter(sql, objectBrowser, &bs.iter, 
					    lob_table[0], lob_table[1], 
					    lob_segment, NULL);
		}

		set_gtktreeview_column_names(objectBrowser, "MB", "Segment",
					     "Index", NULL);


		get_dbms_space(owner, object, NULL, "LOB",
			       &ds, TRUE);
		mb = (float)ds.totbytes_i/(float)1048576;

		g_snprintf(mb_buf, sizeof(mb_buf), "%.02f", mb);
		gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
				   BC_JUNK1, mb_buf, -1);

		g_free(lob_segment);
		g_strfreev(lob_table);
		break;


	case B_QUEUE:
		objectDetailSelected(&bs);
		break;

	case B_SERVICE:
		objectDetailSelected(&bs);
		break;


	case B_SESSIONS:
	case B_SESSIONS_ACTIVE:
		objectDetailSelected(&bs);
		break;
    

	case B_SESSION:
		objectDetailSelected(&bs);
		break;


	case B_SESSIONS_LONGOPS:
		objectDetailSelected(&bs);
		break;



	default:
		g_free(type);
		g_free(partitioned);
		browser_free_selection(&bs);
		return;
	}


	gtk_tree_view_expand_row(objectBrowser, bs.path, FALSE);

	g_free(type);
	g_free(owner);
	g_free(object);
	g_free(partitioned);
	browser_free_selection(&bs);
}
