/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <gtk/gtk.h>
#include <oci.h>

#include "gutil.h"
#include "orautil.h"
#include "dbms_metadata.h"
#include "dbms_space.h"
#include "plobject.h"
#include "table.h"
#include "partition.h"
#include "syntax.h"
#include "query.h"
#include "appconfig.h"
#include "browser.h"
#include "browser_selected_sql.h"
#include "browser_selected.h"


enum {NB_DETAIL,
      NB_CONSTRAINTS,
      NB_GRANTS,
      NB_DEPENDENCIES,
      NB_DDL
};

static gboolean metadata_query=FALSE;

extern int oramajor;
extern OCIEnv *envhp;
extern OCIError *errhp;
extern OCISvcCtx *svchp;
extern gulong comment_signal;
extern gchar fg_normal[32];

void add_other_schema_nodes(GtkTreeIter *,
			    const gchar *);


void objectDetailSelected(browserSelection *bs)
{
	GtkTreeView *detail_view=0;
	GtkTreeView *browser_view=0;
	GtkTreeModel *detail_model=0;
	GtkTreeStore *store=0;
	GtkTreeIter detail_iter;
	GtkTreeIter iter_save;
	GtkCellRenderer *rend=0;
	GtkTreeViewColumn *col=0;
	GList *dt_list=0;
	GList *rends=0;
	gchar *type=0, *owner=0, *object=0, *parent_object=0, *tabcomment=0;
	gchar *grandparent_object=0, *type_name=0, *partitioned=0;
	gchar type_arr[8], sql[16000], syn_like[64];
	gchar **type_part=0;
	guint unhandled=0;
	const gchar *font = config_get("listfont");


	if ((detail_view = GTK_TREE_VIEW(lw("detailList"))) == NULL) {
		return;
	}

	if ((browser_view = GTK_TREE_VIEW(lw("objectList"))) == NULL) {
		return;
	}

	gtk_tree_model_get(bs->model, &bs->iter, 
			   BC_TYPEID, &type,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   BC_PARTITIONED, &partitioned, 
			   -1);

	gtk_widget_hide(lw("tableCommentEntry"));

	switch (atoi(type)) {

	case B_SCHEMAS:
    
		g_snprintf(sql, sizeof(sql), schema_other_sql, B_SCHEMA_OTHER);
		execute_sql_gtktreeview(sql, detail_view, get_schema(), NULL);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Other Schemas</b>");

		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF,
						 4, BC_OBJECT,
						 -1);
		set_gtktreeview_column_names(browser_view, " ", " ", NULL);
		break;


	case B_SCHEMA_OTHER:

		unhandled = TRUE;
		add_other_schema_nodes(&bs->iter, owner);
		break;


	case B_TABLES:

		g_snprintf(sql, sizeof(sql), 
			   tables_sql,
			   B_TABLE_HEAP,
			   B_TABLE_IOT);

		execute_sql_gtktreeview(sql, detail_view, owner, NULL);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Tables</b>");

		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF,
						 4, BC_OBJECT,
						 5, 7, 
						 -1);
		set_gtktreeview_column_names(browser_view, "MB", "Tablespace",
					     NULL);
		break;


	case B_TABLE_HEAP:
	case B_TABLE_IOT:
	case B_VIEW:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), table_sql,
			   B_TABLE_COLUMN);
			    
		execute_sql_gtktreeview(sql, detail_view,
					owner, object, NULL); 
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Table Columns</b>");


		/* Table comments */
		if ((tabcomment = get_table_comments(owner, object)) != NULL) {

			g_signal_handler_block((gpointer) lw("tableCommentEntry"),
					       comment_signal);
			gtk_entry_set_text(GTK_ENTRY(lw("tableCommentEntry")),
					   tabcomment );
			g_signal_handler_unblock((gpointer) lw("tableCommentEntry"),
						 comment_signal);
			g_free(tabcomment);
		}
		gtk_widget_show(lw("tableCommentEntry"));
    
		detail_model = gtk_tree_view_get_model(detail_view);

		/* Object type column display */
		if (detail_model) {
			gchar *objtype=0;
			gboolean next;

			next = gtk_tree_model_get_iter_first(detail_model,
							     &detail_iter);
			while (next) {

				gtk_tree_model_get(detail_model, &detail_iter,
						   15, &objtype, -1);

				if (objtype) {
					if (objtype[0] != '\0') {
						show_oracle_type(objtype,
								 detail_view,
								 &detail_iter,
								 2);
					}
					g_free(objtype);
				}

				next = gtk_tree_model_iter_next(detail_model,
								&detail_iter);
			}
		}


		/* Table partition key display */
		if (B_VIEW != atoi(type)) {
			if (atoi(partitioned)) {
				GList *tmp_type=0;

				if (oramajor > 10) {
					g_snprintf(sql, sizeof(sql), partition_v10_sql);
				}
				else {
					g_snprintf(sql, sizeof(sql), partition_v9_sql);
				}
	
				gtk_tree_store_append(GTK_TREE_STORE(detail_model),
						      &detail_iter, NULL);
      
				rends = execute_sql_to_glist(sql, owner,
								 object, owner, object,
								 owner, object, owner,
								 object, NULL);
				if (rends) {
					for(tmp_type=rends;
					    tmp_type;
					    tmp_type=g_list_next(tmp_type)) {
						gtk_tree_store_append(GTK_TREE_STORE(detail_model),
								      &detail_iter, NULL);
						gtk_tree_store_set(GTK_TREE_STORE(detail_model),
								   &detail_iter, 2,
								   (gchar *) tmp_type->data,
								   -1);
						g_free(tmp_type->data);
					}
					g_list_free(rends);
				}
			}
			else {
				gtk_tree_store_append(GTK_TREE_STORE(detail_model),
						      &detail_iter, NULL);
				gtk_tree_store_append(GTK_TREE_STORE(detail_model),
						      &detail_iter, NULL);
				gtk_tree_store_set(GTK_TREE_STORE(detail_model),
						   &detail_iter, 2, (gchar *) "Non-partitioned",
						   -1);
			} 

      
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(detail_model),
					   &detail_iter, 2, "Table info :",
					   -1);
			execute_sql_pivotted("select * from all_tables where owner=:p1 and table_name=:p2",
					     detail_view,
					     &detail_iter,
					     2,
					     owner,
					     object, 
					     NULL);

			/* Column comment editing */
			rends = gtk_tree_view_column_get_cell_renderers(gtk_tree_view_get_column(detail_view, 14));
			if (rends) {
				gchar     table[64];
	
				g_snprintf(table, sizeof(table), "%s.%s", owner, object);
				g_object_set(GTK_CELL_RENDERER(rends->data), "editable",
					     TRUE, NULL);
				g_signal_connect(GTK_CELL_RENDERER(rends->data), "edited",
						 (GCallback) edit_column_comments,
						 (gpointer) g_strdup(table) );
				g_list_free(rends);
			}
		}

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		set_gtktreeview_column_names(browser_view, "MB",
					     "Tablespace", NULL);
		break;



	case B_TABLE_PARTITION:
    
		unhandled=TRUE;
		g_free(owner);
		gtk_tree_model_get(bs->model, &bs->iter_parent,
				   BC_OWNER, &owner, BC_OBJECT,
				   &parent_object, -1);

		execute_sql_pivotted(table_partition_sql,
				     detail_view, NULL,
				     0, owner, parent_object,
				     object, NULL);

		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Table Partition Details</b>");
		set_gtktreeview_column_names(browser_view, "MB",
					     "Tablespace", NULL);
		g_free(parent_object);
		break;


	case B_TABLE_SUBPARTITION:

		unhandled=TRUE;
		g_free(owner);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_OWNER, &owner,
				   BC_OBJECT, &grandparent_object,
				   -1);
		gtk_tree_model_get(bs->model, &bs->iter_parent,
				   BC_OBJECT, &parent_object, -1);
		execute_sql_pivotted(table_subpartition_sql,
				     detail_view,
				     NULL,
				     0,
				     owner,
				     grandparent_object,
				     parent_object,
				     object, 
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Table Subpartition Details</b>");
		g_free(grandparent_object);
		g_free(parent_object);
		set_gtktreeview_column_names(bs->treeview, "MB",
					     "Tablespace", " ",
					     " ", NULL);
		break;

	case B_INDEXES:

		g_snprintf(sql, sizeof(sql), indexes_sql, B_INDEX);

		execute_sql_gtktreeview(sql, detail_view, fg_normal,
					owner, owner, NULL);

		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF,
						 4, BC_OBJECT,
						 10, BC_JUNK2, 
						 -1);

		gtk_tree_store_prepend(GTK_TREE_STORE(gtk_tree_view_get_model(browser_view)),
				       &iter_save, &bs->iter);
		g_snprintf(type_arr, sizeof(type_arr), "%d", B_INDEXES_INVALID);
		gtk_tree_store_set(GTK_TREE_STORE(gtk_tree_view_get_model(browser_view)),
				   &iter_save,
				   0, type_arr, 
				   1, "0", 
				   2, owner,
				   3, query_get_pixbuf("indexinvalid_pixbuf"), 
				   4, "Unusable Indexes", -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Indexes</b>");
		set_gtktreeview_column_names(browser_view, "MB",
					     "Tablespace", NULL);
		break;


	case B_INDEXES_INVALID:

		g_snprintf(sql, sizeof(sql), indexes_invalid_sql);
		execute_sql_gtktreeview(sql, detail_view, owner,
					owner, owner, NULL);
		break;


	case B_INDEX:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), index_sql);

		execute_sql_gtktreeview(sql, detail_view, owner,
					object, owner, object, NULL);
		detail_model = gtk_tree_view_get_model(detail_view);

		if (atoi(partitioned)) {
			GList  *tmp_type=0;

			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
      
			rends = execute_sql_to_glist(index_partitions_sql, owner,
						     object, owner, object, NULL);
			if (rends) {
				for(tmp_type=rends;
				    tmp_type;
				    tmp_type=g_list_next(tmp_type)) {
					gtk_tree_store_append(GTK_TREE_STORE(detail_model),
							      &detail_iter, NULL);
					gtk_tree_store_set(GTK_TREE_STORE(detail_model),
							   &detail_iter, 0, 
							   (gchar *) tmp_type->data, -1);
					g_free(tmp_type->data);
				}
				g_list_free(rends);
			}
		}
		else {
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(detail_model),
					   &detail_iter, 0, 
					   (gchar *) "Global, non-partitioned",
					   -1);
		}

		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &bs->iter, NULL);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &bs->iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &bs->iter, 
				   0, "------------",
				   1, "------------", -1);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &bs->iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &bs->iter, 0, "Index info :", 
				   -1);
		execute_sql_pivotted(index_detail_sql,
				     detail_view, &bs->iter, 0,
				     owner, object, NULL);

		expand_at_iter(detail_view, &bs->iter);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Index Columns</b>");
		set_gtktreeview_column_names(browser_view, "MB", 
					     "Tablespace", NULL);
		break;


	case B_INDEX_PARTITION:
    
		unhandled=TRUE;
		g_free(owner);
		gtk_tree_model_get(bs->model, &bs->iter_parent,
				   BC_OWNER, &owner,
				   BC_OBJECT, &parent_object, -1);
		execute_sql_pivotted(index_partition_sql,
				     detail_view,  NULL,
				     0, owner, parent_object,
				     object, NULL);

		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Index Partition Details</b>");
		set_gtktreeview_column_names(browser_view, "MB",
					     "Tablespace", NULL);
		g_free(parent_object);
		break;


	case B_INDEX_SUBPARTITION:

		unhandled=TRUE;
		g_free(owner);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_OWNER, &owner, BC_OBJECT,
				   &grandparent_object, -1);
		gtk_tree_model_get(bs->model, &bs->iter_parent,
				   BC_OBJECT, &parent_object, -1);
		execute_sql_pivotted(index_subpartition_sql,
				     detail_view, NULL, 0,
				     owner, grandparent_object,
				     parent_object, object, 
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Index Subpartition Details</b>");
		g_free(grandparent_object);
		g_free(parent_object);
		set_gtktreeview_column_names(bs->treeview, "MB", 
					     "Tablespace", " ", " ",
					     NULL);
		break;


	case B_VIEWS:
    
		g_snprintf(sql, sizeof(sql), views_sql, B_VIEW);
		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		set_gtktreeview_column_names(bs->treeview, " ", " ", " ",
					     NULL);

		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Views</b>");
		set_gtktreeview_column_names(browser_view, " ",
					     " ", NULL);
		break;



	case B_CLUSTERS:

		g_snprintf(sql, sizeof(sql), clusters_sql, B_CLUSTER);
		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		set_gtktreeview_column_names(bs->treeview, " ", " ",
					     " ", NULL);

		browser_copy_gtktreeview_columns(detail_view, browser_view, 
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Clusters</b>");
		set_gtktreeview_column_names(browser_view, "MB", 
					     "Tablespace", NULL);
		break;

               
	case B_CLUSTER:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), cluster_sql, B_TABLE_HEAP);

		execute_sql_gtktreeview(sql, detail_view, owner, object, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);

		detail_model = gtk_tree_view_get_model(detail_view);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &bs->iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &bs->iter, 
				   4, "------------",
				   5, "------------", -1);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &bs->iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &bs->iter, 4, "Cluster info :", -1);
		execute_sql_pivotted(cluster_detail_sql,
				     detail_view, &bs->iter,
				     4, owner, object, NULL);
		expand_at_iter(detail_view, &bs->iter);

		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Cluster Tables</b>");
		set_gtktreeview_column_names(browser_view,
					     "MB", "Tablespace", NULL);
		break;



	case B_MVIEWS:
    
		g_snprintf(sql, sizeof(sql), mviews_sql, B_MVIEW);

		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		set_gtktreeview_column_names(bs->treeview, " ", " ", " ",
					     NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Materialized Views</b>");
		break;


	case B_MVIEW:
    
		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), mview_sql, B_TABLE_COLUMN);
			    
		execute_sql_gtktreeview(sql, detail_view, owner,
					object, NULL);

		detail_model = gtk_tree_view_get_model(detail_view);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &detail_iter, 2, "MView info :", -1);
		execute_sql_pivotted(mview_detail_sql,
				     detail_view, &detail_iter,
				     2, owner, object, NULL);
		expand_at_iter(detail_view, &detail_iter);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Materialized View Detail</b>");
		break;


	case B_MVIEW_LOGS:

		g_snprintf(sql, sizeof(sql), mview_logs_sql,
			   B_MVIEW_LOG);

		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		set_gtktreeview_column_names(bs->treeview, " ", " ",
					     " ", NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Materialized View Logs</b>");
		break;



	case B_MVIEW_LOG:
    
		unhandled=TRUE;

		clear_gtktreeview(detail_view);

		rend = gtk_cell_renderer_text_new();
		g_object_set(G_OBJECT(rend), "font", font, "ypad",
			     0, "xpad", 0, NULL);
		col  = gtk_tree_view_column_new_with_attributes(" ", rend,
								"markup", 0,
								NULL);
		gtk_tree_view_append_column(detail_view, col);
		col  = gtk_tree_view_column_new_with_attributes(" ", rend,
								"markup", 1,
								NULL);
		gtk_tree_view_append_column(detail_view, col);
		col  = gtk_tree_view_column_new_with_attributes(" ", rend,
								"markup", 2,
								NULL);
		gtk_tree_view_append_column(detail_view, col);
		col  = gtk_tree_view_column_new_with_attributes(" ", rend,
								"markup", 3,
								NULL);
		gtk_tree_view_append_column(detail_view, col);
		col  = gtk_tree_view_column_new_with_attributes(" ", rend,
								"markup", 4,
								NULL);
		gtk_tree_view_append_column(detail_view, col);

		store = gtk_tree_store_new(5, G_TYPE_STRING, G_TYPE_STRING,
					   G_TYPE_STRING, G_TYPE_STRING,
					   G_TYPE_STRING);
		gtk_tree_view_set_model(detail_view, GTK_TREE_MODEL(store) );
		g_object_unref(G_OBJECT(store));

		detail_model = gtk_tree_view_get_model(detail_view);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &detail_iter, 0, "Info :", -1);
		execute_sql_pivotted(mvlog_detail_sql,
				     detail_view,
				     &detail_iter,
				     0,
				     owner,
				     object,
				     NULL);
		expand_at_iter(detail_view, &detail_iter);


		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &detail_iter, 0, "------------", -1);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &detail_iter, 0,
				   "Registered snapshots :", -1);
		execute_sql_at_iter(registered_snapshot_sql,
				    detail_view, &detail_iter,
				    owner, object, NULL);
		expand_at_iter(detail_view, &detail_iter);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Materialized View Log Detail</b>");

		break;




	case B_QUEUES:
    
		g_snprintf(sql, sizeof(sql), queues_sql, B_QUEUE);
		execute_sql_gtktreeview(sql, detail_view,
					owner, NULL);
		set_gtktreeview_column_names(bs->treeview, " ",
					     " ", " ", NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Queues</b>");
		break;


	case B_QUEUE:

		unhandled=TRUE;
		execute_sql_pivotted(queue_sql, detail_view, NULL,
				     0, owner, object, NULL);

		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Queue Detail</b>");

		dt_list = execute_sql_to_glist(queue_datatype_sql,
					       owner, object, NULL);

		if ((type_name = g_list_nth_data(dt_list, 0)) != NULL) {
 
			detail_model = gtk_tree_view_get_model(detail_view);
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(detail_model),
					   &detail_iter, 0, "Payload Datatype:", -1);
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(detail_model),
					   &detail_iter, 0, type_name, -1);

			show_oracle_type((gchar *) type_name, detail_view,
					 &detail_iter, 0);
			expand_at_iter(detail_view, &detail_iter);
			g_list_free(dt_list);
		}

		dt_list = execute_sql_to_glist(queue_table_sql,
					       owner, object, NULL);

		if( (type_name = g_list_nth_data(dt_list, 0)) != NULL ) {

			detail_model = gtk_tree_view_get_model(detail_view);
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(detail_model),
					   &detail_iter, 0, "Message State Count:",
					   -1);
			g_snprintf(sql, sizeof(sql), queue_msg_state_sql,
				   (gchar *) type_name);

			execute_sql_at_iter(sql, detail_view,
					    &detail_iter, NULL);
			expand_at_iter(detail_view, &detail_iter);
			g_list_free(dt_list);
		}

		break;




	case B_SYNONYMS:

		g_snprintf(sql, sizeof(sql), synonyms_sql, B_SYNONYM);
		execute_sql_gtktreeview(sql, detail_view, owner,
					"%", NULL);
		set_gtktreeview_column_names(bs->treeview, " ",
					     " ", " ", NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 5, BC_JUNK1,
						 6, BC_JUNK2,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Synonyms</b>");
		break;


	case B_PUBLIC_SYNONYM_ALL:
	case B_PUBLIC_SYNONYM_DBA:
	case B_PUBLIC_SYNONYM_USER:
	case B_PUBLIC_SYNONYM_VDOLLAR:

		switch(atoi(type)) {
		case B_PUBLIC_SYNONYM_ALL:
			g_snprintf(syn_like, sizeof(syn_like), "ALL_%%");
			break;
		case B_PUBLIC_SYNONYM_DBA:
			g_snprintf(syn_like, sizeof(syn_like), "DBA_%%");
			break;
		case B_PUBLIC_SYNONYM_USER:
			g_snprintf(syn_like, sizeof(syn_like), "USER_%%");
			break;
		case B_PUBLIC_SYNONYM_VDOLLAR:
			g_snprintf(syn_like, sizeof(syn_like), "V%%");
			break;
		}

		g_snprintf(sql, sizeof(sql), public_synonym_sql, 
			   B_SYNONYM);

		execute_sql_gtktreeview(sql, 
					detail_view,
					"PUBLIC",
					syn_like,
					NULL);
		set_gtktreeview_column_names(bs->treeview, " ",
					     " ", " ", NULL);

		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 5, BC_JUNK1,
						 6, BC_JUNK2,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Synonyms</b>");
		break;


	case B_SYNONYM:

		unhandled=TRUE;
		g_free(owner);
		g_free(object);
		gtk_tree_model_get(bs->model, &bs->iter, BC_JUNK1,
				   &owner, BC_JUNK2, &object, -1);
 
		execute_sql_gtktreeview(synonym_sql, detail_view,
					owner, object, NULL);

		set_gtktreeview_column_names(bs->treeview, " ", " ",
					     " ", NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Synonym Columns</b>");
		break;



	case B_SEQUENCES:
    
		g_snprintf(sql, sizeof(sql), sequences_sql,
			   B_SEQUENCE);
		execute_sql_gtktreeview(sql, detail_view, owner, 
					NULL);
		set_gtktreeview_column_names(bs->treeview, " ", 
					     " ", " ", NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Sequences</b>");
		break;

    
	case B_SEQUENCE:

		unhandled=TRUE;
		execute_sql_pivotted(sequence_sql, detail_view,
				     NULL, 0, owner, object,
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Sequence Detail</b>");
		break;



	case B_LOBS:

		if (oramajor > 9) {

			g_snprintf(sql, sizeof(sql), lobs_v10_sql, B_LOB);
		}
		else {
			g_snprintf(sql, sizeof(sql), lobs_v9_sql, B_LOB);
		}

		execute_sql_gtktreeview(sql, 
					detail_view,
					owner, 
					NULL);
		set_gtktreeview_column_names(bs->treeview, " ", 
					     " ", " ", NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 5, BC_JUNK2,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>LOBs</b>");
		break;



	case B_LOB:

		unhandled=TRUE;
		execute_sql_pivotted(lob_sql, detail_view, NULL,
				     0, owner, object, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>LOB Detail</b>");
		break;



	case B_LINKS:

		g_snprintf(sql, sizeof(sql), links_sql, B_LINK);

		execute_sql_gtktreeview(sql, 
					detail_view,
					owner, 
					NULL);
		set_gtktreeview_column_names(bs->treeview, " ", 
					     " ", " ", NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Database Links</b>");
		break;


	case B_LINK:

		unhandled=TRUE;
		execute_sql_pivotted(link_sql, detail_view,
				     NULL, 0, owner,
				     object, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>DB Link Detail</b>");
		break;



    

	case B_DIRECTORIES:
		g_snprintf(sql, sizeof(sql), directories_sql,
			   B_DIRECTORY);

		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Directories</b>");
		break;


	case B_DIRECTORY:

		unhandled=TRUE;
		execute_sql_pivotted(directory_sql, detail_view,
				     NULL, 0, owner, object,
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Directory Detail</b>");
		break;


	case B_LIBRARIES:

		g_snprintf(sql, sizeof(sql), libraries_sql,
			   B_LIBRARY);

		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Librariess</b>");
		break;


	case B_LIBRARY:

		unhandled=TRUE;
		execute_sql_pivotted(library_sql, detail_view,
				     NULL, 0, owner, object,
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Library Detail</b>");
		break;


	case B_RECYCLEBIN:
    
		g_snprintf(sql, sizeof(sql), recyclebin_sql,
			   B_RECYCLEBIN_ITEM);

		execute_sql_gtktreeview(sql, detail_view, NULL);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Recyclebin Items</b>");
		break;



	case B_FUNCTIONS:
	case B_PROCEDURES:

		g_snprintf(sql, sizeof(sql), plsql_subprograms_sql,
			   atoi(type)==B_FUNCTIONS ? 
			   B_FUNCTION : B_PROCEDURE,
			   atoi(type)==B_FUNCTIONS ? 
			   "function_pixbuf" : "procedure_pixbuf");

		execute_sql_gtktreeview(sql, detail_view, owner,
					atoi(type)==B_FUNCTIONS 
					? "FUNCTION" : "PROCEDURE",
					fg_normal, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 7, BC_OBJECT_INFO,
						 -1);

		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		hide_gtktreeview_column(detail_view, 6);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     atoi(type)==B_FUNCTIONS ? 
				     "<b>Functions</b>" : "<b>Procedures</b>");
		break;



	case B_FUNCTION:
	case B_PROCEDURE:

		unhandled=TRUE;
		if( -1 == atoi(partitioned) ) {

			unhandled=TRUE;
			execute_sql_gtktreeview(plsql_error_sql,
						detail_view, owner, object,
						atoi(type)==B_FUNCTION ? "FUNCTION" : "PROCEDURE", NULL);
			gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
					     "<b>Compilation Error</b>");
		}
		else {
			gchar  *parent_type=0, *package=0;

			gtk_tree_model_get(bs->model, &bs->iter_parent,
					   BC_TYPEID, &parent_type, -1);

			if( B_PACKAGE == atoi(parent_type) ) {

				gtk_tree_model_get(bs->model, &bs->iter_parent,
						   BC_OBJECT, &package, -1);	
				g_snprintf(sql, sizeof(sql), procedure_sql, 
					   B_PLSQL_ARGUMENT,
					   partitioned ? "and (overload=:p4 or overload is null)" : " ");

				execute_sql_gtktreeview(sql, detail_view, owner, object,
							package, partitioned, NULL);
				browser_copy_gtktreeview_columns(detail_view, browser_view,
								 &bs->iter, TRUE,
								 0, BC_TYPEID, 
								 1, BC_PARTITIONED, 
								 2, BC_OWNER, 
								 3, BC_PIXBUF, 
								 4, BC_OBJECT,
								 -1);
				g_free(package);
			}
			else {

				g_snprintf(sql, sizeof(sql), function_sql, B_PLSQL_ARGUMENT);

				execute_sql_gtktreeview(sql, detail_view, owner, object, NULL);
				browser_copy_gtktreeview_columns(detail_view, browser_view,
								 &bs->iter, TRUE,
								 0, BC_TYPEID, 
								 1, BC_PARTITIONED, 
								 2, BC_OWNER, 
								 3, BC_PIXBUF, 
								 4, BC_OBJECT,
								 -1);

				detail_model = gtk_tree_view_get_model(detail_view);
				gtk_tree_store_append(GTK_TREE_STORE(detail_model),
						      &detail_iter, NULL);
				gtk_tree_store_append(GTK_TREE_STORE(detail_model),
						      &detail_iter, NULL);
				gtk_tree_store_set(GTK_TREE_STORE(detail_model),
						   &detail_iter,
						   4, "------------",
						   5, "------------", -1);
				gtk_tree_store_append(GTK_TREE_STORE(detail_model),
						      &detail_iter, NULL);
				gtk_tree_store_set(GTK_TREE_STORE(detail_model),
						   &detail_iter, 4, "PL/SQL Object Info :",
						   -1);
				execute_sql_pivotted(plsql_object_settings_sql,
						     detail_view,
						     &detail_iter,
						     4,
						     owner,
						     object,
						     NULL);
				expand_at_iter(detail_view, &detail_iter);
				gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
						     "<b>PL/SQL Arguments</b>");
			}

			hide_gtktreeview_column(detail_view, 0);
			hide_gtktreeview_column(detail_view, 1);
			hide_gtktreeview_column(detail_view, 2);
			g_free(parent_type);
		}
		break;




	case B_PACKAGES:

		g_snprintf(sql, sizeof(sql), packages_sql, B_PACKAGE);
		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Packages</b>");
		break;



	case B_PACKAGE:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), package_sql, B_FUNCTION, B_PROCEDURE);
		execute_sql_gtktreeview(sql, detail_view, owner, object, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Package Members</b>");
		break;


	case B_TYPES:

		g_snprintf(sql, sizeof(sql), types_sql,
			   B_TYPE);

		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 5, BC_JUNK1,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Types</b>");
		break;


	case B_TYPE:

		if(atoi(partitioned)) {

			unhandled=TRUE;
			execute_sql_gtktreeview(type_error_sql,	detail_view,
						owner, object, "TYPE", NULL);
			gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
					     "<b>Type Compilation Error</b>");

		}
		else {

			gtk_tree_model_get(bs->model, &bs->iter, BC_JUNK1,
					   &parent_object, -1);

			if( !g_ascii_strcasecmp(parent_object, "COLLECTION") ) {
				unhandled=TRUE;
				execute_sql_pivotted(collection_sql,
						     detail_view,
						     NULL,
						     0,
						     owner,
						     object, 
						     NULL);

				gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
						     "<b>Collection Type Details</b>");
			}
			else {

				g_snprintf(sql, sizeof(sql), type_detail_sql,
					   B_TYPE_ATTRIBUTE, B_TYPE_METHOD);

				execute_sql_gtktreeview(sql, detail_view, owner, object,
							owner, object, NULL);
				browser_copy_gtktreeview_columns(detail_view, browser_view,
								 &bs->iter, TRUE,
								 0, BC_TYPEID, 
								 1, BC_PARTITIONED, 
								 2, BC_OWNER, 
								 3, BC_PIXBUF, 
								 4, BC_OBJECT,
								 6, BC_JUNK1,
								 -1);
				hide_gtktreeview_column(detail_view, 0);
				hide_gtktreeview_column(detail_view, 1);
				hide_gtktreeview_column(detail_view, 2);
	
				gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
						     "<b>Object Type Attributes and Methods</b>");
			}
			g_free(parent_object);
		}
		break;


	case B_TYPE_ATTRIBUTE:

		gtk_tree_model_get(bs->model, &bs->iter, BC_JUNK1, &type, -1);
		if(type[0] != '\0') {

			type_part = g_strsplit(type, ".", 2);

#if DEBUG
			fprintf(stderr, "%s:%d  type: %s . %s\n",
				__FILE__, __LINE__, type_part[0], type_part[1]);
#endif

			g_snprintf(sql, sizeof(sql), type_attribute_sql,
				   B_TYPE_ATTRIBUTE, B_TYPE_METHOD);

			execute_sql_gtktreeview(sql, detail_view, type_part[0],
						type_part[1], type_part[0],
						type_part[1], NULL);
			browser_copy_gtktreeview_columns(detail_view, browser_view,
							 &bs->iter, TRUE,
							 0, BC_TYPEID, 
							 1, BC_PARTITIONED, 
							 2, BC_OWNER, 
							 3, BC_PIXBUF, 
							 4, BC_OBJECT,
							 6, BC_JUNK1,
							 -1);
			hide_gtktreeview_column(detail_view, 0);
			hide_gtktreeview_column(detail_view, 1);
			hide_gtktreeview_column(detail_view, 2);
			g_strfreev(type_part);

		}
		break;

	case B_TYPE_METHOD:

		gtk_tree_model_get(bs->model, &bs->iter_parent,
				   BC_OBJECT, &parent_object, -1);
		g_snprintf(sql, sizeof(sql), type_method_sql, B_TYPE_METHOD_ARG);
		execute_sql_gtktreeview(sql, detail_view, owner,
					parent_object, object, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Type Method Args</b>");
		g_free(parent_object);
		break;


	case B_TRIGGERS:

		g_snprintf(sql, sizeof(sql), triggers_sql, B_TRIGGER);
		execute_sql_gtktreeview(sql, detail_view, fg_normal, owner, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);

		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Triggerss</b>");
		break;


	case B_TRIGGER:

		unhandled=TRUE;
		execute_sql_pivotted(trigger_sql,
				     detail_view,
				     NULL,
				     0,
				     owner,
				     object,
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Trigger Detail</b>");
		break;


	case B_JAVA_CLASSES:

		execute_sql_gtktreeview(java_classes_sql, detail_view,
					owner, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Java Classes</b>");
		break;

	case B_JAVA_METHODS:

		execute_sql_gtktreeview(java_methods_sql, detail_view,
					owner, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Java Methods</b>");
		break;


	case B_SCHEDULER_SCHEDULES:

		g_snprintf(sql, sizeof(sql), schedules_sql);
		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Schedules</b>");
		break;


	case B_SCHEDULER_SCHEDULE:

		execute_sql_pivotted(schedule_sql,
				     detail_view,
				     NULL,
				     0,
				     owner,
				     object,
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Shedule Detail</b>");
		break;



	case B_SCHEDULER_WINDOWS:

		g_snprintf(sql, sizeof(sql), windows_sql, B_SCHEDULER_WINDOW);

		execute_sql_gtktreeview(sql, detail_view,fg_normal,
					fg_normal, fg_normal, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED,
						 2, BC_PIXBUF, 
						 3, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Scheduler Windows</b>");
		break;


	case B_SCHEDULER_WINDOW:
		unhandled=TRUE;
		execute_sql_pivotted(window_sql,
				     detail_view,
				     NULL,
				     0,
				     object,
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Sheduler Window Detail</b>");
		break;



	case B_SCHEDULER_JOBS:

		g_snprintf(sql, sizeof(sql), jobs_sql, B_SCHEDULER_JOB);
		execute_sql_gtktreeview(sql, detail_view, fg_normal, owner, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Scheduler Jobs</b>");
		break;
    

	case B_SCHEDULER_JOB:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), job_sql);
		execute_sql_gtktreeview(sql, detail_view, fg_normal,
					owner, object, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Job Log (last 100)</b>");
		break;
    

	case B_SCHEDULER_PROGRAMS:
    
		g_snprintf(sql, sizeof(sql), programs_sql, B_SCHEDULER_PROGRAM);
		execute_sql_gtktreeview(sql, detail_view, owner, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Scheduler Programs</b>");
		break;
    

	case B_SCHEDULER_PROGRAM:

		unhandled=TRUE;
		execute_sql_pivotted(program_sql,
				     detail_view,
				     NULL,
				     0,
				     owner,
				     object, 
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Scheduler Program</b>");
		break;


	case B_INSTANCE:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter, BC_PARTITIONED,
				   &partitioned, -1);

		if(atoi(partitioned)) {

			execute_sql_pivotted(gvinstance_sql,
					     detail_view,
					     NULL,
					     0,
					     partitioned,
					     NULL);
		}
		else {

			g_snprintf(sql, sizeof(sql), instance_sql);
			execute_sql_gtktreeview(sql, detail_view, NULL);
		}
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Instance Details</b>");
		break;


	case B_SERVICES:
    
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_parent, BC_PARTITIONED,
				   &partitioned, -1);
		g_snprintf(sql, sizeof(sql), services_sql, B_SERVICE,
			   atoi(partitioned) ? "and s.inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Services</b>");
		break;

	case B_SERVICE:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), service_sql);

		execute_sql_pivotted(sql,
				     detail_view,
				     NULL,
				     0,
				     object,
				     NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Service Statistics</b>");
		break;


	case B_SESSIONS:
	case B_SESSIONS_ACTIVE:

		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_parent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), sessions_sql, B_SESSION,
			   atoi(partitioned) ? "and s.inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, fg_normal,
					atoi(type)==B_SESSIONS_ACTIVE ? "ACTIVE" : "%",
					partitioned, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 1, BC_OBJECT,
						 7, BC_JUNK1,
						 11, BC_JUNK2,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);

		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Sessions</b>");
		break;


	case B_SESSION:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), session_sql);
		execute_sql_gtktreeview(sql,
					GTK_TREE_VIEW(lw("detailList")),
					object,
					NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Session Statistics</b>");
		break;


	case B_SESSIONS_LONGOPS:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_parent,
				   BC_PARTITIONED, &partitioned, -1);
		g_snprintf(sql, sizeof(sql), longops_sql,
			   atoi(partitioned) ? "and s.inst_id = :p2" : " ");
		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Session LongOps</b>");
		break;


	case B_ADVICE_DBCACHE:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), cache_advice_sql,
			   atoi(partitioned) ? "and inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>DB Cache Advice</b>");
		break;


	case B_ADVICE_PXBUFFER:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), pxbuffer_advice_sql,
			   atoi(partitioned) ? "and inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>PX Buffer Advice</b>");
		break;


	case B_ADVICE_POOL:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		switch(oramajor) {

		case 9:
			g_snprintf(sql, sizeof(sql), pool_advice_v9_sql,
				   atoi(partitioned) ? "and a.inst_id = :p2" : " ");
			break;

		case 10:
		case 11:
			g_snprintf(sql, sizeof(sql), pool_advice_v10_sql,
				   atoi(partitioned) ? "and a.inst_id = :p2" : " ");
			break;
		}

		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Shared Pool Advice</b>");
		break;


	case B_ADVICE_SGA:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), sga_advice_sql,
			   atoi(partitioned) ? "and a.inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, 
					partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>SGA Target Advice</b>");
		break;


	case B_ADVICE_PGA:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), pga_advice_sql,
			   atoi(partitioned) ? "and a.inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view,
					partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>PGA Target Advice</b>");
		break;


	case B_ADVICE_MTTR:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), mttr_advice_sql,
			   atoi(partitioned) ? "and a.inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>MTTR Target Advice</b>");
		break;


	case B_ADVICE_JAVA:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), java_advice_sql,
			   atoi(partitioned) ? "and a.inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>JAVA Pool Advice</b>");
		break;


	case B_ADVICE_STREAMS:

		unhandled=TRUE;
		g_free(partitioned);
		gtk_tree_model_get(bs->model, &bs->iter_grandparent,
				   BC_PARTITIONED, &partitioned, -1);

		g_snprintf(sql, sizeof(sql), streams_advice_sql,
			   atoi(partitioned) ? "and a.inst_id = :p2" : " ");

		execute_sql_gtktreeview(sql, detail_view, partitioned, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>JAVA Pool Advice</b>");
		break;



	case B_USERS:

		g_snprintf(sql, sizeof(sql), users_sql, B_USER);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 5, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Users</b>");
		break;


	case B_USER:

		unhandled=TRUE;
		execute_sql_pivotted(user_sql,
				     detail_view,
				     NULL,
				     0,
				     object,
				     NULL);

		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>User Details</b>");
		break;


	case B_ROLES:

		g_snprintf(sql, sizeof(sql), roles_sql,
			   B_ROLE);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		browser_copy_gtktreeview_columns(detail_view, browser_view,
						 &bs->iter, TRUE,
						 0, BC_TYPEID, 
						 1, BC_PARTITIONED, 
						 2, BC_OWNER, 
						 3, BC_PIXBUF, 
						 4, BC_OBJECT,
						 -1);
		hide_gtktreeview_column(detail_view, 0);
		hide_gtktreeview_column(detail_view, 1);
		hide_gtktreeview_column(detail_view, 2);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Roles</b>");
		break;

	case B_ROLE:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), role_sql);
		execute_sql_gtktreeview(sql, detail_view, object, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Role Grantees</b>");
		break;


	case B_PROFILE:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), profile_sql);
		execute_sql_gtktreeview(sql, detail_view, object, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Profile Resources</b>");
		break;



	case B_DATABASE:

		unhandled=TRUE;
		execute_sql_pivotted("select * from v$database",
				     detail_view, NULL,
				     0, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Database Details</b>");
		unhandled=1;
		break;


	case B_TABLESPACES:

		g_snprintf(sql, sizeof(sql), tablespaces_sql);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Tablespaces</b>");
		break;


	case B_TABLESPACE:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), tablespace_sql);
		execute_sql_gtktreeview(sql, detail_view, object, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Tablespace Segments</b>");
		break;



    
	case B_REDOLOGS:

		g_snprintf(sql, sizeof(sql), redologs_sql);
		execute_sql_gtktreeview(sql, detail_view, fg_normal, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Online Redo Logs</b>");
		break;


	case B_DATAFILES:

		g_snprintf(sql, sizeof(sql), datafiles_sql);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")), 
				     "<b>Datafiles</b>");
		break;


	case B_DATAFILE_10METRIC:
	case B_DATAFILE_1HMETRIC:
    
		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), datafile_metric_sql,
			   atoi(type)==B_DATAFILE_10METRIC ? "v$filemetric" : "v$filemetric_history",
			   atoi(type)==B_DATAFILE_10METRIC ? "d.file_name" : "m.begin_time desc, d.file_name");
		execute_sql_gtktreeview(sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Datafile Metrics</b>");
		break;


	case B_TEMPFILES:

		g_snprintf(sql, sizeof(sql), tempfiles_sql);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Tempfiles</b>");
		break;


	case B_ARCHIVEDLOGS:
    
		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), archived_logs_sql);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>Archived Redo Logs</b>");
		break;



	case B_ASM_DISKGROUPS:

		g_snprintf(sql, sizeof(sql), asm_diskgroups_sql);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>ASM Disk Groups</b>");
		break;


	case B_ASM_DISKGROUP:

		g_snprintf(sql, sizeof(sql), asm_diskgroup_sql);
		execute_sql_gtktreeview(sql, detail_view, partitioned,
					NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>ASM Diskgroup Disks</b>");
		break;



	case B_RMAN:

		unhandled=TRUE;
		execute_sql_gtktreeview(rman_sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>RMAN Status</b>");
		break;


	case B_RMAN_BACKUPS:

		unhandled=TRUE;
		g_snprintf(sql, sizeof(sql), rman_backups_sql);
		execute_sql_gtktreeview(sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>RMAN Backup Jobs</b>");
		break;


	case B_RMAN_ALGOS:
    
		unhandled=TRUE;
		execute_sql_gtktreeview(rman_algos_sql, detail_view, NULL);
		gtk_label_set_markup(GTK_LABEL(lw("detailLabel")),
				     "<b>RMAN Backup Jobs</b>");
		break;


	default:

		unhandled=TRUE;
		break;
	}


	if (!unhandled) {
		gint            nrows=0;
		gchar           obj_info[128];
		GtkTreeModel    *dmodel = gtk_tree_view_get_model(detail_view);

		if (dmodel) {
			nrows = gtk_tree_model_iter_n_children(dmodel, NULL);
			g_snprintf(obj_info, sizeof(obj_info), 
				   "<span foreground=\"blue4\">  (%d)</span>",
				   nrows);
			gtk_tree_store_set(GTK_TREE_STORE(bs->model),
					   &bs->iter, BC_OBJECT_INFO, 
					   obj_info, -1);
		}
	}

	g_free(owner);
	g_free(object);
	g_free(partitioned);
	g_free(type);
}


static gpointer metadata_thread(gpointer data)
{
	metadata_input *meta_info = (metadata_input *)data;

	if (!metadata_query) {

		metadata_query=TRUE;

		dbms_metadata_set_transforms();
		
		if (meta_info->group) {
			dbms_metadata_group_to_textbuffer(meta_info);
		}
		else {
			dbms_metadata_to_textbuffer(meta_info);
		}

		metadata_query=FALSE;
	}
}


static void objectDDLSelected(browserSelection *bs)
{
	OCIStmt *sth=0;
	OCIDefine *defnp;
	GtkTextBuffer *ddl_buffer=0;
	GtkTextIter iter_start, iter_end;
	guint unhandled=0;
	gchar *type=0, *owner=0, *object=0;
	gchar sql_text[4096], type_name[40];
	gboolean group=FALSE;
	metadata_input *me=0;
	gchar *gvsession_sql = "select * \n\
           from table(dbms_xplan.display_cursor((select sql_id from gv$session where sid=:p1), \n\
                                                (select sql_child_number from gv$session where sid=:p1),'ALL' ))";


	if ((ddl_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lw("ddlText")))) == NULL) {
		return;
	}

	gtk_tree_model_get(bs->model, &bs->iter,
			   BC_TYPEID, &type,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   -1);
 
	switch (atoi(type)) {

	case B_TABLES:
		g_snprintf(type_name, sizeof(type_name), "TABLE");
		group=TRUE;
		break;

	case B_TABLE_HEAP:
	case B_TABLE_IOT:
		g_snprintf(type_name, sizeof(type_name), "TABLE");
		break;

	case B_INDEXES:
		g_snprintf(type_name, sizeof(type_name), "INDEX");
		group=TRUE;
		break;
		
	case B_INDEX:
		g_snprintf(type_name, sizeof(type_name), "INDEX");
		break;

	case B_VIEWS:
		g_snprintf(type_name, sizeof(type_name), "VIEW");
		group=TRUE;
		break;

	case B_VIEW:
		g_snprintf(type_name, sizeof(type_name), "VIEW");
		break;

	case B_CLUSTERS:
		g_snprintf(type_name, sizeof(type_name), "CLUSTER");
		group=TRUE;
		break;

	case B_CLUSTER:
		g_snprintf(type_name, sizeof(type_name), "CLUSTER");
		break;

	case B_MVIEWS:
		g_snprintf(type_name, sizeof(type_name), "MATERIALIZED_VIEW");
		group=TRUE;
		break;

	case B_MVIEW:
		g_snprintf(type_name, sizeof(type_name), "MATERIALIZED_VIEW");
		break;

	case B_MVIEW_LOGS:
		g_snprintf(type_name, sizeof(type_name), "MATERIALIZED_VIEW_LOG");
		group=TRUE;
		break;

	case B_MVIEW_LOG:
		g_snprintf(type_name, sizeof(type_name), "MATERIALIZED_VIEW_LOG");
		break;

	case B_SYNONYMS:
		g_snprintf(type_name, sizeof(type_name), "SYNONYM");
		group=TRUE;
		break;

	case B_SYNONYM:
		g_snprintf(type_name, sizeof(type_name), "SYNONYM");
		break;

	case B_QUEUES:
		g_snprintf(type_name, sizeof(type_name), "AQ_QUEUE");
		group=TRUE;
		break;

	case B_QUEUE:
		g_snprintf(type_name, sizeof(type_name), "AQ_QUEUE");
		break;

	case B_SEQUENCES:
		g_snprintf(type_name, sizeof(type_name), "SEQUENCE");
		group=TRUE;
		break;

	case B_SEQUENCE:
		g_snprintf(type_name, sizeof(type_name), "SEQUENCE");
		break;

	case B_LINK:
		g_snprintf(type_name, sizeof(type_name), "DB_LINK");
		break;

	case B_DIRECTORIES:
		g_snprintf(type_name, sizeof(type_name), "DIRECTORY");
		group=TRUE;
		break;

	case B_DIRECTORY:
		g_snprintf(type_name, sizeof(type_name), "DIRECTORY");
		break;

	case B_FUNCTION:
		g_snprintf(type_name, sizeof(type_name), "FUNCTION");
		break;

	case B_PROCEDURE:
		g_snprintf(type_name, sizeof(type_name), "PROCEDURE");
		break;

	case B_PACKAGE:
		g_snprintf(type_name, sizeof(type_name), "PACKAGE");
		break;

	case B_PACKAGE_BODY:
		g_snprintf(type_name, sizeof(type_name), "PACKAGE_BODY");
		break;

	case B_TRIGGER:
		g_snprintf(type_name, sizeof(type_name), "TRIGGER");
		break;

	case B_TYPE:
		g_snprintf(type_name, sizeof(type_name), "TYPE");
		break;

	case B_TYPE_BODY:
		g_snprintf(type_name, sizeof(type_name), "TYPE_BODY");
		break;

	case B_ROLE:
		g_snprintf(type_name, sizeof(type_name), "ROLE");
		break;

	case B_PROFILE:
		g_snprintf(type_name, sizeof(type_name), "PROFILE");
		break;

	case B_SESSION:

		unhandled=1;
		gtk_text_buffer_get_bounds(ddl_buffer, &iter_start,
					   &iter_end);
		gtk_text_buffer_delete(ddl_buffer, &iter_start,
				       &iter_end);

		OCIHandleAlloc((dvoid *)envhp, (dvoid **)&sth,
			       (ub4)OCI_HTYPE_STMT, (size_t)0,
			       (dvoid **)0);
		ociErr(errhp, ociPrepare(sth, errhp, "select t.sql_text stext \n\
     from gv$sqltext_with_newlines t, gv$session s \n\
     where s.sql_address=t.address \n\
       and s.sql_hash_value=t.hash_value \n\
       and s.sid=:p1 \n\
       order by s.sid, t.piece"));
		ociBindString(sth, errhp, 1, (gchar *)object);
		ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 0,
					     0, NULL, NULL, OCI_DEFAULT));
		OCIDefineByPos(sth, &defnp, errhp, (ub4) 1, 
			       (dvoid *) &sql_text, (sb4)sizeof(sql_text)+1,
			       (ub2) SQLT_STR, (dvoid *) 0, (ub2 *) 0,
			       (ub2 *) 0, (ub4) OCI_DEFAULT);
		while (OCI_SUCCESS == OCIStmtFetch2(sth, errhp, 1,
						   OCI_FETCH_NEXT, 0,
						   OCI_DEFAULT)) {
 
			gtk_text_buffer_insert(ddl_buffer, &iter_start, sql_text, -1);
		}
    
		if (oramajor >= 10) {

			gtk_text_buffer_insert(ddl_buffer, &iter_start, 
					       "\n\n\n\nDBMS_XPLAN.DISPLAY_CURSOR():\n",
					       -1);
			ociErr(errhp, ociPrepare(sth, errhp, gvsession_sql));
			ociBindString(sth, errhp, 1, (gchar *)object);
			ociBindString(sth, errhp, 2, (gchar *)object);
			ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 0,
						     0, NULL, NULL, OCI_DEFAULT));
			OCIDefineByPos(sth, &defnp, errhp, (ub4) 1, 
				       (dvoid *) &sql_text, (sb4)sizeof(sql_text)+1,
				       (ub2) SQLT_STR, (dvoid *) 0, (ub2 *) 0,
				       (ub2 *) 0, (ub4) OCI_DEFAULT);

			while (OCI_SUCCESS == OCIStmtFetch2(sth, errhp, 1, OCI_FETCH_NEXT,
							   0, OCI_DEFAULT)) {
 
				gtk_text_buffer_insert(ddl_buffer, &iter_start, sql_text, -1);
				gtk_text_buffer_insert(ddl_buffer, &iter_start, "\n", -1);
			}
		}

		buffer_syntax_highlight(ddl_buffer, -1);
		OCIHandleFree(sth, OCI_HTYPE_STMT);
		break;

	default:
		unhandled=1;
		break;
	}

	if (!unhandled) {
		me = g_malloc(sizeof(metadata_input));
		me->buffer = ddl_buffer;
		me->type_name = g_strdup(type_name);
		me->owner = g_strdup(owner);
		me->object = g_strdup(object);
		me->type = atoi(type);
		me->group = group;
		g_thread_create(metadata_thread,
				(gpointer) me,
				FALSE,
				(GError **)0);
	}

	g_free(owner);
	g_free(type);
	g_free(object);
}




static void objectConstraintsSelected(browserSelection *bs)
{
	GtkTreeView *dep_view = GTK_TREE_VIEW(lw("detailConstraintsList"));
	gchar *type=0;
	gchar *owner=0;
	gchar *object=0;
	gchar sql[2048];
	guint unhandled=0;


	if (!dep_view) {
		return;
	}

	gtk_tree_model_get(bs->model, &bs->iter, 
			   BC_TYPEID, &type, 
			   BC_OWNER, &owner, 
			   BC_OBJECT, &object,
			   -1);
 
	switch (atoi(type)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:

		g_snprintf(sql, sizeof(sql), constraints_sql);
		execute_sql_gtktreeview(sql, dep_view, fg_normal,
					owner, object, NULL);
		break;


	default:
		unhandled=1;
		break;
	}

	if (unhandled) {
		clear_gtktreeview(dep_view);
	}

	g_free(owner);
	g_free(type);
	g_free(object);
}



static void objectGrantsSelected(browserSelection *bs)
{
	GtkTreeView *grants_view = GTK_TREE_VIEW(lw("detailGrantsList"));
	GtkTreeModel *detail_model=0;
	GtkTreeIter detail_iter;
	gchar *type=0, *owner=0, *object=0;
	GList *syn_list=0;
	GList *tmp_list=0;
	static gchar *syn_sql = "select owner||'.'||synonym_name \
          from all_synonyms \
          where table_owner=:p1 and table_name=:p2";


	if (!grants_view) {
		return;
	}

	gtk_tree_model_get(bs->model, &bs->iter, 
			   BC_TYPEID, &type, 
			   BC_OWNER, &owner, 
			   BC_OBJECT, &object, 
			   -1);
 
	switch (atoi(type)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:
	case B_VIEW:
	case B_MVIEW:
	case B_FUNCTION:
	case B_PROCEDURE:
	case B_PACKAGE:
	case B_SYNONYM:
	case B_SEQUENCE:

		execute_sql_gtktreeview(grants_sql, grants_view,
					owner, object, NULL);

		detail_model = gtk_tree_view_get_model(grants_view);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_append(GTK_TREE_STORE(detail_model),
				      &detail_iter, NULL);
		gtk_tree_store_set(GTK_TREE_STORE(detail_model),
				   &detail_iter, 0, (gchar *)"Synonyms pointing here:",
				   -1);
		syn_list = execute_sql_to_glist(syn_sql, owner, object, NULL);
 
		for (tmp_list=syn_list; tmp_list; tmp_list=g_list_next(tmp_list)) {

			gtk_tree_store_append(GTK_TREE_STORE(detail_model),
					      &detail_iter, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(detail_model),
					   &detail_iter, 0, (gchar *) tmp_list->data,
					   -1);
			g_free(tmp_list->data);
		}

		g_list_free(syn_list);
		break;
   

	default:

		clear_gtktreeview(grants_view);
		break;
	}

	g_free(type);
	g_free(owner);
	g_free(object);
}


static void objectDependenciesSelected(browserSelection *bs)
{
	GtkTreeView *dep_view=GTK_TREE_VIEW(lw("detailDependenciesList"));
	gchar *type=0, *owner=0, *object=0;
	gchar type_name[32];
	guint unhandled=0;


	if (!dep_view) {
		return;
	}

	clear_gtktextbuffer(gtk_text_view_get_buffer(GTK_TEXT_VIEW(lw("detailDependenciesText"))));

	gtk_tree_model_get(bs->model, &bs->iter, 
			   BC_TYPEID, &type, 
			   BC_OWNER, &owner, 
			   BC_OBJECT, &object, 
			   -1);
 
	switch (atoi(type)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:
		g_snprintf(type_name, sizeof(type_name), "TABLE");
		break;

	case B_VIEW:
		g_snprintf(type_name, sizeof(type_name), "VIEW");
		break;

	case B_SYNONYM:
		g_snprintf(type_name, sizeof(type_name), "SYNONYM");
		break;

	case B_SEQUENCE:
		g_snprintf(type_name, sizeof(type_name), "SEQUENCE");
		break;

	case B_TYPE:
		g_snprintf(type_name, sizeof(type_name), "TYPE");
		break;

	case B_PACKAGE:
		g_snprintf(type_name, sizeof(type_name), "PACKAGE");
		break;

	case B_FUNCTION:
		g_snprintf(type_name, sizeof(type_name), "FUNCTION");
		break;

	case B_PROCEDURE:
		g_snprintf(type_name, sizeof(type_name), "PROCEDURE");
		break;


	default:
		unhandled=1;
		break;
	}

	if (!unhandled) {

		execute_sql_gtktreeview(dependencies_sql,
					dep_view, owner,
					type_name, object,
					NULL);
	}
			    
	g_free(owner);
	g_free(type);
	g_free(object);
}


void on_dependencySelected(GtkTreeSelection *sel,
			   gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *type=0, *owner=0, *object=0;
	metadata_input *mi=0;


	if (!gtk_tree_selection_get_selected(sel, &model, &iter)) {
		return;
	}

	gtk_tree_model_get(model, &iter, 0, &owner, 1, &object, 2, &type, -1);

	mi = g_malloc(sizeof(metadata_input));
	mi->buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lw("detailDependenciesText")));
	mi->type_name = type;
	mi->owner = owner;
	mi->object = object;
	g_thread_create(metadata_thread, (gpointer) mi,
			FALSE, (GError **)0);

}


void on_objectSelected(GtkTreeSelection *sel,
		       gpointer data)
{
	GtkTreeView *browser_view=0;
	GtkNotebook *detail_book;
	browserSelection bs, bsb;
	gchar *type=0;
	guint pnum=0;
	guint *iptr=0;


	detail_book = GTK_NOTEBOOK(lw("detailNotebook"));
	browser_view = GTK_TREE_VIEW(lw("objectList"));

	if (!browser_view) {
#if DEBUG  
		fprintf(stderr, "%s:%d problem getting pointer to objectList\n",
			__FILE__, __LINE__);
#endif
		return;
	}

	if (!browser_get_selection(browser_view, &bs)) {
		//fprintf(stderr, "no browser selection\n");
		return;
	}

	if (data) {
		iptr = data;
		pnum = *iptr;
	}
	else {
		pnum = gtk_notebook_get_current_page(detail_book);
	}

#if DEBUG
	fprintf(stderr, "%s:%d object_browser.c: detailNotebook: page %d\n",
		__FILE__, __LINE__, pnum);
#endif

	gtk_tree_model_get(bs.model, &bs.iter, 
			   BC_TYPEID, &type,
			   -1);

	switch (atoi(type)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:
	case B_VIEW:
	case B_FUNCTION:
	case B_PROCEDURE:
	case B_PACKAGE:
	case B_SYNONYM:
	case B_SEQUENCE:
	case B_MVIEW:
		gtk_widget_show(gtk_notebook_get_nth_page(detail_book,
							  NB_GRANTS));
		break;


	default:
		if (NB_GRANTS != gtk_notebook_get_current_page(detail_book)) {
			gtk_widget_hide(gtk_notebook_get_nth_page(detail_book,
								  NB_GRANTS));
		}
		break;
	}

	switch (atoi(type)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:
	case B_MVIEW:
		gtk_widget_show(gtk_notebook_get_nth_page(detail_book, NB_CONSTRAINTS));
		break;

	default:
		if (gtk_notebook_get_current_page(detail_book) != NB_CONSTRAINTS) {
			gtk_widget_hide(gtk_notebook_get_nth_page(detail_book, NB_CONSTRAINTS));
		}
		break;
	}
	g_free(type);

	browser_get_selection(browser_view, &bsb);
	objectDetailSelected(&bsb);
	browser_free_selection(&bsb);

	switch (pnum) {

	case NB_DETAIL:
		break;

	case NB_CONSTRAINTS:
		objectConstraintsSelected(&bs);
		break;

	case NB_GRANTS:
		objectGrantsSelected(&bs);
		break;

	case NB_DEPENDENCIES:
		objectDependenciesSelected(&bs);
		break;

	case NB_DDL:
		objectDDLSelected(&bs);
		break;

	default:
		break;
	}

	browser_free_selection(&bs);
}

