#ifndef BROWSER_SELECTED_SQL_H_
#define BROWSER_SELECTED_SQL_H_


gchar *grants_sql =
"select\n\
  grantor \"Granted By\" \n\
 ,grantee \"Granted To\" \n\
 ,privilege \"Privilege\" \n\
 ,grantable \"With Grant\" \n\
 ,hierarchy \"Hierarchy\" \n\
from all_tab_privs \
where table_schema=:p1 \n\
  and table_name=:p2 \n\
order by grantor, grantee, privilege";

gchar *constraints_sql =
"select constraint_name, \n\
 decode(constraint_type, 'C','Check','P','Primary Key',\n\
 'U','Unique','R','Ref Integrity',\n\
 'V','View Check','O','View RO', constraint_type) \"Type\", \n\
 decode(index_owner, NULL, index_name, index_owner || '.' || index_name) \"Index\", \n\
 decode(r_owner, NULL, r_constraint_name, r_owner || '.' ||  r_constraint_name) \"Reference\", \n\
 search_condition \"Condition\",\n\
 delete_rule \"Delete Rule\", status \"Status\", decode(status,'ENABLED','green3', :fg_normal) \"foreground\",\n\
 deferrable \"Deferrable\", deferred \"Deferred\", validated \"Validated\", \n\
 bad \"Bad\", rely \"Rely\",\n\
 last_change\n\
from all_constraints \n\
where owner=:p1 \n\
  and table_name=:p2 \n\
order by constraint_type, constraint_name";

gchar *dependencies_sql =
"select \n\
 owner \"Owner\", \n\
 name \"Name\",\n\
 type \"Type\",\n\
 dependency_type \"Dependency Type\"\n\
from all_dependencies \n\
where referenced_owner=:p1 \n\
  and referenced_type=:p2 \n\
  and referenced_name=:p3 \n\
order by name";

gchar *schema_other_sql =
"select \n\
  %d,\n\
  user_id,\n\
  username un, \n\
  'schema_pixbuf' \"pxb\",\n\
  username \"Schema\", \n\
  created \"Created\"\n\
 from all_users \n\
 where username != :p1 \n\
 order by username";

gchar *tables_sql =
"SELECT \n\
  decode(t.iot_type,'IOT',%d,%d) \"Type\", \n\
  decode(t.partitioned,'YES', 1, 0), \n\
  t.owner,\n\
  decode(t.iot_type, 'IOT', \n\
         decode(t.partitioned, \
          'YES', 'iot_partition_pixbuf', \
          'iot_pixbuf'), \n\
         decode(t.partitioned, \
            'YES', 'partition_pixbuf', \
           decode(temporary,\
             'Y','temptable_pixbuf', 'heap_pixbuf'))) \"pxb\", \n\
  t.table_name \"Table\", \n\
  t.tablespace_name \"Tablespace\",\n\
  t.logging \"Logging\",\n\
  t.degree \"Degree\",\n\
  t.num_rows \"# Rows\",\n\
  t.blocks+t.empty_blocks \"Blocks\",\n\
  t.avg_row_len \"AvgLen\",\n\
  t.sample_size \"Sampled\",\n\
  to_char(t.last_analyzed,'MM/DD HH24:MI') \"Analyzed\",\n\
  t.buffer_pool \"Pool\",\n\
  t.row_movement \"RowMove\",\n\
  t.skip_corrupt \"SkipCorrupt\",\n\
  t.monitoring \"Monitoring\"\n\
 from all_tables t \n\
 where t.owner like :p1 \n\
   and not exists (select 1 from all_mviews mv where mv.owner=t.owner and mv.mview_name=t.table_name) \n\
   and not exists (select 1 from all_mview_logs mvl where mvl.log_owner=t.owner and mvl.log_table=t.table_name) \n\
  order by t.table_name";

gchar *table_sql =
"select %d, 0, \n\
    t.column_name \"Column\",\n\
    max(t.data_type || \n\
    CASE \n\
       WHEN t.char_used='B' THEN '(' || t.data_length || ')'\n\
       WHEN t.data_type like 'TIMESTAMP%%' THEN NULL\n\
       ELSE decode(t.data_scale, NULL ,' ', '(' || t.data_precision || ',' || t.data_scale || ')') \n\
    END) \"Datatype\",\n\
    max(t.nullable) \"Null\",\n\
    max(decode(c.constraint_type,'P','Y',NULL)) \"PK\",\n\
    max(decode(c.r_owner,NULL,NULL,c.r_owner || '.' || c.r_constraint_name)) \"Ref Constaint\",\n\
    max(t.num_distinct) \"Distinct\", \n\
    max(t.num_nulls) \"Nulls\", \n\
    max(t.density) \"Density\",\n\
    max(t.avg_col_len) \"AvgLen\",\n\
    max(t.num_buckets) \"Bckts\",\n\
    max(t.sample_size) \"Sample\", \n\
    max(to_char(t.last_analyzed,'MM/DD HH24:MI')) \"Analyzed\",\n\
    max(com.comments) \"Comments\",\n\
    max(decode(t.data_type_owner, NULL, NULL, t.data_type_owner || '.' || t.data_type)) \"User Datatype\"\n\
   from all_tab_columns t JOIN all_col_comments com on com.owner=t.owner \n\
           and com.table_name=t.table_name and com.column_name=t.column_name \n\
        left outer join all_cons_columns cc on cc.owner=t.owner and cc.table_name=t.table_name \n\
           and cc.column_name=t.column_name \n\
        left outer join all_constraints c on cc.table_name=c.table_name and cc.owner=c.owner \n\
           and c.constraint_name=cc.constraint_name and c.constraint_type in ('R','P') \n\
   where t.owner=:p1 and t.table_name=:p2 \n\
   group by t.column_name \n\
   order by max(t.column_id)";

gchar *table_partition_sql =
"select * from all_tab_partitions \
 where table_owner=:p1 \
 and table_name=:p2 \
 and partition_name=:p3";

gchar *table_subpartition_sql =
"select * from all_tab_subpartitions \
 where table_owner=:p1 \
 and table_name=:p2 \
 and partition_name=:p3 \
 and subpartition_name=:p4";

gchar *partition_v10_sql =
"select val from (\
  select 0 pos, 'Partitioned by ' || partitioning_type || ' ' \
      || decode(interval,NULL,NULL, ' ('||interval||')') || ' on:' val \
  from all_part_tables \
  where owner=:p1 and table_name=:p2 \
  union all \
  select column_position, column_name \
  from all_part_key_columns \
  where owner=:p1 and name=:p2 and object_type='TABLE' \
  union all \
  select decode(subpartitioning_type,'NONE',200,100) pos, 'Subpartitioned by ' \
     || subpartitioning_type || ' on:' val \
  from all_part_tables \
  where owner=:p1 and table_name=:p2 \
  union all \
  select column_position*100, column_name \
  from all_subpart_key_columns \
  where owner=:p1 and name=:p2 and object_type='TABLE' \
 ) \
 where pos < 200 \
 order by pos";

gchar *partition_v9_sql =
"select val from (\
  select 0 pos, 'Partitioned by ' || partitioning_type || ' on:' val \
   from all_part_tables \
   where owner=:p1 and table_name=:p2 \
   union all \
   select column_position, column_name \
   from all_part_key_columns \
   where owner=:p1 and name=:p2 and object_type='TABLE' \
   union all \
   select decode(subpartitioning_type,'NONE',200,100) pos, 'Subpartitioned by ' \
      || subpartitioning_type || ' on:' val \
   from all_part_tables \
   where owner=:p1 and table_name=:p2 \
   union all \
   select column_position*100, column_name \
   from all_subpart_key_columns \
   where owner=:p1 and name=:p2 and object_type='TABLE' \
 ) \
 where pos < 200 \
 order by pos";

gchar *indexes_sql =
"select %d,\n\
     decode(a.partitioned,'NO',0,1) par,\n\
     ai.index_owner,\n\
     decode(a.FUNCIDX_STATUS, NULL, \n\
            decode(a.index_type, \
              'BITMAP', decode(a.partitioned,'YES','bitmap_partition_pixbuf', 'bitmap_pixbuf'), \n\
                   decode(a.partitioned,'YES','iot_partition_pixbuf','iot_pixbuf')), 'function_pixbuf') \"pxb\",\n\
     ai.index_name \"Index\",\n\
     decode(status,'UNUSABLE','red4', :fg_normal) \"foreground\",\n\
     a.table_owner || '.' || a.table_name \"Table\",\n\
     ai.columns \"Columns\",\n\
     a.uniqueness \"Unique\",\n\
     decode(a.compression,'DISABLED',a.compression,a.compression \
      ||'('||a.prefix_length||')') \"Compress\",\n\
     a.tablespace_name \"Tablespace\",\n\
     a.initial_extent \"Initial\",\n\
     a.next_extent \"Next\",\n\
     a.pct_threshold \"PctThreshold\",\n\
     a.logging \"Logging\",\n\
     a.blevel \"BLevel\",\n\
     a.leaf_blocks \"LBlocks\",\n\
     a.distinct_keys \"Distinct\",\n\
     a.AVG_LEAF_BLOCKS_PER_KEY \"AvgLBKey\",\n\
     a.AVG_DATA_BLOCKS_PER_KEY \"AvgDBKey\",\n\
     a.clustering_factor \"ClFact\",\n\
     a.num_rows \"# Rows\",\n\
     a.sample_size \"Sample\",\n\
     to_char(last_analyzed, 'MM/DD HH24:MI') \"Analyzed\",\n\
     a.degree \"Degree\",\n\
     a.instances \"Instances\",\n\
     a.temporary \"Temp\",\n\
     a.buffer_pool \"Pool\",\n\
     a.pct_direct_access \"%% DI\",\n\
     a.ITYP_OWNER || '.' || a.ityp_name \"IType\"\n\
    from\
    (with data as (\n\
     select \n\
      i.owner index_owner,\n\
      c.index_name index_name,\n\
      c.column_name col,\n\
      row_number() over (partition by c.index_name order by c.column_position) rn,\n\
      count(*) over (partition by c.index_name) cnt\n\
     from all_ind_columns c JOIN all_indexes i ON i.index_name = c.index_name\n\
      AND i.owner = c.index_owner AND i.table_owner = c.table_owner AND i.table_name = c.table_name\n\
     where i.owner=:p1\n\
    )\n\
    select index_owner, index_name, ltrim(sys_connect_by_path(col,','),',') columns \n\
    from data\n\
    where rn = cnt\n\
    start with rn = 1\n\
      connect by prior index_name=index_name and prior rn = rn-1\n\
    ) ai,\n\
    all_indexes a\n\
    where a.owner=ai.index_owner and a.index_name=ai.index_name and a.owner=:p2\n\
    order by a.index_name";

gchar *index_sql =
"select \n\
  i.column_name \"Column\",\n\
  e.column_expression \"FBI Expression\", \n\
  i.column_length \"Length\",\n\
   decode(i.descend,'ASC','Ascending','DSC','Descending',i.descend)  \"Order\"\n\
 from all_ind_columns i left outer join \n\
   (select column_expression, column_position from all_ind_expressions\n\
    where index_owner=:p1 and index_name=:p2) e\n\
       ON e.column_position=i.column_position\n\
 where i.index_owner = :p3 and i.index_name = :p4\n\
 order by i.column_position";

gchar *index_detail_sql =
"select * from all_indexes \n\
 where owner=:p1 \n\
   and index_name=:p2";

gchar *indexes_invalid_sql =
 "select index_name \"Index\", NULL \"Partition\" \
 from all_indexes \
 where status = 'UNUSABLE' and owner=:p1\n\
 union all \n\
 select index_name, partition_name \
 from all_ind_partitions \
 where status = 'UNUSABLE' and index_owner=:p1 \n\
 union all \n\
 select index_name, subpartition_name \
 from all_ind_subpartitions \
 where status = 'UNUSABLE' and index_owner=:p1";

gchar *index_partitions_sql = 
"select val from (\n\
  select 0 pos, initcap(locality) || ' partitioned by ' || partitioning_type || ' on:' val \n\
  from all_part_indexes \n\
  where owner=:p1 and index_name=:p2 \n\
  union all \n\
  select column_position, column_name \
  from all_part_key_columns \
  where owner=:p1 and name=:p2 and object_type='INDEX' \
 ) \
 where pos < 200 \
 order by pos";

gchar *index_partition_sql =
"select * \
 from all_ind_partitions \
 where index_owner=:p1 \
 and index_name=:p2 \
 and partition_name=:p3";

gchar *index_subpartition_sql =
"select * from all_ind_subpartitions \n\
 where index_owner=:p1 \n\
 and index_name=:p2 \n\
 and partition_name=:p3 \n\
 and subpartition_name=:p4";

gchar *views_sql =
"select %d, 0, v.owner, 'view_pixbuf' \"pxb\", \n\
  v.view_name \"View\",\n\
  v.view_type_owner \"Type Owner\", \n\
  v.view_type \"View Type\",\n\
  v.superview_name \"Superview Name\" \n\
 from all_views v \n\
 where v.owner=:p1 \n\
 order by v.view_name";

gchar *clusters_sql =
"select %d, 0, owner, 'cluster_pixbuf' \"pxb\", \n\
  cluster_name \"Cluster\", \n\
  cluster_type \"Type\", \n\
  function \"Function\", \n\
  hashkeys \"HashKeys\", \n\
  key_size \"Key Size\",\n\
  single_table \"Single\", tablespace_name \"Tablespace\", degree \"Degree\", \n\
  cache \"Cache\", buffer_pool \"Pool\"\n\
 from all_clusters \n\
 where owner=:p1 order by cluster_name";

gchar *cluster_sql =
"SELECT %d, 0, t.owner, 'heap_pixbuf' \"pxb\", \n\
    t.table_name \"Table\", \n\
    t.tablespace_name \"Tablespace\",\n\
    t.logging \"Logging\",\n\
    t.degree \"Degree\",\n\
    t.num_rows \"# Rows\",\n\
    t.blocks+t.empty_blocks \"Blocks\",\n\
    t.avg_row_len \"AvgLen\",\n\
    t.sample_size \"Sampled\",\n\
    to_char(t.last_analyzed,'MM/DD HH24:MI') \"Analyzed\",\n\
    t.buffer_pool \"Pool\",\n\
    t.row_movement \"RowMove\",\n\
    t.skip_corrupt \"SkipCorrupt\",\n\
    t.monitoring \"Monitoring\"\n\
    from all_tables t \n\
    where t.owner=:p1 and cluster_name=:p2\
    order by t.table_name";

gchar *cluster_detail_sql =
"select * from all_clusters \
 where owner=:p1 \
   and cluster_name=:p2";

gchar *mviews_sql =
"select %d, 0, owner, 'mview_pixbuf' \"pxb\",\n\
    mview_name \"MView\", \n\
    container_name \"Container\",\n\
    updatable \"Updatable\",\n\
    rewrite_enabled \"Rewrite\",\n\
    rewrite_capability \"Rewrite Capability\",\n\
    refresh_mode \"Refresh Mode\",\n\
    refresh_method \"Refresh Method\",\n\
    build_mode \"Build Mode\",\n\
    FAST_REFRESHABLE \"Fast Refresh\",\n\
    LAST_REFRESH_TYPE \"Last Refresh Type\",\n\
    to_char(last_refresh_date, 'YYYY-MM-DD HH24:MI') \"Last Refresh\",\n\
    update_log \"Update Log\",\n\
    master_rollback_seg \"Master RB Seg\",\n\
    master_link \"Master Link\",\n\
    staleness \"Staleness\",\n\
    after_fast_refresh \"After Fast Refresh\",\n\
    compile_state \"Compile State\"\n\
    from all_mviews \n\
    where owner=:p1 order by mview_name";

gchar *mview_sql =
"select %d, 0,\n\
  t.column_name \"Column\",\n\
  max(t.data_type || \n\
  CASE \n\
       WHEN t.char_used='B' THEN '(' || t.data_length || ')'\n\
       WHEN t.data_type like 'TIMESTAMP%%' THEN NULL\n\
       ELSE decode(t.data_scale, NULL ,' ', '(' || t.data_precision || ',' || t.data_scale || ')') \n\
  END) \"Datatype\",\n\
  max(t.nullable) \"Null\",\n\
  max(decode(c.constraint_type,'P','Y',NULL)) \"PK\",\n\
  max(decode(c.r_owner,NULL,NULL,c.r_owner || '.' || c.r_constraint_name)) \"Ref Constaint\",\n\
  max(t.num_distinct) \"Distinct\", \n\
  max(t.num_nulls) \"Nulls\", \n\
  max(t.density) \"Density\",\n\
  max(t.avg_col_len) \"AvgLen\",\n\
  max(t.num_buckets) \"Bckts\",\n\
  max(t.sample_size) \"Sample\", \n\
  max(to_char(t.last_analyzed,'MM/DD HH24:MI')) \"Analyzed\",\n\
  max(com.comments) \"Comments\",\n\
  max(decode(t.data_type_owner, \
    NULL, NULL, \
    t.data_type_owner || '.' || t.data_type)\
     ) \"User Datatype\"\n\
 from all_tab_columns t JOIN all_col_comments com on com.owner=t.owner \n\
      and com.table_name=t.table_name and com.column_name=t.column_name \n\
   left outer join all_cons_columns cc on cc.owner=t.owner and cc.table_name=t.table_name \n\
      and cc.column_name=t.column_name \n\
   left outer join all_constraints c on cc.table_name=c.table_name and cc.owner=c.owner \n\
      and c.constraint_name=cc.constraint_name and c.constraint_type in ('R','P') \n\
 where t.owner=:p1 \n\
   and t.table_name=:p2 \n\
 group by t.column_name \n\
 order by max(t.column_id)";

gchar *mview_detail_sql =
"select * from all_mviews \
 where owner=:p1 \
   and mview_name=:p2";

gchar *mview_logs_sql =
"select %d, 0, log_owner, 'mview_log_pixbuf' \"pxb\",\n\
    log_table \"Log\", \n\
    master \"Master\",\n\
    log_trigger \"Trigger\",\n\
    rowids \"Rowids\",\n\
    primary_key \"Primary Key\",\n\
    object_id \"Object ID\",\n\
    filter_columns \"Filter Columns\",\n\
    sequence \"Sequence\",\n\
    include_new_values \"Include New\"\n\
    from all_mview_logs where log_owner=:p1";

gchar *mvlog_detail_sql =
"select * from all_mview_logs \
 where log_owner=:p1 \
   and log_table=:p2";

gchar *registered_snapshot_sql =
"select \n\
  r.owner ||'@'|| r.snapshot_site \n\
  from all_registered_snapshots r \n\
 where r.name is not null \n\
   and r.name = (select master from all_mview_logs where log_owner=:p2 and log_table=:p3)";

gchar *queue_sql =
"select * from all_queues where owner=:p1 and name=:p2";

gchar *queue_datatype_sql =
"select c.data_type_owner || '.' || c.data_type \
 from all_tab_columns c join all_queues q on q.queue_table=c.table_name \
 where q.owner=:p1 and q.name=:p2 and c.column_name='USER_DATA'";

gchar *queue_table_sql =
"select owner || '.AQ$' || queue_table \
 from all_queues \
 where owner=:p1 and name=:p2";

gchar *queue_msg_state_sql =
"select msg_state || rpad(' ', 10-length(msg_state),' ') || ': ' || to_char(count(*)) \
 from %s group by msg_state";

gchar *queues_sql =
"select \n\
  %d, \n\
  0, \n\
  owner,\n\
  'queue_pixbuf' \"pxb\",\n\
  name, \n\
  queue_table, \n\
  enqueue_enabled, \n\
  dequeue_enabled, \
  qid, \n\
  queue_type, \n\
  max_retries, \n\
  retention, \n\
  user_comment \n\
 from all_queues \n\
 where owner=:p1 \n\
 order by name";

gchar *synonyms_sql =
"select \n\
  %d, \n\
  0, \n\
  owner, \n\
  'synonym_pixbuf' \"pxb\", \n\
  synonym_name \"Synonym\", \n\
  table_owner \"Table Owner\", \n\
  table_name \"Table\", \n\
  db_link \"DB Link\" \n\
 from all_synonyms \n\
 where owner=:p1 \
   and synonym_name like :p2 \n\
 order by synonym_name";

gchar *synonym_sql =
"select  \n\
  t.column_name \"Column\", \n\
  t.nullable \"Null\",\n\
  (CASE WHEN t.data_precision is null THEN t.data_type||'('||t.data_length||')' \n\
     ELSE t.data_type||'('||t.data_precision||','||t.data_scale||')' END) \"Datatype\",\n\
  t.nullable \"Null\",\n\
  t.data_default \"Default\",\n\
  c.comments \"Comments\"\n\
 from all_tab_columns t, all_col_comments c \n\
 where c.owner=t.owner \
   and c.table_name=t.table_name\n\
   and c.column_name = t.column_name \
   and t.owner=:p1 \
   and t.table_name=:p2 \n\
 order by t.column_id";

gchar *public_synonym_sql =
"select \n\
  %d, \n\
  0, \n\
  owner, \n\
  'synonym_pixbuf' \"pxb\", \n\
  synonym_name \"Synonym\", \n\
  table_owner \"Table Owner\", \n\
  table_name \"Table\", \n\
  db_link \"DB Link\" \n\
 from all_synonyms \n\
 where owner=:p1 \
   and synonym_name like :p2 \n\
 order by synonym_name";

gchar *sequences_sql =
"select \n\
  %d, \n\
  0, \n\
  sequence_owner, \n\
  'sequence_pixbuf' \"pxb\",\n\
  sequence_name \"Sequence\",\n\
  last_number \"Last\", \n\
  min_value \"MinValue\",\n\
  max_value \"MaxValue\",\n\
  increment_by \"Increment\",\n\
  cache_size \"Cache Size\",\n\
  cycle_flag \"Cycle Flag\",\n\
  order_flag \"Order Flag\"\n\
 from all_sequences \n\
 where sequence_owner=:p1 \n\
 order by sequence_name";

gchar *sequence_sql =
"select * from all_sequences \
 where sequence_owner=:p1\
   and sequence_name=:p2";

gchar *lobs_v10_sql =
"select \n\
  %d,\n\
  decode(partitioned,'YES',1,0),\n\
  owner,\n\
  'lob_pixbuf' \"pxb\",\n\
  segment_name,\n\
  table_name || '.' || column_name \"Column\",\n\
  tablespace_name \"Tablespace\",\n\
  index_name \"Index\",\n\
  chunk,\n\
  pctversion,\n\
  retention,\n\
  freepools,\n\
  cache,\n\
  logging,\n\
  in_row,\n\
  format,\n\
  partitioned\n\
 from all_lobs\n\
 where owner=:p1\n\
 order by segment_name";

gchar *lobs_v9_sql =
"select\n\
      %d,\n\
      1,\n\
      owner,\n\
      'lob_pixbuf' \"pxb\",\n\
      segment_name,\n\
      table_name || '.' || column_name \"Column\",\n\
      index_name \"Index\",\n\
      chunk,\n\
      pctversion,\n\
      retention,\n\
      freepools,\n\
      cache,\n\
      logging,\n\
      in_row\n\
      from all_lobs\n\
      where owner=:p1\n\
      order by segment_name";

gchar *lob_sql =
"select * from all_lobs \
 where owner=:p1 \
   and segment_name=:p2";

gchar *links_sql =
"select\n\
  %d,\n\
  0,\n\
  owner,\n\
  'dblink_pixbuf' \"pxb\",\n\
  db_link \"Link\",\n\
  username \"Username\",\n\
  host \"Host\",\n\
  to_char(created, 'YYYY-MM-DD HH24:MI:SS') \"Created\"\n\
 from all_db_links\n\
 where owner=:p1\n\
 order by db_link";

gchar *link_sql = 
"select * from all_db_links where owner=:p1 and db_link=:p2";

gchar *directories_sql =
"select\n\
    %d,\n\
    0,\n\
    owner,\n\
    'directory_pixbuf' \"pxb\",\n\
    directory_name \"Directory\",\n\
    directory_path \"Path\"\n\
    from all_directories\n\
    order by directory_name";

gchar *directory_sql =
"select * \
 from all_directories \
 where owner=:p1 \
   and directory_name=:p2";

gchar *libraries_sql =
"select\n\
   %d,\n\
   0,\n\
   owner,\n\
   'library_pixbuf' \"pxb\",\n\
   library_name \"Library\",\n\
   file_spec \"File Spec\",\n\
   dynamic \"Dynamic\",\n\
   status \"Status\"\n\
   from all_libraries\n\
   where owner=:p1\n\
   order by library_name";

gchar *library_sql =
"select * \
 from all_libraries \
 where owner=:p1 \
   and library_name=:p2";

gchar *recyclebin_sql =
"select %d, 0,\n\
  decode(type,\
    'TABLE','heap_pixbuf',\
    'NORMAL INDEX','iot_pixbuf',\
    'BITMAP INDEX','bitmap_pixbuf',\n\
    'LOB','lob_pixbuf',\
    'LOB INDEX','lob_pixbuf', \
    'Table Partition','partition_pixbuf',\n\
    'Index Partition','iot_partition_pixbuf', \
    'LOB Partition','lob_pixbuf',\n\
     NULL) \"pxb\",\n\
  original_name \"Original\", type, \n\
  object_name \"Recycle Seg\",\n\
  ts_name \"Tablespace\", \n\
  createtime  \"Created\",\n\
  droptime  \"Dropped\",\n\
  dropscn \"Drop SCN\",\n\
  can_undrop \"Can Undrop?\",\n\
  can_purge \"Can Unpurge?\",\n\
  related \"Related\",\n\
  base_object \"Base Obj\",\n\
  purge_object \"Purge Obj\",\n\
  space \"# Blocks\"\n\
 from user_recyclebin \n\
 order by original_name";

gchar *procedure_sql =
"select \n\
  %d, 0, owner,\n\
  decode(in_out,'OUT',\
     'plsqlargumentout_pixbuf','IN/OUT',\
     'plsqlargumentinout_pixbuf',\
     'plsqlargument_pixbuf') \"pxb\", \n\
 argument_name \"Name\", \n\
 data_type \"Datatype\", \n\
 in_out \"In/Out\", \n\
 default_value \"Default\", \n\
 decode(type_owner,NULL,NULL,type_owner || '.' || type_name) \"Type\" \n\
 from all_arguments where owner=:p1 and object_name=:p2 and package_name=:p3 %s\n\
 order by sequence";

gchar *function_sql =
"select \n\
  %d, 0, owner,\n\
  decode(in_out,'OUT',\
    'plsqlargumentout_pixbuf','IN/OUT', \
    'plsqlargumentinout_pixbuf',\
    'plsqlargument_pixbuf') \"pxb\", \n\
  argument_name \"Name\", \n\
  data_type \"Datatype\", \n\
  in_out \"In/Out\", \n\
  default_value \"Default\", \n\
  decode(type_owner,NULL,NULL,type_owner || '.' || type_name) \"Type\" \n\
 from all_arguments where owner=:p1 and object_name=:p2 and package_name is null\n\
 order by sequence";

gchar *plsql_object_settings_sql =
"select * from all_plsql_object_settings where owner=:p1 and name=:p2";

gchar *plsql_subprograms_sql =
"with data as (\n\
     select \n\
      o.owner,\n\
      o.object_name,\n\
      o.status,\n\
      nvl(a.argument_name, decode(o.object_type,'FUNCTION','RETURN',' '))  \
       || ' ' || a.in_out || ' ' || a.data_type arg,\n\
      row_number() over (partition by o.object_name order by a.position) rn,\n\
      count(*) over (partition by o.object_name) cnt\n\
     from all_objects o LEFT OUTER JOIN all_arguments a ON o.owner=a.owner \n\
        AND o.object_name=a.object_name \n\
     where o.owner=:p1 \n\
       and o.object_type=:p2\n\
    ) \n\
    select\n\
     %d,\n\
     decode(status,'INVALID',-1,0),\n\
     owner,\n\
     '%s' \"pxb\", \n\
     object_name \"Function\", \n\
     decode(status,'INVALID','red3', :fg_normal) \"foreground\", \n\
     decode(status, 'INVALID', NULL, '(' || ltrim(sys_connect_by_path(arg,', '),', ') || ')') \"Arguments\",\n\
     decode(status, 'INVALID', '<span foreground=\"red3\">!!</span>', NULL) \"Invalid?\"\n\
    from data \n\
    where rn=cnt \n\
    start with rn=1 connect by prior object_name=object_name and prior rn=rn-1";


gchar *plsql_error_sql =
"select line \"Line\", position \"Position\", text \"Message\", 'red4' \"foreground\"\n\
 from all_errors where owner=:p1 and name=:p2 and type=:p3 \n\
 order by sequence";

gchar *packages_sql =
"select %d, 0, owner, 'package_pixbuf' \"pxb\", package_name \n\
 from all_arguments where owner=:p1 and package_name is not null \n\
 group by owner, package_name \n\
 order by owner, package_name";

gchar *package_sql =
"select \n\
  decode(min(decode(position, 0, 0, 1)), 0, %d, %d),\n\
  min(overload),\n\
  owner,\n\
  decode(min(decode(position, 0, 0, 1)), 0, 'function_pixbuf', 'procedure_pixbuf') \"pxb\",\n\
  object_name \"Member\", \n\
  min(overload) \"Overload\" \n\
 from all_arguments \
 where owner=:p1 and package_name=:p2 \n\
 group by owner, object_name, overload\n\
 order by object_name";

gchar *types_sql =
"select \n\
  %d,\n\
  (select count(*) \
   from all_errors \
   where owner=t.owner \
   and type='TYPE'\
   and name=t.type_name) \"Errors\", \n\
  owner,\n\
  'type_pixbuf' \"pxb\",\n\
  type_name \"Type\", \n\
  typecode \"TypeCode\",\n\
  attributes \"Attributes\",\n\
  methods \"# Methods\",\n\
  predefined \"Predefined\",\n\
  incomplete \"Incomplete\",\n\
  final \"Final\",\n\
  INSTANTIABLE \"Instantiable\",\n\
  decode(supertype_name, NULL, NULL, supertype_owner ||'.'||supertype_name) \"Supertype\",\n\
  local_attributes \"Local Attr\",\n\
  local_methods \"Local Methods\",\n\
  typeid \"TypeID\"\n\
 from all_types t \n\
 where owner=:p1 \n\
 order by type_name";

gchar *type_error_sql =
"select line \"Line\", position \"Position\", text \"Message\", 'red4' \"foreground\"\n\
 from all_errors where owner=:p1 and name=:p2 and type=:p3 \n\
 order by sequence";


gchar *collection_sql =
"select * from all_coll_types where owner=:p1 and type_name=:p2";

gchar *type_detail_sql =
"select typeid, par, own, pic \"pxb\", n \"Name\", d \"Datatype\", o \"Object Type\" from (\n\
  select\n\
   %d typeid, \n\
   0 par, \n\
   owner own,\n\
   'plsqlattribute_pixbuf' pic,\n\
   attr_name n,\n\
   case \n\
   when precision is not null then decode(scale, NULL, attr_type_name || '(' || precision || ')', \n\
   attr_type_name || '(' || precision || ',' || scale || ')') \n\
   when scale is not null then attr_type_name || '(' || scale || ')' \n\
   when length is not null then attr_type_name || '(' || length || ')' \n\
   else attr_type_name  \n\
   end d, \n\
   decode(attr_type_owner, NULL, NULL, attr_type_owner || '.' || attr_type_name) o, \n\
   attr_no r\n\
   from all_type_attrs \n\
   where owner=:p1 and type_name=:p2 \n\
   union all\n\
   select %d,\n\
   0,\n\
   owner,\n\
   'typemethod_pixbuf',\n\
   method_name, \n\
   NULL, \n\
   NULL, \n\
   method_no+100\n\
   from all_type_methods where owner=:p1 and type_name=:p2\n\
   ) order by o";


gchar *type_attribute_sql =
"select typeid, par, own, pic \"pxb\", n \"Name\", d \"Datatype\", o \"Object Type\" from (\n\
   select \n\
    %d typeid, \n\
    0 par, \n\
    owner own,\n\
    'plsqlattribute_pixbuf' pic,\n\
    attr_name n,\n\
    case \n\
    when precision is not null then decode(scale, NULL, attr_type_name || '(' || precision || ')', \n\
    attr_type_name || '(' || precision || ',' || scale || ')') \n\
    when scale is not null then attr_type_name || '(' || scale || ')' \n\
    when length is not null then attr_type_name || '(' || length || ')' \n\
    else attr_type_name  \n\
    end d, \n\
    decode(attr_type_owner, NULL, NULL, attr_type_owner || '.' || attr_type_name) o, \n\
    attr_no r\n\
    from all_type_attrs \n\
    where owner=:p1 and type_name=:p2 \n\
    union all\n\
    select %d,\n\
    0,\n\
    owner,\n\
    'typemethod_pixbuf',\n\
    method_name, \n\
    NULL, \n\
    NULL, \n\
    method_no+100\n\
    from all_type_methods where owner=:p1 and type_name=:p2\n\
    ) order by o";


gchar *type_method_sql =
"select %d, 0, owner, 'plsqlattribute_pixbuf' \"pxb\",\n\
  argument_name, \n\
  decode(data_type,'OBJECT',type_owner||'.'||type_name,data_type) data_type, \n\
  in_out \n\
 from all_arguments \n\
 where owner=:p1 and package_name=:p2 and object_name=:p3 \n\
 order by sequence";

gchar *triggers_sql =
"select %d, 0, owner, NULL \"pxb\", trigger_name \"Trigger\", \n\
 status \"Status\", decode(status,'ENABLED','green4','DISABLED','red4', :fg_normal) \"foreground\",\n\
 trigger_type \"Type\",\n\
 triggering_event \"Event\", \n\
 table_owner || '.' || table_name \"Table\",\n\
 column_name \"Column\",\n\
 referencing_names \"Referencing Names\", \n\
 substr(when_clause,1,1024) \
   || case when length(when_clause) >= 1024 \
   then '....' else NULL end \"When Clause\" \n\
 from all_triggers \
 where owner=:p1 order by trigger_name";

gchar *trigger_sql =
"select owner, trigger_name, trigger_type, triggering_event\n\
 table_owner, table_name, column_name, base_object_type, referencing_names,\n\
 when_clause, status, description, action_type\n\
 from all_triggers where owner=:p1 and trigger_name=:p2";

gchar *java_classes_sql =
"select * from all_java_classes where owner=:p1 order by name";

gchar *java_methods_sql =
"select * from all_java_methods where owner=:p1 order by name";

gchar *schedules_sql =
"select \n\
 owner, schedule_name, 'schedule_pixbuf' \"pxb\",\n\
 schedule_name \"Schedule\",\n\
 schedule_type \"Type\",\n\
 to_char(start_date, 'YYYY-MM-DD HH24:MI') \"Start Date\", \n\
 to_char(end_date, 'YYYY-MM-DD HH24:MI') \"End Date\", \n\
 repeat_interval \"Repeat Interval\",\n\
 event_queue_owner || '.' || event_queue_name \"Event Queue\",\n\
 event_queue_agent \"Event Queue Agent\",\n\
 event_condition \"Event Condition\",\n\
 comments \"Comments\"\n\
 from all_scheduler_schedules\n\
 where owner=:p1 \n\
 order by 4";

gchar *schedule_sql =
"select * from all_scheduler_schedules \
 where owner=:p1 and schedule_name=:p2";

gchar *windows_sql =
"select %d, 0, 'schedule_pixbuf' \"pxb\", \n\
 window_name \"Window\",\n\
 enabled \"Enabled\", decode(enabled, 'TRUE', 'green4', :fg_normal) \"foreground\", \n\
 active \"Active\", decode(active, 'TRUE', 'green4', :fg_normal) \"foreground\", \n\
 resource_plan \"Resource Plan\",\n\
 decode(schedule_owner, NULL, schedule_name, schedule_owner || '.' || schedule_name) \"Schedule\",\n\
 schedule_type \"Schedule Type\",\n\
 to_char(start_date, 'YYYY-MM-DD HH24:MI') \"Start Date\", \n\
 to_char(end_date, 'YYYY-MM-DD HH24:MI') \"End Date\", \n\
 repeat_interval \"Repeat Interval\",\n\
 duration \"Duration\",\n\
 window_priority \"Priority\", decode(window_priority, 'HIGH', 'red4', :fg_normal) \"foreground\",\n\
 to_char(next_start_date, 'YYYY-MM-DD HH24:MI') \"Next Start Date\", \n\
 to_char(last_start_date, 'YYYY-MM-DD HH24:MI') \"Last Start Date\", \n\
 manual_open_time \"Manual Open Time\",\n\
 manual_duration \"Manual Duration\",\n\
 comments \"Comments\"\n\
 from all_scheduler_windows\n\
 order by window_name";

gchar *window_sql =
"select * from all_scheduler_windows where window_name=:p1";

gchar *jobs_sql =
"select %d, 0, owner, 'schedule_pixbuf' \"pxb\",\n\
 job_name \"Job\",\n\
 enabled \"Enabled\", decode(enabled,'TRUE','green4', :fg_normal) \"foreground\",\n\
 state \"State\",\n\
 job_creator \"Creator\",\n\
 decode(schedule_owner, NULL, schedule_name, schedule_owner || '.' || schedule_name) \"Schedule\",\n\
 job_type \"Type\",\n\
 decode(program_owner, NULL, program_name, program_owner || '.' || program_name) \"Program\",\n\
 substr(job_action,1,32) || case when length(job_action)>32 then '...' else NULL end \"Action\",\n\
 schedule_type \"Sched Type\",\n\
 number_of_arguments \"# Arguments\",\n\
 to_char(start_date, 'YYYY-MM-DD HH24:MI') \"Start Date\", \n\
 to_char(end_date, 'YYYY-MM-DD HH24:MI') \"End Date\", \n\
 repeat_interval \"Repeat Interval\",\n\
 to_char(last_start_date, 'YYYY-MM-DD HH24:MI') \"Last Start\", \n\
 last_run_duration \"Last Duration\",\n\
 to_char(next_run_date, 'YYYY-MM-DD HH24:MI') \"Next Date\", \n\
 schedule_limit \"Limit\",\n\
 max_run_duration \"Max Duration\",\n\
 logging_level \"Log Level\",\n\
 raise_events \"Raise Events\",\n\
 instance_stickiness \"Stickiness\",\n\
 event_queue_owner || '.' || event_queue_name \"Event Queue\",\n\
 event_queue_agent \"Event Queue Agent\",\n\
 event_condition \"Event Condition\",\n\
 event_rule \"Event Rule\",\n\
 comments \"Comments\"\n\
 from all_scheduler_jobs where owner=:p1 order by job_name";

gchar *job_sql =
"select * from (\n\
 select \n\
 l.job_name \"Job\" \n\
 ,l.operation \"Operation\" \n\
 ,l.status \"Status\", decode(l.status,'SUCCEEDED','green4','FAILED','red4', :fg_normal) \"foreground\"\n\
 ,d.req_start_date \"Req Start\" \n\
 ,d.actual_start_date \"Actual Start\" \n\
 ,d.run_duration \"Duration\" \n\
 ,d.instance_id \"Instance\" \n\
 ,d.session_id \"SID\" \n\
 ,d.cpu_used \"CPU Used\" \n\
 ,d.error# \"Error\" \n\
 ,d.additional_info \"Additional Info\" \n\
 from all_scheduler_job_log l left outer join all_scheduler_job_run_details d on l.log_id=d.log_id \n\
 where l.owner=:p1 and l.job_name=:p2 \n\
 order by l.log_date desc)\n\
 where rownum <=100";

gchar *programs_sql =
"select %d, 0, owner, 'schedule_pixbuf' \"pxb\",\n\
    program_name \"Program\",\n\
    enabled \"Enabled\",\n\
    detached \"Detached\",\n\
    program_type \"Type\",\n\
    number_of_arguments \"# Arguments\",\n\
    substr(program_action,1,32) || case when length(program_action) > 32 then '...' else NULL end \"Action\",\n\
    comments \"Comments\"\n\
    from all_scheduler_programs where owner=:p1 order by program_name";

gchar *program_sql =
"select * from all_scheduler_programs where owner=:p1 and program_name=:p2";

gchar *gvinstance_sql =
"select * from gv$instance where inst_id=:p1";

gchar *instance_sql =
"select * from v$instance order by instance_name";

gchar *services_sql =
"select %d, s.service_id, NULL, NULL \"pxb\",\n\
               s.name \"Service\",\n\
               s.service_id \"ID\",\n\
               s.network_name \"Network Name\",\n\
               i.instance_name \"Instance\",\n\
               s.name_hash \"Hash\",\n\
               s.creation_date \"Created\"\n\
              from gv$services s join gv$instance i on s.inst_id=i.inst_id \
              where 1=1 %s\n\
              order by name";

gchar *service_sql =
"select\
       service_name\n\
      ,max(decode(stat_name, 'DB CPU', value, 0)) \"DB CPU\"\n\
      ,max(decode(stat_name, 'DB time', value, 0)) \"DB time\"\n\
      ,max(decode(stat_name, 'execute count', value, 0)) \"Execute Count\"\n\
      ,max(decode(stat_name, 'parse count (total)', value, 0)) \"Parse Count\"\n\
      ,max(decode(stat_name, 'session cursor cache hits', value, 0)) \"Cursor Cache Hits\"\n\
      ,max(decode(stat_name, 'physical reads', value, 0)) \"Physical Reads\"\n\
      ,max(decode(stat_name, 'session logical reads', value, 0)) \"Session Logical Reads\"\n\
      ,max(decode(stat_name, 'application wait time', value, 0)) \"App Wait Time\"\n\
      ,max(decode(stat_name, 'concurrency wait time', value, 0)) \"Concurrency Wait Time\"\n\
      ,max(decode(stat_name, 'cluster wait time', value, 0)) \"Cluster Wait Time\"\n\
      ,max(decode(stat_name, 'user I/O wait time', value, 0)) \"I/O Wait Time\"\n\
      ,max(decode(stat_name, 'db block changes', value, 0)) \"Block Changes\"\n\
      ,max(decode(stat_name, 'physical writes', value, 0)) \"Physical Writes\"\n\
      ,max(decode(stat_name, 'redo size', value, 0)) \"Redo Size\"\n\
      ,max(decode(stat_name, 'user calls', value, 0)) \"User Calls\"\n\
      ,max(decode(stat_name, 'user commits', value, 0)) \"User Commits\"\n\
      ,max(decode(stat_name, 'user rollbacks', value, 0)) \"User Rollbacks\"\n\
      ,max(decode(stat_name, 'logons cumulative', value, 0)) \"Logons\"\n\
      ,max(decode(stat_name, 'gc cr block receive time', value, 0)) \"GC CR Block Receive Time\"\n\
      ,max(decode(stat_name, 'gc cr blocks received', value, 0)) \"GC CR Blocks Received\"\n\
      ,max(decode(stat_name, 'gc current block receive time', value, 0)) \"GC CR Block Receive Time\"\n\
      ,max(decode(stat_name, 'gc current blocks received', value, 0)) \"GC CR Blocks Received\"\n\
      ,max(decode(stat_name, 'workarea executions - multipass', value, 0)) \"Workarea Execs Multi\"\n\
      ,max(decode(stat_name, 'workarea executions - onepass', value, 0)) \"Workarea Execs Onepass\"\n\
      ,max(decode(stat_name, 'workarea executions - optimal', value, 0)) \"Workarea Execs Optimal\"\n\
      from v$service_stats\n\
      where service_name = :p1 \n\
      group by service_name";


gchar *sessions_sql =
"select\n\
  %d, \n\
  s.sid, \n\
  NULL, \n\
  decode(s.status,'ACTIVE','sessionactive_pixbuf','session_pixbuf') \"pxb\",\n\
  i.instance_name \"Instance\", \n\
  s.sid || ',' || s.serial# \"SID,Serial#\",\n\
  s.process \"Process\",\n\
  s.username \"User\",\n\
  s.machine \"Machine\",\n\
  s.status \"Status\",\n\
  decode(s.status,'ACTIVE','green4', :fg_normal) \"foreground\",\n\
  substr(s.module,1,20) \"Module\",\n\
  substr(s.action,1,16) \"Action\", \n\
  CASE WHEN s.type='USER' \
    THEN to_char(floor(s.last_call_et/60))||':'|| ltrim(to_char(mod(s.last_call_et,60),'00')) ELSE '--:--' END \"Min/sec\",\n\
  w.event \"Event\", \n\
  substr(q.sql_text,1,128) \"SQL\",\n\
  q.hash_value \"SQL Hash\",\n\
  q.plan_hash_value \"Plan Hash\",\n\
  s.row_wait_obj# \"Row Wait Obj#\",\n\
  s.row_wait_file# \"Row Wait File#\"\n\
  from gv$session s left outer join gv$session_wait w on s.sid=w.sid and w.inst_id=s.inst_id \n\
   left outer join gv$sql q on s.sql_address=q.address and s.sql_hash_value=q.hash_value \n\
    and q.inst_id=s.inst_id and q.child_number=0 and q.object_status='VALID'\n\
   join gv$instance i on i.inst_id=s.inst_id \n\
 where s.username is not null \n\
   and s.status like :p1 \n\
   %s \n\
 order by s.status, s.module";

gchar *session_sql =
"select \n\
     initcap(sn.name) \"Statistic\", ss.value \"Value\",\n\
     decode(sn.class,1,'User',2,'Redo',4,'Enqueue',8,'Cache',16,'OS',32,'RAC',64,'SQL',128,'Debug','N/A') \"Event Class\"\n\
    from gv$sesstat ss join gv$statname sn on ss.statistic#=sn.statistic#\
    where ss.sid=:p1\n\
      and sn.name IN (\n\
  'CPU used by this session',\n\
  'SQL*Net roundtrips to/from client',\n\
  'SQL*Net roundtrips to/from dblink',\n\
  'application wait time',\n\
  'bytes received via SQL*Net from client',\n\
  'bytes received via SQL*Net from dblink',\n\
  'bytes sent via SQL*Net to client', \n\
  'bytes sent via SQL*Net to dblink',\n\
  'consistent changes', \n\
  'consistent gets',\n\
  'db block changes',\n\
  'db block gets direct',\n\
  'db block gets from cache',\n\
  'db block gets',\n\
  'messages received',\n\
  'messages sent',\n\
  'parse count (failures)',\n\
  'parse count (hard)',\n\
  'parse count (total)',\n\
  'parse time cpu',\n\
  'parse time elapsed',\n\
  'physical reads',\n\
  'recursive calls',\n\
  'redo blocks written',\n\
  'redo entries',\n\
  'redo size',\n\
  'redo wastage',\n\
  'redo writes',\n\
  'session logical reads',\n\
  'session pga memory max',\n\
  'session pga memory',\n\
  'sorts (disk)',\n\
  'sorts (memory)',\n\
  'sorts (rows)',\n\
  'table fetch by rowid',\n\
  'table scan rows gotten',\n\
  'user I/O wait time',\n\
  'user calls',\n\
  'user commits',\n\
  'user rollbacks'\n\
  ) \n\
  order by sn.class, sn.name";

gchar *longops_sql =
"select\n\
  i.instance_name \"Instance\"\n\
  ,s.sid || ',' || s.serial# \"SID\"\n\
  ,s.process \"PID\"\n\
  ,s.username \"User\"\n\
  ,s.machine \"Machine\"\n\
  ,s.module \"Module\"\n\
  ,s.action \"Action\"\n\
  ,to_char(start_time, 'MM/DD HH24:MI') \"Start\"\n\
  ,sofar || '/' || totalwork \"Work\"\n\
  ,round(sofar/decode(totalwork,0,1,totalwork),4)*100 \"GtkProgress%%Done         \"\n\
  ,round((totalwork-sofar) / decode((sofar / ((sysdate - start_time) * 86400)),0,1),0) \"Secs to go\"\n\
  ,substr(q.sql_text,1,128) \"SQL\" \n\
  from gv$session_longops l join gv$session s \
       on s.sid=l.sid and s.serial#=l.serial# \
       and s.inst_id=l.inst_id\
  left outer join gv$sql q \
       on s.sql_address=q.address\
       and s.sql_hash_value=q.hash_value \
       and s.inst_id=q.inst_id \n\
  join gv$instance i on s.inst_id=i.inst_id \n\
  where s.status='ACTIVE' \n\
  %s \n\
  and round(sofar/decode(totalwork,0,1,totalwork),4)*100 != 100 \n\
  order by start_time desc";

gchar *cache_advice_sql =
"select\n\
  * \n\
 from gv$db_cache_advice\n\
 where 1=1 \n\
 %s \n\
 order by inst_id, size_for_estimate";

gchar *pxbuffer_advice_sql =
"select\n\
 * \n\
 from gv$px_buffer_advice\n\
 where 1=1 \n\
 %s \n\
 order by inst_id, statistic";

gchar *pool_advice_v9_sql =
"select \n\
 i.instance_name, \n\
 a.SHARED_POOL_SIZE_FOR_ESTIMATE \"Pool MB for Est\", \n\
 a.SHARED_POOL_SIZE_FACTOR \"Factor\",\n\
 a.estd_lc_size \"# LC Objects\", \n\
 a.estd_lc_time_saved \"LC Time Saved\", \n\
 a.estd_lc_time_saved_factor \"LC Time Saved Factor\", \n\
 a.ESTD_LC_MEMORY_OBJECT_HITS \"LC Obj Hits\" \n\
 from gv$shared_pool_advice a join gv$instance i on a.inst_id=i.inst_id \n\
 where 1=1 \n\
 %s \n\
 order by 1,2";

gchar *pool_advice_v10_sql =
"select \n\
 i.instance_name, \n\
 a.SHARED_POOL_SIZE_FOR_ESTIMATE \"Pool MB for Est\", \n\
 a.SHARED_POOL_SIZE_FACTOR \"Factor\",\n\
 a.estd_lc_size \"# LC Objects\", \n\
 a.estd_lc_time_saved \"LC Time Saved\", \n\
 a.estd_lc_time_saved_factor \"LC Time Saved Factor\", \n\
 a.estd_lc_load_time \"LC Load Time\", \n\
 a.estd_lc_load_time_factor \"LC Load Time Factor\", \n\
 a.ESTD_LC_MEMORY_OBJECT_HITS \"LC Obj Hits\" \n\
 from gv$shared_pool_advice a join gv$instance i on a.inst_id=i.inst_id \n\
 where 1=1 \n\
 %s \n\
 order by 1,2";

gchar *sga_advice_sql =
"select\n\
 i.instance_name, \n\
 a.sga_size \"SGA Size\",\n\
 a.sga_size_factor \"Factor\",\n\
 a.estd_db_time \"Estd DB Time\",\n\
 a.estd_db_time_factor \"Estd DB Time Factor\",\n\
 a.estd_physical_reads \"Estd Phys Reads\"\n\
 from gv$sga_target_advice a join gv$instance i on a.inst_id=i.inst_id \n\
 where 1=1 \n\
  %s \n\
 order by 1,2";

gchar *pga_advice_sql =
"select\n\
 i.instance_name, \n\
 a.pga_target_for_estimate \"PGA for Estimate\",\n\
 a.pga_target_factor \"Factor\",\n\
 a.advice_status \"Advice Status\",\n\
 a.bytes_processed \"Bytes Processed\",\n\
 a.estd_extra_bytes_rw \"Estd Extra Bytes RW\",\n\
 a.ESTD_PGA_CACHE_HIT_PERCENTAGE \"Estd Cache Hit %%\",\n\
 a.estd_overalloc_count \"Estd Over Allocs\"\n\
 from gv$pga_target_advice a join gv$instance i on a.inst_id=i.inst_id \n\
 where 1=1 \n\
  %s \n\
 order by 1,2";

gchar *mttr_advice_sql =
"select\n\
 i.instance_name, \n\
 a.mttr_target_for_estimate \"Target for Est\",\n\
 a.advice_status \"Advice Status\",\n\
 a.dirty_limit \"Dirty Limit\",\n\
 a.estd_cache_writes \"Estd Cache Writes\",\n\
 a.estd_cache_write_factor \"Estd Cache W Factor\",\n\
 a.estd_total_writes \"Estd Total Writes\",\n\
 a.estd_total_write_factor \"Estd Total W Factor\",\n\
 a.estd_total_ios \"Estd Total IO\",\n\
 a.estd_total_io_factor \"Estd Total IO Factor\"\n\
 from gv$mttr_target_advice a join gv$instance i on a.inst_id=i.inst_id \n\
 where 1=1 \n\
  %s \n\
 order by 1,2";

gchar *java_advice_sql =
"select\n\
  * \n\
 from gv$java_pool_advice a \n\
 where 1=1 \n\
  %s \n\
 order by 1,2";

gchar *streams_advice_sql =
"select\n\
 * \n\
 from gv$streams_pool_advice a \n\
 where 1=1 \n\
  %s \n\
 order by 1,2";

gchar *users_sql =
"select %d, 0, username, NULL \"pxb\",\n\
 user_id \"ID\",\n\
 username \"Username\",\n\
 account_status \"Status\",\n\
 default_tablespace \"Default Tbs\",\n\
 temporary_tablespace \"Temp Tbs\",\n\
 created \"Created\",\n\
 lock_date \"Lock Date\",\n\
 expiry_date \"Expired\",\n\
 profile \"Profile\",\n\
 INITIAL_RSRC_CONSUMER_GROUP \"Consumer Group\",\n\
 external_name \"External Name\"\n\
 from dba_users \
 order by username";

gchar *user_sql =
"select * from dba_users where username=:p1";

gchar *roles_sql =
"select %d, 0, NULL, NULL \"pxb\",\n\
 role \"Role\", password_required \"Password Req\"\n\
 from dba_roles \
 order by role";

gchar *role_sql =
"select \n\
 grantee \"Grantee\", admin_option \"Admin Option\", default_role \"Default?\"\n\
 from dba_role_privs \
 where granted_role=:p1 \
 order by grantee";

gchar *profile_sql =
"select\n\
 resource_name \"Resource\",\n\
 resource_type \"Type\",\n\
 limit \"Limit\"\n\
 from dba_profiles where profile=:p1\n\
 order by resource_name";

gchar *tablespaces_sql =
"select \n\
 tablespace_name \"Tablespace\", \n\
 round( sum(byte) / 1048576, 2) \"Size MB\", \n\
 round( (sum(byte) - sum(fbyte)) /1048576, 2) \"Used MB\",\n\
 round( sum(fbyte) / 1048576, 2) \"Free MB\", \n\
 round( (sum(byte) - sum(fbyte)) / decode(sum(byte),0,1,sum(byte) ) * 100, 0) \"GtkProgressPercent Full\", \n\
 sum(obj_cnt) \"# Objects\", \n\
 max(cts) \"Contents\",\n\
 max(st) \"Status\",\n\
 max(lg) \"Logging\",\n\
 max(fl) \"Force Log\",\n\
 max(em) \"ExtManage\",\n\
 max(at) \"Allocation\",\n\
 max(ssm) \"Space Mgmt\"\n\
 from\n\
 (select tablespace_name, 0 obj_cnt, 0 ini_ext, 0 byte, sum(bytes) fbyte, count(*) frags, max(bytes) largest,\n\
 NULL cts, NULL st, NULL lg, NULL fl, NULL em, NULL at, NULL ssm\n\
 from sys.dba_free_space group by tablespace_name\n\
 union\n\
 select tablespace_name, 0, 0, sum(bytes), 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL\n\
 from sys.dba_data_files group by tablespace_name\n\
 union \n\
 select tablespace_name, 0, initial_extent/1024 ini_ext, 0, 0, 0, 0, \n\
 contents, status, logging, force_logging, extent_management, allocation_type, segment_space_management from sys.dba_tablespaces \n\
 union \n\
 select tablespace_name, count(*) obj_cnt, 0, 0, 0, 0, 0,\n\
 NULL, NULL, NULL, NULL, NULL, NULL, NULL\n\
 from sys.dba_segments group by tablespace_name)\n\
 group by tablespace_name";

gchar *tablespace_sql =
"select \n\
 decode(segment_type, \n\
  'TABLE', 'heap_pixbuf', \n\
  'TABLE PARTITION', 'heap_pixbuf', \n\
  'INDEX', 'iot_pixbuf', \n\
  'INDEX PARTITION', 'iot_pixbuf', \n\
  'LOBINDEX', 'lob_pixbuf',\n\
  'LOB PARTITION', 'lob_pixbuf',\n\
  'LOBSEGMENT', 'lob_pixbuf',\n\
  'none') \"pxb\",\n\
 owner \"Owner\",\n\
 segment_name \"Segment\", \n\
 partition_name \"Partition\", \n\
 round(bytes/1048576,2) \"MB\", \n\
 blocks \"Blocks\", \n\
 extents \"# Extents\",\
 initial_extent/1024 \"Initial KB\",\n\
 next_extent/1024 \"Next KB\",\n\
 pct_increase \"PctIncrease\",\n\
 freelists \"Freelists\",\n\
 freelist_groups \"FL Groups\",\n\
 buffer_pool \"Buffer Pool\",\n\
 header_file \"Header File\",\n\
 relative_fno \"Rel Fno\",\n\
 header_block \"Header Block\",\n\
 segment_type \"Type\" \n\
 from dba_segments \n\
 where tablespace_name=:p1 \n\
 order by segment_name, partition_name";

gchar *redologs_sql =
"select \n\
 l.group# groupno,\n\
 l.thread# threadno,\n\
 l.bytes/1048576 \"MB\",\n\
 l.archived,\n\
 l.status \"Log Status\",\n\
 decode(l.status, 'CURRENT', 'green4', :fg_normal) \"foreground\",\n\
 to_char(l.first_time,'MM/DD HH24:MI:SS') \"First Time\",\n\
 l.first_change# \"First Change\",\n\
 f.member \"Logfile\" \n\
 from v$log l join v$logfile f on l.group# = f.group# \n\
 order by 1,2";

gchar *datafiles_sql =
"select\n\
 df.tablespace_name \"Tablespace\",\n\
 df.file_name \"Filename\",\n\
 df.file_id \"File ID\",\n\
 round(df.bytes/1048576,2) \"MB\",\n\
 round( sum(nvl(fs.bytes,df.bytes)) / 1048576, 2) \"Free\",\n\
 round( (df.bytes - sum(nvl(fs.bytes,df.bytes))) / df.bytes*100, 0) \"GtkProgressPercent Full\",\n\
 max(df.autoextensible) \"Autoextensible\",\n\
 max(df.status) \"Status\"\n\
 from dba_data_files df left outer join dba_free_space fs on df.file_id = fs.file_id\n\
 group by df.tablespace_name, df.file_name, df.file_id, df.bytes";

gchar *datafile_metric_sql =
"select d.file_name, d.file_id, m.begin_time \"Sampled\", \n\
 m.average_read_time, m.average_write_time,\n\
 m.physical_reads, m.physical_writes, m.physical_block_reads, m.physical_block_writes\n\
from dba_data_files d left outer join %s m on d.file_id=m.file_id order by %s";

gchar *tempfiles_sql =
"select\n\
 df.tablespace_name \"Tablespace\",\n\
 df.file_name \"Filename\",\n\
 df.file_id \"File ID\",\n\
 round( df.bytes/1048576, 2) \"MB\",\n\
 round( fs.bytes_free / 1048576, 2) \"Free\",\n\
 round( fs.bytes_used / df.bytes*100, 0) \"GtkProgressPercent Full\",\n\
 df.autoextensible \"Autoextensible\",\n\
 df.status \"Status\"\n\
 from dba_temp_files df left outer join v$temp_space_header fs on df.file_id = fs.file_id";


gchar *archived_logs_sql =
"select * from \
  (select * from v$archived_log \
   order by completion_time desc)\
 where rownum <= 100";

gchar *asm_diskgroups_sql =
"select * from v$asm_diskgroup order by name";

gchar *asm_diskgroup_sql =
"select * from v$asm_disk where group_number=:p1 order by name";

gchar *rman_sql =
"select * from v$rman_status order by start_time desc";

gchar *rman_backups_sql =
"select * from v$rman_backup_job_details order by start_time desc";

gchar *rman_algos_sql =
"select * from V$RMAN_ENCRYPTION_ALGORITHMS order by algorithm_name";


#endif
