/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <oci.h>

#include "callbacks.h"
#include "interface.h"
#include "browser.h"
#include "browser_activated.h"
#include "browser_selected.h"
#include "support.h"
#include "gutil.h"
#include "orautil.h"
#include "dbms_metadata.h"
#include "dbms_space.h"
#include "describe_object.h"
#include "statement.h"
#include "explain.h"
#include "query.h"
#include "query_async.h"
#include "dbms_stats.h"
#include "table.h"
#include "index.h"
#include "syntax.h"
#include "cursor_cache.h"
#include "trace.h"
#include "result_clip.h"
#include "keepalive.h"
#include "command_tab.h"
#include "bind_variable.h"
#include "file_ops.h"
#include "dbms_output.h"
#include "inotify_edit.h"
#include "appconfig.h"
#include "finder.h"
#include "dbms_connection.h"

#include "icons/mini_close_over.h"

#include "icons/gtko_icon_gray_pixbuf.h"
#include "icons/fonts.h"
#include "icons/rollback.h"
#include "icons/commit_pixbuf.h"
#include "icons/connect_pixbuf.h"
#include "icons/disconnect_pixbuf.h"
#include "icons/newtab_pixbuf.h"


extern GtkWidget *window1;
extern const guint gtk_major_version;
extern const guint gtk_minor_version;
extern const guint gtk_micro_version;
extern const guint glib_major_version;
extern const guint glib_minor_version;
extern const guint glib_micro_version;

extern int     errno;

OCIEnv         *envhp  = (OCIEnv *)0;
OCIError       *errhp  = (OCIError *)0;
OCISvcCtx      *svchp  = (OCISvcCtx *)0;
OCISvcCtx      *bsvchp = (OCISvcCtx *)0;
OCIServer      *svrhp  = (OCIServer *)0;
OCIServer      *bsvrhp = (OCIServer *)0;
OCISession     *seshp  = (OCISession *)0;
OCISession     *bseshp = (OCISession *)0;

int oramajor=10, oraminor=0;
int traceLevel=12, session_id=0;

static guint dba_role=0;
static GtkWidget *connDialog=0;
static GtkWidget *tab_popup=0, *clip_popup=0, *jobPopup=0, *recompilePopup=0;
static GtkWidget *connect_image;
static GdkPixbuf *connect_buf, *disconnect_buf;
static gboolean recompile_debug=FALSE, recompile_reuse=FALSE;

static GtkEntryCompletion *find_entry_completion=0;
static GtkWidget *find_entry=0;
GtkTreeModel *completion_model=0;

gint epoch_sec=0;
static GTimer *server_timer;

GtkWidget *schemaCombo=0;
static guint indcost_comp;





#define GLADE_HOOKUP_OBJECT(component,widget,name) \
	g_object_set_data_full (G_OBJECT (component), name, \
				gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)



void on_executeButton_clicked(GtkButton *button,
			      gpointer user_data)
{
	query_async_execute();
}


void on_bindButton_clicked(GtkButton *button,
			   gpointer user_data)
{
	create_bind_entries();
}


void on_breakButton_clicked(GtkButton *button,
			    gpointer user_data)
{
	query_async_break();
}


void on_commitButton_clicked(GtkButton *button,
			     gpointer user_data)
{
	sword ret = OCI_SUCCESS;

	ret = ociErr(errhp, OCITransCommit(bsvchp, errhp, OCI_DEFAULT));
	if (!ret) {
		gtk_widget_set_sensitive (lw("commitButton"), FALSE);
		gtk_widget_set_sensitive (lw("rollbackButton"), FALSE);
		gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), "<b>Committed</b>");
	}
}



void on_rollbackButton_clicked(GtkButton *button,
			       gpointer user_data)
{
	sword ret = OCI_SUCCESS;
	
	ret = ociErr(errhp, OCITransRollback(bsvchp, errhp, OCI_DEFAULT));
	if (!ret) {
		gtk_widget_set_sensitive (lw("commitButton"), FALSE);
		gtk_widget_set_sensitive (lw("rollbackButton"), FALSE);
		gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), "<b>Rolled back</b>");
	}
}






void on_conncancelButton_clicked(GtkButton *button,
				 gpointer user_data)
{
	gtk_widget_destroy(connDialog);
}




void on_schemaCombo_changed(GtkComboBox *combobox,
			    gpointer user_data)
{
	OCIStmt *sth = (OCIStmt *)0;
	gchar sql[128];
	gchar *current_schema=0;
	gchar *upper_schema=0;
	qCommand *qs=0;


	current_schema = gtk_combo_box_get_active_text(combobox);

	if (!current_schema) {
#if DEBUG
		fprintf(stderr, "%s:%d could not get current_schema from combo\n",
			__FILE__, __LINE__);
#endif
		return;
	}

	upper_schema = g_ascii_strup(current_schema, -1);
	g_snprintf(sql, sizeof(sql),
		   "alter session set current_schema = %s",
		   upper_schema);
	g_free(current_schema);

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&sth, 
		       (ub4)OCI_HTYPE_STMT, 0, (dvoid **)0);
	ociPrepare(sth, errhp, sql);
	ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 
				     1, 0, NULL, NULL, 
				     OCI_DEFAULT));


	qs = alloc_qCommand("resList");
	qs->offset = -1;
	qs->sql = g_strdup(sql);
	qs->page = command_tab_get_page();
	
	ociPrepare(qs->sth, errhp, qs->sql);
	query_async_push(qs);

	OCIHandleFree(sth, OCI_HTYPE_STMT);

	clearList(GTK_TREE_VIEW(lw("detailList")));

	object_browser_init(GTK_TREE_VIEW(lw("objectList")),
			    dba_role);

	g_timeout_add(100, fill_completion_model_timeout, 
		      (gpointer) g_strdup(upper_schema));
	g_free(upper_schema);
}




static void connect_free_handles()
{

	OCIHandleFree(seshp,  OCI_HTYPE_SESSION);
	OCIHandleFree(bseshp, OCI_HTYPE_SESSION);
	OCIHandleFree(svrhp,  OCI_HTYPE_SERVER);
	OCIHandleFree(bsvrhp, OCI_HTYPE_SERVER);
	OCIHandleFree(svchp,  OCI_HTYPE_SVCCTX);
	OCIHandleFree(bsvchp, OCI_HTYPE_SVCCTX);

	svchp  = (OCISvcCtx *)0;
	bsvchp = (OCISvcCtx *)0;
	seshp  = (OCISession *)0;
	bseshp = (OCISession *)0;
	svrhp  = (OCIServer *)0;
	bsvrhp = (OCIServer *)0;
}





static void connect_manager_selected (GtkTreeSelection *sel, 
				      gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkBin *classcombo = GTK_BIN(lookup_widget(connDialog, "cmanSIDCombo"));
	gchar *user=0;
	gchar *sid=0;
	gchar *servername=0;
	gchar *portnum=0;
	gint mode;
  

	if (!gtk_tree_selection_get_selected(sel, &model, &iter)) {
		return;
	}

	gtk_tree_model_get(model, &iter, 0, &user, 
			   1, &sid, 3, &mode, 4, &servername, 
			   5, &portnum, -1);

	gtk_entry_set_text(GTK_ENTRY(lookup_widget(connDialog, "cmanUserEntry")), user);
	gtk_entry_set_text(GTK_ENTRY(classcombo->child), sid);
	gtk_combo_box_set_active(GTK_COMBO_BOX(lookup_widget(connDialog, "cmanModeCombo")),
				 mode);
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(connDialog, "cmanHostEntry")), 
			   servername ? servername : "");
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(connDialog, "cmanPortEntry")), 
			   portnum ? portnum : "");
}


static void connect_build_string() 
{
	GString *connect_str=0;
	GtkTreeView *view = GTK_TREE_VIEW(lookup_widget(connDialog, "cmanLoginView"));
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(view);
	guint ret;


	for (ret=gtk_tree_model_get_iter_first(model, &iter); 
	     ret!=FALSE; 
	     ret=gtk_tree_model_iter_next(model, &iter)) 
	{
		gchar entry[128];
		gchar *username=0;
		gchar *osid=0;
		gchar *hostname=0;
		gchar *portnum=0;
		gint mode;

		gtk_tree_model_get(model, &iter, 0, &username, 
				   1, &osid, 3, &mode, 
				   4, &hostname, 5, &portnum, -1);
		g_snprintf(entry, sizeof(entry), "%s,%s,%d,%s,%s:", 
			   username, osid, mode, 
			   hostname ? hostname : "", 
			   portnum ? portnum : "");

		if (!connect_str) {
			connect_str = g_string_new(entry);
		}
		else {
			connect_str = g_string_append(connect_str, entry);
		}
		g_free(username);
		g_free(osid);
		g_free(hostname);
		g_free(portnum);
	}

	connect_str = g_string_truncate(connect_str, connect_str->len-1);
	config_set("connections", connect_str->str );

#if DEBUG
	fprintf(stderr, "%s:%d connections = '%s'\n",
		__FILE__, __LINE__, connect_str->str);
#endif
	g_string_free(connect_str, TRUE);
}




static gboolean server_time_update(gpointer data)
{
	static gdouble elapsed;
	static time_t  esec=0;
	struct tm *tmstruct=0;


	if (server_timer) {

		elapsed = g_timer_elapsed(server_timer, NULL);
		esec = (time_t) (epoch_sec + (int) elapsed);

		if ((tmstruct = gmtime(&esec)) != NULL) {
			static gchar    stbuf[32];

			g_snprintf(stbuf, sizeof(stbuf), 
				   "%04d-%02d-%02d %02d:%02d:%02d ", 
				   tmstruct->tm_year+1900, tmstruct->tm_mon+1, 
				   tmstruct->tm_mday, tmstruct->tm_hour, 
				   tmstruct->tm_min, tmstruct->tm_sec);

			gtk_label_set_text(GTK_LABEL(lw("serverTimeLabel")), stbuf);
		}
		return TRUE;
	}
	return FALSE;
}


sb4 callback_fn(dvoid *svchp, 
		dvoid *envhp, 
		dvoid *fo_ctx, 
		ub4 fo_type, 
		ub4 fo_event)
{


	switch (fo_event) {

	case OCI_FO_BEGIN:
    
		fprintf(stderr, " Failing Over ... Please stand by \n");
		fprintf(stderr, " Failover type was found to be %s \n",
			((fo_type==OCI_FO_SESSION) ? "SESSION" 
			 :(fo_type==OCI_FO_SELECT) ? "SELECT"
			 : "UNKNOWN!")); 
		break;
     

	case OCI_FO_ABORT:
    
		fprintf(stderr, " Failover aborted. Failover will not take place.\n");
		break;


	case    OCI_FO_END:
      
		fprintf(stderr, " Failover ended ...resuming services\n");
		break;
 

	case OCI_FO_REAUTH:
		fprintf(stderr, " Failed over user. Resuming services\n");
		break;
     

	default:
		fprintf(stderr, "Bad Failover Event: %d.\n",  fo_event);
		break;

	}

	return 0;
}



void on_cmanConnectButton_clicked(GtkButton *button,
				  gpointer user_data)
{
	GList *schema_list=0;
	GList *tmplist=0;
	gchar title_buf[128];
	gchar ouser[32];
	gchar current_schema[32];
	gchar opass[32];
	gchar osid[40];
	gchar conn_sid[512];
	gchar ohost[128];
	gchar oport[8];
	gchar mode[8];
	text vbuf[512];
	gchar dvbuf[512];
	gchar **verparts=0;
	gint cmode;
	gint plustrce_role=0;
	gint v;
	ub4 connMode[3] = {OCI_DEFAULT, OCI_SYSOPER, OCI_SYSDBA};
	ub4 cache_size=40;

	OCIStmt *sth = (OCIStmt *)0;
	OCIDefine *defnp = (OCIDefine *)0;
	GtkBin *classcombo = GTK_BIN(lookup_widget(connDialog, "cmanSIDCombo"));
	OCIFocbkStruct failover; 
	sword ret;
     
	g_snprintf(ouser, sizeof(ouser),
		   gtk_entry_get_text(GTK_ENTRY(lookup_widget(connDialog, "cmanUserEntry"))));
	g_snprintf(current_schema, sizeof(current_schema), "%s", 
		   g_ascii_strup(ouser, -1));
	g_snprintf(opass, sizeof(opass), 
		   gtk_entry_get_text(GTK_ENTRY(lookup_widget(connDialog, "cmanPassEntry"))));
	g_snprintf(osid,  sizeof(osid),  
		   gtk_entry_get_text(GTK_ENTRY(classcombo->child)));
	g_snprintf(oport, sizeof(oport), 
		   gtk_entry_get_text(GTK_ENTRY(lookup_widget(connDialog, "cmanPortEntry"))));
	g_snprintf(ohost, sizeof(ohost), 
		   gtk_entry_get_text(GTK_ENTRY(lookup_widget(connDialog, "cmanHostEntry"))));

	cmode = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget(connDialog, "cmanModeCombo")));
	g_snprintf(mode, sizeof(mode), "%d", cmode);

	config_set("oracle_user", ouser);
	config_set("oracle_sid",  osid);
	config_set("oracle_host", ohost);
	config_set("oracle_port", oport);
	config_set("oracle_mode", mode);

	if(gtk_expander_get_expanded(GTK_EXPANDER(lookup_widget(connDialog, "cmanFormatExpander")))) {

		g_snprintf(conn_sid, sizeof(conn_sid),
			   "(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL=TCP)(HOST=%s)(PORT=%s)))(CONNECT_DATA=(SERVICE_NAME=%s)%s))",
			   ohost, oport, osid,
			   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget(connDialog, "cmanDedicatedCheck"))) ? "(SERVER=dedicated)" : " "
			);
	}
	else {
		snprintf(conn_sid, sizeof(conn_sid), osid);
	}
  

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&errhp, (ub4)OCI_HTYPE_ERROR, 
		       (size_t)0, (dvoid **)0);

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&svrhp, (ub4)OCI_HTYPE_SERVER,
		       (size_t)0, (dvoid **)0);

	if (OCIServerAttach(svrhp, errhp, (text *)conn_sid, (sb4)strlen(conn_sid),
			    (ub4)OCI_DEFAULT) != OCI_SUCCESS ) {
		text errbuf[1000];
		text status_buf[1000];
		sword errcode = (sb4) 0;
 
		OCIErrorGet((dvoid *)errhp, (ub4)1, (text *)NULL, 
			    &errcode, errbuf, (ub4) sizeof(errbuf), 
			    (ub4)OCI_HTYPE_ERROR);

		g_strchomp((gchar *)errbuf);

		g_snprintf((gchar *)status_buf, sizeof(status_buf),
			   "<span color=\"red3\">%s</span>", errbuf);
		gtk_label_set_markup(GTK_LABEL(lookup_widget(connDialog, "cmanStatusLabel")),
				     (gchar *)status_buf);

		OCIHandleFree((dvoid *)svrhp, OCI_HTYPE_SERVER);
		return;
	}


	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&svchp, (ub4)OCI_HTYPE_SVCCTX, 
		       (size_t)0, (dvoid **)0);

	OCIAttrSet((dvoid *)svchp, (ub4)OCI_HTYPE_SVCCTX, (dvoid *)svrhp,
		   (ub4)0, (ub4)OCI_ATTR_SERVER, errhp);

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&seshp, (ub4)OCI_HTYPE_SESSION,
		       (size_t)0, (dvoid **)0);
	OCIAttrSet((dvoid *)seshp, (ub4)OCI_HTYPE_SESSION,
		   (dvoid *)ouser, (ub4)strlen(ouser),
		   (ub4)OCI_ATTR_USERNAME, errhp);
	OCIAttrSet((dvoid *)seshp, (ub4)OCI_HTYPE_SESSION,
		   (dvoid *)opass, (ub4)strlen(opass),
		   (ub4)OCI_ATTR_PASSWORD, errhp);
	OCIAttrSet((dvoid *)svchp, (ub4)OCI_HTYPE_SVCCTX, (dvoid *)seshp, (ub4)0,
		   (ub4)OCI_ATTR_SESSION, errhp);
	ret = OCISessionBegin(svchp, errhp, seshp, (ub4)OCI_CRED_RDBMS, 
			      (ub4)connMode[cmode]);

	if (OCI_SUCCESS != ret)	{
		text errbuf[1000];
		text status_buf[1000];
		sword errcode = (sb4) 0;
 
		OCIErrorGet((dvoid *)errhp, (ub4)1, (text *)NULL, 
			    &errcode, errbuf, (ub4) sizeof(errbuf),
			    (ub4)OCI_HTYPE_ERROR);

		g_strchomp((gchar *)errbuf);
		g_snprintf((gchar *)status_buf, sizeof(status_buf),
			   "<span color=\"red3\">%s</span>", errbuf);
		gtk_label_set_markup(GTK_LABEL(lookup_widget(connDialog, "cmanStatusLabel")),
				     (gchar *)status_buf);

		if (28001 == errcode) {
			create_password_change_dialog(lookup_widget(connDialog, "cmanPassEntry"),
						      lookup_widget(connDialog, "cmanStatusLabel"),
						      ouser, opass);
		}
		else {
			OCIHandleFree(seshp, OCI_HTYPE_SESSION);
			OCIHandleFree(svchp, OCI_HTYPE_SVCCTX);
			OCIServerDetach(svrhp, errhp, OCI_DEFAULT);
			OCIHandleFree(svrhp, OCI_HTYPE_SERVER);
			seshp=0;
			svchp=0;
			svrhp=0;
		}

		return;
	}

	OCIAttrSet((dvoid *)svchp, (ub4)OCI_HTYPE_SVCCTX, (dvoid *)&cache_size,
		   (ub4)0, (ub4)OCI_ATTR_STMTCACHESIZE, errhp);
	cache_size=0;


	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&bsvrhp, (ub4)OCI_HTYPE_SERVER,
		       (size_t)0, (dvoid **)0);
	ociErr(errhp, OCIServerAttach(bsvrhp, errhp, (text *)conn_sid, 
				      (sb4)strlen(conn_sid), (ub4)OCI_DEFAULT));
 
	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&bsvchp, (ub4)OCI_HTYPE_SVCCTX,
		       (size_t)0, (dvoid **)0);

	OCIAttrSet((dvoid *)bsvchp, (ub4)OCI_HTYPE_SVCCTX, (dvoid *)bsvrhp,
		   (ub4)0, (ub4)OCI_ATTR_SERVER, errhp);
	OCIAttrSet((dvoid *)bsvchp, (ub4)OCI_HTYPE_SVCCTX, (dvoid *)&cache_size,
		   (ub4)0, (ub4)OCI_ATTR_STMTCACHESIZE, errhp);

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&bseshp, (ub4)OCI_HTYPE_SESSION,
		       (size_t)0, (dvoid **)0);
	OCIAttrSet((dvoid *)bseshp, (ub4)OCI_HTYPE_SESSION, (dvoid *)ouser,
		   (ub4)strlen(ouser), (ub4)OCI_ATTR_USERNAME, errhp);
	OCIAttrSet((dvoid *)bseshp, (ub4)OCI_HTYPE_SESSION, (dvoid *)opass,
		   (ub4)strlen(opass), (ub4)OCI_ATTR_PASSWORD, errhp);
	ociErr(errhp, OCISessionBegin(bsvchp, errhp, bseshp, (ub4)OCI_CRED_RDBMS,
				      (ub4)connMode[cmode]) );

	OCIAttrSet((dvoid *)bsvchp, (ub4)OCI_HTYPE_SVCCTX, (dvoid *)bseshp, (ub4)0,
		   (ub4)OCI_ATTR_SESSION, errhp);

	OCIAttrSet(seshp, OCI_HTYPE_SESSION,(dvoid *)"GTKO",(ub4)strlen("GTKO"),
		   OCI_ATTR_MODULE, errhp);
	OCIAttrSet(bseshp, OCI_HTYPE_SESSION,(dvoid *)"GTKO bg",(ub4)strlen("GTKO bg"),
		   OCI_ATTR_MODULE, errhp);
  
	/* compmodel_init(ouser, opass, conn_sid); */

	connection_data_set(ouser, opass, conn_sid, cmode);

	snprintf(title_buf, sizeof(title_buf), "GTKO - %s", osid);
	gtk_window_set_title(GTK_WINDOW(window1), title_buf);

	g_snprintf((gchar *)vbuf, sizeof(vbuf), "(unknown)");
	OCIServerVersion(svchp, errhp, (text *)vbuf, (ub4)sizeof(vbuf), (ub1)OCI_HTYPE_SVCCTX);

	verparts = g_strsplit((gchar *)vbuf, " ", -1);
	if(verparts) {
		for(v=0; verparts[v]; v++) {
			if( !g_ascii_strncasecmp(verparts[v], "release", 7) && verparts[v+1] ) {
				g_snprintf((gchar *)vbuf, sizeof(vbuf), "%s", verparts[v+1] );
				break;
			}
		}
		g_strfreev(verparts);
	}

	verparts = g_strsplit((gchar *)vbuf, ".", -1);
	if(verparts) {
		if(verparts[0]) {
			oramajor = (int) g_ascii_strtod(verparts[0], NULL);
			if(verparts[1]) {
				oraminor = (int) g_ascii_strtod(verparts[1], NULL);
			}
		}
	}
	g_strfreev(verparts);

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, (size_t)0, (dvoid **)0);
	ociPrepare(sth, errhp, "select \
                          max(decode(granted_role, 'PLUSTRACE', 1, 0)) plustrace, \
                          max(decode(granted_role, 'DBA', 1, 0)) dba \
                          from user_role_privs");

	OCIDefineByPos((OCIStmt *)sth,
		       (OCIDefine **)&defnp,
		       (OCIError *)errhp, 
		       (ub4) 1,
		       (dvoid *) &plustrce_role, sizeof(plustrce_role), 
		       (ub2) SQLT_INT, 
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);
	OCIDefineByPos((OCIStmt *)sth,
		       (OCIDefine **)&defnp,
		       (OCIError *)errhp, 
		       (ub4) 2,
		       (dvoid *) &dba_role, sizeof(dba_role), 
		       (ub2) SQLT_INT, 
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);
	ociExecute(bsvchp, sth, errhp, 1);

	if(plustrce_role) {
		const gchar *tlist = config_get("trace_list");

		gtk_tooltips_set_tip(gtk_tooltips_new(), 
				     lw("traceCheck"), 
				     "Enable V$SESSTAT session tracing", NULL);

		ociPrepare(sth, errhp, "select max(sid) from v$mystat");
		ociDefineNumber(sth, errhp, 1, &session_id, sizeof(session_id) );
		if(ociErr(errhp, ociExecute(bsvchp, sth, errhp, 1)) == OCI_SUCCESS) {

			gtk_widget_set_sensitive(lw("traceCheck"), TRUE);

			if(tlist)
				session_trace_init(envhp, errhp, session_id, tlist);
		}
	}
	else {

		gtk_widget_set_sensitive(lw("traceCheck"), FALSE);
		gtk_tooltips_set_tip(gtk_tooltips_new(), 
				     lw("traceCheck"), 
				     "PLUSTRACE role not enabled for this user, tracing disabled", 
				     NULL);
	}
	OCIHandleFree((dvoid *)sth, (ub4)OCI_HTYPE_STMT);


	/* get schema list */
	schemaCombo = gtk_combo_box_new_text();
	GLADE_HOOKUP_OBJECT (window1, schemaCombo, "schemaCombo");
	gtk_widget_show(schemaCombo);
	gtk_box_pack_start(GTK_BOX(lw( "schemaBox")), schemaCombo, TRUE, TRUE, 0);
	gtk_combo_box_append_text(GTK_COMBO_BOX(schemaCombo), current_schema);

	schema_list = execute_sql_to_glist("select username from all_users where username!=:p1 order by 1", current_schema, NULL);
	if (schema_list) {
		for(tmplist=schema_list; tmplist; tmplist = g_list_next(tmplist)) {

			gtk_combo_box_append_text(GTK_COMBO_BOX(schemaCombo), 
						  (gchar *) tmplist->data);
			g_free(tmplist->data);
			tmplist->data = 0;
		}
		g_list_free(schema_list);
	}
  
	gtk_combo_box_set_active(GTK_COMBO_BOX(schemaCombo), 0);
	g_signal_connect((gpointer) schemaCombo, "changed", 
			 G_CALLBACK(on_schemaCombo_changed), NULL);

	object_browser_init(GTK_TREE_VIEW(lw("objectList")), dba_role);

	/*   get server time */
	schema_list =  execute_sql_to_glist("select round((sysdate - to_date('1970-01-01','YYYY-MM-DD'))*86400,0) from dual", NULL);
	if (schema_list) {    
		epoch_sec = atoi((gchar *)schema_list->data);
		server_timer = g_timer_new();
		g_timeout_add(1000, server_time_update, (gpointer) NULL);
		g_free(schema_list->data);
		g_list_free(schema_list);
		schema_list=0;
	}


	if(oramajor > 9 ) {
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(tab_popup, "for_all_columns")), TRUE);
		gtk_widget_set_sensitive(lw( "awrTrackButton"), TRUE);
		gtk_widget_hide(lw( "awrReportButton"));
	}
	else {
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(tab_popup, "for_all_indexes2")), TRUE);
		gtk_widget_set_sensitive(lw( "awrTrackButton"), TRUE);
		gtk_widget_set_sensitive(lw( "awrPlanButton"), TRUE);
		gtk_widget_show(lw( "awrReportButton") );
	}


	gtk_combo_box_set_active(GTK_COMBO_BOX(lw("cursorSourceCombo")), 0);

	g_snprintf((gchar *)dvbuf, sizeof(dvbuf), "Connected to Oracle Version %s",
		   vbuf);
	setLabel("statusLabel", (gchar *)dvbuf);

	gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(lw("connect1")))),
			   "Disconnect");
	gtk_image_set_from_pixbuf(GTK_IMAGE(connect_image), disconnect_buf);

	gtk_widget_set_sensitive (lw( "executeButton"), TRUE);
	gtk_widget_set_sensitive (lw( "bindButton"), TRUE);
	gtk_widget_set_sensitive (lw( "versionButton"), TRUE);
	gtk_widget_set_sensitive (lw( "mainNotebook"), TRUE);
	gtk_combo_box_set_active(GTK_COMBO_BOX(lw( "awrOrderbyCombo")), 0);

	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(lw("dbmsoutput"))) )
		dbms_output_enable(envhp, bsvchp, errhp, TRUE);
	else
		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lw("dbmsoutput")),
					       TRUE);

	gtk_widget_destroy(connDialog);

	create_completion_model(g_strdup(current_schema));
}


void on_cmanUserEntry_activate(GtkEntry *entry,
			       gpointer user_data)
{
	on_cmanConnectButton_clicked(NULL, NULL);
}


void on_cmanPassEntry_activate(GtkEntry *entry,
			       gpointer user_data)
{
	on_cmanConnectButton_clicked(NULL, NULL);
}




static void connect_edit_entry(GtkCellRendererText *cell,
			       gchar *path_string,
			       gchar *new_text,
			       gpointer user_data) 
{
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(lookup_widget(connDialog, "cmanLoginView")));
	GtkTreeStore *store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lookup_widget(connDialog, "cmanLoginView"))));
	guint stat = GPOINTER_TO_INT(user_data);


	gtk_tree_model_get_iter_from_string(model, &iter, path_string);
	gtk_tree_store_set(store, &iter, stat, new_text, -1);

	switch (stat) {

	case 0:
		gtk_entry_set_text(GTK_ENTRY(lookup_widget(connDialog, "cmanUserEntry")), new_text);
		break;

	default:
		break;
	}

	connect_build_string();
}


void create_connection_dialog()
{
	GtkTreeView *view;
	GtkCellRenderer *crend;
	GtkTreeViewColumn *column;
	GtkTreeStore *store;
	GtkTreeIter iter;
	GtkBin *osidCombo;
	GList *tnsentries=0;
	gchar **login_entries=0;
	gchar **entry;
	guint x;
	gint cmode;
	const gchar *connections;
	const gchar *font;
	gchar *colnames[] = {"Username", 
			     "SID", 
			     "Mode", 
			     "ModeInt", 
			     "Server", 
			     "Port"};


	connDialog = create_connectionManagerWindow();

	g_signal_connect((gpointer) gtk_bin_get_child(GTK_BIN(lookup_widget(connDialog, "cmanSIDCombo"))),
			 "activate",
			 G_CALLBACK (on_cmanUserEntry_activate),
			 NULL);

	view = GTK_TREE_VIEW(lookup_widget(connDialog, "cmanLoginView"));
	font = config_get("listfont");

	for(x=0; x < sizeof(colnames)/sizeof(colnames[0]); x++) {

		crend  = gtk_cell_renderer_text_new();
      
		if(font)
			g_object_set(G_OBJECT(crend), "font", font, NULL);

		g_object_set(G_OBJECT(crend), "ypad", 0, NULL);

		column = gtk_tree_view_column_new_with_attributes(colnames[x], crend, "text", x, NULL);
		gtk_tree_view_append_column(view, column);
		gtk_tree_view_column_set_sort_column_id(column, x);

		if (x<2) {
			g_object_set(crend, "editable", TRUE, NULL);
			g_signal_connect(crend, "edited", (GCallback) connect_edit_entry, GUINT_TO_POINTER(x) );
		}

	}
	gtk_tree_view_column_set_visible(gtk_tree_view_get_column(view, 3), FALSE);

	store = gtk_tree_store_new(6, 
				   G_TYPE_STRING, 
				   G_TYPE_STRING,
				   G_TYPE_STRING,
				   G_TYPE_INT,
				   G_TYPE_STRING,
				   G_TYPE_STRING);
	gtk_tree_view_set_model(view, GTK_TREE_MODEL(store) );

	connections = config_get("connections");
	if(connections) {

		login_entries = g_strsplit_set(connections, ":", -1);

		for(x=0; login_entries[x]; x++) {
			gchar     *modelabel[3] = {"Normal", "SYSOPER", "SYSDBA"};

			entry = g_strsplit_set(login_entries[x], ",",-1);
			gtk_tree_store_append(store, &iter, NULL);
			gtk_tree_store_set(store, &iter, 0, entry[0], 
					   1, entry[1], 
					   2, modelabel[atoi(entry[2])], 
					   3, atoi(entry[2]),
					   -1);

			if( g_strv_length(entry) > 4) {

				gtk_tree_store_set(store, &iter, 4, entry[3], 
						   5, entry[4], -1);
			}

			g_strfreev(entry);
			entry=0;
		}
	}
	g_strfreev(login_entries);
 
	gtk_widget_grab_focus(lookup_widget(connDialog, "cmanUserEntry"));

	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(view)),
			 "changed", G_CALLBACK(connect_manager_selected), NULL);

	gtk_widget_show(connDialog);
	tnsentries = get_tns_entries(lookup_widget(connDialog, "cmanStatusLabel"));
	if (!tnsentries) {
		gtk_expander_set_expanded(GTK_EXPANDER(lookup_widget(connDialog, "cmanFormatExpander")), TRUE);
	}
	else {
		while (tnsentries) {
			gtk_combo_box_append_text(GTK_COMBO_BOX(lookup_widget(connDialog, "cmanSIDCombo")),
						  g_strdup((gchar *)tnsentries->data) );
			tnsentries = g_list_next(tnsentries);
		}
	}
	g_list_free(tnsentries);

	osidCombo = GTK_BIN(lookup_widget(connDialog, "cmanSIDCombo"));

	cmode = config_get_integer("oracle_mode");
	if(cmode<3)
		gtk_combo_box_set_active(GTK_COMBO_BOX(lookup_widget(connDialog, "cmanModeCombo")), cmode);
  
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(connDialog, "cmanUserEntry")),
			   config_get("oracle_user") );
	gtk_entry_set_text(GTK_ENTRY(osidCombo->child), config_get("oracle_sid") );
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(connDialog, "cmanHostEntry")),
			   config_get("oracle_host") );
	gtk_entry_set_text(GTK_ENTRY(lookup_widget(connDialog, "cmanPortEntry")),
			   config_get("oracle_port") );
	gtk_editable_select_region(GTK_EDITABLE(lookup_widget(connDialog, "cmanUserEntry")), 0, -1);
}

void on_connect1_activate (GtkMenuItem *menuitem,
			   gpointer user_data)
{

	if (!seshp) {
		create_connection_dialog();
	}
	else {

		g_timer_stop(server_timer);
		g_timer_destroy(server_timer);
		server_timer = 0;

		gtk_widget_destroy(schemaCombo);

		OCISessionEnd(svchp, errhp, seshp, OCI_DEFAULT);
		OCISessionEnd(bsvchp, errhp, seshp, OCI_DEFAULT);
		OCIServerDetach(svrhp, errhp, OCI_DEFAULT);
		OCIServerDetach(bsvrhp, errhp, OCI_DEFAULT);
   
		connect_free_handles();
		gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), "<b>Disconnected</b>");
    
		gtk_widget_set_sensitive (lw( "executeButton"), FALSE);
		gtk_widget_set_sensitive (lw( "bindButton"), FALSE);
		gtk_widget_set_sensitive (lw( "mainNotebook"), FALSE);
		gtk_notebook_set_current_page(GTK_NOTEBOOK(lw( "mainNotebook")), 0);

		clearList(GTK_TREE_VIEW(lw("resList")));
		clearList(GTK_TREE_VIEW(lw("planPredicateTree")));
		clearList(GTK_TREE_VIEW(lw("detailList")));
		clearList(GTK_TREE_VIEW(lw("planTree")));

		gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(lw( "connect1")))), "Connect");
		gtk_image_set_from_pixbuf(GTK_IMAGE(connect_image), connect_buf);
	}
}



void on_cmanSaveButton_clicked(GtkButton *button,
			       gpointer user_data)
{

	GtkTreeView *view = GTK_TREE_VIEW(lookup_widget(connDialog, "cmanLoginView"));
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(view);
	GtkTreeStore *store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
	GtkBin *classcombo = GTK_BIN(lookup_widget(connDialog, "cmanSIDCombo"));
	const gchar *username=0, *osid=0;
	guint mode = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget(connDialog, "cmanModeCombo")));
	guint  ret;
	gchar *modelabel[3] = {"Normal",
			       "SYSOPER",
			       "SYSDBA"};

	username = gtk_entry_get_text(GTK_ENTRY(lookup_widget(connDialog, "cmanUserEntry")));
	osid = gtk_entry_get_text(GTK_ENTRY(classcombo->child));

	if( !username || !osid)
		return;

	for(ret=gtk_tree_model_get_iter_first(model, &iter);
	    ret!=FALSE;
	    ret=gtk_tree_model_iter_next(model, &iter) ) {
		gchar *user;
		gchar *sid;
		gint login_mode;

		gtk_tree_model_get(model, &iter, 0, &user, 1, &sid, 3, &login_mode, -1);
 
		if( !g_ascii_strncasecmp(username, user, strlen(username)) ) 
			if( !g_ascii_strncasecmp(osid, sid, strlen(osid)) ) 
				if( mode == login_mode) {
					g_free(user);
					g_free(sid);
					return;
				}
    
		g_free(user);
		g_free(sid);
	}


	gtk_tree_store_append(store, &iter, NULL);
	gtk_tree_store_set(store, &iter, 0, username, 
			   1, osid, 2, modelabel[mode], 
			   3, mode, -1); 

	if(gtk_expander_get_expanded(GTK_EXPANDER(lookup_widget(connDialog, "cmanFormatExpander"))) ) {
		const gchar *hostname=0, *portnum=0;

		hostname = gtk_entry_get_text(GTK_ENTRY(lookup_widget(connDialog, "cmanHostEntry")));
		portnum  = gtk_entry_get_text(GTK_ENTRY(lookup_widget(connDialog, "cmanPortEntry")));
		gtk_tree_store_set(store, &iter, 4, hostname, 5, portnum, -1); 
	}

	connect_build_string();
}


void on_cmanCancelButton_clicked(GtkButton *button,
				 gpointer user_data)
{
	gtk_widget_destroy(connDialog);
	connDialog = NULL;
}


void on_cmanDeleteButton_clicked(GtkButton *button,
				 gpointer user_data)
{
	GtkTreeView *view = GTK_TREE_VIEW(lookup_widget(connDialog, "cmanLoginView"));
	GtkTreeSelection *sel = gtk_tree_view_get_selection(view);
	GtkTreeIter iter;
	GtkTreeModel *model=0;

	if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
		gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
	}
}






void on_quit1_activate(GtkMenuItem *menuitem,
		       gpointer user_data)
{
	ub4 serverStatus = 0;

	config_write();

	OCIAttrGet((dvoid *)svrhp, OCI_HTYPE_SERVER, 
		   (dvoid *)&serverStatus, (ub4 *)0,
		   OCI_ATTR_SERVER_STATUS, errhp);

	if (serverStatus == OCI_SERVER_NORMAL) {
		OCIBreak(bsvchp, errhp);
		OCITransCommit(bsvchp, errhp, OCI_DEFAULT);
		OCILogoff(svchp, errhp);
		OCILogoff(bsvchp, errhp);
		OCITerminate(OCI_DEFAULT);
	}

	gtk_main_quit();
	exit(0);
}


gboolean on_app_delete(GtkWidget *widget,
		       GdkEvent *event,
		       gpointer user_data)
{
	on_quit1_activate((GtkMenuItem *) NULL, (gpointer) NULL);
	return FALSE;
}



void on_versionButton_clicked(GtkButton *button,
			      gpointer user_data)
{
	gchar oraversion[1024];
	gchar gtkversion[512];
	gchar gsversion[128];
	GtkWidget *infoWindow;
	GtkWidget *frame3;
	GtkWidget *alignment31;
	GtkWidget *vbox15;
	GtkWidget *gtkversionLabel;
	GtkWidget *hseparator3;
	GtkWidget *oraversionLabel;
	GtkWidget *hseparator14;
	GtkWidget *hbox80;
	GtkWidget *label225;
	GtkWidget *versionLabel;

	infoWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (infoWindow), "GTKO Version Info");
	gtk_window_set_position (GTK_WINDOW (infoWindow), GTK_WIN_POS_CENTER);
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (infoWindow), TRUE);
	gtk_window_set_skip_pager_hint (GTK_WINDOW (infoWindow), TRUE);

	frame3 = gtk_frame_new (NULL);
	gtk_widget_show (frame3);
	gtk_container_add (GTK_CONTAINER (infoWindow), frame3);
	gtk_frame_set_shadow_type (GTK_FRAME (frame3), GTK_SHADOW_NONE);

	alignment31 = gtk_alignment_new (0.5, 0.5, 1, 1);
	gtk_widget_show (alignment31);
	gtk_container_add (GTK_CONTAINER (frame3), alignment31);
	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment31), 0, 0, 12, 0);

	vbox15 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox15);
	gtk_container_add (GTK_CONTAINER (alignment31), vbox15);

	gtkversionLabel = gtk_label_new ("");
	gtk_widget_show (gtkversionLabel);
	gtk_box_pack_start (GTK_BOX (vbox15), gtkversionLabel, FALSE, FALSE, 0);

	hseparator3 = gtk_hseparator_new ();
	gtk_widget_show (hseparator3);
	gtk_box_pack_start (GTK_BOX (vbox15), hseparator3, FALSE, FALSE, 0);

	oraversionLabel = gtk_label_new ("Oracle Version");
	gtk_widget_show (oraversionLabel);
	gtk_box_pack_start (GTK_BOX (vbox15), oraversionLabel, FALSE, FALSE, 0);
	gtk_misc_set_padding (GTK_MISC (oraversionLabel), 6, 4);

	hseparator14 = gtk_hseparator_new ();
	gtk_widget_show (hseparator14);
	gtk_box_pack_start (GTK_BOX (vbox15), hseparator14, TRUE, TRUE, 0);

	hbox80 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox80);
	gtk_frame_set_label_widget (GTK_FRAME (frame3), hbox80);

	label225 = gtk_label_new ("<b><span size=\"xx-large\">GTK</span><span foreground=\"red\" size=\"xx-large\">O</span></b>");
	gtk_widget_show (label225);
	gtk_box_pack_start (GTK_BOX (hbox80), label225, FALSE, FALSE, 0);
	gtk_label_set_use_markup (GTK_LABEL (label225), TRUE);

	versionLabel = gtk_label_new ("<b><span size=\"xx-large\">1.33</span></b>");
	gtk_widget_show (versionLabel);
	gtk_box_pack_start (GTK_BOX (hbox80), versionLabel, FALSE, FALSE, 0);
	gtk_label_set_use_markup (GTK_LABEL (versionLabel), TRUE);
	gtk_misc_set_padding (GTK_MISC (versionLabel), 7, 0);

	snprintf(gsversion, sizeof(gsversion), "<b><span size=\"xx-large\">%s</span></b>", VERSION);
	gtk_label_set_markup(GTK_LABEL(versionLabel), gsversion);

	snprintf(gtkversion, 512, "Glib %d.%d.%d\n GTK %d.%d.%d\n",
		 glib_major_version, glib_minor_version, glib_micro_version,
		 gtk_major_version, gtk_minor_version, gtk_micro_version);
	   

	if(svchp) 
		OCIServerVersion(svchp, errhp, (void *)oraversion, sizeof(oraversion), OCI_HTYPE_SVCCTX);
	else
		snprintf(oraversion, 1024, "(Oracle version not available)");

	gtk_label_set_text(GTK_LABEL(oraversionLabel), oraversion);
	gtk_label_set_text(GTK_LABEL(gtkversionLabel), gtkversion);
	gtk_widget_show(infoWindow);
}



void on_clearButton_clicked(GtkButton *button,
			    gpointer user_data)
{
	text_buffer_clear();
}


void on_multiBlockReadSpin_changed(GtkEditable *editable,
				   gpointer user_data)
{
	gchar sql[128];

	snprintf(sql, 128, "alter session set db_file_multiblock_read_count=%d", 
		 gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw("multiBlockReadSpin"))));
	execute_sql(sql, "resList", NULL);
}


void on_indexCachingSpin_changed(GtkEditable *editable,
				 gpointer user_data)
{
	gchar sql[128];

	snprintf(sql, 128, "alter session set optimizer_index_caching=%d", 
		 gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw("indexCachingSpin"))));
	execute_sql(sql, "resList", NULL);
}


void on_indexCostSpin_changed(GtkEditable *editable,
			      gpointer user_data)
{
	gchar sql[128];

	snprintf(sql, 128, "alter session set optimizer_index_cost_adj=%d", 
		 gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw("indexCostSpin"))));
	execute_sql(sql, "resList", NULL);
}


void on_expPlanButton_clicked(GtkButton *button,
			      gpointer user_data)
{
	explain_plan(-1, (char *)NULL, (char *)NULL, (char *)NULL);
}


void on_sqlTabEntry_changed(GtkEditable *editable,
			    gpointer user_data)
{
	GtkWidget *label = command_tab_get_page_label();

	gtk_label_set_text(GTK_LABEL(label), 
			   gtk_entry_get_text(GTK_ENTRY(lw("sqlTabEntry"))) );
}



void on_traceItem_activate(GtkMenuItem *menuitem,
			   gpointer user_data)
{

}


void
on_l4trace_activate(GtkMenuItem *menuitem,
		    gpointer user_data)
{
	traceLevel = 4;
}


void on_l8trace_activate(GtkMenuItem *menuitem,
			 gpointer user_data)
{
	traceLevel = 8;
}


void on_l12trace_activate(GtkMenuItem *menuitem,
			  gpointer user_data)
{
	traceLevel = 12;
}


void on_session_trace_active1_activate(GtkMenuItem *menuitem,
				       gpointer user_data)
{
	OCIStmt       *sth = (OCIStmt *)0;
	gchar         sql[256];

	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(lw("session_trace_active1")))) {
		snprintf(sql, 256, 
			 "alter session set events '10046 trace name context forever, level %d'",
			 traceLevel);
	}
	else {
		snprintf(sql, 256, "alter session set sql_trace=FALSE");
	}

	OCIHandleAlloc((CONST dvoid *)envhp, (dvoid **)&sth,
		       (ub4)OCI_HTYPE_STMT,
		       (size_t)0, (dvoid **)0);
	OCIStmtPrepare(sth, errhp, (text *)sql,
		       (ub4) strlen(sql),
		       (ub4)OCI_NTV_SYNTAX,
		       (ub4)OCI_DEFAULT);
	OCIStmtExecute(bsvchp, sth, errhp, (ub4)1,
		       (ub4)0, (OCISnapshot *)0,
		       (OCISnapshot *)0, (ub4)OCI_DEFAULT);
	OCIHandleFree(sth, OCI_HTYPE_STMT);
}



void on_dbmsoutput_activate(GtkMenuItem *menuitem,
			    gpointer user_data)
{
	if (bsvchp) {

		if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(lw("dbmsoutput")))) {
			dbms_output_enable(envhp, bsvchp, errhp, TRUE);
		}
		else {
			dbms_output_enable(envhp, bsvchp, errhp, FALSE);
		}
	}
}



void on_sqltabAddButton_clicked(GtkButton *button,
				gpointer user_data)
{
	command_tab_add(NULL);
}



static gint results_popup(GtkWidget *widget, 
			  GdkEvent *event)
{
	GtkMenu        *menu;
	GdkEventButton *event_button;

	g_return_val_if_fail(widget != NULL, FALSE);
	g_return_val_if_fail(GTK_IS_MENU (widget), FALSE);
	g_return_val_if_fail(event != NULL, FALSE);

	menu = GTK_MENU(widget);
	if (event->type == GDK_BUTTON_PRESS) {
		event_button = (GdkEventButton *) event;
		if (event_button->button == 3) {
			gtk_menu_popup(menu, NULL, NULL, NULL,
				       NULL, event_button->button,
				       event_button->time);
			return TRUE;
		}
	}
	return FALSE;
}




static gint browser_popup (GtkWidget *widget,
			   GdkEvent  *event)
{
	GtkMenu *menu=0;
	GdkEventButton *event_button=0;


	g_return_val_if_fail(event != NULL, FALSE);

	menu = GTK_MENU(tab_popup);

	if (event->type == GDK_BUTTON_PRESS) {
		event_button = (GdkEventButton *) event;
    
		if (event_button->button == 3) {
			gint64      typeid=-1;

			gtk_treeview_get_value_gint64("objectList", 0, &typeid);

			switch( typeid ) {

			case B_TABLE_HEAP:
			case B_TABLE_IOT:
				gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(lookup_widget(GTK_WIDGET(menu), "analyze_label")))), "Analyze Table");
				gtk_widget_show(lookup_widget(tab_popup, "for_all_indexes2"));
				gtk_widget_show(lookup_widget(tab_popup, "for_all_columns"));
				gtk_widget_show(lookup_widget(tab_popup, "compute1"));
				gtk_widget_show(lookup_widget(tab_popup, "cascade1"));
				gtk_widget_show(lookup_widget(tab_popup, "block_sample1"));
				break;

			case B_INDEX:
				gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(lookup_widget(GTK_WIDGET(menu), "analyze_label")))), "Analyze Index");
				gtk_widget_hide(lookup_widget(tab_popup, "for_all_indexes2"));
				gtk_widget_hide(lookup_widget(tab_popup, "for_all_columns"));
				gtk_widget_hide(lookup_widget(tab_popup, "compute1"));
				gtk_widget_hide(lookup_widget(tab_popup, "cascade1"));
				gtk_widget_hide(lookup_widget(tab_popup, "block_sample1"));
				break;

			case B_INDEX_PARTITION:
				gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(lookup_widget(GTK_WIDGET(menu), "analyze_label")))), "Analyze Index Partition");
				gtk_widget_hide(lookup_widget(tab_popup, "for_all_indexes2"));
				gtk_widget_hide(lookup_widget(tab_popup, "for_all_columns"));
				gtk_widget_hide(lookup_widget(tab_popup, "compute1"));
				gtk_widget_hide(lookup_widget(tab_popup, "cascade1"));
				gtk_widget_hide(lookup_widget(tab_popup, "block_sample1"));
				break;

			case B_TABLE_PARTITION:
				gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(lookup_widget(GTK_WIDGET(menu), "analyze_label")))), "Analyze Table Partition");
				gtk_widget_show(lookup_widget(tab_popup, "for_all_indexes2"));
				gtk_widget_show(lookup_widget(tab_popup, "for_all_columns"));
				gtk_widget_show(lookup_widget(tab_popup, "compute1"));
				gtk_widget_show(lookup_widget(tab_popup, "cascade1"));
				gtk_widget_show(lookup_widget(tab_popup, "block_sample1"));
				break;

			case B_SCHEDULER_JOB:
				menu = GTK_MENU(jobPopup);
				break;

			case B_PROCEDURE:
			case B_FUNCTION:
				menu = GTK_MENU(recompilePopup);
				break;

			default:
				return FALSE;
			}


			gtk_menu_popup(menu, NULL, NULL, NULL, NULL, 
				       event_button->button, event_button->time);
			return TRUE;
		}
	}
	return FALSE;
}





static void on_find_entry_selected(GtkEntryCompletion *widget,
				   GtkTreeModel *model,
				   GtkTreeIter *iter,
				   gpointer user_data)
{
	GtkWidget *descWin=0;
	gint type=0;
	gchar *name=0;
	gchar owner[40];
	gchar object[40];
	gchar **name_p=0;


	gtk_tree_model_get(model, iter, 0, &type, 1, &name, -1);

	name_p = g_strsplit(name, ".", -1);
	g_free(name);

	if (!name_p) {
		return;
	}

	if (2 == g_strv_length(name_p) ) {

		g_snprintf((gchar *)owner, sizeof(owner), "%s", name_p[0]);
		g_snprintf((gchar *)object, sizeof(object), "%s", name_p[1]);
	}
	else {
		g_snprintf((gchar *)owner, sizeof(owner), "%s", get_schema());
		g_snprintf((gchar *)object, sizeof(object), "%s", name_p[0]);
	}
	g_strfreev(name_p);

	switch (type) {

	case 1:
	case 2:
	case 3:
		if( (descWin = create_describeWindow(owner, object)) != NULL)
			gtk_widget_show(descWin);
		break;

	case 4:
	case 5:
	case 6:
		if( (descWin = create_plsqldescribeWindow(owner, object, type)) != NULL)
			gtk_widget_show(descWin);
		break;

	default:
		break;
	}

}


static gboolean write_config_timeout(gpointer data)
{

	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(lw("autosaveconfigCheck"))) ) {

		config_write();

#if DEBUG
		fprintf(stderr, "%s:%d config saved\n", __FILE__, __LINE__);
#endif
	}

	return TRUE;
}


void on_window1_realize(GtkWidget       *widget,
			gpointer         user_data)
{
	gchar            homebuf[128];
	const gchar      *oracle_home=0, *font=0;

	GdkPixbuf *rollback_buf = gdk_pixbuf_new_from_inline(-1, rollback_icon, FALSE, NULL);
	GdkPixbuf *commit_buf = gdk_pixbuf_new_from_inline(-1, commit_pixbuf, FALSE, NULL);
	GdkPixbuf *newtab_buf = gdk_pixbuf_new_from_inline(-1, newtab_pixbuf, FALSE, NULL);
	GtkWidget *newtab_image;


	create_syntax_tag_table(NULL);
	create_syntax_tag_table(gtk_text_view_get_buffer(GTK_TEXT_VIEW(lw("ddlText"))) );
	create_syntax_tag_table(gtk_text_view_get_buffer(GTK_TEXT_VIEW(lw("detailDependenciesText"))) );

	config_read();

	if ((font = config_get("textfont")) != NULL) {
		gtk_widget_set_font(GTK_WIDGET(lw("ddlText")), font);
		gtk_widget_set_font(GTK_WIDGET(lw("detailDependenciesText")), font);
	}


	query_create_pixbufs();

	connect_buf = gdk_pixbuf_new_from_inline(-1, connect_pixbuf, FALSE, NULL);
	disconnect_buf = gdk_pixbuf_new_from_inline(-1, disconnect_pixbuf, FALSE, NULL);
	gtk_image_set_from_pixbuf(GTK_IMAGE(lw("commitImage")), commit_buf);
	gtk_image_set_from_pixbuf(GTK_IMAGE(lw("rollbackImage")), rollback_buf);
	gtk_image_set_from_pixbuf(GTK_IMAGE(lw("find_entry_image")), 
				  gdk_pixbuf_new_from_inline(-1, mini_close_over_pixbuf, FALSE, NULL));

	gtk_window_set_icon(GTK_WINDOW(window1), query_get_pixbuf("gtko_pixbuf") );

	connect_image = gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(lw("connect1")));
	gtk_image_set_from_pixbuf(GTK_IMAGE(connect_image), connect_buf);

	newtab_image = GTK_WIDGET(gtk_image_new_from_pixbuf(newtab_buf));
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(lw("add_tab")), newtab_image);

	tab_popup  = create_tablePopup();
	clip_popup = create_clipPopup();
	jobPopup   = create_jobPopup();
	recompilePopup = create_recompilePopup();

	query_async_init();

#if GTK_CHECK_VERSION(2,10,0)
	gtk_notebook_set_group_id(GTK_NOTEBOOK(lw("sqlNotebook")), 1);
	gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(lw("planTree")), TRUE);
	gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(lw("objectList")), TRUE);
	gtk_tree_view_set_grid_lines(GTK_TREE_VIEW(lw("resList")), GTK_TREE_VIEW_GRID_LINES_BOTH);
#endif

	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(lw("resList"))),
				    GTK_SELECTION_MULTIPLE);

	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(lw("detailDependenciesList")))), 
			 "changed",
			 G_CALLBACK(on_dependencySelected),
			 (gpointer) NULL);

	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(lw("objectList")))), 
			 "changed",
			 G_CALLBACK(on_objectSelected),
			 (gpointer) NULL);

	g_signal_connect(G_OBJECT(GTK_TREE_VIEW(lw("objectList"))), 
			 "row-activated",
			 G_CALLBACK(on_objectActivated),
			 (gpointer) NULL);

	g_signal_connect(G_OBJECT(GTK_TREE_VIEW(lw("detailList"))), 
			 "row-activated",
			 G_CALLBACK(on_detailListActivated),
			 (gpointer) NULL);

	g_signal_connect(G_OBJECT(GTK_TREE_VIEW(lw("planTree"))), 
			 "row-activated",
			 G_CALLBACK(on_planTreeActivated),
			 (gpointer) NULL);


	g_signal_connect_swapped(lw("resList"), "button_press_event"
				 , G_CALLBACK(results_popup), clip_popup);

	g_signal_connect_swapped(lw("objectList"), "button_press_event",
				 G_CALLBACK(browser_popup), (gpointer)NULL);

	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(lw("planTree")))), 
			 "changed", G_CALLBACK(explain_line_selected), NULL);
 
	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(lw("awrList")))), 
			 "changed", G_CALLBACK(cursor_cache_line_selected_cb), NULL);



	/* Create our entry */
	find_entry = gtk_entry_new ();
	gtk_widget_set_size_request(find_entry, -1, -1);
	gtk_box_pack_start (GTK_BOX (lw("findBox")), find_entry, TRUE, TRUE, 0);

	/* Create the completion object */
	find_entry_completion = gtk_entry_completion_new ();
	gtk_entry_completion_set_minimum_key_length(find_entry_completion, 1);

	/* Assign the completion to the entry */
	gtk_entry_set_completion (GTK_ENTRY (find_entry), find_entry_completion);
	g_object_unref (find_entry_completion);
    
	/* Create a tree model and use it as the completion model */
	completion_model = GTK_TREE_MODEL(gtk_list_store_new(2, G_TYPE_INT, G_TYPE_STRING));
	gtk_entry_completion_set_model(find_entry_completion, completion_model);
	g_object_unref(completion_model);
    
	/* Use model column 0 as the text column */
	gtk_entry_completion_set_text_column (find_entry_completion, 1);
	gtk_widget_show(find_entry);
 
	g_signal_connect_swapped(find_entry_completion, "match-selected",
				 G_CALLBACK(on_find_entry_selected), (gpointer) NULL);


	set_analyze_options();
	indcost_comp=0;

	OCIEnvCreate(&envhp, (ub4)OCI_THREADED | OCI_OBJECT | OCI_EVENTS,
		     (dvoid *)0, (dvoid *)0, (dvoid *)0, (dvoid *)0,
		     (size_t)0, (dvoid **)0);

	oracle_home = g_getenv("ORACLE_HOME");
	if (!oracle_home) {
		setLabel("statusLabel", "ORACLE_HOME not set");
	}
	else if (!g_ascii_strncasecmp(oracle_home, ".", 1) ) {
		setLabel("statusLabel", "ORACLE_HOME not set (library set to '.') ");
	}
	else {
		snprintf(homebuf, sizeof(homebuf), "ORACLE_HOME=%s", oracle_home);
		setLabel("statusLabel", homebuf);
	}

	g_timeout_add(5*60*1000, write_config_timeout, (gpointer)NULL);
}



void on_index_cost1_activate(GtkMenuItem     *menuitem,
			     gpointer         user_data)
{
	calc_index_cost(indcost_comp);
}


void compress_none_cb(GtkMenuItem     *menuitem,
		      gpointer         user_data)
{
	indcost_comp=0;
}


void compress_1_cb(GtkMenuItem     *menuitem,
		   gpointer         user_data)
{
	indcost_comp=1;
}


void compress_2_cb(GtkMenuItem     *menuitem,
		   gpointer         user_data)
{
	indcost_comp=2;
}


void on_granularity1_activate(GtkMenuItem     *menuitem,
			      gpointer         user_data)
{
	
}



void on_granularity2_activate(GtkMenuItem     *menuitem,
			      gpointer         user_data)
{

}


void on_clip_all_activate(GtkMenuItem     *menuitem,
			  gpointer         user_data)
{
	clip_results(GTK_TREE_VIEW(lw("resList")), 0);
}


void on_clip_selected_activate(GtkMenuItem     *menuitem,
			       gpointer         user_data)
{
	clip_results(GTK_TREE_VIEW(lw("resList")), 1);
}


void on_with_titles_activate(GtkMenuItem     *menuitem,
			     gpointer         user_data)
{
	set_clip_titles(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) );
}


gboolean on_clipWindow_delete_event(GtkWidget       *widget,
				    GdkEvent        *event,
				    gpointer         user_data)
{
	popdown_clip_window();
	return FALSE;
}


gboolean on_clipWindow_destroy_event(GtkWidget *widget,
				     GdkEvent *event,
				     gpointer user_data)
{
	popdown_clip_window();
	return FALSE;
}





gboolean send_keepalive(gpointer data)
{
	return(FALSE);
}



void on_runItem_activate(GtkMenuItem *menuitem,
			 gpointer user_data)
{
	
}


void on_execute_buffer_activate(GtkMenuItem *menuitem,
				gpointer user_data)
{
	on_executeButton_clicked(NULL, NULL);
}


void on_execute_line_activate(GtkMenuItem *menuitem,
			      gpointer user_data)
{
	GtkTextBuffer *buffer=0;
	GtkTextMark *cursor=0;
	GtkTextIter start;
	GtkTextIter end;

   
	buffer = getCurrentSqlBuffer(-1);
	
	cursor = gtk_text_buffer_get_insert(buffer);
	gtk_text_buffer_get_iter_at_mark(buffer, &start, cursor);

	if (gtk_text_iter_ends_line(&start)) {
		gtk_text_iter_backward_char(&start);
	}
	end = start;

	while (!gtk_text_iter_starts_line(&start) ) {
		gtk_text_iter_backward_chars(&start, 1);
	}

	gtk_text_iter_forward_to_line_end(&end);
	gtk_text_buffer_select_range(buffer, &start, &end);

	on_executeButton_clicked((GtkButton *)NULL, (gpointer) NULL);
}


void on_execute_block_activate(GtkMenuItem     *menuitem,
			       gpointer         user_data)
{
	GtkTextBuffer *buffer=0;
	GtkTextMark *cursor=0;
	GtkTextIter start;
	GtkTextIter end;
	GtkTextIter iter;
	gulong nchars;

	buffer = getCurrentSqlBuffer(-1);

	cursor = gtk_text_buffer_get_insert(buffer);
	gtk_text_buffer_get_iter_at_mark(buffer, &start, cursor);
	end = start;

	nchars = 1;
	for( ; gtk_text_iter_backward_line(&start) && nchars; ) {
		iter=start;

		for (nchars=0;
		     !gtk_text_iter_ends_line(&iter);
		     gtk_text_iter_forward_char(&iter) ) {
			gchar   c = gtk_text_iter_get_char(&iter);
			
			if( !g_ascii_isspace(c) ) {
				nchars++;
			}
		}

		if (!nchars) {
			gtk_text_iter_forward_line(&start);
			break;
		}
	}

	nchars = 1;
	for( ; gtk_text_iter_forward_line(&end) && nchars; ) {
		iter=end;

		for(nchars=0; !gtk_text_iter_ends_line(&iter); gtk_text_iter_forward_char(&iter) ) {
			gchar   c = gtk_text_iter_get_char(&iter);
      
			if( !g_ascii_isspace(c) )
				nchars++;
		}

		if(!nchars) {
			break;
		}
	}

	while( !gtk_text_iter_starts_line(&start) )
		gtk_text_iter_backward_char(&start);

	gtk_text_buffer_select_range(buffer, &start, &end);
	on_executeButton_clicked((GtkButton *)NULL, (gpointer) NULL);
}






void on_save_resultsItem_activate(GtkMenuItem     *menuitem,
				  gpointer         user_data)
{
	create_resultsSaveDialog("resList");
}


void on_results_saveItem_activate(GtkMenuItem     *menuitem,
				  gpointer         user_data)
{
	create_resultsSaveDialog("resList");
}


/*
void on_polling_activate(GtkMenuItem     *menuitem,
			 gpointer         user_data)
{

}
*/

void on_sqlNotebook_switch_page (GtkNotebook     *notebook,
				 GtkNotebookPage *page,
				 guint            page_num,
				 gpointer         user_data)
{
	GtkWidget   *npage = gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw("sqlNotebook")), page_num);
	const gchar *tablabel = gtk_label_get_text(GTK_LABEL(g_object_get_data(G_OBJECT(npage), "labelptr")));

	g_signal_handlers_disconnect_by_func(G_OBJECT(lw("sqlTabEntry")),
					     on_sqlTabEntry_changed, (gpointer) NULL);
	gtk_entry_set_text(GTK_ENTRY(lw("sqlTabEntry")), tablabel); 
	g_signal_connect(G_OBJECT(lw("sqlTabEntry")), 
			 "changed", G_CALLBACK(on_sqlTabEntry_changed),
			 (gpointer) NULL);
}



void on_pop_selected1_activate(GtkMenuItem     *menuitem,
			       gpointer         user_data)
{
	pop_results(1);
}


void on_pop_all_activated(GtkMenuItem     *menuitem,
			  gpointer         user_data)
{
	pop_results(0);
}


void on_traceconfOKButton_clicked(GtkButton       *button,
				  gpointer         user_data)
{
	trace_config_ok();
}


void on_traceconfCancelButton_clicked(GtkButton       *button,
				      gpointer         user_data)
{
	trace_config_cancel();
}


void on_configure_tracing1_activate(GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
	trace_config_pop(envhp, svchp, errhp);
}


void on_add_tab_activate(GtkMenuItem     *menuitem,
			 gpointer         user_data)
{
	command_tab_add(NULL);
}




void on_tableDataPageSpin_changed(GtkEditable     *editable,
				  gpointer         user_data)
{
  
	gchar *owner = getSelected("tableList", 1);
	gchar *table = getSelected("tableList", 2);

	if (!owner || !table) {
		return;
	}

	update_table_data(owner, table);

	g_free(owner);
	g_free(table);
}




void on_explain_plan_item_activate(GtkMenuItem     *menuitem,
				   gpointer         user_data)
{
	explain_plan(-1, (char *)NULL, (char *)NULL, (char *)NULL);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(lw("mainNotebook")), 2);
}



void on_preferences_activate(GtkMenuItem     *menuitem,
			     gpointer         user_data)
{
	config_window_show();
}



void on_open_external_activate(GtkMenuItem     *menuitem,
			       gpointer         user_data)
{
	result_to_external_app(0);
}


void on_external_editor_activate(GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
	inotify_edit_open(0); 
}


void on_external_viewer_activate(GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
	result_to_external_app(0);
}



void on_outputSaveButton_clicked(GtkButton       *button,
				 gpointer         user_data)
{
	create_resultsSaveDialog("resList");
}


void on_outputFileButton_clicked(GtkButton       *button,
				 gpointer         user_data)
{
	create_outputSaveChooser();
}


void on_traceConfigureButton_clicked(GtkButton       *button,
				     gpointer         user_data)
{
	trace_config_pop(envhp, svchp, errhp);
}


void on_traceFileChooserButton_clicked(GtkButton       *button,
				       gpointer         user_data)
{
	create_traceFileChooser();
}


void on_outputFileCheck_toggled(GtkToggleButton *togglebutton,
				gpointer         user_data)
{
	const gchar *file = g_object_get_data(G_OBJECT(lw("outputFileButton")), "filename");

	if (!file) {
		create_outputSaveChooser();
	}
}

void on_traceFileCheck_toggled(GtkToggleButton *togglebutton,
			       gpointer         user_data)
{
	const gchar *file = g_object_get_data(G_OBJECT(lw("traceFileChooserButton")), "filename");

	if (!file) {
		create_traceFileChooser();
	}
}


void on_send_keepalives_activate(GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) ) {
		keepalive_stop();
	}
	else {
		keepalive_window_show();
	}
}


void on_tableGranularityItem_activate(GtkMenuItem     *menuitem,
				      gpointer         user_data)
{

}



void on_detailNotebook_switch_page(GtkNotebook     *notebook,
				   GtkNotebookPage *page,
				   guint            page_num,
				   gpointer         user_data)
{

#if DEBUG
	fprintf(stderr, "callbacks.c: on_detailNotebook_switch_page(%d)\n", page_num);
#endif

	on_objectSelected(NULL, &page_num );
}


void on_traceSampleButton_clicked(GtkButton       *button,
				  gpointer         user_data)
{
	session_trace_display(svchp, errhp, "traceList", 0, 0);
}



void on_traceCheck_toggled(GtkToggleButton *togglebutton,
			   gpointer         user_data)
{
	guint       x;
	gboolean    toggled = gtk_toggle_button_get_active(togglebutton);
	const gchar *widgets[] = {"traceConfigureButton",
				  "traceFileCheck",
				  "traceFileChooserButton",
				  "traceWithQueryCheck",
				  "traceWithPlanCheck",
				  "traceAppendCheck",
				  "traceSampleButton",
				  NULL};

	for(x=0; widgets[x]; x++) {
		gtk_widget_set_sensitive(lw((gchar *)widgets[x]), toggled);
	}
}


void on_enable_job_activate(GtkMenuItem     *menuitem,
			    gpointer         user_data)
{
	browserSelection bs;
	gchar *type=0;
	gchar *owner=0;
	gchar *object=0;
	gchar *enabled=0;


	if (!browser_get_selection(GTK_TREE_VIEW(lw("objectList")), &bs) ) {
		return;
	}

	gtk_tree_model_get(bs.model,
			   &bs.iter, 
			   BC_TYPEID, &type,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   BC_PARTITIONED, &enabled,
			   -1);

	if (atoi(type) == B_SCHEDULER_JOB) {
		gchar   ownobj[64];

		g_snprintf(ownobj, sizeof(ownobj), "%s.%s", owner, object);

		if (1 == atoi(enabled) ) {
			execute_sql_to_glist("begin dbms_scheduler.disable(name=>:p1, force=>FALSE); end;",
					     ownobj, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
					   BC_PARTITIONED, "0",
					   BC_OBJECT_INFO, "<b>-</b>", -1);
			gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), "Job Disabled");
		}
		else {
			execute_sql_to_glist("begin dbms_scheduler.enable(name=>:p1); end;",
					     ownobj, NULL);
			gtk_tree_store_set(GTK_TREE_STORE(bs.model), &bs.iter,
					   BC_PARTITIONED, "1",
					   BC_OBJECT_INFO, "<b>+</b>", -1);
			gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), "Job Enabled");
		}
	}

	g_free(owner);
	g_free(object);
	g_free(type);
	browser_free_selection(&bs);
}


void on_run_job_activate(GtkMenuItem     *menuitem,
			 gpointer         user_data)
{
	browserSelection    bs;
	gchar *type=0;
	gchar *owner=0;
	gchar *object=0;


	if (!browser_get_selection(GTK_TREE_VIEW(lw("objectList")), &bs) ) {
		return;
	}

	gtk_tree_model_get(bs.model,
			   &bs.iter, 
			   BC_TYPEID, &type,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   -1);

	if( atoi(type) == B_SCHEDULER_JOB) {
		gchar   ownobj[64];

		g_snprintf(ownobj, sizeof(ownobj), "%s.%s", owner, object);
		execute_sql_to_glist("begin dbms_scheduler.run_job(job_name=>:p1, use_current_session=>FALSE); end;",
				     ownobj, NULL);
  }
	
	g_free(owner);
	g_free(object);
	g_free(type);
	browser_free_selection(&bs);
}


void on_purge_job_log_activate(GtkMenuItem     *menuitem,
			       gpointer         user_data)
{
	browserSelection    bs;
	gchar *type=0;
	gchar *owner=0;
	gchar *object=0;


	if (!browser_get_selection(GTK_TREE_VIEW(lw("objectList")), &bs) ) {
		return;
	}

	gtk_tree_model_get(bs.model,
			   &bs.iter, 
			   BC_TYPEID, &type,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   -1);

	if( atoi(type) == B_SCHEDULER_JOB) {
		gchar   ownobj[64];

		g_snprintf(ownobj, sizeof(ownobj), "%s.%s", 
			   owner, object);
		execute_sql_to_glist("begin dbms_scheduler.purge_log(log_history=>0, job_name=>:p1); end;", ownobj, NULL);
		clear_gtktreeview(GTK_TREE_VIEW(lw("detailList")));
	}

	g_free(owner);
	g_free(object);
	g_free(type);
	browser_free_selection(&bs);
}


void on_find_object_activate(GtkMenuItem     *menuitem,
			     gpointer         user_data)
{
	gtk_widget_show(lw("findBox"));
	gtk_notebook_set_current_page(GTK_NOTEBOOK(lw("mainNotebook")), 1);
	gtk_widget_grab_focus(find_entry);
}


void on_findCloseButton_clicked(GtkButton       *button,
				gpointer         user_data)
{
	gtk_widget_hide(lw("findBox"));
}


void on_table_info_activate(GtkMenuItem     *menuitem,
			    gpointer         user_data)
{
	browserSelection bs;
	gchar *typeid=0;
	gchar *owner=0;
	gchar *object=0;
	GtkWidget *descWin=0;

	if (!browser_get_selection(GTK_TREE_VIEW(lw("objectList")), &bs) ) {
		return;
	}

	gtk_tree_model_get(bs.model, 
			   &bs.iter,
			   BC_TYPEID, &typeid,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   -1);

	switch(atoi(typeid)) {
		
	case B_TABLE_HEAP:
	case B_TABLE_IOT:

		if ((descWin = create_describeWindow(owner, object)) != NULL) {
			gtk_widget_show(descWin);
		}
		break;

	default:
		break;
	}

	g_free(typeid);
	g_free(owner);
	g_free(object);
}


void on_recompile_item_activate(GtkMenuItem     *menuitem,
				gpointer         user_data)
{
	browserSelection    bs;
	gchar *type=0;
	gchar *owner=0;
	gchar *object=0;
	gchar *enabled=0;
	gchar plstype[32];
	gchar sql[256];


	if (!browser_get_selection(GTK_TREE_VIEW(lw("objectList")), &bs) ) {
		return;
	}

	gtk_tree_model_get(bs.model,
			   &bs.iter, 
			   BC_TYPEID, &type,
			   BC_OWNER, &owner,
			   BC_OBJECT, &object,
			   BC_PARTITIONED, &enabled,
			   -1);

	switch(atoi(type)) {

	case B_PROCEDURE:
		g_snprintf(plstype, sizeof(plstype), "procedure");
		break;

	case B_FUNCTION:
		g_snprintf(plstype, sizeof(plstype), "function");
		break;

	case B_PACKAGE:
		g_snprintf(plstype, sizeof(plstype), "package");
		break;

	default:
		break;
	}
	
	g_snprintf(sql, sizeof(sql),
		   "alter %s %s.%s compile %s %s",
		   plstype, owner, object,
		   recompile_debug ? "debug" : " ",
		   recompile_reuse ? "reuse settings" : " ");

	execute_sql(sql, "objectList", NULL);

	g_free(type);
	g_free(owner);
	g_free(object);
	g_free(enabled);
}


void on_recompile_debug_item_activate(GtkMenuItem     *menuitem,
				      gpointer         user_data)
{

	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) ) {
		recompile_debug=TRUE;
	}
	else {
		recompile_debug=FALSE;
	}
}


void on_recompile_reuse_item_activate(GtkMenuItem     *menuitem,
				      gpointer         user_data)
{

	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) ) {
		recompile_reuse=TRUE;
	}
	else {
		recompile_reuse=FALSE;
	}
}


void on_segmentAttrCheck_toggled(GtkToggleButton *togglebutton,
				 gpointer         user_data)
{

	if( !gtk_toggle_button_get_active(togglebutton) ) {
		gtk_widget_set_sensitive(lw("storageCheck"), FALSE);
		gtk_widget_set_sensitive(lw("tablespaceCheck"), FALSE);
	}
	else {
		gtk_widget_set_sensitive(lw("storageCheck"), TRUE);
		gtk_widget_set_sensitive(lw("tablespaceCheck"), TRUE);
	}
}


void on_ddlConsasalterCheck_toggled(GtkToggleButton *togglebutton,
				    gpointer         user_data)
{

	if( gtk_toggle_button_get_active(togglebutton) ) {
		gtk_widget_set_sensitive(lw("ddlConstraintsCheck"), TRUE);
		gtk_widget_set_sensitive(lw("ddlRefconstraintsCheck"), TRUE);

	}
	else {
		gtk_widget_set_sensitive(lw("ddlConstraintsCheck"), FALSE);
		gtk_widget_set_sensitive(lw("ddlRefconstraintsCheck"), FALSE);
	}
}


void on_autosaveconfigCheck_activate(GtkMenuItem     *menuitem,
				     gpointer         user_data)
{

}


