/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <glib.h>
#include <gtk/gtk.h>
 
#include "command_tab.h"
#include "appconfig.h"
#include "syntax.h"
#include "gutil.h"
#include "appconfig.h"
#include "icons/mini_close.h"
#include "icons/mini_close_over.h"

#if WITH_GTKSOURCEVIEW
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcelanguage.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#endif


static GdkPixbuf *mini_close=0;
static GdkPixbuf *mini_close_over=0;


GtkWidget *command_tab_get_page() {
	guint      page_num = gtk_notebook_get_current_page(GTK_NOTEBOOK(lw("sqlNotebook")));

	return(gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw("sqlNotebook")), page_num));
}


GtkWidget *command_tab_get_page_label()
{
	GtkWidget   *page = command_tab_get_page();
	
	return(g_object_get_data(G_OBJECT(page), "labelptr") );
}


GtkWidget *command_tab_get_page_button()
{
	GtkWidget   *page = command_tab_get_page();

	return(g_object_get_data(G_OBJECT(page), "buttonptr") );
}



GtkTextView *command_tab_get_current_textview()
{
	return(GTK_TEXT_VIEW(gtk_bin_get_child(GTK_BIN(command_tab_get_page() ))));
}


GtkTextBuffer *command_tab_get_current_textbuffer(gint pagenum) 
{
	GtkWidget      *scrolledWindow;
  
	if (-1 == pagenum) {
		pagenum = gtk_notebook_get_current_page(GTK_NOTEBOOK(lw("sqlNotebook")));
	}
	scrolledWindow = gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw( "sqlNotebook")), pagenum);
	return(gtk_text_view_get_buffer(GTK_TEXT_VIEW(gtk_bin_get_child(GTK_BIN(scrolledWindow)))));
}



static gboolean delete_button_enter(GtkWidget        *widget,
				    GdkEventCrossing *event,
				    gpointer          user_data) 
{
	gtk_image_set_from_pixbuf(GTK_IMAGE(user_data), mini_close_over);
	return FALSE;
}


static gboolean delete_button_leave(GtkWidget        *widget,
				    GdkEventCrossing *event,
				    gpointer          user_data)
{
	gtk_image_set_from_pixbuf(GTK_IMAGE(user_data), mini_close);
	return FALSE;
}



static gboolean command_tab_delete(GtkWidget *widget, 
				   GdkEventButton *event, 
				   gpointer user_data) 
{
	gint pagenum = gtk_notebook_page_num(GTK_NOTEBOOK(lw("sqlNotebook")), 
					     GTK_WIDGET(user_data));

	if (1 == event->button && -1 != pagenum) {
	  GtkTextBuffer      *buffer;
  
	  buffer = command_tab_get_current_textbuffer(pagenum);
	  if (buffer) {
		  g_object_unref(G_OBJECT(buffer));
	  }

	  gtk_notebook_remove_page(GTK_NOTEBOOK(lw("sqlNotebook")), pagenum);
	}

	return FALSE;
}



GtkWidget *command_tab_add(const gchar *tabname) 
{
	GtkWidget       *page = gtk_scrolled_window_new(NULL, NULL);
	GtkWidget       *tview;
	GtkWidget       *hbox, *align, *tab_label;
	GtkWidget       *event_box;
	GtkWidget       *mini_close_image=0;
	GtkTextTagTable *table=0;
	GtkTextTag      *tag=0;
	const gchar     *font=0;


#if WITH_GTKSOURCEVIEW

	GtkSourceBuffer           *buffer=0; 
	GtkSourceLanguagesManager *lm=0;
	GtkSourceLanguage         *language=0;
	GtkSourceTagStyle         *paren_style=0;

	paren_style = gtk_source_tag_style_new();
	gdk_color_parse("black", &paren_style->foreground);
	gdk_color_parse("darkseagreen2", &paren_style->background);
	paren_style->bold = TRUE;
	paren_style->mask = GTK_SOURCE_TAG_STYLE_USE_BACKGROUND && GTK_SOURCE_TAG_STYLE_USE_FOREGROUND;

	lm = gtk_source_languages_manager_new();
	language = gtk_source_languages_manager_get_language_from_mime_type(lm, "text/x-sql");

	buffer = GTK_SOURCE_BUFFER(gtk_source_buffer_new_with_language(language));
	gtk_source_buffer_set_highlight(buffer, TRUE);
	gtk_source_buffer_set_check_brackets(buffer, TRUE);
	gtk_source_buffer_set_bracket_match_style(buffer, paren_style);
	gtk_source_tag_style_free(paren_style);

	tview  = gtk_source_view_new_with_buffer(buffer);
	gtk_source_view_set_show_line_markers(GTK_SOURCE_VIEW(tview), TRUE);
	gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(tview), TRUE);
	gtk_source_view_set_highlight_current_line(GTK_SOURCE_VIEW(tview), TRUE);
	gtk_source_view_set_auto_indent(GTK_SOURCE_VIEW(tview), TRUE);
	add_error_tags(gtk_text_buffer_get_tag_table(buffer));
#else
	GtkTextBuffer             *buffer;

	buffer = gtk_text_buffer_new(get_syntax_tag_table());
	tview  = gtk_text_view_new_with_buffer(buffer);

	g_signal_connect_after(G_OBJECT(buffer), "insert_text", 
			 G_CALLBACK(buffer_insert_text), 
			 (gpointer) NULL);

	g_signal_connect_after(G_OBJECT(tview), "move_cursor", 
			 G_CALLBACK(text_move_cursor_cb),
			 (gpointer) NULL);
#endif

	if (!mini_close) {
		mini_close = gdk_pixbuf_new_from_inline(-1, mini_close_pixbuf,
							FALSE, NULL);
	}

	if (!mini_close_over) {
		mini_close_over = gdk_pixbuf_new_from_inline(-1, mini_close_over_pixbuf,
							     FALSE, NULL);
	}

	mini_close_image = gtk_image_new_from_pixbuf(mini_close);

	tab_label = gtk_label_new(tabname ? tabname : "SQL");

	hbox = gtk_hbox_new(FALSE, 0);
	event_box = gtk_event_box_new();  
	align = gtk_alignment_new(1.0, 0.2, 0.0, 0.0);
	gtk_container_add(GTK_CONTAINER(align), event_box);
	gtk_container_add(GTK_CONTAINER(event_box), mini_close_image);
	gtk_widget_set_events(event_box, 
			      GDK_BUTTON_PRESS_MASK 
			      | GDK_ENTER_NOTIFY_MASK 
			      | GDK_LEAVE_NOTIFY_MASK);

	gtk_box_pack_start(GTK_BOX(hbox), tab_label, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
	gtk_widget_show_all(hbox);


	g_object_set_data(G_OBJECT(page), "labelptr",  tab_label);
	g_object_set_data(G_OBJECT(page), "buttonptr", mini_close_image);
 

	gtk_notebook_append_page(GTK_NOTEBOOK(lw("sqlNotebook")), page, hbox );
	g_signal_connect(GTK_OBJECT(event_box), "button-press-event", 
			 G_CALLBACK(command_tab_delete), (gpointer) page);
	g_signal_connect(GTK_OBJECT(event_box), "enter-notify-event", 
			 G_CALLBACK(delete_button_enter), (gpointer) mini_close_image);
	g_signal_connect(GTK_OBJECT(event_box), "leave-notify-event", 
			 G_CALLBACK(delete_button_leave), (gpointer) mini_close_image);

#if GTK_CHECK_VERSION(2,10,0)
	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(lw("sqlNotebook")), page, TRUE);
	gtk_notebook_set_tab_detachable(GTK_NOTEBOOK(lw("sqlNotebook")), page, TRUE);
#endif

	gtk_text_view_set_left_margin(GTK_TEXT_VIEW(tview), 6);
	gtk_container_add(GTK_CONTAINER(page), tview);

	font = config_get("textfont");
	if(font) {
		gtk_widget_set_font(tview, font);
	}

#if WITH_GTKSOURCEVIEW
	gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(tview), 
					      config_get_integer("textview_linenumbers") );
	gtk_source_view_set_show_line_markers(GTK_SOURCE_VIEW(tview), 
					      config_get_integer("textview_linemarkers") );
	gtk_source_view_set_show_margin(GTK_SOURCE_VIEW(tview), 
					config_get_integer("textview_margin") );
	gtk_source_view_set_highlight_current_line(GTK_SOURCE_VIEW(tview), 
						   config_get_integer("textview_highlight_current") );
#endif

	gtk_widget_show(tview);
	gtk_widget_show(page);

	gtk_notebook_set_current_page(GTK_NOTEBOOK(lw("sqlNotebook")), -1);
  
	return(tview);
}





