/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <oci.h>
#include <gtk/gtk.h>

#include "gutil.h"
#include "orautil.h"
#include "cursor_cache.h"
#include "query.h"
#include "explain.h"
#include "appconfig.h"

extern OCIEnv      *envhp;
extern OCIError    *errhp;
extern OCISvcCtx   *svchp;
extern gint        cmode;
extern int         oramajor;
extern int         oraminor;

enum {S_SQLAREA, 
      S_STATSPACK, 
      S_AWR,
      S_ACTIVE};


static void cache_query_vsqlarea(gchar *nrows)
{

	guint       order=0;
	char        sql[8196], orderby[8];
	gint        cmode = config_get_integer("oracle_mode");

	order = gtk_combo_box_get_active(GTK_COMBO_BOX(lw( "awrOrderbyCombo")));
	snprintf(orderby, sizeof(orderby), "%d", order+7 );

	snprintf(sql, sizeof(sql),
		 "select * from (\
   select '0' \"Source\",\
    rawtohex(address) \"Address\",\
    to_char(v.hash_value) \"Hash\",\
    to_char(v.child_number) \"Child\",\
    u.username \"Parsing User\",\
    substr(v.sql_text,1,64) \"SQL Text\", \
    v.buffer_gets \"Buffer Gets\",\
    v.executions \"Execs\", \
    v.parse_calls \"Parses\", \
    round(v.buffer_gets/decode(v.executions,0,1,v.executions)) \"Gets/Exec\", \
    v.disk_reads \"Disk Reads\", \
    CASE WHEN v.executions>0 THEN round(v.disk_reads/v.executions,0) ELSE 0 END \"Reads/Exec\",\
    v.rows_processed \"Rows\",\
    v.fetches \"Fetches\", \
    CASE WHEN v.executions>0 THEN round(v.rows_processed/v.executions,0) ELSE 0 END \"Rows/Exec\", \
    CASE WHEN v.fetches>0 THEN round(v.rows_processed/v.fetches,0) ELSE 0 END \"Rows/Fetch\", \
    v.sorts \"Sorts\", \
    v.runtime_mem \"RunTimeMem\",\
    v.sharable_mem \"SharableMem\",\
    v.persistent_mem \"PersistentMem\"\
   from %s v, sys.dba_users u where u.user_id = v.parsing_user_id and u.username != 'SYS' \
   order by %s %s) where rownum <= :1",
		 cmode < 2 ? "sys.v_$sql" : "sys.v$sql",
		 orderby,
		 gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("awrOrderCheck"))) ? "ASC" : "DESC");

	execute_sql(sql, "awrList", nrows, NULL);
	hideColumn("awrList", 0); 
	hideColumn("awrList", 1);
/*   hideColumn("awrList", 2); */
	hideColumn("awrList", 3);
}





static void cache_query_statspack(gchar *nrows)
{
	gchar        begin[8], end[8];
	guint        order=0;
	gchar        sql[8192], orderby[8], descend[16];


	order = gtk_combo_box_get_active(GTK_COMBO_BOX(lw( "awrOrderbyCombo")));
	snprintf(orderby, sizeof(orderby), "%d", order+7);

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw( "awrOrderCheck")))) {
		snprintf(descend, 16, "ASC");
	}
	else {
		snprintf(descend, 16, "DESC");
	}

	snprintf(sql, 8192, "select * from (\
  select '1' \"Source\",\
   rawtohex(l.address) \"Address\",\
   to_char(l.hash_value) \"Hash\",\
   '0' \"Child\",\
   '--' \"Parsing User\",\
   st.sql_text \"SQL Text\",\
   (l.buffer_gets - nvl(f.buffer_gets,0)) \"Gets\",\
   (l.executions - nvl(f.executions,0)) \"Executions\", \
   (l.parse_calls - nvl(f.parse_calls,0)) \"Parses\", \
   CASE WHEN l.executions-nvl(f.executions,0) > 0 \
     THEN round((l.buffer_gets-nvl(f.buffer_gets,0))/(l.executions-nvl(f.executions,0)),1) \
     ELSE 0 END \"Gets/Exec\", \
   (l.disk_reads - nvl(f.disk_reads,0)) \"Disk Reads\", \
   CASE WHEN l.executions-nvl(f.executions,0) > 0 \
     THEN round((l.disk_reads-nvl(f.disk_reads,0))/(l.executions-nvl(f.executions,0)),1) \
     ELSE 0 END \"Reads/Exec\", \
   (l.rows_processed - nvl(f.rows_processed,0)) \"Rows\", \
   (l.fetches - nvl(f.fetches,0)) \"Fetches\", \
   CASE WHEN l.executions-nvl(f.executions,0) > 0 \
     THEN round((l.rows_processed-nvl(f.rows_processed,0))/(l.executions-nvl(f.executions,0)),1) \
     ELSE 0 END \"Rows/Exec\", \
   CASE WHEN l.fetches-nvl(f.fetches,0) > 0 \
     THEN round((l.rows_processed-nvl(f.rows_processed,0))/(l.fetches-nvl(f.fetches,0)),1) \
     ELSE 0 END \"Rows/Fetch\", \
   (l.elapsed_time - nvl(f.elapsed_time,0))/1000000 \"Elapsed sec\", \
  CASE WHEN l.executions-nvl(f.executions,0) > 0 \
     THEN round((l.elapsed_time-nvl(f.elapsed_time,0))/1000000/(l.executions-nvl(f.executions,0)),4) \
     ELSE 0 END \"Elap/Exec\", \
   (l.cpu_time - nvl(f.cpu_time,0))/1000 \"CPU sec\" \
   from stats$sql_summary l, stats$sql_summary f, stats$sqltext st \
   where f.snap_id(+) = :bid \
    and f.dbid(+) = l.dbid \
    and f.instance_number(+) = l.instance_number \
    and f.hash_value(+) = l.hash_value \
    and f.address(+) = l.address \
    and f.text_subset(+) = l.text_subset \
    and l.snap_id = :eid \
    and l.instance_number = 1 \
    and l.hash_value = st.%s \
    and l.text_subset = st.text_subset \
    and st.piece = 0 \
    order by %s %s, l.hash_value, st.piece \
  ) where rownum < :p3", oramajor>9 ? "old_hash_value" : "hash_value", orderby, descend);

	snprintf(begin, 8, "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorBeginSpin"))));
	snprintf(end, 8, "%d",  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorEndSpin"))));

	execute_sql(sql, "awrList", begin, end, nrows, NULL);

	hideColumn("awrList", 0); 
	hideColumn("awrList", 1); 
	hideColumn("awrList", 3); 
	hideColumn("awrList", 4); 
}




static void cache_query_awr(gchar *nrows) 
{
	gchar       begin[8], end[8], sql[8192], orderby[8], descend[8];
	int order = gtk_combo_box_get_active(GTK_COMBO_BOX(lw( "awrOrderbyCombo")));


	snprintf(orderby, sizeof(orderby), "%d", order+7 );

	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw( "awrOrderCheck")))) {
		snprintf(descend, 16, "ASC");
	}
	else {
		snprintf(descend, 16, "DESC");
	}

	snprintf(sql, 8192, "select * from (\
  select '2' \"Source\",\
   '0' \"Address\", \
   q.sql_id \"SQL_ID\",\
   '0' \"Child\",\
   u.username \"Parsing User\",\
   dbms_lob.substr(t.sql_text,64) \"SQL Text\",\
   sum(q.buffer_gets_delta) \"Buffer Gets\", \
   sum(q.executions_delta) \"Execs\",\
   sum(q.parse_calls_delta) \"Parses\", \
   CASE WHEN sum(q.executions_delta)>0 THEN round(sum(q.buffer_gets_delta)/sum(q.executions_delta),0)  ELSE 0 END \"Gets/Exec\", \
   sum(q.disk_reads_delta) \"Disk Reads\", \
   CASE WHEN sum(q.executions_delta)>0 THEN round(sum(q.disk_reads_delta)/sum(q.executions_delta),0) ELSE 0 END \"Reads/Exec\",\
   sum(q.rows_processed_delta) \"Rows\", \
   sum(q.fetches_delta) \"Fetch\", \
   CASE WHEN sum(q.executions_delta)>0 THEN round(sum(q.rows_processed_delta)/sum(q.executions_delta),0) ELSE 0 END \"Rows/Exec\", \
   CASE WHEN sum(q.fetches_delta)>0 THEN round(sum(q.rows_processed_delta)/sum(q.fetches_delta),0) ELSE 0 END \"Rows/Fetch\", \
   round(sum(q.elapsed_time_delta)/1000000,0) \"Elap Sec\", \
   CASE WHEN sum(q.executions_delta)>0 THEN round(sum(q.elapsed_time_delta)/1000000/sum(q.executions_delta),4) ELSE 0 END \"Elapsed/Exec\", \
   round(sum(q.cpu_time_delta)/1000000,0) \"CPU Sec\", \
   round(sum(q.iowait_delta)/1000000,0) \"IO Wait\", \
   round(sum(q.clwait_delta)/1000000,0) \"Clust Wait\", \
   round(sum(q.apwait_delta)/1000000,0) \"App Wait\", \
   round(sum(q.ccwait_delta)/1000000,0) \"Concur Wait\", \
   round(sum(q.PLSEXEC_TIME_DELTA)/1000000,0) \"PL/SQL Exec\", \
   round(sum(q.javexec_time_delta)/1000000,0) \"Java Exec\" \
   from dba_hist_sqlstat q, dba_hist_sqltext t, dba_users u \
  where t.sql_id = q.sql_id \
    and q.snap_id >= :1 and q.snap_id <= :2 \
    and u.user_id = q.parsing_schema_id and u.username != 'SYS' \
  group by q.sql_id, u.username, dbms_lob.substr(t.sql_text,64), t.command_type \
  order by %s %s)\
  where rownum < :p3", orderby, descend);

	snprintf(begin, 8, "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorBeginSpin"))));
	snprintf(end, 8, "%d",  gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorEndSpin"))));

	execute_sql(sql, "awrList", begin, end, nrows, NULL);
	hideColumn("awrList", 0); 
	hideColumn("awrList", 1);
	hideColumn("awrList", 3); 

}



static void  cache_query_active(gchar *nrows)
{
	gchar       sql[8192];
	gint        cmode = config_get_integer("oracle_mode");

	g_snprintf(sql, sizeof(sql), "\
  select '3' \"Source\", \
   rawtohex(s.sql_address) \"Address\", \
   to_char(s.sql_hash_value) \"Hash\", \
   0 \"Child\",\
   s.username || '(' || s.sid || ',' || s.serial# || ')' \"User\", \
   s.machine \"Machine\",\
   s.module \"Module\",\
   s.action \"Action\",\
   t.sql_text \"SQL\" \
  from %s t, %s s \
  where s.sql_address=t.address \
  and s.sql_hash_value=t.hash_value \
  and s.status = 'ACTIVE' \
  and t.piece=0",
		   cmode < 2 ? "sys.v_$sqltext" : "sys.v$sqltext",
		   cmode < 2 ? "sys.v_$session" : "sys.v$session"
		);

	execute_sql(sql, "awrList", NULL);
	hideColumn("awrList", 0); 
}



void cursorQueryButton_clicked(GtkButton *button,
			       gpointer user_data)
{
	gchar nrows[8];
  
	snprintf(nrows, sizeof(nrows), "%d", 
		 gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorRowsSpin"))) );

	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(lw( "cursorSourceCombo")))) {

	case S_SQLAREA:

		cache_query_vsqlarea(nrows);
		break;

	case S_STATSPACK:

		cache_query_statspack(nrows);
		break;

	case S_AWR:

		cache_query_awr(nrows);
		break;

	case S_ACTIVE:

		cache_query_active(nrows);
		break;

	default:
		break;
	}

}



static void cursor_cache_set_ranges(guint type) 
{
	gchar          sql[512]; 
	gint           min, max;
	OCIStmt        *sth;
	OCIDefine      *defnp;
	GtkSpinButton *bsb = GTK_SPIN_BUTTON(lw( "cursorBeginSpin"));
	GtkSpinButton *esb = GTK_SPIN_BUTTON(lw( "cursorEndSpin"));

	switch (type) {

	case S_SQLAREA:
		return;

	case S_STATSPACK:

		snprintf(sql, sizeof(sql),  "select snap_id, prev_snap_id from (\
    select snap_id, lead(snap_id) over(order by snap_id) prev_snap_id from (\
      select snap_id from stats$snapshot order by snap_id desc)\
       where rownum<3\
    ) where prev_snap_id is not null");
		break;

	case S_AWR:
		snprintf(sql, sizeof(sql), "select snap_id, prev_snap_id from (\
    select snap_id, lead(snap_id) over(order by snap_id) prev_snap_id from (\
      select snap_id from dba_hist_snapshot order by snap_id desc)\
       where rownum<3\
    ) where prev_snap_id is not null");
		break;

	default:
		return;
	}
    
	OCIHandleAlloc(envhp, (dvoid **)&sth,
		       (ub4)OCI_HTYPE_STMT, 0, NULL);
	OCIStmtPrepare(sth, errhp, (text *)sql,
		       (ub4)strlen(sql), OCI_NTV_SYNTAX,
		       OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 1,
		       &min, sizeof(min), SQLT_INT,
		       0, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 2,
		       &max, sizeof(max), SQLT_INT,
		       0, 0, 0, OCI_DEFAULT);
	if (OCI_SUCCESS == ociErr(errhp, OCIStmtExecute(svchp, sth, errhp,
							1, 0, NULL, NULL,
							OCI_DEFAULT))) {
		gtk_spin_button_set_range(bsb, 0, max);
		gtk_spin_button_set_range(esb, 0, max);
		gtk_spin_button_set_value(esb, max);
		gtk_spin_button_set_value(bsb, min);
	}
	OCIHandleFree(sth, OCI_HTYPE_STMT);
	return;
}




void on_cursorSourceCombo_changed(GtkComboBox *combobox,
				  gpointer user_data)
{
	gint combo = gtk_combo_box_get_active(combobox);

	switch (combo) {

	case S_SQLAREA:

		gtk_widget_hide(lw("cursorBeginLabel"));
		gtk_widget_hide(lw("cBeginLabel"));
		gtk_widget_hide(lw("cursorBeginSpin"));
		gtk_widget_hide(lw("cursorEndLabel"));
		gtk_widget_hide(lw("cEndLabel"));
		gtk_widget_hide(lw("cursorEndSpin"));
		gtk_widget_hide(lw("cursorSnapshotButton"));
		gtk_widget_hide(lw("statLevelSpin"));
		gtk_widget_hide(lw("awrReportButton"));
		gtk_widget_hide(lw("awrTrackButton"));
		gtk_widget_hide(lw("awrPlanButton"));
		break;

	case S_STATSPACK:
		gtk_widget_show(lw("cursorBeginLabel"));
		gtk_widget_show(lw("cBeginLabel"));
		gtk_widget_show(lw("cursorBeginSpin"));
		gtk_widget_show(lw("cursorEndLabel"));
		gtk_widget_show(lw("cEndLabel"));
		gtk_widget_show(lw("cursorEndSpin"));
		gtk_widget_show(lw("cursorSnapshotButton"));
		gtk_widget_show(lw("statLevelSpin"));
		gtk_widget_hide(lw("awrReportButton"));
		gtk_widget_hide(lw("awrTrackButton"));
		gtk_widget_hide(lw("awrPlanButton"));
		break;

	case S_AWR:

		gtk_widget_show(lw("cursorBeginLabel"));
		gtk_widget_show(lw("cBeginLabel"));
		gtk_widget_show(lw("cursorBeginSpin"));
		gtk_widget_show(lw("cursorEndLabel"));
		gtk_widget_show(lw("cEndLabel"));
		gtk_widget_show(lw("cursorEndSpin"));
		gtk_widget_show(lw("cursorSnapshotButton"));
		gtk_widget_show(lw("statLevelSpin"));
		gtk_widget_show(lw("awrReportButton"));
		gtk_widget_show(lw("awrTrackButton"));
		gtk_widget_show(lw("awrPlanButton"));
		break;

	case S_ACTIVE:

		gtk_widget_hide(lw("cursorBeginLabel"));
		gtk_widget_hide(lw("cBeginLabel"));
		gtk_widget_hide(lw("cursorBeginSpin"));
		gtk_widget_hide(lw("cursorEndLabel"));
		gtk_widget_hide(lw("cEndLabel"));
		gtk_widget_hide(lw("cursorEndSpin"));
		gtk_widget_hide(lw("cursorSnapshotButton"));
		gtk_widget_hide(lw("statLevelSpin"));
		gtk_widget_hide(lw("awrReportButton"));
		gtk_widget_hide(lw("awrTrackButton"));
		gtk_widget_show(lw("awrPlanButton"));
		break;

	default:
		break;
	}

	cursor_cache_set_ranges(combo);
}




void on_awrSnapshotButton_clicked(GtkButton *button,
				  gpointer user_data)
{

	guint combo;
	gchar statlevel[8];

	combo  = gtk_combo_box_get_active(GTK_COMBO_BOX(lw( "cursorSourceCombo")));
	switch( combo ) {

	case S_STATSPACK:
    
		snprintf(statlevel, sizeof(statlevel), "%d",
			 gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw("statLevelSpin"))) );
		execute_sql("BEGIN STATSPACK.SNAP(i_snap_level=>:p1); END;",
			    "awrList",
			    statlevel,
			    NULL);
		break;

	case S_AWR:

		execute_sql("select DBMS_WORKLOAD_REPOSITORY.CREATE_SNAPSHOT() from dual",
			    "awrList", NULL);
		break;

	default:
		break;
	}

	cursor_cache_set_ranges(combo);
}



void on_awrReportButton_clicked(GtkButton *button,
				gpointer user_data)
{
	gchar       *sql = "select output \"AWR Full Report\" \
  from table(DBMS_WORKLOAD_REPOSITORY.AWR_REPORT_TEXT((select dbid from v$database),\
  1, :p1, :p2))";
	gchar       bstr[16], estr[16];


	snprintf(bstr, sizeof(bstr), "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorBeginSpin"))));
	snprintf(estr, sizeof(estr), "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorEndSpin"))));

	execute_sql(sql, "awrList", bstr, estr, NULL);
}




void awrBeginSpin_value_changed(GtkSpinButton *spinbutton,
				gpointer user_data)
{
	gchar sql[512], bt[24];
	OCIStmt *sth;
	OCIDefine *defnp;
	OCIBind *bhp;
	guint snapid = gtk_spin_button_get_value_as_int(spinbutton);


	snprintf(bt, 24, "--/--/-- --:--");

	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(lw( "cursorSourceCombo")))) {
	case 1:
		snprintf(sql, 512, "select to_char(snap_time, 'YYYY/MM/DD HH24:MI') from stats$snapshot where snap_id = :1");
		break;
	case 2:
		snprintf(sql, 512, "select to_char(begin_interval_time, 'YYYY/MM/DD HH24:MI') from dba_hist_snapshot where snap_id = :1");
		break;
	default:
		break;
	}

	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT,
		       0, NULL);
	OCIStmtPrepare(sth, errhp, (text *)sql, (ub4)strlen(sql),
		       OCI_NTV_SYNTAX, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 1, (dvoid *)&snapid,
		     sizeof(snapid), SQLT_INT, 0, 0, 0,
		     0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 1, bt,
		       sizeof(bt)+1, SQLT_STR, 0, 0,
		       0, OCI_DEFAULT);
	ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 1,
				     0, NULL, NULL, OCI_DEFAULT));

	OCIHandleFree(sth, OCI_HTYPE_STMT);
  	setLabel("cursorBeginLabel", bt);
}


void awrEndSpin_value_changed(GtkSpinButton *spinbutton,
			      gpointer user_data)
{
	gchar sql[512], et[24];
	OCIStmt *sth;
	OCIDefine *defnp;
	OCIBind *bhp;
	guint snapid = gtk_spin_button_get_value_as_int(spinbutton);


	snprintf(et, 24, "--/--/-- --:--");

	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(lw( "cursorSourceCombo")))) {
	case 1:
		snprintf(sql, 512, "select to_char(snap_time, 'YYYY/MM/DD HH24:MI') from stats$snapshot where snap_id = :1");
		break;
	case 2:
		snprintf(sql, 512, "select to_char(end_interval_time, 'YYYY/MM/DD HH24:MI') from dba_hist_snapshot where snap_id = :1");
		break;
	default:
		break;
	}

	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT,
		       0, NULL);
	OCIStmtPrepare(sth, errhp, (text *)sql, (ub4)strlen(sql),
		       OCI_NTV_SYNTAX, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 1, (dvoid *)&snapid,
		     sizeof(snapid), SQLT_INT, 0, 0, 0, 0,
		     0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 1, et, sizeof(et)+1,
		       SQLT_STR, 0, 0, 0, OCI_DEFAULT);
	ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 1,
				     0, NULL, NULL, OCI_DEFAULT));

	OCIHandleFree(sth, OCI_HTYPE_STMT);
  	setLabel("cursorEndLabel", et);
}







void cursor_cache_line_selected_cb(GtkTreeSelection *sel,
				   gpointer data)
{
	GtkTextBuffer *buffer = getCurrentSqlBuffer(-1);
	GtkTextIter start;
	OCIStmt *sth;
	OCIBind *bhp;
	OCIDefine *defnp;
	gchar *source, *hash, *child, *address;
	gchar piece[32768], vsqlstmt[256];
	GList            *column_list = gtk_tree_view_get_columns(GTK_TREE_VIEW(lw("awrList")));
	gint             ncols;
	gint             cmode = config_get_integer("oracle_mode");
	const char       *awrstmt = "select sql_text from dba_hist_sqltext where sql_id=:p1";


	if (!column_list) {
		return;
	}

	ncols = g_list_length(column_list);
	g_list_free(column_list);
  
	if (ncols < 2) {
		return;
	}

	source  = getSelected("awrList", 0);
	address = getSelected("awrList", 1);
	hash    = getSelected("awrList", 2);
	child   = getSelected("awrList", 3);

	if (!source || !address || !hash) {
		return;
	}
 

	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("appendCursorCheck")))) {
		text_buffer_clear();
		gtk_text_buffer_get_iter_at_offset(buffer, &start, -1);
	}
	else {
		gtk_text_buffer_get_iter_at_offset(buffer, &start, -1);
		gtk_text_buffer_insert(buffer, &start, "\n", -1);
	}

	snprintf(vsqlstmt, sizeof(vsqlstmt), 
		 "select sql_text from %s where address=hextoraw(:p1) and hash_value=:p2 order by piece",
		 cmode < 2 ? "sys.v_$sqltext_with_newlines" : "sys.v$sqltext_with_newlines");

	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, 0, NULL);
  
	switch (atoi(source)) {

	case S_SQLAREA:
	case S_ACTIVE:

		OCIStmtPrepare(sth, errhp, (text *)vsqlstmt,
			       (ub4)strlen(vsqlstmt), OCI_NTV_SYNTAX,
			       OCI_DEFAULT);
		OCIBindByPos(sth, &bhp, errhp, 1, (dvoid *)address,
			     strlen(address)+1, SQLT_STR, 0, 0, 0,
			     0, 0, OCI_DEFAULT);
		OCIBindByPos(sth, &bhp, errhp, 2, (dvoid *)hash,
			     strlen(hash)+1, SQLT_STR, 0, 0, 0,
			     0, 0, OCI_DEFAULT);
		OCIDefineByPos(sth, &defnp, errhp, 1, piece,
			       sizeof(piece)+1, SQLT_STR, 0, 0, 0,
			       OCI_DEFAULT);
		break;
    
	case S_STATSPACK:

		OCIStmtPrepare(sth, errhp, (text *)vsqlstmt,
			       (ub4)strlen(vsqlstmt), OCI_NTV_SYNTAX,
			       OCI_DEFAULT);
		OCIBindByPos(sth, &bhp, errhp, 1, (dvoid *)address,
			     strlen(address)+1, SQLT_STR, 0, 0, 0,
			     0, 0, OCI_DEFAULT);
		OCIBindByPos(sth, &bhp, errhp, 2, (dvoid *)hash,
			     strlen(hash)+1, SQLT_STR, 0, 0, 0,
			     0, 0, OCI_DEFAULT);
		OCIDefineByPos(sth, &defnp, errhp, 1, piece,
			       sizeof(piece)+1, SQLT_STR, 0, 0, 0,
			       OCI_DEFAULT);
		break;

	case S_AWR:

		OCIStmtPrepare(sth, errhp, (text *)awrstmt,
			       (ub4)strlen(awrstmt), OCI_NTV_SYNTAX,
			       OCI_DEFAULT);
		OCIBindByPos(sth, &bhp, errhp, 1, (dvoid *)hash,
			     strlen(hash)+1, SQLT_STR, 0, 0, 0,
			     0, 0, OCI_DEFAULT);
		OCIDefineByPos(sth, &defnp, errhp, 1, piece,
			       sizeof(piece)+1, SQLT_STR, 0, 0, 0,
			       OCI_DEFAULT);
		break;

	default:
		return;

	}

	if(OCI_SUCCESS == ociErr(errhp, OCIStmtExecute(svchp, sth, errhp,
						      1, 0, NULL, NULL,
						      OCI_DEFAULT))) {
		GString    *str = g_string_new(piece);

		while (OCI_SUCCESS == OCIStmtFetch2(sth, errhp, 1,
						    OCI_FETCH_NEXT, 0,
						    OCI_DEFAULT))
			str = g_string_append(str, piece);

		gtk_text_buffer_insert(buffer, &start, str->str, -1); 
		g_string_free(str, TRUE);
	}

	OCIHandleFree(sth, OCI_HTYPE_STMT);  
	gtk_text_buffer_insert(buffer, &start, "\n", -1);
}



void on_awrPlanButton_clicked(GtkButton *button,
			      gpointer user_data)
{
	gchar     *source, *address, *hash, *child;

	source = getSelected("awrList", 0);
	address = getSelected("awrList", 1);
	hash = getSelected("awrList", 2);
	child = getSelected("awrList", 3);

	if (!source || !address || !hash || !child) {
		return;
	}

	explain_plan(atoi(source), address, hash, child);

	gtk_notebook_set_current_page(GTK_NOTEBOOK(lw( "mainNotebook")), 2);

	g_free(source);
	g_free(address);
	g_free(hash);
	g_free(child);
}


void cursor_cache_line_activated(GtkTreeView       *treeview, 
				 GtkTreePath       *path,
				 GtkTreeViewColumn *arg2,
				 gpointer          user_data)
{
	GtkTreeIter iter;
	GtkTreeModel *model=0;
	gchar bsnap[12], esnap[12];
	gchar *sqlid=0;

	model = gtk_tree_view_get_model(treeview);

	if (!gtk_tree_model_get_iter(model, &iter, path)) {
		return;
	}

	g_snprintf(bsnap, sizeof(bsnap), "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorBeginSpin"))));
	g_snprintf(esnap, sizeof(esnap), "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorEndSpin"))));

	sqlid = getSelected("awrList", 2);
	if (!sqlid) {
		return;
	}

	execute_sql_at_iter("\
   select '2', \
    '0', \
    s.sql_id, \
    s.snap_id, \
    NULL, \
    to_char(h.begin_interval_time,'MM/DD HH24:MI') || ' to '||to_char(h.end_interval_time,'MM/DD HH24:MI'), \
    s.buffer_gets_delta, \
    s.executions_delta, \
    s.parse_calls_delta, \
    CASE WHEN s.executions_delta>0 THEN round(s.buffer_gets_delta/s.executions_delta, 0) ELSE 0 END, \
    s.disk_reads_delta, \
    CASE WHEN s.executions_delta>0 THEN round(s.disk_reads_delta/s.executions_delta, 0) ELSE 0 END,\
    s.rows_processed_delta, \
    s.fetches_delta,\
    CASE WHEN s.executions_delta>0 THEN round(s.rows_processed_delta/s.executions_delta, 0) ELSE 0 END,\
    CASE WHEN s.executions_delta>0 THEN round(s.fetches_delta/s.executions_delta, 0) ELSE 0 END,\
    round(s.elapsed_time_delta/1000,2), \
    round(s.cpu_time_delta/1000,2), \
    round(s.iowait_delta/1000,2), \
    round(s.clwait_delta/1000,2) \
  from sys.dba_hist_sqlstat s, sys.dba_hist_snapshot h \
  where h.snap_id = s.snap_id \
    and h.dbid = s.dbid \
    and h.instance_number=s.instance_number \
    and s.snap_id >= :p1 \
    and s.snap_id <= :p2 \
    and s.sql_id = :p3 \
  order by s.snap_id, s.instance_number",
			    GTK_TREE_VIEW(lw("awrList")), &iter, bsnap, esnap, sqlid, NULL); 

}



void on_awrTrackButton_clicked(GtkButton *button,
			       gpointer user_data)
{
	gchar bSnap[12], eSnap[12];
	gchar *sqlid;
	gchar *stmt = "select s.sql_id \"SQL Id\", \
  to_char(h.begin_interval_time,'MM/DD HH24:MI') \"Snap Time\", \
  s.instance_number \"Instance\", \
  s.snap_id \"Snap\", \
  s.optimizer_cost \"Cost\", \
  s.parse_calls_delta \"Parses\", \
  s.executions_delta \"Execs\", \
  s.buffer_gets_delta \"Gets\", \
  CASE WHEN s.executions_delta>0 THEN round(s.buffer_gets_delta/s.executions_delta,0) ELSE 0 END \"Gets/Exec\",\
  s.rows_processed_delta \"Rows\",\
  s.disk_reads_delta \"DReads\",\
  s.sorts_delta \"Sorts\", \
  round(s.elapsed_time_delta/1000,2) \"Elapsed ms\", \
  round(s.cpu_time_delta/1000,2) \"CPU ms\", \
  round(s.iowait_delta/1000,2) \"IO Wait ms\", round(s.clwait_delta/1000,2) \"Clust Wt ms\", \
  round(s.apwait_delta/1000,2) \"App Wt ms\", round(s.plsexec_time_delta/1000,2) \"PL/SQL Exec ms\", \
  round(s.javexec_time_delta/1000,2) \"Java Exec ms\" \
  from sys.dba_hist_sqlstat s, sys.dba_hist_snapshot h \
  where h.snap_id = s.snap_id and h.dbid = s.dbid and h.instance_number = s.instance_number and \
  s.snap_id >= :1 and s.snap_id <= :2 and s.sql_id = :3 order by s.snap_id, s.instance_number";


	sqlid = getSelected("awrList", 2);
	if (!sqlid) {
		return;
	}
  
	g_snprintf(bSnap, sizeof(bSnap), "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorBeginSpin"))));
	g_snprintf(eSnap, sizeof(eSnap), "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "cursorEndSpin"))));

	execute_sql(stmt, "awrList", bSnap, eSnap, sqlid, NULL);
	g_free(sqlid);
}

