/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <oci.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "dbms_aq.h"
#include "plobject.h"
#include "gutil.h"
#include "orautil.h"
#include "syntax.h"
#include "query.h"

extern OCIEnv      *envhp;
extern OCIError    *errhp;
extern OCISvcCtx   *svchp;

extern gchar       *font;

void purge_queue() 
{
	gchar *owner=0, *queue=0;
	gchar *sql="DECLARE po aq$_purge_options_t; BEGIN DBMS_AQADM.PURGE_QUEUE_TABLE(queue_table=>:p1); END;";


	owner      = getSelected("queueList", 0);
	queue      = getSelected("queueList", 1);

	if (!owner || !queue) {
		return;
	}

	execute_sql(sql, "queueList", queue);

	g_free(owner);
	g_free(queue);
}


void alter_queue(GtkCellRendererText *cell,
		 gchar               *path_string,
		 gchar               *new_text,
		 gpointer             user_data) 
{
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(lw("queueList")));
	GtkTreeStore *store;
	gchar *queue=0, *retention=0, *retries=0, *delay=0, *comment=0;
	gchar queuebuf[128];
	guint statn = GPOINTER_TO_UINT(user_data);
	gchar *sql="\
 BEGIN DBMS_AQADM.ALTER_QUEUE(\
  queue_name=>:p1,\
  max_retries=>:p2,\
  retry_delay=>:p3,\
  retention_time=>:p4,\
  comment=>:p5);\
 END;";

	gtk_tree_model_get_iter_from_string(model, &iter,
					    path_string);
	store = GTK_TREE_STORE(model);
	gtk_tree_store_set(store, &iter, statn,
			   new_text, -1);
 
	gtk_tree_model_get(model, &iter, 1, &queue,
			   5, &retries, 6, &delay, 
			   7, &retention, 8, &comment, -1);

	if (!queue || !retries || !delay || !retention) {
		return;
	}

	snprintf(queuebuf, sizeof(queuebuf), "%s.%s", 
		 get_schema(), queue);

	execute_sql(sql, "resList", queuebuf, retries, 
		    delay, retention, comment, NULL);
 
	g_free(queue);
	g_free(retention);
	g_free(retries);
	g_free(delay);
	g_free(comment);
}


void set_queue_payload(GtkCellRendererText *cell,
		       gchar               *path_string,
		       gchar               *new_text,
		       gpointer             user_data) 
{
	GtkTreeIter      iter;
	GtkTreeModel     *model = gtk_tree_view_get_model(GTK_TREE_VIEW(lw("qPayloadList")));
	GtkTreeStore     *store;


	gtk_tree_model_get_iter_from_string(model, &iter,
					    path_string);
	store = GTK_TREE_STORE(model);
	gtk_tree_store_set(store, &iter, 2, new_text, -1);
}



void update_queue_list() 
{
	const gchar *schema = get_schema();
	GList *rends=0;
	guint x;

	execute_sql("\
  select q.owner,\
   q.name \"Queue\",\
   q.queue_table \"Q Table\",\
   decode(q.queue_type,'EXCEPTION_QUEUE','Except','NORMAL_QUEUE','Normal',q.queue_type) \"Type\",\
   t.object_type \"Payload\",\
   to_char(q.max_retries) \"Retries\",\
   to_char(q.retry_delay) \"Delay\",\
   to_char(q.retention) \"Reten\",\
   q.user_comment \"Comment\",\
   q.enqueue_enabled \"Enqueue\",\
   q.dequeue_enabled \"Dequeue\"\
  from all_queues q, all_queue_tables t\
  where q.queue_type = 'NORMAL_QUEUE'\
    and q.owner = :p1\
    and t.owner=:p2\
    and q.queue_table=t.queue_table\
  order by q.name", 
		    "queueList", schema, schema, NULL );

	for(x=5; x<=8; x++) {
		GtkTreeViewColumn *col;

		col = gtk_tree_view_get_column(GTK_TREE_VIEW(lw("queueList")), x);
	
		rends = gtk_tree_view_column_get_cell_renderers(col);
		
		g_object_set(GTK_CELL_RENDERER(rends->data), "editable", TRUE, NULL);
		g_object_set(GTK_CELL_RENDERER(rends->data), "weight", 800, NULL);
		g_signal_connect(GTK_CELL_RENDERER(rends->data), "edited", 
				 (GCallback) alter_queue, GUINT_TO_POINTER(x) );
		g_list_free(rends);
	}

	hideColumn("queueList", 0);
}






void queueSelected_cb(GtkTreeSelection *sel,
		      gpointer data) 
{

	gchar *owner=0, *queue=0, *queuetable=0, *typename=0;
	GtkTextIter start, end;
	GtkTextBuffer *buffer=0;
	GList *rends=0;
	guint page =  gtk_notebook_get_current_page(GTK_NOTEBOOK(lw("queueNotebook")));
	static char ddlbuf[256000];
	OCIStmt *sth;
	OCIDefine *defnp;
	gchar *sql = "SELECT DBMS_METADATA.GET_DDL(:1, :2, :3) from dual";
	gchar stat_sql[512];


	owner      = getSelected("queueList", 0);
	queue      = getSelected("queueList", 1);
	queuetable = getSelected("queueList", 2);
	typename   = getSelected("queueList", 4);

	if (!owner || !queue || !queuetable || !typename || strlen(typename)==0) {
		return;
	}

	if(data) {
		page = GPOINTER_TO_UINT(data);
	}

	switch (page) {

	case 0:
		snprintf(stat_sql, sizeof(stat_sql), "SELECT '<b>Ready:</b> ' || sum(decode(state, 0, 1, 0)) || \
    '   <b>Delayed:</b> ' || sum(decode(state, 1, 1, 0)) ||\
    '   <b>Retained:</b> ' || sum(decode(state, 2, 1, 0)) || '   <b>Exception:</b> ' || sum(decode(state, 2, 1, 0))\
    FROM %s.%s", owner, queuetable); 

		OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, 0, (dvoid **)0);
		ociPrepare(sth, errhp, stat_sql);
		OCIDefineByPos((OCIStmt *) sth,
			       (OCIDefine **) &defnp,
			       (OCIError *)errhp, 
			       (ub4) 1,
			       (dvoid *)&ddlbuf, sizeof(ddlbuf)+1, 
			       (ub2)SQLT_STR, 
			       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
			       (ub4)OCI_DEFAULT);

		if (OCI_SUCCESS == ociErr(errhp, OCIStmtExecute(svchp, sth, 
								errhp, 1, 0, 
								NULL, NULL, 
								OCI_DEFAULT))) {
			gtk_label_set_markup(GTK_LABEL(lw("qStatsLabel")), ddlbuf);
		}

		OCIHandleFree(sth, OCI_HTYPE_STMT);
 
		rends = gtk_tree_view_column_get_cell_renderers(gtk_tree_view_get_column(GTK_TREE_VIEW(lw("qPayloadList")), 2));
		if (rends) {
			g_object_set(GTK_CELL_RENDERER(rends->data),
				     "editable", TRUE, NULL);
			g_signal_connect(GTK_CELL_RENDERER(rends->data), 
					 "edited", (GCallback) set_queue_payload,
					 (gpointer) NULL );
			g_list_free(rends);
		}
		break;

	case 1:
	    
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lw("queueDDLText")));
		gtk_text_buffer_get_bounds(buffer, &start, &end);
		gtk_text_buffer_delete(buffer, &start, &end);

		OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT,
			       0, (dvoid **)0);
		ociPrepare(sth, errhp, sql);
		ociBindString(sth, errhp, 1, "AQ_QUEUE");
		ociBindString(sth, errhp, 2, queue);
		ociBindString(sth, errhp, 3, (gchar *)owner);
		if(ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 0, 0,
						NULL, NULL, OCI_DEFAULT)))
			return;
		OCIDefineByPos((OCIStmt *) sth,
			       (OCIDefine **) &defnp,
			       (OCIError *)errhp, 
			       (ub4) 1,
			       (dvoid *)&ddlbuf, sizeof(ddlbuf)+1, 
			       (ub2)SQLT_STR, 
			       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
			       (ub4)OCI_DEFAULT);

		while (OCI_SUCCESS == OCIStmtFetch2(sth, errhp, 1, OCI_FETCH_NEXT,
						    0, OCI_DEFAULT)) {
			gtk_text_buffer_insert(buffer, &start, ddlbuf, -1);
		}

		if (queuetable) {
			ociBindString(sth, errhp, 1, "AQ_QUEUE_TABLE");
			ociBindString(sth, errhp, 2, queuetable);
			ociBindString(sth, errhp, 3, (gchar *)owner);
			if(ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 0,
							0, NULL, NULL, OCI_DEFAULT)))
				return;

			OCIDefineByPos((OCIStmt *) sth,
				       (OCIDefine **) &defnp,
				       (OCIError *)errhp, 
				       (ub4) 1,
				       (dvoid *)&ddlbuf, sizeof(ddlbuf)+1, 
				       (ub2)SQLT_STR, 
				       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
				       (ub4)OCI_DEFAULT);
 
			while (OCI_SUCCESS == OCIStmtFetch2(sth, errhp, 1,
							    OCI_FETCH_NEXT, 0,
							    OCI_DEFAULT)) {
				gtk_text_buffer_insert(buffer, &start, ddlbuf, -1);
			}
      
		}
		OCIHandleFree(sth, OCI_HTYPE_STMT);
  		buffer_syntax_highlight(buffer, -1);
		break;
	}

	g_free(queue);
	g_free(queuetable);
}


void switch_queueNotebook_page(GtkNotebook *nb, 
			       GtkNotebookPage *page,
			       guint page_num)
{
	queueSelected_cb( (GtkTreeSelection *) NULL,
			  GUINT_TO_POINTER(page_num) );
}


void start_queue() 
{

	gchar *qstart = "BEGIN DBMS_AQADM.START_QUEUE(:p1); END;";
	gchar *qstop  = "BEGIN DBMS_AQADM.STOP_QUEUE(:p1); END;";
	gchar *queue = getSelected("queueList", 0);
	gchar *qstat = getSelected("queueList", 8);

	if (!queue || !qstat) {
		return;
	}

	if (!g_ascii_strncasecmp(qstat, "  NO", 4)) {
		execute_sql(qstart, "resList", queue, NULL);
	}
	else {
		execute_sql(qstop, "resList", queue, NULL);
	}
 
	update_queue_list(NULL);
	g_free(queue);
	g_free(qstat);
}
