/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "support.h"
#include <string.h>
#include <oci.h>
#include <gtk/gtk.h>

static gchar username[64];
static gchar password[64];
static gchar service_name[1024];
static gint conn_mode;
static GtkWidget *password_change=0;
static GtkWidget *passwordNewEntry1=0;
static GtkWidget *passwordNewEntry2=0;
static GtkWidget *passwordStatusLabel=0;
static GtkWidget *conn_dialog_password=0;
static GtkWidget *conn_dialog_label=0;

extern OCISvcCtx *svchp;
extern OCIError *errhp;

void connection_data_set(const gchar *user,
			 const gchar *pass,
			 const gchar *sid,
			 gint  mode)
{
	g_snprintf(username, sizeof(username), "%s", user);
	g_snprintf(password, sizeof(password), "%s", pass);
	g_snprintf(service_name, sizeof(service_name), "%s", sid);
	conn_mode = mode;
}

gchar *connection_get_user()
{
	return(username);
}

gchar *connection_get_pass()
{
	return(password);
}

gchar *connection_get_service()
{
	return(service_name);
}


static void on_passwordCancelButton_clicked(GtkButton *button,
					    gpointer user_data)
{
	gtk_widget_destroy(password_change);
	password_change=0;
}


void on_passwordOKButton_clicked(GtkButton *button,
				 gpointer user_data)
{
	sword ret=OCI_SUCCESS;
	const gchar *new_password1=0;
	const gchar *new_password2=0;

	new_password1 = gtk_entry_get_text(GTK_ENTRY(passwordNewEntry1));
	new_password2 = gtk_entry_get_text(GTK_ENTRY(passwordNewEntry2));

	if (g_ascii_strncasecmp(new_password1, new_password2, 
				strlen(new_password1) > strlen(new_password2) 
				? strlen(new_password1) : strlen(new_password2))) 
	{
		gtk_label_set_text(GTK_LABEL(passwordStatusLabel), 
				   "New passwords do not match");
		return;
	}

	ret = OCIPasswordChange(svchp, errhp, 
				(CONST text *) username, strlen(username),
				(CONST text *) password, strlen(password),
				(CONST text *) new_password1, strlen(new_password1),
				OCI_AUTH);

	if (OCI_SUCCESS != ret) {
		gchar errmsg[256];
		sb4 errcode;

		OCIErrorGet((dvoid *)errhp, 1, (text *)0,
			    &errcode, (text *)errmsg,
			    sizeof(errmsg), OCI_HTYPE_ERROR);

		g_strchomp(errmsg);
		gtk_label_set_text(GTK_LABEL(passwordStatusLabel), errmsg);
	}
	else {
		gtk_entry_set_text(GTK_ENTRY(conn_dialog_password),
				   new_password1);
		gtk_label_set_markup(GTK_LABEL(conn_dialog_label),
				     "<b>Password changed</b>");

		gtk_widget_destroy(password_change);
		password_change=0;
	}
}


GtkWidget* create_password_change_dialog(GtkWidget *login_password_entry,
					 GtkWidget *login_status_label,
					 const gchar *dialog_username,
					 const gchar *old_password)
{

  GtkWidget *dialog_vbox1=0;
  GtkWidget *vbox1=0;
  GtkWidget *hbox1=0;
  GtkWidget *label1=0;
  GtkWidget *passwordOldEntry=0;
  GtkWidget *hbox2=0, *hbox3=0;
  GtkWidget *label2=0, *label3=0;
  GtkWidget *dialog_action_area1=0;
  GtkWidget *passwordCancelButton=0;
  GtkWidget *passwordOKButton=0;

  if (password_change) {
	  return;
  }

  g_snprintf(username, sizeof(username), "%s", dialog_username);
  g_snprintf(password, sizeof(password), "%s", old_password);
  conn_dialog_password = login_password_entry;
  conn_dialog_label = login_status_label;

  password_change = gtk_dialog_new ();
  gtk_window_set_title(GTK_WINDOW(password_change),
		       _("Change Oracle Password"));
  gtk_window_set_modal(GTK_WINDOW(password_change), TRUE);
  gtk_window_set_type_hint(GTK_WINDOW(password_change),
			   GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG(password_change)->vbox;
  gtk_widget_show(dialog_vbox1);

  vbox1 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox1);
  gtk_box_pack_start(GTK_BOX(dialog_vbox1), vbox1, FALSE, FALSE, 0);

  hbox1 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox1);
  gtk_box_pack_start(GTK_BOX(vbox1), hbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new (_("Old Password:"));
  gtk_widget_show(label1);
  gtk_box_pack_start(GTK_BOX(hbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_width_chars(GTK_LABEL(label1), 20);

  passwordOldEntry = gtk_entry_new ();
  gtk_widget_show(passwordOldEntry);
  gtk_box_pack_start(GTK_BOX(hbox1), passwordOldEntry, FALSE, FALSE, 0);
  gtk_entry_set_visibility(GTK_ENTRY(passwordOldEntry), FALSE);
  gtk_entry_set_width_chars(GTK_ENTRY(passwordOldEntry), 12);
  gtk_entry_set_text(GTK_ENTRY(passwordOldEntry), old_password);

  hbox2 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox2);
  gtk_box_pack_start(GTK_BOX(vbox1), hbox2, TRUE, TRUE, 0);

  label2 = gtk_label_new (_("New Password:"));
  gtk_widget_show(label2);
  gtk_box_pack_start(GTK_BOX(hbox2), label2, FALSE, FALSE, 0);
  gtk_label_set_width_chars(GTK_LABEL(label2), 20);

  passwordNewEntry1 = gtk_entry_new ();
  gtk_widget_show(passwordNewEntry1);
  gtk_box_pack_start(GTK_BOX(hbox2), passwordNewEntry1, FALSE, TRUE, 0);
  gtk_entry_set_visibility(GTK_ENTRY(passwordNewEntry1), FALSE);
  gtk_entry_set_width_chars(GTK_ENTRY(passwordNewEntry1), 12);

  hbox3 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox3);
  gtk_box_pack_start(GTK_BOX(vbox1), hbox3, TRUE, TRUE, 0);

  label3 = gtk_label_new (_("Confirm New Password:"));
  gtk_widget_show(label3);
  gtk_box_pack_start(GTK_BOX(hbox3), label3, FALSE, FALSE, 0);
  gtk_label_set_width_chars(GTK_LABEL(label3), 20);

  passwordNewEntry2 = gtk_entry_new ();
  gtk_widget_show(passwordNewEntry2);
  gtk_box_pack_start(GTK_BOX(hbox3), passwordNewEntry2, FALSE, TRUE, 0);
  gtk_entry_set_visibility(GTK_ENTRY(passwordNewEntry2), FALSE);
  gtk_entry_set_width_chars(GTK_ENTRY(passwordNewEntry2), 12);

  passwordStatusLabel = gtk_label_new ("");
  gtk_widget_show(passwordStatusLabel);
  gtk_box_pack_start(GTK_BOX(dialog_vbox1), passwordStatusLabel,
		     FALSE, FALSE, 0);
  gtk_misc_set_alignment(GTK_MISC(passwordStatusLabel), 0.04, 0.5);

  dialog_action_area1 = GTK_DIALOG(password_change)->action_area;
  gtk_widget_show(dialog_action_area1);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area1),
			    GTK_BUTTONBOX_END);

  passwordCancelButton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show(passwordCancelButton);
  gtk_dialog_add_action_widget(GTK_DIALOG(password_change),
			       passwordCancelButton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS(passwordCancelButton, GTK_CAN_DEFAULT);

  passwordOKButton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show(passwordOKButton);
  gtk_dialog_add_action_widget(GTK_DIALOG(password_change),
			       passwordOKButton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS(passwordOKButton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) passwordCancelButton, "clicked",
                    G_CALLBACK(on_passwordCancelButton_clicked),
                    NULL);
  g_signal_connect ((gpointer) passwordOKButton, "clicked",
                    G_CALLBACK(on_passwordOKButton_clicked),
                    NULL);

  gtk_widget_show(password_change);

  return password_change;
}
