/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <oci.h>
#include <gtk/gtk.h>

#include "dbms_metadata.h"
#include "orautil.h"
#include "gutil.h"
#include "syntax.h"
#include "browser.h"
#include "query.h"

extern OCIEnv *envhp;
extern OCIError *errhp;
extern OCISvcCtx *svchp;

typedef struct _migroup {
	gint type;
	gchar *select_name;
	gchar *table_name;
} migroup;



void dbms_metadata_set_transforms()
{
	gchar *sql = "\
  DECLARE \n\
    sa_b boolean := TRUE;\n\
    st_b boolean := TRUE;\n\
    ca_b boolean := TRUE;\n\
    cn_b boolean := TRUE;\n\
    rc_b boolean := TRUE;\n\
    ts_b boolean := TRUE;\n\
    sq_b boolean := TRUE;\n\
  BEGIN\n\
   if :sa_bind = 0 then sa_b := FALSE; end if;\n\
   if :st_bind = 0 then st_b := FALSE; end if;\n\
   if :ca_bind = 0 then ca_b := FALSE; end if;\n\
   if :cn_bind = 0 then cn_b := FALSE; end if;\n\
   if :rc_bind = 0 then rc_b := FALSE; end if;\n\
   if :ts_bind = 0 then ts_b := FALSE; end if;\n\
   if :sq_bind = 0 then sq_b := FALSE; end if;\n\
   DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM, 'SEGMENT_ATTRIBUTES', sa_b);\n\
   DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM, 'STORAGE', st_b);\n\
   DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM, 'CONSTRAINTS_AS_ALTER', ca_b);\n\
   DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM, 'CONSTRAINTS', cn_b);\n\
   DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM, 'REF_CONSTRAINTS', rc_b);\n\
   DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM, 'TABLESPACE', ts_b);\n\
   DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM, 'SQLTERMINATOR', sq_b);\n\
  END;";
  
	execute_sql(sql, 
		    "resList", 
		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("segmentAttrCheck"))) ? "1" : "0",
		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("storageCheck"))) ? "1" : "0",
		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("ddlConsasalterCheck"))) ? "1" : "0",
		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("ddlConsasalterCheck"))) ? "0" : "1",
		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("ddlConsasalterCheck"))) ? "0" : "1",
		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("tablespaceCheck"))) ? "1" : "0",
		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("terminatorCheck"))) ? "1" : "0",
		    NULL);
}


static void clear_metadata_textbuffer(GtkTextBuffer *buffer)
{
	GtkTextIter iter_start, iter_end;

	gdk_threads_enter();
	
	gtk_text_buffer_get_bounds(buffer, &iter_start, &iter_end);
	gtk_text_buffer_delete(buffer, &iter_start, &iter_end);

	gdk_threads_leave();
}

static void free_metadata_info(metadata_input *mi)
{
	g_free(mi->type_name);
	g_free(mi->owner);
	g_free(mi->object);
	g_free(mi);
}


static void dbms_metadata_dependent_ddl(metadata_input *md_info)
{
	OCIStmt *sth=0;
	GtkTextIter iter;
	sword ret=OCI_SUCCESS;

	gtk_text_buffer_get_end_iter(md_info->buffer, &iter);

	if (!g_ascii_strncasecmp(md_info->type_name, "TABLE", 5) 
	    || !g_ascii_strncasecmp(md_info->type_name, "MATERIALIZED_VIEW", 17)
	    || !g_ascii_strncasecmp(md_info->type_name, "VIEW", 4)) 
	{
		guint x;
		const gchar *dsql = "select DBMS_METADATA.GET_DEPENDENT_DDL(:p1, :p2, :p3) from dual";
		gchar *dtype[] = {"INDEX", 
				  "CONSTRAINT", 
				  "REF_CONSTRAINT", 
				  "OBJECT_GRANT", 
				  "COMMENT", 
				  "MATERIALIZED_VIEW_LOG", 
				  "TRIGGER", 
				  "TABLE_STATISTICS", 
				  "INDEX_STATISTICS", 
				  "AQ_QUEUE"};
		gchar *dcheck[] = {"ddlIndexesCheck", 
				   "ddlConstraintsCheck", 
				   "ddlRefconstraintsCheck",
				   "ddlGrantsCheck",
				   "ddlCommentsCheck",
				   "ddlMviewlogCheck",
				   "ddlTriggersCheck",
				   "ddlTablestatsCheck",
				   "ddlIndexstatsCheck",
				   "ddlQueueCheck"};

		ret = ociErr(errhp, OCIStmtPrepare2(svchp, &sth,
						    errhp, (text *)dsql,
						    strlen(dsql),
						    (text *)0, 0,
						    OCI_NTV_SYNTAX, OCI_DEFAULT));
		if (OCI_SUCCESS != ret) {
			return;
		} 

		for(x=0; x < sizeof(dtype)/sizeof(dtype[0]); x++) {
			GString *ddl_str=0;
			gchar comment_buf[256];
 
			if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw(dcheck[x])))) {
				continue;
			}

			if (!GTK_WIDGET_IS_SENSITIVE(lw(dcheck[x]))) {
				continue;
			}

			ociBindString(sth, errhp, 1, dtype[x] );
			ociBindString(sth, errhp, 2, (gchar *)md_info->object);
			ociBindString(sth, errhp, 3, (gchar *)md_info->owner);

#if DEBUG
			fprintf(stderr, "%s:%d  %s %s %s\n", __FILE__, __LINE__,
				dtype[x], md_info->owner, md_info->object);
#endif

			ret = OCIStmtExecute(svchp, sth, errhp, 
					     0, 0, 0, 0, OCI_DEFAULT);
				   
			if (OCI_SUCCESS != ret) {
				return;
			}

			ddl_str = oci_piecewise_fetch_to_gstring(sth);

			g_snprintf(comment_buf, sizeof(comment_buf),
				   "\n/* %s */", dtype[x]);

			gdk_threads_enter();

			gtk_text_buffer_insert(md_info->buffer, &iter,
					       comment_buf, -1);
			if(ddl_str) {
				gtk_text_buffer_insert(md_info->buffer, &iter,
						       ddl_str->str, -1);
				gtk_text_buffer_insert(md_info->buffer, &iter,
						       "\n", -1);
				g_string_free(ddl_str, TRUE);
			}
			else {
				gtk_text_buffer_insert(md_info->buffer,
						       &iter, "\n", -1);
			}

			gdk_threads_leave();
		}

		OCIStmtRelease(sth, errhp, (OraText *) 0, 0, OCI_DEFAULT);
	}
}


static void dbms_metadata_execute(metadata_input *mi, OCIStmt *sth)
{
	sword ret;
	GtkTextIter iter;
	GString *ddl_str=0;


	clear_metadata_textbuffer(mi->buffer);
	gtk_text_buffer_get_start_iter(mi->buffer, &iter);

	ret = OCIStmtExecute(svchp, sth, errhp, 0, 0,
			     NULL, NULL, OCI_DEFAULT);

	if (OCI_SUCCESS != ret) {
		sb4 errcode;
		gchar errmsg[256];

		OCIErrorGet((dvoid *)errhp, 1, (text *)0,
			    &errcode, (text *)errmsg,
			    sizeof(errmsg), OCI_HTYPE_ERROR);

		gdk_threads_enter();
		gtk_text_buffer_insert(mi->buffer, &iter, 
				       "Error fetching metadata:\n",
				       -1);
		gtk_text_buffer_insert(mi->buffer, &iter, 
				       errmsg,
				       -1);
		gdk_threads_leave();
		return;
	}

	ddl_str = oci_piecewise_fetch_to_gstring(sth);

	if (ddl_str) {
		gdk_threads_enter();
		gtk_text_buffer_insert(mi->buffer, &iter, ddl_str->str, -1);
		buffer_syntax_highlight(mi->buffer, -1);
		gdk_threads_leave();

		g_string_free(ddl_str, TRUE);
	}

	OCIStmtRelease(sth, errhp, (OraText *) 0, 0, OCI_DEFAULT);
}


void dbms_metadata_to_textbuffer(metadata_input *mi)
{
	OCIStmt *sth=0;
	OCIBind *bhp1=0, *bhp2=0, *bhp3=0;
	GString *ddl_str=0;
	GtkTextIter iter;
	sword ret=OCI_SUCCESS;
	const gchar *sql = "SELECT DBMS_METADATA.GET_DDL(:p1, :p2, :p3) from dual";


	OCIStmtPrepare2(svchp, &sth, errhp, 
			(text *)sql, strlen(sql), 
			(text *)0, 0, 
			OCI_NTV_SYNTAX, OCI_DEFAULT);

	OCIBindByPos((OCIStmt *)sth, (OCIBind **)&bhp1, 
		     (OCIError *)errhp, (ub4)1, 
		     (dvoid *) mi->type_name, (sb4) strlen(mi->type_name)+1, 
		     (ub2)SQLT_STR, (dvoid *)0,
		     (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);
	OCIBindByPos((OCIStmt *)sth, (OCIBind **)&bhp2, 
		     (OCIError *)errhp, (ub4)2, 
		     (dvoid *) mi->object, (sb4) strlen(mi->object)+1,
		     (ub2)SQLT_STR, (dvoid *)0, (ub2 *)0,
		     (ub2 *)0, (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);
	OCIBindByPos((OCIStmt *)sth, (OCIBind **)&bhp2, 
		     (OCIError *)errhp, (ub4)3, 
		     (dvoid *) mi->owner, (sb4) strlen(mi->owner)+1, 
		     (ub2)SQLT_STR, (dvoid *)0, (ub2 *)0,
		     (ub2 *)0, (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);

	dbms_metadata_execute(mi, sth);

	dbms_metadata_dependent_ddl(mi);

	gdk_threads_enter();
	buffer_syntax_highlight(mi->buffer, -1);
	gdk_threads_leave();

	free_metadata_info(mi);
}


void dbms_metadata_group_to_textbuffer(metadata_input *mi)
{
	OCIStmt *sth=0;
	OCIBind *bhp1=0, *bhp2=0, *bhp3=0;
	GtkTextIter iter;
	sword ret=OCI_SUCCESS;
	gchar sql[256];
	gint x;
	static migroup migroup_config[] = {
		{B_TABLES, "t.table_name", "all_tables t where t.owner=:p3"},
		{B_INDEXES, "i.index_name", "all_indexes i where i.owner=:p3"},
		{B_VIEWS, "v.view_name", "all_views v where v.owner=:p3"},
		{B_CLUSTERS, "c.cluster_name", "all_clusters c where c.owner=:p3"},
		{B_MVIEWS, "m.mview_name", "all_mviews m where m.owner=:p3"},
		{B_MVIEW_LOGS, "m.log_table", "all_mview_logs m where m.log_owner=:p3"},
		{B_SYNONYMS, "s.synonym_name", "all_synonyms s where s.owner=:p3"},
		{B_QUEUES, "q.name", "all_queues q where q.owner=:p3"},
		{B_SEQUENCES, "s.sequence_name", "all_sequences s where s.sequence_owner=:p3"},
		{B_DIRECTORIES, "d.directory_name", "all_directories d where d.owner=:p3"},
		{-1, 0, 0}
	};

	for (x=0; ; x++) {
		if (migroup_config[x].type == mi->type) {
			break;
		}

		if (-1 == migroup_config[x].type) {
			return;
		}
	}

	g_snprintf(sql, sizeof(sql), 
		   "select dbms_metadata.get_ddl(:p1, %s, :p2) from %s",
		   migroup_config[x].select_name,
		   migroup_config[x].table_name);

	OCIStmtPrepare2(svchp, &sth, errhp, 
			(text *)sql, strlen(sql), 
			(text *)0, 0, 
			OCI_NTV_SYNTAX, OCI_DEFAULT);

	OCIBindByPos((OCIStmt *)sth, (OCIBind **)&bhp1, 
		     (OCIError *)errhp, (ub4)1, 
		     (dvoid *) mi->type_name,
		     (sb4)strlen(mi->type_name)+1, 
		     (ub2)SQLT_STR, (dvoid *)0, (ub2 *)0,
		     (ub2 *)0, (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);
	OCIBindByPos((OCIStmt *)sth, (OCIBind **)&bhp2, 
		     (OCIError *)errhp, (ub4)2, 
		     (dvoid *) mi->owner,
		     (sb4) strlen(mi->owner)+1,
		     (ub2)SQLT_STR, (dvoid *)0, (ub2 *)0,
		     (ub2 *)0, (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);
	OCIBindByPos((OCIStmt *)sth, (OCIBind **)&bhp3, 
		     (OCIError *)errhp, (ub4)3,
		     (dvoid *) mi->owner,
		     (sb4) strlen(mi->owner)+1,
		     (ub2)SQLT_STR, (dvoid *)0, (ub2 *)0,
		     (ub2 *)0, (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);

	dbms_metadata_execute(mi, sth);

	free_metadata_info(mi);
}
