/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <oci.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "dbms_output.h"
#include "gutil.h"
#include "orautil.h"
#include "query.h"
#include "appconfig.h"


extern GtkWidget      *window1;


void
dbms_output_enable(OCIEnv *env, 
		   OCISvcCtx *svc, 
		   OCIError *errhp, 
		   gboolean enable) 
{
	OCIStmt *sth = NULL;
	gchar sql[256];


	if (enable) {
		g_snprintf(sql, sizeof(sql), "BEGIN DBMS_OUTPUT.ENABLE; END;");
	}
	else {
		g_snprintf(sql, sizeof(sql), "BEGIN DBMS_OUTPUT.DISABLE; END;");
	}
#if DEBUG
	fprintf(stderr, "dbms_output.c: %s\n", sql);
#endif

	OCIHandleAlloc(env, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, 
		       (size_t)0, (dvoid **)0);
	OCIStmtPrepare(sth, errhp, (text *)sql, strlen(sql), 
		       OCI_NTV_SYNTAX, OCI_DEFAULT);
	ociErr(errhp, OCIStmtExecute(svc, sth, errhp, (ub4)1, 
				     0, NULL, NULL, OCI_DEFAULT));
	OCIHandleFree((dvoid *)sth, (ub4)OCI_HTYPE_STMT);
}




void show_dbms_output(OCIEnv *envhp,
		      OCIError *errhp,
		      OCISvcCtx *svc)
{
	OCIStmt *sth = (OCIStmt *)0;
	OCIBind *bhp1=0, *bhp2=0;
	gchar value[1024];
	gint status=0;
	const gchar *font = config_get("listfont");
	sb2 indp = OCI_IND_NULL;
	gchar *sql = "BEGIN DBMS_OUTPUT.GET_LINE(line=>:p1, status=>:p2); END;";
	int x=0;

	GtkTreeStore *store=0;
	GList *lcols;
	GtkCellRenderer *rend;
	GtkTreeViewColumn *column;
	GtkTreeView *list = GTK_TREE_VIEW(lw("resList"));
	GtkTreeIter iter;
	sword ret;
 

	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(lw("dbmsoutput")))) {
		return;
	}

	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("outputScreenCheck")))) {
		return;
	}

	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, 
		       (size_t)0, (dvoid **)0);
	OCIStmtPrepare(sth, errhp, (text *)sql, strlen(sql), 
		       OCI_NTV_SYNTAX, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp1, errhp, 1, (dvoid *)value, sizeof(value)+1, 
		     SQLT_STR, &indp, 0, 0, 0, 0, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp2, errhp, 2, (dvoid *)&status, sizeof(status), 
		     SQLT_INT, &indp, 0, 0, 0, 0, OCI_DEFAULT);

	ret = OCIStmtExecute(svc, sth, errhp, (ub4)1, 
			     0, NULL, NULL, OCI_DEFAULT);
	while (status != 1 
	       & (ret==OCI_SUCCESS || ret==OCI_STILL_EXECUTING)) {

		if (!x++) {

			store = GTK_TREE_STORE(gtk_tree_view_get_model(list));

			if (store) {
				gtk_tree_store_clear(GTK_TREE_STORE(store)); 
			}

			lcols = gtk_tree_view_get_columns(list);
			g_list_foreach(lcols, (GFunc) delcolumn, (gpointer) list);
			g_list_free(lcols);

			store = gtk_tree_store_new (1, G_TYPE_STRING);
 
			rend = gtk_cell_renderer_text_new();
			column = gtk_tree_view_column_new_with_attributes("DBMS Output",
									  rend, "text", 0, NULL);
			gtk_tree_view_append_column(list, column);

			if (font) {
				g_object_set(G_OBJECT(rend), "font", font, NULL);
			}

			g_object_set(G_OBJECT(rend), "ypad", 0, "xpad", 0, NULL);
		}
 
		if (OCI_SUCCESS == ret) {
			gtk_tree_store_append(store, &iter, NULL);
			gtk_tree_store_set(store, &iter, 0, value, -1);
		}

		ret = OCIStmtExecute(svc, sth, errhp, (ub4)1, 
				     0, NULL, NULL, OCI_DEFAULT);
	}

	if (x > 0) {
		gtk_tree_view_set_model(list, GTK_TREE_MODEL(store));
	}

	OCIHandleFree((dvoid *)sth, (ub4)OCI_HTYPE_STMT);
}
