/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <oci.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>

#include "dbms_space.h"
#include "orautil.h"
#include "gutil.h"


extern OCIError    *errhp;
extern OCISvcCtx   *svchp;


gint
get_dbms_space(const gchar *owner,
	       const gchar *objname,
	       const gchar *parname,
	       const gchar *objtype,
	       dspace      *ds, 
	       gint        simple) 
{
	OCIStmt            *sth = (OCIStmt *) 0;
	OCIBind            *bhp = (OCIBind *) 0;
	sb2                indp=-1;
	sword              ret = OCI_SUCCESS;

	if (parname) {
		indp=0;
	}

	gchar *sql = "BEGIN DBMS_SPACE.UNUSED_SPACE(\
  segment_owner=>:p0,\
  segment_name=>:p1,\
  segment_type=>:p2,\
  total_blocks=>:p3,\
  total_bytes=>:p4,\
  unused_blocks=>:p5,\
  unused_bytes=>:p6,\
  last_used_extent_file_id=>:p7,\
  last_used_extent_block_id=>:p8,\
  last_used_block=>:p9,\
  partition_name=>:p10);\
  END;";

	g_snprintf(ds->totblocks, sizeof(ds->totblocks), "0");
	g_snprintf(ds->totbytes, sizeof(ds->totbytes), "0");
	g_snprintf(ds->unblocks, sizeof(ds->unblocks), "0");
	g_snprintf(ds->unbytes, sizeof(ds->unbytes), "0");
	g_snprintf(ds->luefi, sizeof(ds->luefi), "0");
	g_snprintf(ds->luebi, sizeof(ds->luebi), "0");
	g_snprintf(ds->lub, sizeof(ds->lub), "0");

	OCIStmtPrepare2(svchp, &sth, errhp, (OraText *)sql, strlen(sql), 
			(OraText *)0, 0, OCI_NTV_SYNTAX, OCI_DEFAULT);

	OCIBindByPos(sth, &bhp, errhp, 1, (dvoid *)owner, strlen(owner)+1, 
		     SQLT_STR, 0, 0, 0, 
		     0, 0, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 2, (dvoid *)objname, strlen(objname)+1, 
		     SQLT_STR, 0, 0, 0, 0, 0, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 3, (dvoid *)objtype, strlen(objtype)+1, 
		     SQLT_STR, 0, 0, 0, 0, 0, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 4, (dvoid *)ds->totblocks, 
		     sizeof(ds->totblocks)+1, SQLT_STR, 0, 0, 0, 0, 0, 
		     OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 5, (dvoid *)ds->totbytes, 
		     sizeof(ds->totbytes)+1, SQLT_STR, 0, 0, 0, 0, 0,
		     OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 6, (dvoid *)ds->unblocks, 
		     sizeof(ds->unbytes)+1, SQLT_STR, 0, 0, 0, 0, 0,
		     OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 7, (dvoid *)ds->unbytes, 
		     sizeof(ds->unbytes)+1, SQLT_STR, 0, 0, 0, 0, 0,
		     OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 8, (dvoid *)ds->luefi, 
		     sizeof(ds->luefi), SQLT_STR, 0, 0, 0, 0, 0, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 9, (dvoid *)ds->luebi, 
		     sizeof(ds->luebi), SQLT_STR, 0, 0, 0, 0, 0, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 10, (dvoid *)ds->lub, 
		     sizeof(ds->lub), SQLT_STR, 0, 0, 0, 0, 0, OCI_DEFAULT);
	OCIBindByPos(sth, &bhp, errhp, 11, (dvoid *)parname, 
		     parname ? strlen(parname)+1 : 0, SQLT_STR, &indp, 0, 
		     0, 0, 0, OCI_DEFAULT);
	ret = OCIStmtExecute(svchp, sth, errhp, (ub4)1, 0, 
			     NULL, NULL, OCI_DEFAULT);

	if (ret != OCI_SUCCESS) {

		OCIStmtRelease(sth, errhp, (OraText *)0, 0, OCI_DEFAULT);
		ds->totbytes_i  = 0;
		ds->unbytes_i   = 0;
		ds->unblocks_i  = 0;
		ds->totblocks_i = 0;
		ds->blocksize = 0;
    		return 1;
	}
	OCIStmtRelease(sth, errhp, (OraText *)0, 0, OCI_DEFAULT);

	ds->totbytes_i  = strtoull(ds->totbytes, NULL, 10);
	ds->unbytes_i   = strtoull(ds->unbytes, NULL, 10);
	ds->unblocks_i  = strtoull(ds->unblocks, NULL, 10);
	ds->totblocks_i = strtoull(ds->totblocks, NULL, 10);
	ds->blocksize = ds->totbytes_i / ds->totblocks_i;

	return(0);
}



