/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "gutil.h"
#include "orautil.h"
#include "query.h"
#include "query_async.h"
#include "dbms_stats.h"
#include "browser.h"
#include "callbacks.h"

extern OCIEnv         *envhp;
extern OCIError       *errhp;
extern OCISvcCtx      *svchp;

static AnalyzeOptions analyzeopts;


void set_analyze_options() 
{

	analyzeopts.method=1;
	analyzeopts.compute=0;
	analyzeopts.cascade=1;
	analyzeopts.block_sample=1;
	analyzeopts.degree=32767;
	analyzeopts.indcompute=0;
	analyzeopts.inddegree=32767;
	analyzeopts.tabpar_granularity=0;
	analyzeopts.indpar_granularity=0;
}




void gather_stats(GtkMenuItem  *menuitem,
		  gpointer     user_data)
{
	OCIBind *bhp = (OCIBind *)0;
	qCommand *qs=0;
	GtkTreeSelection *sel=0;
	GtkTreeModel *model=0;
	GtkTreeIter iter, parent_iter;
	gchar *owner=0, *object=0, *parent=0, *partitioned=0, *type=0;
	gchar sql[4096];
	gchar *granularity[5] = {"DEFAULT",
				 "ALL",
				 "GLOBAL",
				 "PARTITION",
				 "SUBPARTITION"};




	if ( !(sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw("objectList"))))) {
		return;
	}
  
	gtk_tree_selection_get_selected(sel, &model, &iter);
	gtk_tree_model_get(model, &iter, 
			   BC_TYPEID, &type, 
			   BC_OWNER, &owner, 
			   BC_OBJECT, &object, 
			   BC_PARTITIONED, &partitioned,
			   -1);

	qs = alloc_qCommand(NULL);

	switch (atoi(type)) {

	case B_TABLE_HEAP:
	case B_TABLE_IOT:

		g_snprintf(sql, sizeof(sql), "\
    DECLARE \n\
      blk_bool   BOOLEAN:=TRUE; \n\
      cas_bool   BOOLEAN:=TRUE; \n\
      est_pct    NUMBER:=DBMS_STATS.AUTO_SAMPLE_SIZE; \n\
    BEGIN \n\
     if :p1 = 0 then blk_bool:=FALSE; end if; \n\
     if :p2 = 0 then cas_bool:=FALSE; end if; \n\
     if :p3 = 1 then est_pct:=100; end if; \n\
     DBMS_STATS.GATHER_TABLE_STATS(\n\
        ownname=>:p4, \n\
        tabname=>:p5, \n\
        degree=>:p6, \n\
        block_sample=>blk_bool, \n\
        cascade=>cas_bool, \n\
        method_opt=>:p7, \n\
        estimate_percent=>est_pct, \n\
        granularity=>:p8); \n\
     END;");

		qs->sql = g_strdup(sql);
		OCIStmtPrepare(qs->sth, errhp, (text *)qs->sql, 
			       (ub4)strlen(qs->sql), OCI_NTV_SYNTAX,
			       OCI_DEFAULT);
		ociBindString(qs->sth, errhp, 1, 
			      analyzeopts.block_sample ? "1" : "0");
		ociBindString(qs->sth, errhp, 2, 
			      analyzeopts.cascade ? "1" : "0");
		ociBindString(qs->sth, errhp, 3, 
			      analyzeopts.compute ? "1" : "0");
		ociBindString(qs->sth, errhp, 4, 
			      (gchar *)g_strdup(owner) );
		ociBindString(qs->sth, errhp, 5, 
			      g_strdup(object) );
		ociBindNumber(qs->sth, errhp, 6, 
			      &analyzeopts.degree, sizeof(analyzeopts.degree));
		ociBindString(qs->sth, errhp, 7, 
			      analyzeopts.method==0 ? "for all indexed columns" : "for all columns");
		ociBindString(qs->sth, errhp, 8, 
			      g_strdup(granularity[analyzeopts.tabpar_granularity]) );
		break;

	case B_INDEX:

		g_snprintf(sql, 1024, "BEGIN dbms_stats.gather_index_stats(\
    ownname=>:p1, indname=>:p2, degree=>:p3, granularity=>:p4); END;");

		qs->sql = g_strdup(sql);
		ociPrepare(qs->sth, errhp, sql);
		ociBindString(qs->sth, errhp, 1,
			      (gchar *)g_strdup(owner) );
		ociBindString(qs->sth, errhp, 2,
			      g_strdup(object) );
		ociBindNumber(qs->sth, errhp, 3,
			      &analyzeopts.degree, sizeof(analyzeopts.degree));
		ociBindString(qs->sth, errhp, 4,
			      g_strdup(granularity[analyzeopts.tabpar_granularity]) );
		break;

	case B_INDEX_PARTITION:

		gtk_tree_model_iter_parent(model, &parent_iter,
					   &iter);
		gtk_tree_model_get(model, &parent_iter,
				   BC_OBJECT, &parent, -1);

		g_snprintf(sql, 1024, "BEGIN dbms_stats.gather_index_stats(\
    ownname=>:p1, indname=>:p2, degree=>:p3, granularity=>:p4, partname=>:p5); END;");

		qs->sql = g_strdup(sql);
		ociPrepare(qs->sth, errhp, sql);
		ociBindString(qs->sth, errhp, 1, 
			      (gchar *)g_strdup(owner) );
		ociBindString(qs->sth, errhp, 2,
			      g_strdup(parent) );
		ociBindNumber(qs->sth, errhp, 3,
			      &analyzeopts.degree, sizeof(analyzeopts.degree));
		ociBindString(qs->sth, errhp, 4,
			      g_strdup(granularity[analyzeopts.tabpar_granularity]) );
		ociBindString(qs->sth, errhp, 5,
			      g_strdup(object));
		break;

	case B_TABLE_PARTITION:
    
		gtk_tree_model_iter_parent(model, &parent_iter,
					   &iter);
		gtk_tree_model_get(model, &parent_iter,
				   BC_OBJECT, &parent, -1);

		g_snprintf(sql, 1024, "BEGIN dbms_stats.gather_table_stats(ownname=>:1, tabname=>:2, degree=>:p3,\
    block_sample=>%s, cascade=>%s, method_opt=>:p4,\
    estimate_percent=>%s, granularity=>:p5, partname=>:p6); END;", 
			   analyzeopts.block_sample==1 ? "TRUE" : "FALSE",
			   analyzeopts.cascade==1 ? "TRUE" : "FALSE", 
			   analyzeopts.compute==1 ? "100" : "DBMS_STATS.AUTO_SAMPLE_SIZE");

		qs->sql = g_strdup(sql);
		ociPrepare(qs->sth, errhp, sql);
		ociBindString(qs->sth, errhp, 1,
			      (gchar *)g_strdup(owner) );
		ociBindString(qs->sth, errhp, 2,
			      g_strdup(parent) );
		ociBindNumber(qs->sth, errhp, 3,
			      &analyzeopts.degree, sizeof(analyzeopts.degree));
		ociBindString(qs->sth, errhp, 4,
			      analyzeopts.method==0 ? "for all indexed columns" : "for all columns");
		ociBindString(qs->sth, errhp, 5,
			      g_strdup(granularity[analyzeopts.tabpar_granularity]) );
		ociBindString(qs->sth, errhp, 6,
			      g_strdup(object)); 
		break;


	default:
		break;
	}

#if DEBUG
	fprintf(stderr, "%s\n%s %s %s\n", 
		sql, owner, 
		parent ? parent : object,
		parent ? object : " ");
#endif

	query_async_push(qs);

	if (parent) {
		g_free(parent);
	}

	g_free(owner);
	g_free(object);
}






void on_for_all_indexes2_activate(GtkMenuItem *menuitem,
				  gpointer user_data)
{
	analyzeopts.method=0;
}


void on_for_all_columns_activate(GtkMenuItem *menuitem,
				 gpointer user_data)
{
	analyzeopts.method=1;
}


void on_compute1_activate(GtkMenuItem *menuitem,
			  gpointer user_data)
{

	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
		analyzeopts.compute=1;
	}
	else {
		analyzeopts.compute=0;
	}
}


void on_show_comments1_activate(GtkMenuItem *menuitem,
				gpointer user_data)
{

}


void on_cascade1_activate(GtkMenuItem *menuitem,
			  gpointer user_data)
{
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) 
		analyzeopts.cascade=1;
	else 
		analyzeopts.cascade=0;
}


void on_block_sample1_activate(GtkMenuItem *menuitem,
			       gpointer user_data)
{
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) 
		analyzeopts.block_sample=1;
	else 
		analyzeopts.block_sample=0;
}



void on_tabledegreeDefault_activate(GtkMenuItem *menuitem,
				    gpointer user_data)
{
	analyzeopts.degree=32767;
}


void on_tabledegree1_activate(GtkMenuItem *menuitem,
			      gpointer user_data)
{
	analyzeopts.degree=1;
}


void on_tabledegree2_activate(GtkMenuItem *menuitem,
			      gpointer user_data)
{
	analyzeopts.degree=2;
}


void on_tabledegree4_activate(GtkMenuItem *menuitem,
			      gpointer user_data)
{
	analyzeopts.degree=4;
}


void on_tabledegree8_activate(GtkMenuItem *menuitem,
			      gpointer user_data)
{
	analyzeopts.degree=8;
}


void analyze_table_activate(GtkMenuItem *menuitem,
			    gpointer user_data)
{
	gather_stats(NULL, NULL);
}


void on_analyzeIndexCompute_activate(GtkMenuItem *menuitem,
				     gpointer user_data)
{
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) 
		analyzeopts.indcompute=1;
	else 
		analyzeopts.indcompute=0;
}



void on_inddegreeDefault_activate(GtkMenuItem *menuitem,
				  gpointer user_data)
{
	analyzeopts.inddegree=32767;
}


void on_analyzeIndexDegree1_activate(GtkMenuItem *menuitem,
				     gpointer user_data)
{
	analyzeopts.inddegree=1;
}


void on_analyzeIndexDegree2_activate(GtkMenuItem *menuitem,
				     gpointer user_data)
{
	analyzeopts.inddegree=2;
}


void on_analyzeIndexDegree4_activate(GtkMenuItem *menuitem,
				     gpointer user_data)
{
	analyzeopts.inddegree=4;
}


void on_analyzeIndexDegree8_activate(GtkMenuItem *menuitem,
				     gpointer user_data)
{
	analyzeopts.inddegree=8;
}


void on_tabparDefault_activate(GtkMenuItem *menuitem,
			       gpointer user_data)
{
	analyzeopts.tabpar_granularity=0;
}


void on_tabparAll_activate(GtkMenuItem *menuitem,
			   gpointer user_data)
{
	analyzeopts.tabpar_granularity=1;
}

void on_tabparGlobal_activate(GtkMenuItem     *menuitem,
			      gpointer         user_data)
{
	analyzeopts.tabpar_granularity=2;
}


void on_tabparPartition_activate(GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
	analyzeopts.tabpar_granularity=3;
}


void on_tabparSubpartition_activate(GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
	analyzeopts.tabpar_granularity=4;
}


void on_indparDefault_activate(GtkMenuItem     *menuitem,
			       gpointer         user_data)
{
	analyzeopts.indpar_granularity=0;
}


void on_indparAll_activate(GtkMenuItem     *menuitem,
			   gpointer         user_data)
{
	analyzeopts.indpar_granularity=1;
}


void on_indparGlobal_activate(GtkMenuItem     *menuitem,
			      gpointer         user_data)
{
	analyzeopts.indpar_granularity=2;
}


void on_indparPartition_activate(GtkMenuItem     *menuitem,
				 gpointer         user_data)
{
	analyzeopts.indpar_granularity=3;
}


void on_indparSubpartition_activate(GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
	analyzeopts.indpar_granularity=4;
}



void on_granularityDefaultItem_activate(GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	analyzeopts.indpar_granularity=0;
}


void on_granularitySubpartitionItem_activate(GtkMenuItem     *menuitem,
					     gpointer         user_data)
{
	analyzeopts.indpar_granularity=1;
}


void on_granularityPartitionItem_activate(GtkMenuItem     *menuitem,
					  gpointer         user_data)
{
	analyzeopts.indpar_granularity=2;
}


void on_granularityGlobalItem_activate(GtkMenuItem     *menuitem,
				       gpointer         user_data)
{
	analyzeopts.indpar_granularity=3;
}


void on_granularityAllItem_activate(GtkMenuItem     *menuitem,
				    gpointer         user_data)
{
	analyzeopts.indpar_granularity=4;
}

