/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <oci.h>
 
#include "describe_object.h"
#include "gutil.h"
#include "query.h"

enum {TINFO_COLUMNS, TINFO_INDEXES, TINFO_SEGMENTS, TINFO_INFO};


static gchar   *tab_column_sql = "select\
    t.column_name \"Column\",\
    t.data_type || \
    CASE \
       WHEN t.char_used='B' THEN '(' || t.data_length || ')'\
       WHEN t.data_type like 'TIMESTAMP%%' THEN NULL\
       ELSE decode(t.data_scale, NULL ,' ', '(' || t.data_precision || ',' || t.data_scale || ')') \
    END \"Datatype\",\
    t.nullable \"Null\",\
    t.data_default \"Default\",\
    to_char(t.last_analyzed,'MM/DD HH24:MI') \"Analyzed\",\
    com.comments \"Comments\"\
   from all_tab_columns t JOIN all_col_comments com on com.owner=t.owner \
           and com.table_name=t.table_name and com.column_name=t.column_name \
   where t.owner=:p1 and t.table_name=:p2";


static void
on_describeClose_clicked(GtkButton *button,
			 gpointer   user_data)
{
  
  if(GTK_IS_WIDGET(user_data)) {

    gtk_widget_destroy(user_data);
  }

} 



static void
on_describeNotebook_switched(GtkNotebook     *notebook,
			     GtkNotebookPage *page,
			     guint            page_num,
			     gpointer         user_data)
{
  gchar      *owner = g_object_get_data(G_OBJECT(user_data), "owner");
  gchar      *table = g_object_get_data(G_OBJECT(user_data), "table");
  GtkWidget  *describeColumnsList = g_object_get_data(G_OBJECT(user_data), "columnList");
  GtkWidget  *describeIndexesList = g_object_get_data(G_OBJECT(user_data), "indexesList");
  GtkWidget  *describeInfoList = g_object_get_data(G_OBJECT(user_data), "infoList");
  GtkWidget  *segmentsList = g_object_get_data(G_OBJECT(user_data), "segmentsList");


  switch(page_num) {

  case TINFO_COLUMNS:

    execute_sql_gtktreeview(tab_column_sql,
			    GTK_TREE_VIEW(describeColumnsList), 
			    owner, 
			    table, 
			    NULL);
    break;


  case TINFO_SEGMENTS:

    execute_sql_gtktreeview("SELECT \
      decode(t.subpartition_count,0, 'heap_pixbuf', 'partition_pixbuf') \"pxb\", \n \
      t.partition_name \"Partition\", \
      t.high_value \"High Value\",\
      t.tablespace_name \"Tablespace\",\
      t.logging \"Logging\",\
      t.num_rows \"# Rows\",\
      t.blocks \"Blocks\",\
      t.empty_blocks \"Empty Blocks\",\
      t.avg_row_len \"AvgLen\",\
      t.sample_size \"Sampled\",\
      to_char(t.last_analyzed,'MM/DD HH24:MI') \"Analyzed\"\
      from all_tab_partitions t where t.table_owner=:p1 and t.table_name=:p2 \
      order by t.partition_position",
			    GTK_TREE_VIEW(segmentsList),
			    owner,
			    table,
			    NULL);
    break;


  case TINFO_INDEXES:

  execute_sql_gtktreeview("select \
  ai.index_owner \"Owner\", ai.index_name \"Index\", \
  CASE a.status WHEN 'UNUSABLE' THEN 'red3' ELSE 'black' END \"foreground\",\
  ai.columns \"Columns\",\
  CASE a.partitioned WHEN 'NO' THEN a.tablespace_name ELSE '(Partitioned)' END \"Tablespace\",\
  a.index_type \"Type\",\
  a.uniqueness \"Unique\", CASE a.uniqueness WHEN 'UNIQUE' THEN 'green4' ELSE 'black' END \"foreground\",\
  CASE a.compression WHEN 'ENABLED' THEN 'COMPRESS '||a.prefix_length ELSE 'NO' END \"Compress\",\
  CASE a.compression WHEN 'ENABLED' THEN 'green4' ELSE 'black' END \"foreground\",\
  a.num_rows \"# Rows\", a.blevel \"Level\", a.leaf_blocks \"Leaves\",\
  a.clustering_factor \"Cl Fact\",\
  CASE WHEN a.num_rows>0 THEN round((a.clustering_factor/a.num_rows)*100,0)||'%' ELSE NULL END \"Cl Coeff\",\
  to_char(a.last_analyzed,'MM/DD HH24:MI') \"Analyzed\"\
  from\
  (with data as (\
   select \
     i.owner index_owner,\
     c.index_name index_name,\
     c.column_name col,\
     row_number() over (partition by c.index_name order by c.column_position) rn,\
     count(*) over (partition by c.index_name) cnt\
   from all_ind_columns c JOIN all_indexes i ON i.index_name = c.index_name\
     AND i.owner = c.index_owner AND i.table_owner = c.table_owner AND i.table_name = c.table_name\
   where  i.table_owner=:p1 and i.table_name = :p2\
   )\
   select index_owner, index_name, ltrim(sys_connect_by_path(col,','),',') columns \
   from data\
   where rn = cnt\
   start with rn = 1\
     connect by prior index_name=index_name and prior rn = rn-1\
   ) ai,\
  all_indexes a\
  where a.index_name = ai.index_name and a.owner = ai.index_owner and a.table_owner = :p3 and a.table_name=:p4", 
			  GTK_TREE_VIEW(describeIndexesList), 
			  owner, 
			  table, 
			  owner, 
			  table, 
			  NULL);
    break;
    
  case TINFO_INFO:

    execute_sql_pivotted("select * from all_tables where owner=:p1 and table_name=:p2",
			 GTK_TREE_VIEW(describeInfoList),
			 NULL,
			 0,
			 owner,
			 table,
			 NULL);
    break;

  default:
    break;
  }


}



GtkWidget*
create_describeWindow(const gchar *object_owner,
		      const gchar *object_name)
{
  GtkWidget *describeWindow;
  GtkWidget *vbox1, *vbox2;
  GtkWidget *describeNotebook;
  GtkWidget *describeColumnsList, *describeIndexesList, *describeInfoList, *segmentsList;
  GtkWidget *swin1;
  GtkWidget *commentEntry;
  GtkWidget *label1;
  GtkWidget *scrolledwindow2;
  GtkWidget *label2;
  GtkWidget *hbox1;
  GtkWidget *describeCloseButton;
  gchar     *object_name_upper=0, *object_owner_upper=0, title[128];


  g_snprintf(title, sizeof(title), "GTKO: Info for %s.%s", 
	     object_owner, object_name);
  describeWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (describeWindow), title);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (describeWindow), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (describeWindow), TRUE);
  gtk_window_set_focus_on_map (GTK_WINDOW (describeWindow), TRUE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (describeWindow), vbox1);

  describeNotebook = gtk_notebook_new ();
  gtk_widget_show (describeNotebook);
  gtk_box_pack_start (GTK_BOX (vbox1), describeNotebook, TRUE, TRUE, 0);
  gtk_widget_set_size_request (describeNotebook, 400, 400);


  /* Columns */
  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (describeNotebook), vbox2);

  commentEntry = gtk_entry_new();
  gtk_widget_show(commentEntry);
  gtk_box_pack_start(GTK_BOX(vbox2), commentEntry, FALSE, TRUE, 0);

  swin1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (swin1);
  gtk_box_pack_start (GTK_BOX (vbox2), swin1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin1), 
				       GTK_SHADOW_IN);

  describeColumnsList = gtk_tree_view_new ();
  gtk_widget_show (describeColumnsList);
  gtk_container_add (GTK_CONTAINER (swin1), describeColumnsList);

  label1 = gtk_label_new("Columns");
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (describeNotebook), 
			      gtk_notebook_get_nth_page(GTK_NOTEBOOK (describeNotebook), 0), 
			      label1);


  /* Indexes */
  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_container_add (GTK_CONTAINER (describeNotebook), scrolledwindow2);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_SHADOW_IN);

  describeIndexesList = gtk_tree_view_new ();
  gtk_widget_show (describeIndexesList);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), describeIndexesList);

  label2 = gtk_label_new ("Indexes");
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (describeNotebook),
			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (describeNotebook), 1),
			      label2);


  /* Segments */
  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow2);
  gtk_container_add (GTK_CONTAINER (describeNotebook), scrolledwindow2);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_SHADOW_IN);

  segmentsList = gtk_tree_view_new ();
  gtk_widget_show (segmentsList);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), segmentsList);

  label2 = gtk_label_new ("Segments");
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (describeNotebook), 
			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (describeNotebook), 2), 
			      label2);



  /* Table Info */
  swin1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (swin1);
  gtk_container_add (GTK_CONTAINER (describeNotebook), swin1);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin1), GTK_SHADOW_IN);

  describeInfoList = gtk_tree_view_new ();
  gtk_widget_show (describeInfoList);
  gtk_container_add (GTK_CONTAINER (swin1), describeInfoList);

  label1 = gtk_label_new("Info");
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (describeNotebook), 
			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (describeNotebook), 3), 
			      label1);



  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

  describeCloseButton = gtk_button_new_with_mnemonic ("Close");
  gtk_widget_show (describeCloseButton);
  gtk_box_pack_start (GTK_BOX (hbox1), describeCloseButton, FALSE, FALSE, 16);
  gtk_container_set_border_width (GTK_CONTAINER (describeCloseButton), 2);

  g_signal_connect((gpointer) describeCloseButton, "clicked", 
		   G_CALLBACK(on_describeClose_clicked), 
		   (gpointer) describeWindow);
  g_signal_connect((gpointer) describeNotebook, "switch-page", 
		   G_CALLBACK(on_describeNotebook_switched), 
		   (gpointer) describeWindow);
 
  object_owner_upper = g_ascii_strup(object_owner, -1);
  object_name_upper = g_ascii_strup(object_name, -1);

  g_object_set_data_full(G_OBJECT(describeWindow), "owner", 
			 (gpointer) object_owner_upper, 
			 (GDestroyNotify) g_free);
  g_object_set_data_full(G_OBJECT(describeWindow), "table", 
			 (gpointer) object_name_upper, 
			 (GDestroyNotify) g_free);
  g_object_set_data(G_OBJECT(describeWindow), "columnList", 
		    (gpointer) describeColumnsList);
  g_object_set_data(G_OBJECT(describeWindow), "indexesList",
		    (gpointer) describeIndexesList);
  g_object_set_data(G_OBJECT(describeWindow), "infoList",
		    (gpointer) describeInfoList);
  g_object_set_data(G_OBJECT(describeWindow), "segmentsList",
		    (gpointer) segmentsList);

  execute_sql_gtktreeview(tab_column_sql,
			  GTK_TREE_VIEW(describeColumnsList), 
			  object_owner_upper, 
			  object_name_upper, 
			  NULL);



  return describeWindow;
}


static void 
edit_plsql_args(GtkCellRendererText *cell,
		gchar               *path_string,
		gchar               *new_text,
		gpointer             user_data) 
{
  GtkTreeIter      iter;
  GtkTreeView      *treeview = GTK_TREE_VIEW(user_data);
  GtkTreeModel     *model = gtk_tree_view_get_model(treeview);
  gchar            *inout=0;

  gtk_tree_model_get_iter_from_string(model, &iter, path_string);
  gtk_tree_model_get(model, &iter, 1, &inout, -1);

  if( g_ascii_strncasecmp(inout, "OUT", 3) )
    gtk_tree_store_set(GTK_TREE_STORE(model), &iter, 4, new_text, -1);

  g_free(inout);
}



GtkWidget*
create_plsqldescribeWindow (const gchar *object_owner,
			    const gchar *object_name,
			    gint object_type)
{
	GtkWidget *plsqldescribeWindow;
	GtkWidget *vbox1;
	GtkWidget *toolbar1;
	GtkIconSize tmp_toolbar_icon_size;
	GtkWidget *scrolledwindow1;
	GtkWidget *treeview1;
	GList     *rends=0;
	gchar     *object_name_upper=0, *object_owner_upper=0;

	plsqldescribeWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (plsqldescribeWindow), "PL/SQL Info");

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (plsqldescribeWindow), vbox1);

	toolbar1 = gtk_toolbar_new ();
	gtk_widget_show (toolbar1);
	gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);
	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_ICONS);
	tmp_toolbar_icon_size = gtk_toolbar_get_icon_size(GTK_TOOLBAR (toolbar1));

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW (scrolledwindow1),
					     GTK_SHADOW_IN);

	treeview1 = gtk_tree_view_new ();
	gtk_widget_show (treeview1);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), treeview1);
	gtk_widget_set_size_request (treeview1, 360, 200);

	object_owner_upper = g_ascii_strup(object_owner, -1);
	object_name_upper = g_ascii_strup(object_name, -1);

	execute_sql_gtktreeview("select\
   argument_name \"Argument Name\",\
   in_out \"In/Out\",\
   data_type \"Datatype\",\
   default_value \"Default\",\
   NULL \"Value\"\
  from all_arguments where owner=:p1 and object_name=:p2\
  order by sequence",
				GTK_TREE_VIEW(treeview1),
				object_owner,
				object_name,
				NULL);

	/* Column comment editing */
	if( (rends = gtk_tree_view_column_get_cell_renderers(gtk_tree_view_get_column(GTK_TREE_VIEW(treeview1), 4))) != NULL) {
    
		g_object_set(GTK_CELL_RENDERER(rends->data), "editable", TRUE, NULL);
		g_signal_connect(GTK_CELL_RENDERER(rends->data), "edited", (GCallback) edit_plsql_args, (gpointer) treeview1);
		g_list_free(rends);
	}

	return plsqldescribeWindow;
}


void
on_describe_object_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkTextBuffer   *buffer;
	GtkTextMark     *cursor;
	GtkTextIter     start, end;
	gchar           *object_selected=0, *owner=0, *object=0;
	gchar           **object_arr=0;


	buffer = getCurrentSqlBuffer(-1);

	cursor = gtk_text_buffer_get_insert(buffer);
	gtk_text_buffer_get_iter_at_mark(buffer, &start, cursor);
	end = start;

	while( !gtk_text_iter_starts_line(&start)) {

		if(g_ascii_isspace(gtk_text_iter_get_char(&start))) {
			gtk_text_iter_forward_chars(&start, 1);
			break;
		}

		gtk_text_iter_backward_chars(&start, 1);
	}

	while( !gtk_text_iter_ends_line(&end) 
	       && !g_ascii_isspace(gtk_text_iter_get_char(&end)) ) {
		gtk_text_iter_forward_chars(&end, 1);
	}

	gtk_text_buffer_select_range(buffer, &start, &end);

  
	object_selected = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
	object_arr = g_strsplit_set(object_selected, ".", -1);
	g_free(object_selected);

	switch( g_strv_length(object_arr) ) {

	case 1:
		owner  = g_strdup(get_schema());
		object = g_strdup(object_arr[0]);
		break;

	case 2:
		owner  = g_strdup(object_arr[0]);
		object = g_strdup(object_arr[1]);
		break;

	default:
		break;
	}


	if( object && owner ) {
		GtkWidget   *descWin=0;

#if DEBUG
		fprintf(stderr, "%s:%d  %d - %s.%s\n",
			__FILE__, __LINE__, owner, object);
#endif

		if( (descWin = create_describeWindow(owner, object)) != NULL) {
			gtk_widget_show_all(descWin);
		}
    
	}

	g_free(object);
	g_free(owner);
	g_strfreev(object_arr);
}
