/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <oci.h>
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

#include "gutil.h"
#include "statement.h"
#include "orautil.h"
#include "table.h"
#include "index.h"
#include "query.h"
#include "partition.h"
#include "appconfig.h"
#include "describe_object.h"


extern OCIError *errhp;
extern OCIEnv *envhp;
extern OCISvcCtx *svchp;

static GtkTreeStore *planStore=0;


void on_optimizerModeCombo_changed(GtkComboBox *combobox,
				   gpointer user_data)
{
	gchar sql[200];
	gchar *mode[7] = {"CHOOSE", 
			  "ALL_ROWS", 
			  "FIRST_ROWS", 
			  "FIRST_ROWS_1",
			  "FIRST_ROWS_10",
			  "FIRST_ROWS_100",
			  "FIRST_ROWS_1000"};
	gint modesel = gtk_combo_box_get_active(GTK_COMBO_BOX(lw("optimizerModeCombo")));
  
	if (-1 == modesel) {
		return;
	}
  
	snprintf(sql, sizeof(sql), "alter session set optimizer_mode=%s", 
		 mode[modesel]);
	execute_sql(sql, "resList", NULL);
}


void  on_planTreeActivated(GtkTreeView *treeview,
			   GtkTreePath *path,
			   GtkTreeViewColumn *arg2,
			   gpointer user_data)
{
	GtkTreeModel *model = gtk_tree_view_get_model(treeview);
	GtkTreeIter iter;
	gchar *owner=0, *object=0, *objtype=0;


	if (!model) {
		return;
	}

	if (!gtk_tree_model_get_iter(model, &iter, path)) {
		return;
	}

	gtk_tree_model_get(model, &iter, 1, &owner, 3, &object, 
			   14, &objtype, -1);

	if(objtype) {
		if(atoi(objtype) == 1) {
			GtkWidget   *descWin = create_describeWindow(owner, object);

			if(descWin) {
				gtk_widget_show(descWin);
			}

		}
	}
	g_free(owner);
	g_free(object);
	g_free(objtype);
}



void explain_line_selected(GtkTreeSelection *sel, 
			   gpointer data)
{
	GtkTreeModel *model=0;
	GtkTreeIter  iter;
	gchar *object=0, *objecttype=0, *owner=0;
	OCIStmt *sth;
	OCIBind *bhp;
	gchar mbuf[1024];
	long int nrows=0, nblocks=0, avglen=0;
	long int ndist=0, clstfct=0, indlevel=0;
	double avglblk=0, avgdblk=0;
	sb2 indp=0;
	gchar *index_columns=0, *part_key=0;
	GList *cols=0;
	guint ncols=0;


	cols = gtk_tree_view_get_columns(GTK_TREE_VIEW(lw("planTree")));
	ncols = g_list_length(cols);
	g_list_free(cols);
 
	if (ncols < 3) {
		return;
	}
  
	mbuf[0] = '\0';
	if(gtk_tree_selection_get_selected(sel, &model, &iter)) {

		gtk_tree_model_get(model, &iter, 1, &owner, 
				   3, &object, 14, &objecttype, -1);

		if(!owner || !object || !objecttype)
			return;

		OCIHandleAlloc(envhp, (dvoid **)&sth, 
			       (ub4)OCI_HTYPE_STMT, 0, NULL);

		if( 1 == atoi(objecttype) ) {
			gchar *sql = "BEGIN DBMS_STATS.GET_TABLE_STATS(ownname=>:1, tabname=>:2, numrows=>:3, numblks=>:4, avgrlen=>:5); END;";
			ociPrepare(sth, errhp, sql);
			ociBindString(sth, errhp, 1, owner);
			ociBindString(sth, errhp, 2, object);
			ociBindNumber(sth, errhp, 3, &nrows, sizeof(nrows));
			ociBindNumber(sth, errhp, 4, &nblocks, sizeof(nblocks));
			ociBindNumber(sth, errhp, 5, &avglen, sizeof(avglen));
			if(ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 1, 0, NULL, NULL, OCI_DEFAULT))) {
				snprintf(mbuf, sizeof(mbuf), "<span color=\"red4\">No Table Stats</span>");
			}
			else {
				snprintf(mbuf, sizeof(mbuf),
					 "<b>Table Stats:</b>    <b>Blocks:</b> %ld   <b>Rows:</b> %ld   <b>Avglen:</b> %ld", 
					 nblocks, nrows, avglen);
			}

			if( (part_key = get_partition_keys("TABLE", owner, object)) != NULL) {
				g_strlcat(mbuf, "   <b>Partition Key:</b>", sizeof(mbuf) );
				g_strlcat(mbuf, part_key, sizeof(mbuf) );
				g_free(part_key);
			}

		}

		if (2 == atoi(objecttype)) {
			guint x;
			gchar *sql = "BEGIN DBMS_STATS.GET_INDEX_STATS(ownname=>:p1, indname=>:p2, numrows=>:p3, numlblks=>:p4, numdist=>:p5,\
      avglblk=>:p6, avgdblk=>:p7, clstfct=>:p8, indlevel=>:p9); END;";

			for(x=0; x<strlen(object); x++)
				if(object[x] == ' ') {
					object[x]='\0';
					break;
				}

			ociPrepare(sth, errhp, sql);
			OCIBindByPos(sth, &bhp, errhp, 1, (dvoid *)owner, 
				     strlen(owner)+1, SQLT_STR, 0, 0, 0, 
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 2, (dvoid *)object, 
				     strlen(object)+1, SQLT_STR, 0, 0, 0, 
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 3, (dvoid *)&nrows, 
				     sizeof(nrows), SQLT_INT, &indp, 0, 0, 
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 4, (dvoid *)&nblocks, 
				     sizeof(nblocks), SQLT_INT, &indp, 0, 0, 
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 5, (dvoid *)&ndist,
				     sizeof(ndist), SQLT_INT, &indp, 0, 0, 
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 6, (dvoid *)&avglblk,
				     sizeof(avglblk), SQLT_FLT, &indp, 0, 0,
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 7, (dvoid *)&avgdblk,
				     sizeof(avgdblk), SQLT_FLT, &indp, 0, 0,
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 8, (dvoid *)&clstfct,
				     sizeof(clstfct), SQLT_INT, &indp, 0, 0,
				     0, 0, OCI_DEFAULT);
			OCIBindByPos(sth, &bhp, errhp, 9, (dvoid *)&indlevel,
				     sizeof(indlevel), SQLT_INT, &indp, 0, 0,
				     0, 0, OCI_DEFAULT);

			if(ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 1, 0, NULL, NULL, OCI_DEFAULT)) != OCI_SUCCESS) {
				snprintf(mbuf, sizeof(mbuf), "<span color=\"red4\">No Index Stats</span>  <b>Columns:</b>");
			}
			else {
				snprintf(mbuf, sizeof(mbuf), 
					 "<b>Index Stats:</b>    <b>Level:</b> %ld   <b>Rows:</b> %ld   <b>Distinct:</b> %ld  <b>ClFact:</b> %ld(%.02f)  <b>Leaf:</b> %ld   <b>Columns:</b> ",
					 indlevel+(long int)1, nrows, ndist, clstfct, nrows>0 ? (double)clstfct/(double)nrows : 0, nblocks);
			}
      
			if( (index_columns = get_index_columns(owner, object)) != NULL) {
				g_strlcat(mbuf, index_columns, sizeof(mbuf) );
				g_free(index_columns);
			}

		}

		gtk_label_set_markup(GTK_LABEL(lw("objectLabel")), mbuf);
		OCIHandleFree(sth, OCI_HTYPE_STMT);
		g_free(object);
		g_free(objecttype);
		g_free(owner);
	}
}



void explain_plan(int source, 
		  gchar *address, 
		  gchar *hash, 
		  gchar *child)
{
	OCIStmt *sth=0;
	OCIDefine *defnp=0;
	gchar *exsql=0;
	GtkTreeIter iter[1024];
	gchar op[128], object_pbf[32], object[64];
	gchar objtype[16], cost[32], card[32];
	gchar bytes[32], cpu_cost[32], io_cost[32];
	gchar owner[32], distribution[32], temp[32], pstart[32], pstop[32], other_tag[64];
	gint id, pid, optimizer_mode;
	sb2 id_ind, pid_ind, indp;
	GtkTreeView *list = GTK_TREE_VIEW(lw("planTree"));
	GtkCellRenderer *renderer=0;
	GtkTreeViewColumn *column;
	GSList *stmtlist, *t_stmtlist;
	GtkCellRenderer *pbf_renderer=0;
	const gchar *font = config_get("listfont");
	stmt *ls=0;
	ub2 pe_offset;

	gchar *sql = "select id, parent_id,\
  (CASE WHEN access_predicates is not null OR filter_predicates is not null \
        THEN '<span foreground=\"red\">'||id||'</span>' \
        ELSE to_char(id) END) || \
  ' <b>' || operation || '</b> ' || options || ' ' || \
  CASE WHEN access_predicates IS NOT NULL THEN ' <b>&lt;&lt;</b>'  WHEN filter_predicates IS NOT NULL THEN ' <b>&gt;&gt;</b>' END, \
  object_owner,\
  decode(operation, 'TABLE ACCESS', decode(partition_start, NULL, 'heap_pixbuf', 'partition_pixbuf'), 'INDEX', 'iot_pixbuf', 'BITMAP INDEX', 'bitmap_pixbuf', NULL) pixbuf,\
  decode(operation, 'INDEX', object_name || decode(search_columns, NULL, NULL, ' (' || search_columns || ')'), \
         'BITMAP INDEX', object_name || ' (' || search_columns || ')', object_name),\
  case when cost > 1000000 then to_char(round(cost/1000000,0)) || 'M' when cost >1000 then to_char(round(cost/1000,0))||'K' else to_char(cost) end cost,\
  cardinality,\
  case when bytes > 1000000 then to_char(round(bytes/1000000,0)) || 'M' when bytes >1000 then to_char(round(bytes/1000,0))||'K' else to_char(bytes) end bytes,\
  case when cpu_cost > 1000000 then to_char(round(cpu_cost/1000000,0)) || 'M' when cpu_cost >1000 then to_char(round(cpu_cost/1000,0))||'K' else to_char(cpu_cost) end cpu_cost,\
  case when io_cost > 1000000 then to_char(round(io_cost/1000000,0)) || 'M' when io_cost >1000 then to_char(round(io_cost/1000,0))||'K' else to_char(io_cost) end io_cost,\
  case when temp_space > 1000000 then to_char(round(temp_space/1000000,0)) || 'M' when temp_space >1000 then to_char(round(temp_space/1000,0))||'K' else to_char(temp_space) end temp_space,\
  partition_start,partition_stop,distribution,\
  decode(other_tag, 'PARALLEL_TO_SERIAL', 'P->S',\
         'PARALLEL_FROM_SERIAL', 'S->P', \
         'PARALLEL_TO_PARALLEL', 'P->P',\
         'PARALLEL_COMBINED_WITH_CHILD', 'PCWC', \
         'PARALLEL_COMBINED_WITH_PARENT', 'PCWP', other_tag), \
  decode(optimizer, 'CHOOSE', 0, 'ALL_ROWS', 1, 'FIRST_ROWS', 2, 'FIRST_ROWS_1', 3, 'FIRST_ROWS_10', 4, 'FIRST_ROWS_100', 5, 'FIRST_ROWS_1000', 6, -1),\
  case when operation like 'TABLE%' then 1 when operation like '%INDEX%' then 2 else 0 end\
  from plan_table\
  connect by prior id = parent_id and prior statement_id = statement_id\
  start with id = 0 and statement_id = 'gtko' order by id";

	gchar *awrsql = "select id, parent_id,\
  (CASE WHEN access_predicates is not null OR filter_predicates is not null \
        THEN '<span foreground=\"red\">'||id||'</span>' \
        ELSE to_char(id) END) || \
  ' ' || operation || ' ' || options || ' ' || \
  CASE WHEN access_predicates IS NOT NULL THEN ' <b>&lt;&lt;</b>'  WHEN filter_predicates IS NOT NULL THEN ' <b>&gt;&gt;</b>' END, \
  object_owner,\
  decode(operation, 'TABLE ACCESS', decode(partition_start, NULL, 'heap_pixbuf', 'partition_pixbuf'), 'INDEX', 'iot_pixbuf', 'BITMAP INDEX', 'bitmap_pixbuf', NULL) pixbuf,\
  decode(operation, 'INDEX', object_name || decode(search_columns, NULL, NULL, ' (' || search_columns || ')'), \
         'BITMAP INDEX', object_name || ' (' || search_columns || ')', object_name),\
  case when cost > 1000000 then to_char(round(cost/1000000,0)) || 'M' when cost >1000 then to_char(round(cost/1000,0))||'K' else to_char(cost) end cost,\
  cardinality,\
  case when bytes > 1000000 then to_char(round(bytes/1000000,0)) || 'M' when bytes >1000 then to_char(round(bytes/1000,0))||'K' else to_char(bytes) end bytes,\
  case when cpu_cost > 1000000 then to_char(round(cpu_cost/1000000,0)) || 'M' when cpu_cost >1000 then to_char(round(cpu_cost/1000,0))||'K' else to_char(cpu_cost) end cpu_cost,\
  case when io_cost > 1000000 then to_char(round(io_cost/1000000,0)) || 'M' when io_cost >1000 then to_char(round(io_cost/1000,0))||'K' else to_char(io_cost) end io_cost,\
  case when temp_space > 1000000 then to_char(round(temp_space/1000000,0)) || 'M' when temp_space >1000 then to_char(round(temp_space/1000,0))||'K' else to_char(temp_space) end temp_space,\
  partition_start,partition_stop,distribution,\
  decode(other_tag, 'PARALLEL_TO_SERIAL', 'P->S',\
         'PARALLEL_FROM_SERIAL', 'S->P', \
         'PARALLEL_TO_PARALLEL', 'P->P',\
         'PARALLEL_COMBINED_WITH_CHILD', 'PCWC', \
         'PARALLEL_COMBINED_WITH_PARENT', 'PCWP', other_tag), \
  decode(optimizer, 'CHOOSE', 0, 'ALL_ROWS', 1, 'FIRST_ROWS', 2, 'FIRST_ROWS_1', 3, 'FIRST_ROWS_10', 4, 'FIRST_ROWS_100', 5, 'FIRST_ROWS_1000', 6, -1),\
  case when operation like 'TABLE%' then 1 when operation like '%INDEX%' then 2 else 0 end\
  from dba_hist_sql_plan\
  connect by prior id = parent_id and prior sql_id = sql_id start with id = 0 and sql_id = :p1 order by id";

	gchar *vsql = "select id, parent_id,\
  (CASE WHEN access_predicates is not null OR filter_predicates is not null \
        THEN '<span foreground=\"red\">'||id||'</span>' \
        ELSE to_char(id) END) || \
  ' ' || operation || ' ' || options || ' ' || \
  CASE WHEN access_predicates IS NOT NULL THEN ' <b>&lt;&lt;</b>'  WHEN filter_predicates IS NOT NULL THEN ' <b>&gt;&gt;</b>' END operation, \
  object_owner,\
  decode(operation, 'TABLE ACCESS', decode(partition_start, NULL, 'heap_pixbuf', 'partition_pixbuf'), 'INDEX', 'iot_pixbuf', NULL) pixbuf,\
  decode(operation, 'INDEX', object_name || decode(search_columns, NULL, NULL, ' (' || search_columns || ')'), \
         'BITMAP INDEX', object_name || ' (' || search_columns || ')', object_name) object,\
  case when cost > 1000000 then to_char(round(cost/1000000,0)) || 'M' when cost >1000 then to_char(round(cost/1000,0))||'K' else to_char(cost) end cost,\
  cardinality,\
  case when bytes > 1000000 then to_char(round(bytes/1000000,0)) || 'M' when bytes >1000 then to_char(round(bytes/1000,0))||'K' else to_char(bytes) end bytes,\
  case when cpu_cost > 1000000 then to_char(round(cpu_cost/1000000,0)) || 'M' when cpu_cost >1000 then to_char(round(cpu_cost/1000,0))||'K' else to_char(cpu_cost) end cpu_cost,\
  case when io_cost > 1000000 then to_char(round(io_cost/1000000,0)) || 'M' when io_cost >1000 then to_char(round(io_cost/1000,0))||'K' else to_char(io_cost) end io_cost,\
  case when temp_space > 1000000 then to_char(round(temp_space/1000000,0)) || 'M' when temp_space >1000 then to_char(round(temp_space/1000,0))||'K' else to_char(temp_space) end temp_space,\
  partition_start,partition_stop,distribution,\
  decode(other_tag, 'PARALLEL_TO_SERIAL', 'P->S',\
         'PARALLEL_FROM_SERIAL', 'S->P', \
         'PARALLEL_TO_PARALLEL', 'P->P',\
         'PARALLEL_COMBINED_WITH_CHILD', 'PCWC', \
         'PARALLEL_COMBINED_WITH_PARENT', 'PCWP', other_tag) parallel, \
  decode(optimizer, 'CHOOSE', 0, 'ALL_ROWS', 1, 'FIRST_ROWS', 2, 'FIRST_ROWS_1', 3, 'FIRST_ROWS_10', 4, 'FIRST_ROWS_100', 5, 'FIRST_ROWS_1000', 6, -1) mde,\
  case when operation like 'TABLE%' then 1 when operation like '%INDEX%' then 2 else 0 end typ\
  from v$sql_plan\
  where address=hextoraw(:p1) and hash_value=:p2 and child_number=:p3 order by id";

	gchar *statsql = "select id, parent_id, \
       (CASE WHEN access_predicates is not null OR filter_predicates is not null \
       THEN '<span foreground=\"red\">'||id||'</span>' ELSE to_char(id) END) || \
       (CASE WHEN options is NULL THEN ' '||operation ELSE ' '||operation||' ('|| options ||')' END),\
       object_owner, object_name, \
       decode(operation,'TABLE ACCESS','blue4','INDEX','green4','BITMAP INDEX','green4','black') \"foreground\",\
       cost,cardinality,bytes,cpu_cost,io_cost,temp_space,partition_start, partition_stop, distribution,\
       decode(other_tag, 'PARALLEL_TO_SERIAL', 'P->S', 'PARALLEL_FROM_SERIAL', 'S->P', \
         'PARALLEL_TO_PARALLEL', 'P->P', 'PARALLEL_COMBINED_WITH_CHILD', 'PCWC', \
         'PARALLEL_COMBINED_WITH_PARENT', 'PCWP', other_tag) \
       from stats$sql_plan\
       connect by prior id = parent_id and prior\
       plan_hash_value=plan_hash_value start with id = 0 and plan_hash_value = :p1 order by id";

	gchar *predsql = "select id, 'red3' \"foreground\", access_predicates \"Access Predicates\",\
       filter_predicates \"Filter Predicates\" from plan_table \
       where (access_predicates is not null or filter_predicates is not null) and statement_id='gtko' order by id";

	gchar *delsql = "DELETE FROM PLAN_TABLE WHERE statement_id = 'gtko'";

	clear_gtktreeview(GTK_TREE_VIEW(lw("planTree")));
	planStore=NULL;

	setLabel("objectLabel", " ");

	if (-1 == source) {
		GtkTextIter begin, end;
		GtkTextBuffer *buffer;

		buffer = getCurrentSqlBuffer(-1);
		gtk_text_buffer_get_bounds(buffer, &begin, &end);
		gtk_text_buffer_remove_tag_by_name(buffer, "error",
						   &begin, &end);
		gtk_text_buffer_remove_tag_by_name(buffer, "red",
						   &begin, &end);

		execute_sql(delsql, "resList", NULL);
		OCITransCommit(svchp, errhp, 0);

		stmtlist = get_statements(getCurrentTextView());
		if (!stmtlist) {
			return;
		}
 
		ls = (stmt *) g_slist_nth_data(stmtlist, 0);
		if( !ls ) {
#if DEBUG
			fprintf(stderr, "%s:%d explain.c: no statements found\n",
				__FILE__, __LINE__);
#endif
			return;
		}

		if( !ls->sql ) {
#if DEBUG
			fprintf(stderr, "%s:%d explain.c: no sql text found\n",
				__FILE__, __LINE__);
#endif
			return;
		}


		exsql = g_strconcat("EXPLAIN PLAN SET STATEMENT_ID='gtko' INTO PLAN_TABLE FOR ",
				    (gchar *) ls->sql, NULL );

		OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, 0, NULL);
		ociPrepare(sth, errhp, exsql);

		if(ociExecute(svchp, sth, errhp, 1) != OCI_SUCCESS) {
			sword errcode=0;
			gchar errbuf[1024], errstr[1024];

			OCIErrorGet((dvoid *)errhp, (ub4)1, (text *)NULL, &errcode, 
				    (text *)errbuf, (ub4) sizeof(errbuf), (ub4)OCI_HTYPE_ERROR);
			g_strchomp(errbuf);
			g_snprintf(errstr, sizeof(errstr), "Explain Plan: %s", errbuf);
			gtk_label_set_text(GTK_LABEL(lw("statusLabel")), errstr);
      
			if(ls->offset >= 0) {
				query_mark_error(sth, errhp, getCurrentSqlBuffer(-1),
						 ls->offset - 59, strlen(ls->sql));
 
			}
 
			OCIHandleFree(sth, OCI_HTYPE_STMT);
			return;
		}

		//OCITransCommit(svchp, errhp, 0);
		OCIHandleFree(sth, OCI_HTYPE_STMT);

		for(t_stmtlist = stmtlist; t_stmtlist; t_stmtlist=g_slist_next(t_stmtlist) ) {
			stmt  *sn = (stmt *) t_stmtlist->data;

			g_free(sn->sql);
			g_free(sn);
		}
		g_slist_free(stmtlist);

		execute_sql(predsql, "planPredicateTree", NULL);
	}

 
	if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("explainXplanCheck"))) ) {

		execute_sql("select * from table(dbms_xplan.display)", "planTree", NULL);
		return;
	}

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Operation", renderer, "markup", 0, NULL);
	gtk_tree_view_append_column(list, column);
  
	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Owner", renderer, "text", 1, NULL);
	gtk_tree_view_append_column(list, column);
  

	/* OBJECT NAME */
	column = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column, "Object");
	gtk_tree_view_append_column(list, column);

	pbf_renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column, pbf_renderer, FALSE);
	gtk_tree_view_column_add_attribute(column, pbf_renderer, "pixbuf", 2);   

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	gtk_tree_view_column_pack_start(column, renderer, TRUE);
	gtk_tree_view_column_add_attribute(column, renderer, "markup", 3);



	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Rows", renderer, "text", 4, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Cost", renderer, "text", 5, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Bytes", renderer, "text", 6, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("CPU Cost", renderer, "text", 7, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("IO Cost", renderer, "text", 8, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Temp", renderer, "text", 9, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Pstart", renderer, "text", 10, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Pstop", renderer, "text", 11, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Distrib", renderer, "text", 12, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Other", renderer, "text", 13, NULL);
	gtk_tree_view_append_column(list, column);

	renderer = gtk_cell_renderer_text_new();
	if(font)
		g_object_set(renderer, "font", font, NULL);
	column = gtk_tree_view_column_new_with_attributes("Type", renderer, "text", 14, NULL);
	gtk_tree_view_append_column(list, column);

	planStore = gtk_tree_store_new(15, 
				       G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,
				       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
				       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model(list, GTK_TREE_MODEL(planStore) );
  


	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, 0, NULL);
	if(source == -1) {
		OCIStmtPrepare(sth, errhp, (text *)sql, (ub4)strlen(sql), OCI_NTV_SYNTAX, OCI_DEFAULT);
	}
	else {

		switch(source) {
		case 0:
		case 3:
			ociPrepare(sth, errhp, vsql);
			ociBindString(sth, errhp, 1, address);
			ociBindString(sth, errhp, 2, hash);
			ociBindString(sth, errhp, 3, child);
			break;
		case 1:
		default:
			ociPrepare(sth, errhp, statsql);
			ociBindString(sth, errhp, 1, hash);
			break;
		case 2:
			ociPrepare(sth, errhp, awrsql);
			ociBindString(sth, errhp, 1, hash);
			break;
		}
	}
	OCIDefineByPos(sth, &defnp, errhp, 1, &id, sizeof(id), SQLT_INT, &id_ind, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 2, &pid, sizeof(pid), SQLT_INT, &pid_ind, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 3, op, sizeof(op)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 4, owner, sizeof(owner)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 5, object_pbf, sizeof(object_pbf)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 6, object, sizeof(object)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 7, card, sizeof(card)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 8, cost, sizeof(cost)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 9, bytes, sizeof(bytes)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 10, cpu_cost, sizeof(cpu_cost)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 11, io_cost, sizeof(io_cost)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 12, temp, sizeof(temp)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 13, pstart, sizeof(pstart)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 14, pstop, sizeof(pstop)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 15, distribution, sizeof(distribution)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 16, other_tag, sizeof(other_tag)+1, SQLT_STR, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 17, &optimizer_mode, sizeof(optimizer_mode), SQLT_INT, &indp, 0, 0, OCI_DEFAULT);
	OCIDefineByPos(sth, &defnp, errhp, 18, objtype, sizeof(objtype)+1, SQLT_STR, 0, 0, 0, OCI_DEFAULT);
	if(ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, 0, 0, NULL, NULL, OCI_DEFAULT)) != OCI_SUCCESS) return;

	while(ociErr(errhp, OCIStmtFetch2(sth, errhp, 1, OCI_FETCH_NEXT, 0, OCI_DEFAULT)) == OCI_SUCCESS) {

		if( id==0) {
			gtk_combo_box_set_active(GTK_COMBO_BOX(lw("optimizerModeCombo")), optimizer_mode);
		}

		if(pid_ind == -1)
			gtk_tree_store_append(planStore, &iter[0], NULL);
		else 
			gtk_tree_store_append(planStore, &iter[id], &iter[pid]);

		gtk_tree_store_set(planStore, &iter[id], 
				   0, op, 
				   1, owner,
				   2, query_get_pixbuf(object_pbf), 
				   3, object,
				   4, card, 
				   5, cost,
				   6, bytes,
				   7, cpu_cost, 
				   8, io_cost, 
				   9, temp,
				   10, pstart,
				   11, pstop,
				   12, distribution, 
				   13, other_tag,
				   14, objtype,
				   -1);

	}

	if( !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("explainOwnerCheck"))) )
		hideColumn("planTree", 1);

	hideColumn("planTree", 13);
 
	gtk_tree_view_expand_all(list);
	OCIHandleFree(sth, OCI_HTYPE_STMT);
}




void on_explainOwnerCheck_toggled(GtkToggleButton *togglebutton,
				  gpointer user_data)
{
	GtkTreeView *treeview = GTK_TREE_VIEW(lw("planTree"));
	gboolean visible = gtk_toggle_button_get_active(togglebutton);
  
	if (treeview) {
		gtk_tree_view_column_set_visible(gtk_tree_view_get_column(treeview, 1), visible);
	}
}

