/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include <glib.h>
#include <gtk/gtk.h>

#include "file_ops.h"
#include "gutil.h"
#include "interface.h"
#include "command_tab.h"
#include "syntax.h"
#include "appconfig.h"


static   GtkWidget *rsdelimCombo=0, *rsquoteCheck=0, *rsoverwriteCheck=0;


static void open_sql_file(GtkWidget *fileDialog)
{
	GSList *flist, *fnode;
	gint file;
	GtkTextIter start;
	GtkTextBuffer *buffer;
	size_t r;


	flist = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(fileDialog) );

	for(fnode=flist; fnode != NULL; fnode = g_slist_next(fnode)) {
		gchar *bname, *dname;
		gchar buf[8196];

		file = open((gchar *)fnode->data, O_RDONLY, "r"); 
    
		if (-1 == file) {
			gchar errbuf[256];

			g_snprintf(errbuf, 
				   sizeof(errbuf),
				   "Could not open file '%s' for read: %d",
				   (gchar *)fnode->data, errno);
			gtk_label_set_text(GTK_LABEL(lw("statusLabel")), errbuf);
      
			g_slist_free(flist);
			return;
		}

		bname = g_path_get_basename((const gchar *)fnode->data);
		dname = g_path_get_dirname((const gchar *)fnode->data);

		command_tab_add(NULL);

		buffer = getCurrentSqlBuffer(-1);  

		gtk_entry_set_text(GTK_ENTRY(lw("sqlTabEntry")), bname);

		gtk_text_buffer_get_start_iter(buffer, &start);

		while( (r = read(file, buf, sizeof(buf)-1)) > 0 )
			gtk_text_buffer_insert(buffer, &start, buf, r);
    
		close(file);

		chdir(dname);

		g_free(bname);
		g_free(dname);
	}

	g_slist_free(flist);
}



static void create_fileOpenChooser (void)
{
	GtkWidget *fileChooser;
	GtkWidget *vbox;
	GtkWidget *action_area;
	GtkWidget *fileChooserCancelButton;
	GtkWidget *fileChooserOKButton;
	const gchar *sqlpath = g_getenv("SQLPATH");


	fileChooser = gtk_file_chooser_dialog_new("Open SQL File", NULL,
						  GTK_FILE_CHOOSER_ACTION_OPEN,
						  NULL, NULL);

	g_object_set(fileChooser,"select-multiple", TRUE, NULL);

	gtk_window_set_skip_taskbar_hint(GTK_WINDOW (fileChooser),
					 TRUE);
	gtk_window_set_skip_pager_hint(GTK_WINDOW (fileChooser),
				       TRUE);
	gtk_window_set_type_hint(GTK_WINDOW (fileChooser),
				 GDK_WINDOW_TYPE_HINT_DIALOG);
 
	if (sqlpath) {
		gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(fileChooser),
						     sqlpath, NULL);
	}

	vbox = GTK_DIALOG (fileChooser)->vbox;
	gtk_widget_show (vbox);

	action_area = GTK_DIALOG (fileChooser)->action_area;
	gtk_widget_show (action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area),
				   GTK_BUTTONBOX_END);

	fileChooserCancelButton = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (fileChooserCancelButton);
	gtk_dialog_add_action_widget (GTK_DIALOG (fileChooser),
				      fileChooserCancelButton, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS (fileChooserCancelButton, GTK_CAN_DEFAULT);

	fileChooserOKButton = gtk_button_new_from_stock ("gtk-open");
	gtk_widget_show(fileChooserOKButton);
	gtk_dialog_add_action_widget(GTK_DIALOG (fileChooser),
				      fileChooserOKButton, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS(fileChooserOKButton, GTK_CAN_DEFAULT);

	gtk_widget_grab_default(fileChooserOKButton);

	switch(gtk_dialog_run(GTK_DIALOG(fileChooser))) {

	case GTK_RESPONSE_OK:
		open_sql_file(fileChooser);
		break;

	default:
		break;
	}

	gtk_widget_destroy(fileChooser);
	return;
}



void open_file_dialog_pop(GtkMenuItem *menuitem,
			  gpointer user_data)
{
	create_fileOpenChooser();
	return;
}







static gint save_sql_file(const gchar *filename)
{
	gchar *sql, *bname, *dname;
	gchar filebuf[256];
	int file;


	if (!filename) {
		return -1;
	}

	file = open(filename, O_WRONLY | O_CREAT | O_TRUNC, "w");

	if (-1 == file) { 
		gchar errbuf[256];

		g_snprintf(errbuf, sizeof(errbuf),
			   "Could not open file '%s' for write: %d",
			   filename, errno);
		setLabel("statusLabel", errbuf);
		return -1;
	}

	sql = getCommandText(-1);
	write(file, sql, strlen(sql));
	close(file);
	chmod(filename, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );

	bname = g_path_get_basename((const gchar *)filename);
	dname = g_path_get_dirname((const gchar *)filename);

	gtk_entry_set_text(GTK_ENTRY(lw("sqlTabEntry")), bname);
	snprintf(filebuf, sizeof(filebuf), "File '%s' saved.", filename);
	setLabel("statusLabel", filebuf);

	chdir(dname);

	g_free(bname);
	g_free(dname);
	g_free(sql);
	return 0;
}



static void on_fileSaveChooser_ok(GtkButton *button,
				  gpointer user_data)
{
	gchar     *filename;


	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(user_data));
	if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
		GtkWidget *saveConfirmDialog;
		GtkWidget *dialog_vbox1;
		GtkWidget *label1;
		GtkWidget *dialog_action_area1;
		GtkWidget *cancelbutton1;
		GtkWidget *okbutton1;
		gchar     filemsg[512];

		saveConfirmDialog = gtk_dialog_new();
		gtk_window_set_title (GTK_WINDOW (saveConfirmDialog),
				      "Save confirmation");
		gtk_window_set_type_hint (GTK_WINDOW (saveConfirmDialog),
					  GDK_WINDOW_TYPE_HINT_DIALOG);
		gtk_window_set_position(GTK_WINDOW(saveConfirmDialog),
					GTK_WIN_POS_MOUSE);

		dialog_vbox1 = GTK_DIALOG (saveConfirmDialog)->vbox;
		gtk_widget_show (dialog_vbox1);

		g_snprintf(filemsg, sizeof(filemsg), "'%s' exists, overwrite?",
			   filename);
		label1 = gtk_label_new (filemsg);
		gtk_widget_show (label1);
		gtk_box_pack_start (GTK_BOX (dialog_vbox1), label1, FALSE,
				    FALSE, 0);

		dialog_action_area1 = GTK_DIALOG (saveConfirmDialog)->action_area;
		gtk_widget_show (dialog_action_area1);
		gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1),
					   GTK_BUTTONBOX_END);

		cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
		gtk_widget_show (cancelbutton1);
		gtk_dialog_add_action_widget (GTK_DIALOG (saveConfirmDialog),
					      cancelbutton1, GTK_RESPONSE_CANCEL);
		GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

		okbutton1 = gtk_button_new_from_stock ("gtk-ok");
		gtk_widget_show (okbutton1);
		gtk_dialog_add_action_widget (GTK_DIALOG (saveConfirmDialog),
					      okbutton1, GTK_RESPONSE_OK);
		GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

		switch(gtk_dialog_run(GTK_DIALOG(saveConfirmDialog)) ) {

		case GTK_RESPONSE_OK:

			gtk_widget_destroy(saveConfirmDialog);
      
			if( !save_sql_file(filename) )
				gtk_widget_destroy(GTK_WIDGET(user_data));

			break;

		default:
			gtk_widget_destroy(saveConfirmDialog);
			break;
		}

	}
	else {

		save_sql_file(filename);
		gtk_widget_destroy(GTK_WIDGET(user_data));
	}

	g_free(filename);
}


static void on_fileSaveChooser_cancel(GtkButton *button,
				      gpointer user_data)
{  
	gtk_widget_destroy(GTK_WIDGET(user_data));
}



static void create_fileSaveChooser (void)
{
	GtkWidget *saveChooser;
	GtkWidget *vbox;
	GtkWidget *action_area;
	GtkWidget *saveFileCancel;
	GtkWidget *saveFileButton;


	saveChooser = gtk_file_chooser_dialog_new (" ", NULL,
						   GTK_FILE_CHOOSER_ACTION_SAVE,
						   NULL, NULL);
	gtk_window_set_type_hint (GTK_WINDOW (saveChooser),
				  GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_window_set_position(GTK_WINDOW(saveChooser),
				GTK_WIN_POS_CENTER_ON_PARENT);

	vbox = GTK_DIALOG (saveChooser)->vbox;
	gtk_widget_show (vbox);

	action_area = GTK_DIALOG (saveChooser)->action_area;
	gtk_widget_show (action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area),
				   GTK_BUTTONBOX_END);

	saveFileCancel = gtk_dialog_add_button(GTK_DIALOG(saveChooser),
					       "gtk-cancel", 0);
	gtk_widget_show (saveFileCancel);
	GTK_WIDGET_SET_FLAGS (saveFileCancel, GTK_CAN_DEFAULT);

	saveFileButton = gtk_dialog_add_button(GTK_DIALOG(saveChooser),
					       "gtk-save", 0);
	gtk_widget_show (saveFileButton);
	GTK_WIDGET_SET_FLAGS (saveFileButton, GTK_CAN_DEFAULT);

	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(saveChooser),
					  gtk_label_get_text(GTK_LABEL(command_tab_get_page_label()))
		);

	gtk_widget_grab_default (saveFileButton);

	g_signal_connect((gpointer) saveFileButton, "clicked",
			 G_CALLBACK(on_fileSaveChooser_ok), 
			 (gpointer) saveChooser);
	g_signal_connect((gpointer) saveFileCancel, "clicked",
			 G_CALLBACK(on_fileSaveChooser_cancel),
			 (gpointer) saveChooser);
  
	gtk_widget_show(saveChooser);

	return;
}



void save_file_dialog_pop(GtkMenuItem *menuitem,
			  gpointer user_data)
{
	create_fileSaveChooser();
}



static gint save_query_results(gchar *treeview,
			       gchar *filename,
			       gchar delim,
			       gboolean quote)
{
	GtkTreeIter iter;
	GList *cols = NULL;
	GtkTreeModel *model = NULL;
	guint ret, ncols;
	FILE *fd;
	guint x, cwidth;
 

	cols  = gtk_tree_view_get_columns(GTK_TREE_VIEW(lw(treeview)) );
	ncols = g_list_length(cols);
	if (!ncols) {
		return -1;
	}

	model = gtk_tree_view_get_model(GTK_TREE_VIEW(lw(treeview)));
	if (!model) {
		return -2;
	}

	fd = fopen(filename, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rsoverwriteCheck)) ? "w+" : "a+");
	if(!fd) {
		setLabel("statusLabel", "Couldn't open file for writing");
		return -3;
	}

	if (delim=='h') {
		fprintf(fd, "<HTML>\n<TABLE BORDER>\n<TR>");
	}


	for (x=0; x < ncols; x++) {
		GList *cl = g_list_nth(cols, x);
		GtkTreeViewColumn *column;
		guint title_width=0;
		gchar *column_name;


		cwidth = 0;
		for (ret=gtk_tree_model_get_iter_first(model, &iter);
		     ret!=FALSE;
		     ret=gtk_tree_model_iter_next(model, &iter) ) {
			static guint width;
			gchar *value=0;

			gtk_tree_model_get(model, &iter, x, &value, -1);
			if(value) {

				width = strlen(value);

				if(width > cwidth) {
					cwidth = width;
				}

				g_free(value);
			}
		}

		column = GTK_TREE_VIEW_COLUMN(cl->data);
		column_name = g_object_get_data(G_OBJECT(column),
						"oracle_column_name");

		if(column_name) {

			title_width = strlen(column_name);
			if(title_width > cwidth)
				cwidth = title_width;

			g_object_set_data(G_OBJECT(cl->data), "cwidth",
					  GUINT_TO_POINTER(cwidth));
    
			switch(delim) {

			case ' ':
				fprintf(fd, "%-*s ", 
					cwidth<11 ? 10 : cwidth,
					column_name ? column_name : " ");
				break;

			case ',':
			case '\t':
				fprintf(fd, "%s%c",
					column_name,
					x<ncols-1 ? delim : ' ');
				break;

			case 'h':
				fprintf(fd, "<TH bgcolor=\"lemonchiffon\">%s</TH>",
					column_name);
				break;
	
			default:
				break;
			}
		}
	}
	fprintf(fd, "%s", delim=='h' ? "</TR>\n" : "\n");

	for (ret=gtk_tree_model_get_iter_first(model, &iter);
	     ret!=FALSE;
	     ret=gtk_tree_model_iter_next(model, &iter) ) {

		for (x=0; x < ncols; x++) {
			GList *cl = g_list_nth(cols, x);
			gchar *value=0;

			cwidth = GPOINTER_TO_UINT(g_object_get_data(cl->data,
								    "cwidth"));

			if (quote && delim != 'h') {
				fprintf(fd, "\"");
			}

			if (G_TYPE_STRING == gtk_tree_model_get_column_type(model, x)) {

				gtk_tree_model_get(model, &iter, x, &value, -1);

				switch(delim) {
	
				case ' ':
					fprintf(fd, "%-*s%s ",
						cwidth<11 ? 10 : cwidth,
						value ? value : " ",
						quote ? "\"" : "");
					break;

				case ',':
				case '\t':
					fprintf(fd, "%s%s%c",
						value,
						quote ? "\"" : "",
						x<ncols-1 ? delim : ' ');
					break;

				case 'h':
					fprintf(fd, "<TD>%s</TD>",
						value);
					break;

				default:
					break;
				}
			}

			g_free(value);
		}

		fprintf(fd, "%s", delim=='h' ? "</TR>\n" : "\n");
	}

	if(delim=='h') {
		fprintf(fd, "</TABLE>\n</HTML>\n");
	}

	g_list_free(cols);
	fclose(fd);
	return 0;
}


void create_resultsSaveDialog(gchar *treeview)
{
	GtkWidget *resultsChooser;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *dialog_action_area3;
	GtkWidget *resultsCancelButton;
	GtkWidget *resultsSaveButton;
	gchar *filename, *dirname;
	gboolean quote=FALSE;
	gchar delims[] = {' ', ',', '\t', 'h'};
	gchar *delim_text[] = {"Column-width formatted",
			       "CSV file",
			       "TSV file",
			       "HTML Table"};
	gint      delim_choice=0, x;


	resultsChooser = gtk_file_chooser_dialog_new ("Save Results....",
						      NULL,
						      GTK_FILE_CHOOSER_ACTION_SAVE,
						      NULL, NULL);
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (resultsChooser), TRUE);
	gtk_window_set_skip_pager_hint (GTK_WINDOW (resultsChooser), TRUE);
	gtk_window_set_type_hint (GTK_WINDOW (resultsChooser),
				  GDK_WINDOW_TYPE_HINT_DIALOG);

	vbox = GTK_DIALOG (resultsChooser)->vbox;
	gtk_widget_show (vbox);

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	label = gtk_label_new ("File Format:    ");
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

	rsdelimCombo = gtk_combo_box_new_text();
	for (x=0; x < sizeof(delims)/sizeof(delims[0]); x++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(rsdelimCombo),
					  delim_text[x]);
	}

	gtk_combo_box_set_active(GTK_COMBO_BOX(rsdelimCombo), 1);
	gtk_widget_show (rsdelimCombo);
	gtk_box_pack_start (GTK_BOX (hbox), rsdelimCombo, FALSE, FALSE, 0);


	rsquoteCheck = gtk_check_button_new_with_label("Quote column values");
	gtk_widget_show(rsquoteCheck);
	gtk_box_pack_end(GTK_BOX (hbox), rsquoteCheck, FALSE, FALSE, 0);

	rsoverwriteCheck = gtk_check_button_new_with_label("Overwrite");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rsoverwriteCheck), TRUE);
	gtk_widget_show(rsoverwriteCheck);
	gtk_box_pack_end(GTK_BOX (hbox), rsoverwriteCheck, FALSE, FALSE, 0);  

	dialog_action_area3 = GTK_DIALOG (resultsChooser)->action_area;
	gtk_widget_show (dialog_action_area3);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area3),
				   GTK_BUTTONBOX_END);

	resultsCancelButton = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (resultsCancelButton);
	gtk_dialog_add_action_widget (GTK_DIALOG (resultsChooser),
				      resultsCancelButton,
				      GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS (resultsCancelButton, GTK_CAN_DEFAULT);

	resultsSaveButton = gtk_button_new_from_stock ("gtk-save");
	gtk_widget_show (resultsSaveButton);
	gtk_dialog_add_action_widget (GTK_DIALOG (resultsChooser),
				      resultsSaveButton,
				      GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS (resultsSaveButton, GTK_CAN_DEFAULT);

	gtk_widget_grab_default (resultsSaveButton);

	switch(gtk_dialog_run (GTK_DIALOG (resultsChooser)) ) {

	case GTK_RESPONSE_OK:

		delim_choice = gtk_combo_box_get_active(GTK_COMBO_BOX(rsdelimCombo));

		quote = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rsquoteCheck));

		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(resultsChooser));
		if (!filename) {
			return;
		}

		if (!save_query_results(treeview, filename, delims[delim_choice], quote)) {
			gtk_widget_destroy(resultsChooser);
		}

		dirname = g_path_get_dirname((const gchar *)filename);
		chdir(dirname);
		g_free(dirname);
		break;

	default:
		gtk_widget_destroy(resultsChooser);

		break;
	}

	return;
}




void result_to_external_app(guint app)
{
	const gchar *ext_viewer = config_get("external_viewer");
	gchar *exe, *filename;
	gchar argbuf[256], msg[512], fbuf[512];
	GError *gerror=0;
	gint x;


	if (!ext_viewer) {
		return;
	}

	if ((exe = g_find_program_in_path(ext_viewer)) == NULL) {
		g_snprintf(msg, sizeof(msg),
			   "<b>Error: could not find '%s' in path</b>",
			   ext_viewer);
		gtk_label_set_markup(GTK_LABEL(lw("statusLabel")),
				     msg);
		return;
	}


	g_snprintf(fbuf, sizeof(fbuf), "%s.csv",
		   gtk_label_get_text(GTK_LABEL(command_tab_get_page_label())) );
	filename = g_build_filename(g_get_home_dir(), fbuf, NULL);

	for (x=1; g_file_test(filename, G_FILE_TEST_EXISTS); x++) {

		g_free(filename);
		g_snprintf(fbuf, sizeof(fbuf), "%s(%d).csv",
			   gtk_label_get_text(GTK_LABEL(command_tab_get_page_label())), x);
		filename = g_build_filename(g_get_home_dir(), fbuf, NULL);
	}

	save_query_results("resList", filename, ',', FALSE);
	g_snprintf(argbuf, sizeof(argbuf), "%s %s", exe, filename);

	if (!g_spawn_command_line_async(argbuf, &gerror)) {
 
		g_snprintf(msg, sizeof(msg), 
			   "<span color=\"red3\">Error executing '%s %s': %s</span>",
			   exe, filename, gerror->message);
		gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), msg);
		g_error_free(gerror);
	}
	else {
		g_snprintf(msg, sizeof(msg), "Spawned external viewer <b>%s %s</b>",
			   exe, filename);
		gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), msg);
	}

	g_free(filename);
}


void create_outputSaveChooser (void)
{
	GtkWidget *saveChooser;
	GtkWidget *vbox;
	GtkWidget *action_area;
	GtkWidget *saveFileCancel;
	GtkWidget *saveFileButton;
	gchar     *filename, *basename, suggested[256];
	GtkTooltips *tip = gtk_tooltips_new();


	saveChooser = gtk_file_chooser_dialog_new("Select query output CSV file...",
						   NULL, GTK_FILE_CHOOSER_ACTION_SAVE,
						  NULL, NULL);
	gtk_window_set_type_hint (GTK_WINDOW (saveChooser),
				  GDK_WINDOW_TYPE_HINT_DIALOG);

	vbox = GTK_DIALOG (saveChooser)->vbox;
	gtk_widget_show (vbox);

	action_area = GTK_DIALOG (saveChooser)->action_area;
	gtk_widget_show (action_area);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area),
				   GTK_BUTTONBOX_END);

	saveFileCancel = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (saveFileCancel);
	gtk_dialog_add_action_widget (GTK_DIALOG (saveChooser),
				      saveFileCancel, GTK_RESPONSE_CANCEL);
	GTK_WIDGET_SET_FLAGS (saveFileCancel, GTK_CAN_DEFAULT);

	saveFileButton = gtk_button_new_from_stock ("gtk-apply");
	gtk_widget_show (saveFileButton);
	gtk_dialog_add_action_widget (GTK_DIALOG (saveChooser),
				      saveFileButton, GTK_RESPONSE_OK);
	GTK_WIDGET_SET_FLAGS (saveFileButton, GTK_CAN_DEFAULT);

	g_snprintf(suggested, sizeof(suggested), "%s.csv",
		   gtk_label_get_text(GTK_LABEL(command_tab_get_page_label())) );
	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(saveChooser), suggested);

	gtk_widget_grab_default (saveFileButton);

	switch(gtk_dialog_run (GTK_DIALOG (saveChooser)) ) {

	case GTK_RESPONSE_OK:

		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(saveChooser) );

		if (filename) {
  
			basename = g_path_get_basename(filename);
			
			if(basename) {
				gtk_label_set_text(GTK_LABEL(lw("outputFileLabel")), basename);
			}
			g_object_set_data_full(G_OBJECT(lw("outputFileButton")),
					       "filename", (gpointer) filename, g_free);
			g_free(basename);

			gtk_tooltips_set_tip(tip, lw("outputFileButton"), filename, NULL);
			gtk_widget_destroy(saveChooser);
		}
		break;

	default:
		gtk_widget_destroy(saveChooser);
		break;
	}

	return;
}
