/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <oci.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "gutil.h"
#include "orautil.h"
#include "dbms_connection.h"
#include "finder.h"

#define COMPMODEL_SIZE 200

extern OCIEnv *envhp;
extern OCIError *errhp;
extern GtkTreeModel *completion_model;
extern gboolean polling;

static gchar object_name[COMPMODEL_SIZE][48];
static gint object_type[COMPMODEL_SIZE];

static OCISvcCtx *fsvchp;
static OCIServer *fsvrhp;



static gboolean compmodel_fetch_timeout(gpointer data)
{
	OCIStmt *sth = (OCIStmt *)data;
	sword ret = OCI_SUCCESS;
	gchar label[256];
	gint nrows=0, x;
	ub4 sizep=sizeof(ub4);
	GtkTreeIter iter;


	ret = OCIStmtFetch(sth, errhp, (ub4)COMPMODEL_SIZE,
			   (ub2)OCI_FETCH_NEXT, (ub4)OCI_DEFAULT);

	if (OCI_STILL_EXECUTING == ret) {
		g_timeout_add(200, compmodel_fetch_timeout, (gpointer) sth);
		return FALSE;
	}

	while (OCI_SUCCESS == ret) {

		OCIAttrGet((dvoid *) sth, (ub4) OCI_HTYPE_STMT,
			   (dvoid *) &nrows, (ub4 *) &sizep,
			   (ub4)OCI_ATTR_ROWS_FETCHED, errhp);

		for (x=0; x<nrows; x++) {
			gtk_list_store_append(GTK_LIST_STORE(completion_model),
					      &iter);
			gtk_list_store_set(GTK_LIST_STORE(completion_model),
					   &iter, 
					   0, object_type[x], 
					   1, object_name[x], -1);
		}

		ret = OCIStmtFetch(sth, errhp, (ub4)COMPMODEL_SIZE,
				   (ub2)OCI_FETCH_NEXT, (ub4)OCI_DEFAULT);
	}

	if (OCI_STILL_EXECUTING == ret) {

		OCIAttrGet((dvoid *)sth, (ub4) OCI_HTYPE_STMT,
			   (dvoid *) &nrows, (ub4 *) &sizep,
			   (ub4)OCI_ATTR_ROW_COUNT, errhp);
		g_snprintf(label, sizeof(label), "Loaded <b>%d</b> objects into finder",
			   nrows);
		gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), label);

		g_timeout_add(100, compmodel_fetch_timeout, (gpointer) sth);
		return FALSE;
	}


	OCIAttrGet((dvoid *) sth, (ub4) OCI_HTYPE_STMT, (dvoid *) &nrows,
		   (ub4 *) &sizep, (ub4)OCI_ATTR_ROWS_FETCHED, errhp);

	for (x=0; x<nrows; x++) {
		gtk_list_store_append(GTK_LIST_STORE(completion_model),
				      &iter);
		gtk_list_store_set(GTK_LIST_STORE(completion_model),
				   &iter, 0, object_type[x],
				   1, object_name[x], -1);
	}

	OCIAttrGet((dvoid *)sth, (ub4) OCI_HTYPE_STMT, (dvoid *) &nrows,
		   (ub4 *) &sizep, (ub4)OCI_ATTR_ROW_COUNT, errhp);
	g_snprintf(label, sizeof(label), "Finished loading <b>%d</b> objects into finder",
		   nrows);
	gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), label);

	OCIHandleFree(sth, OCI_HTYPE_STMT);
	OCILogoff(fsvchp, errhp);
	return FALSE;
}


static gboolean compmodel_timeout(gpointer data)
{
	OCIStmt *sth = (OCIStmt *) data;
	sword ret = OCI_SUCCESS;


	ret = OCIStmtExecute(fsvchp, sth, errhp, (ub4)0,
			     (ub4)0, (OCISnapshot *)0,
			     (OCISnapshot *)0, (ub4)OCI_DEFAULT);

	if (OCI_STILL_EXECUTING == ret) {
		g_timeout_add(200, compmodel_timeout, (gpointer) sth);
		return FALSE;
	}

	if (OCI_SUCCESS == ret) {
		g_timeout_add(10, compmodel_fetch_timeout, (gpointer) sth);
		return FALSE;
	}

	return FALSE;
}



void create_completion_model(const gchar *schema_name)
{
	OCIStmt *sth = (OCIStmt *)0;
	OCIBind *bhp = (OCIBind *)0;
	OCIDefine *defnp = (OCIDefine *)0;
	sword ret = OCI_SUCCESS;
	gchar *username, *password, *service;
	const gchar *sql = "\
  select \
   decode(object_type,'TABLE',1, 'VIEW',2, 'SYNONYM',3, 'FUNCTION',4, 'PROCEDURE',5, 'PACKAGE',6, 0), \
   decode(owner, :p1, object_name, owner || '.' || object_name) \
  from all_objects \
  where (owner=:p1 and object_type IN ('TABLE','VIEW','SYNONYM','FUNCTION','PROCEDURE','PACKAGE') ) \
    or (owner='SYS' and object_type IN ('TABLE','VIEW','SYNONYM') \
       and (object_name like 'USER_%' or object_name like 'DBA_%' or object_name like 'V$%' or object_name like 'ALL_%') ) \
    or (owner='SYS' and object_type IN ('PACKAGE'))\
  order by object_name";

	username = connection_get_user();
	password = connection_get_pass();
	service  = connection_get_service();

	ret = OCILogon(envhp, errhp, &fsvchp, 
		       (CONST OraText *)username, strlen(username), 
		       (CONST OraText *)password, strlen(password), 
		       (CONST OraText *)service, strlen(service)
		);
	if (OCI_SUCCESS != ret) {
		return;
	}

	gtk_list_store_clear(GTK_LIST_STORE(completion_model));

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&sth,
		       (ub4)OCI_HTYPE_STMT, (size_t)0,
		       (dvoid **)0);
	ret = ociErr(errhp, OCIStmtPrepare(sth, errhp, (text *)sql,
					   strlen(sql), OCI_NTV_SYNTAX,
					   OCI_DEFAULT));
	if (ret != OCI_SUCCESS && ret != OCI_STILL_EXECUTING) {
		OCIHandleFree(sth, OCI_HTYPE_STMT);
		return;
	}

	OCIBindByName((OCIStmt *)sth, 
		      (OCIBind **)&bhp, 
		      (OCIError *)errhp, 
		      (text *)":p1", (sb4)strlen(":p1"), 
		      (dvoid *)schema_name, (sb4)strlen(schema_name)+1, 
		      (ub2)SQLT_STR, 
		      (dvoid *)0, (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0, 
		      (ub4)OCI_DEFAULT);

	OCIDefineByPos((OCIStmt *) sth,
		       (OCIDefine **) &defnp,
		       (OCIError *)errhp, 
		       (ub4) 1,
		       (dvoid *)&object_type, sizeof(object_type[0]), 
		       (ub2)SQLT_INT, 
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);
	OCIDefineByPos((OCIStmt *) sth,
		       (OCIDefine **) &defnp,
		       (OCIError *)errhp, 
		       (ub4) 2,
		       (dvoid *)&object_name, sizeof(object_name[0]), 
		       (ub2)SQLT_STR, 
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);

	OCIAttrGet((dvoid *)fsvchp, (ub4)OCI_HTYPE_SVCCTX, (dvoid *)&fsvrhp,
		   (ub4 *)0, (ub4)OCI_ATTR_SERVER, errhp);
	OCIAttrSet((dvoid *)fsvrhp, (ub4)OCI_HTYPE_SERVER, (dvoid *)TRUE,
		   (ub4)0, (ub4)OCI_ATTR_NONBLOCKING_MODE, errhp);

	ret = ociErr(errhp, OCIStmtExecute(fsvchp, sth, errhp, (ub4)0,
					   (ub4)0, (OCISnapshot *)0,
					   (OCISnapshot *)0, (ub4)OCI_DEFAULT));
	if (ret == OCI_STILL_EXECUTING || ret == OCI_SUCCESS) {
		g_timeout_add(50, compmodel_timeout, (gpointer) sth);
		return;
	}
  

}



gboolean fill_completion_model_timeout(gpointer data)
{
	create_completion_model((gchar *)data);
	return FALSE;
}

