/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <glib.h>

#include "gutil.h"
#include "interface.h"
#include "syntax.h"


extern GtkWidget       *window1;
extern GtkWidget       *schemaCombo;


const gchar  *get_schema(void) 
{
	static gchar    *current_schema=0, username[128];
	GtkTreeIter     iter;

	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(schemaCombo), &iter);
	gtk_tree_model_get(gtk_combo_box_get_model(GTK_COMBO_BOX(schemaCombo)),
			   &iter, 0, &current_schema, -1);

	if(current_schema) {
		g_snprintf(username, sizeof(username), "%s", current_schema);
		g_free(current_schema);
		return(username);
	}
	else {
		g_snprintf(username, sizeof(username), "(unknown)" );
		return(username);
	}

}


void gtk_widget_set_font(GtkWidget *widget,
			 const gchar *font)
{
	PangoFontDescription  *font_desc;

	if(font) {
		font_desc = pango_font_description_from_string(font);
		gtk_widget_modify_font(widget, font_desc);
		pango_font_description_free(font_desc);
	}


}



void set_gtktreeview_column_names(GtkTreeView *treeview,
				  ...)
{
	va_list       ap;
	gint          x=0;
	gchar         *colname;


	va_start(ap, treeview);

	while ((colname = va_arg(ap, gchar *)) != NULL) {
		GtkTreeViewColumn   *col;

		col = gtk_tree_view_get_column(treeview, x+4);
		if(g_ascii_strncasecmp(colname, "skip", 4)) {
			gtk_tree_view_column_set_title(col, colname);
		}
		x++;
	}
	va_end(ap);
}



void hide_gtktreeview_column(GtkTreeView *treeview,
			     gint colnum)
{
	GList      *cols=0;

	if (!treeview) {
		return;
	}

	cols = gtk_tree_view_get_columns(treeview);
	if (!cols) {
		return;
	}

	if (g_list_length(cols) > colnum) {
		gtk_tree_view_column_set_visible(gtk_tree_view_get_column(treeview, colnum), FALSE);
	}
	g_list_free(cols);
}


void expand_at_iter(GtkTreeView *treeview,
		    GtkTreeIter *iter)
{
	GtkTreeModel  *model=0;
	GtkTreePath   *tpath=0;

	model = gtk_tree_view_get_model(treeview);
	if (!model) {
		return;
	}

	if ((tpath=gtk_tree_model_get_path(model, iter)) != NULL ) {
		gtk_tree_view_expand_row(treeview, tpath, FALSE);
		gtk_tree_path_free(tpath);
	}
}



void hideColumn(gchar *view, gint position) 
{
	GList *cols = gtk_tree_view_get_columns(GTK_TREE_VIEW(lw(view)) );

	if (!cols) {
		return;
	}

	if (g_list_length(cols) > position) {
		gtk_tree_view_column_set_visible(gtk_tree_view_get_column(GTK_TREE_VIEW(lw(view)), position), FALSE);
	}
	g_list_free(cols);
}




GtkWidget *lw(gchar *widgetname) 
{
	return(lookup_widget(window1, widgetname));
}




void setLabel(gchar *label, gchar *value) 
{
	gtk_label_set_text(GTK_LABEL(lookup_widget(window1, label)), value);
}




void delcolumn(gpointer *data, gpointer *ud)
{
	gtk_tree_view_remove_column((GtkTreeView *)ud, (GtkTreeViewColumn *)data);
}



void clear_gtktextbuffer(GtkTextBuffer *buffer)
{
	GtkTextIter        iter_start, iter_end;


	if( !buffer ) {
		return;
	}

	gtk_text_buffer_get_bounds(buffer, &iter_start, &iter_end);
	gtk_text_buffer_delete(buffer, &iter_start, &iter_end);
}





void clear_gtktreeview(GtkTreeView *treeview) 
{
	GList          *cols; 
	GtkTreeStore   *store; 

	if (!treeview) {
		return;
	}

	store  = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
	if (store) {
		gtk_tree_store_clear(GTK_TREE_STORE(store));
	}

	cols = gtk_tree_view_get_columns(treeview);
	g_list_foreach(cols, (GFunc) delcolumn, (gpointer) treeview);
	g_list_free(cols);
}



void clearList(GtkTreeView *list) 
{
	GList          *columns = gtk_tree_view_get_columns(list);

	g_list_foreach(columns, (GFunc) delcolumn, (gpointer) list);
	g_list_free(columns);
}


GtkWidget *get_current_page() 
{

	return(gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw("sqlNotebook")),
					 gtk_notebook_get_current_page(GTK_NOTEBOOK(lw("sqlNotebook")))
		       )
		);
}


GtkTextView *getCurrentTextView() 
{
	return(GTK_TEXT_VIEW(gtk_bin_get_child(GTK_BIN(get_current_page() ))));
}


GtkTextBuffer *getCurrentSqlBuffer(int pagenum) 
{
	GtkWidget      *scrolledWindow;
  
	if(pagenum == -1) {
		pagenum = gtk_notebook_get_current_page(GTK_NOTEBOOK(lw("sqlNotebook")));
	}
	scrolledWindow = gtk_notebook_get_nth_page(GTK_NOTEBOOK(lw( "sqlNotebook")), pagenum);
	return(gtk_text_view_get_buffer(GTK_TEXT_VIEW(gtk_bin_get_child(GTK_BIN(scrolledWindow)))));
}


void text_buffer_clear() 
{
	GtkTextIter      start, end;
	GtkTextBuffer    *buffer = getCurrentSqlBuffer(-1);
	GSList           *blist;

	gtk_text_buffer_get_bounds(buffer, &start, &end);
	gtk_text_buffer_delete(buffer, &start, &end);
  
	blist = g_object_get_data(G_OBJECT(getCurrentTextView()), "bindlist");
	if (blist) {
		g_slist_free(blist);
		g_object_set_data(G_OBJECT(getCurrentTextView()), "bindlist", (gpointer *)NULL);
	}
}



char *getCommandText(int pagenum) 
{
	GtkTextIter     start, end;
	GtkTextBuffer   *buffer;

	buffer = getCurrentSqlBuffer(pagenum);
	gtk_text_buffer_get_bounds(buffer, &start, &end);

	return(gtk_text_buffer_get_text(buffer, &start, &end, FALSE));
}




gchar *getComboSelected(char *combo) 
{
	GtkTreeIter        iter;
	char               *value;

	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(lw(combo)), &iter);
	gtk_tree_model_get(gtk_combo_box_get_model(GTK_COMBO_BOX(lw( combo))),
			   &iter, 0, &value, -1);
	return(value);
}



gchar *getSelected(char *treeview, int column) 
{
	GtkTreeSelection   *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw( treeview)));
	GtkTreeIter        iter;
	GtkTreeModel       *model;
	gchar              *value=0;

	if (!sel) {
		return(NULL);
	}

	if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
		gtk_tree_model_get(model, &iter, column, &value, -1);
		return(value);
	}
	else {
		return((gchar *)NULL);
	}
}



gboolean gtk_treeview_get_value_gint(const gchar *treeview,
				     int column,
				     gint *value) 
{
	GtkTreeSelection   *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw((gchar *)treeview)));
	GtkTreeIter        iter;
	GtkTreeModel       *model;
	gchar              *val=0;

	if (!sel) {
		return FALSE;
	}

	if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
		gtk_tree_model_get(model, &iter, column, &val, -1);
		if(val) {
			*value = atoi(val);
			return TRUE;
		}
		else {
			return FALSE;
		}
	}
	else {
		return FALSE;
	}
}


gboolean gtk_treeview_get_value_gint64(const gchar *treeview,
				       int column,
				       gint64 *value) 
{
	GtkTreeSelection   *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw((gchar *)treeview)));
	GtkTreeIter        iter;
	GtkTreeModel       *model;
	gchar              *val=0;

	if (!sel) {
		return FALSE;
	}

	if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
		gtk_tree_model_get(model, &iter, column, &val, -1);
		if(val) {
			*value = strtoll(val, NULL, 10);
			return TRUE;
		}
		else {
			return FALSE;
		}
	}
	else {
		return FALSE;
	}
}


GList *getSelectedMulti(char *treeview,
			int column) 
{
	GtkTreeSelection     *sel;
	GList                *selList=0, *retList=0;
	GtkTreeModel         *model;
	GtkTreeIter          iter;
	gchar                *value;

	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(lw(treeview)) );
	selList = gtk_tree_selection_get_selected_rows(sel, &model);

	while (selList != NULL) {
		gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)selList->data);
		gtk_tree_model_get(model, &iter, column, &value, -1);
		retList = g_list_append(retList, value);
		selList = g_list_next(selList);
	}
	g_list_free(selList);
	return(retList);
}



GArray *g_strsplit_garray(const gchar *string,
			  const gchar *delimiter,
			  gint max_tokens)
{
	GArray *ret=0;
	gchar **toks=0;
	guint x;

	toks = g_strsplit_set(string, delimiter, max_tokens);

	ret = g_array_new(TRUE, TRUE, sizeof(gchar *));

	for (x=0; x < g_strv_length(toks); x++) {

		if (toks[x][0] != '\0') {
			gchar *val = g_strdup(toks[x]);

			ret = g_array_append_val(ret, val);
		}

	}

	g_strfreev(toks);
	return ret;
}


void g_strsplit_garray_free(GArray *array)
{
	int x;

	for (x=0; x < array->len; x++) {
		gchar *val;

		val = (gchar *) g_array_index(array, gchar *, x);
		g_free(val);
	}

	g_array_free(array, TRUE);
}

