/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <oci.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "table.h"
#include "gutil.h"
#include "orautil.h"
#include "plobject.h"
#include "query.h"

extern OCIEnv      *envhp;
extern OCIError    *errhp;
extern OCISvcCtx   *svchp;



gchar *get_index_columns(const gchar *owner,
			 const gchar *index) 
{
	OCIStmt *sth;
	OCIDefine *defnp;
	gchar column_buf[512];
	gchar *columns=0;
	gchar *sql = "with data as (\
   select  \
     i.owner index_owner,\
     c.index_name index_name,\
     c.column_name col,\
     row_number() over (partition by c.index_name order by c.column_position) rn,\
     count(*) over (partition by c.index_name) cnt\
   from all_ind_columns c JOIN all_indexes i ON i.index_name = c.index_name\
     AND i.owner = c.index_owner AND i.table_owner = c.table_owner AND i.table_name = c.table_name\
   where  i.owner=:p1 and i.index_name=:p2\
   )\
   select  ltrim(sys_connect_by_path(col,','),',') columns\
   from data\
   where rn = cnt\
   start with rn = 1\
   connect by prior index_name=index_name and prior rn = rn-1";


	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT,
		       0, (dvoid **)0);
	ociPrepare(sth, errhp, sql);
	ociBindString(sth, errhp, 1, (gchar *)owner);
	ociBindString(sth, errhp, 2, (gchar *)index);
	OCIDefineByPos((OCIStmt *) sth,
		       (OCIDefine **) &defnp,
		       (OCIError *)errhp, 
		       (ub4) 1,
		       (dvoid *)&column_buf, sizeof(column_buf)+1, 
		       (ub2)SQLT_STR, 
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);

	if (OCI_SUCCESS == ociErr(errhp, OCIStmtExecute(svchp, sth,
							errhp, 1, 0,
							NULL, NULL, OCI_DEFAULT))) {
		columns = g_strdup(column_buf);
	}

	OCIHandleFree(sth, OCI_HTYPE_STMT);
	return(columns);
}


void edit_index_stats(GtkCellRendererText *cell,
		      gchar *path_string,
		      gchar *new_text,
		      gpointer user_data) 
{
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(lw("indexList")));
	GtkTreeStore *store = GTK_TREE_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(lw("indexList"))));
	gchar *owner, *indname, c_nrows[16], c_ndist[16], c_clstfact[16];
	gint64 new_int64, nrows, ndist, clstfact;
	guint statn = GPOINTER_TO_UINT(user_data);

	new_int64 = strtoll(new_text, NULL, 10);

	gtk_tree_model_get_iter_from_string(model, &iter, path_string);
	gtk_tree_store_set(store, &iter, statn, new_int64, -1);

	gtk_tree_model_get(model, &iter, 1, &owner, 2, &indname,
			   5, &nrows, 6, &ndist, 7, &clstfact, -1);
	g_snprintf(c_nrows, sizeof(c_nrows), "%lld", nrows);
	g_snprintf(c_ndist, sizeof(c_ndist), "%lld", ndist);
	g_snprintf(c_clstfact, sizeof(c_clstfact), "%lld", clstfact);

	execute_sql("BEGIN DBMS_STATS.SET_INDEX_STATS(ownname=>:p1, indname=>:p2, \
  numrows=>:p3, numdist=>:p4, clstfct=>:p5); END;",
		    "resList", owner, indname, c_nrows, c_ndist, c_clstfact, NULL);
 
	g_free(indname);
}





void calc_index_cost(guint compress)
{

	OCIStmt            *sth;
	unsigned long int  used_bytes=0, alloc_bytes=0;
	gchar              ddl[1024], res[512], *table;
	GList              *rows;
	gchar *sql="BEGIN DBMS_SPACE.CREATE_INDEX_COST(ddl=>:p1, used_bytes=>:p2, alloc_bytes=>:p3); END;";


	if ((table = getSelected("tableList", 1)) == NULL) {
		return;
	}

	snprintf(ddl, sizeof(ddl), "CREATE INDEX xxxxxx ON %s (", table);

	rows =  getSelectedMulti("tableColumnList", 0);
	while (rows) {
		strncat(ddl, rows->data, strlen(rows->data));
		strncat(ddl, ",", 2);
		rows = g_list_next(rows);
	}
	g_list_free(rows);

	switch (compress) {
	case 0:
		snprintf(&ddl[strlen(ddl)-1], sizeof(ddl), ")");
		break;
	case 1:
		snprintf(&ddl[strlen(ddl)-1], sizeof(ddl), ") COMPRESS 1");
		break;
	case 2:
		snprintf(&ddl[strlen(ddl)-1], sizeof(ddl), ") COMPRESS 2");
		break;
	default:
		break;
	}


	fprintf(stderr, "%s\n", ddl);
 
	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT, 0, NULL);
	if( ! ociErr(errhp, ociPrepare(sth, errhp, sql))) {
		ociBindString(sth, errhp, 1, ddl);
		ociBindNumber(sth, errhp, 2, &used_bytes, sizeof(used_bytes) );
		ociBindNumber(sth, errhp, 3, &alloc_bytes, sizeof(alloc_bytes) );

		if( ! ociErr(errhp, ociExecute(svchp, sth, errhp, 1))) {
			snprintf(res, sizeof(res), "<b>Used MB:</b> %ld   <b>Alloc MB:</b> %ld",
				 used_bytes/1048576, alloc_bytes/1048576);
			gtk_label_set_markup(GTK_LABEL(lw("statusLabel")), res);
		}
	}
	OCIHandleFree(sth, OCI_HTYPE_STMT);
	g_free(table);
	return;
}
