/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "keepalive.h"
#include "interface.h"
#include "gutil.h"


static guint alive = 0;
static GtkWidget *kaWindow=0;
static GtkWidget *kaHostnameEntry=0;
static GtkWidget *kaIntervalSpin=0;

static gchar ka_host[128];
static gint ka_interval;

static gboolean exe_timeout(gpointer data)
{
	GError *gerror=0;
	GPid pid=0;
	gchar *args[5] = {"ping", "-c", "2", NULL, NULL};


	args[3] = g_strdup(ka_host);

	if (alive) {

		if(g_spawn_async_with_pipes(NULL,
					    args,
					    NULL,
					    G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
					    NULL,
					    NULL,
					    &pid,
					    NULL,
					    NULL,
					    NULL,
					    &gerror)) {
#if DEBUG
			fprintf(stderr, "%s:%d pid: %d\n",
				__FILE__, __LINE__, pid);
#endif

			return TRUE;
		}
		else {
			alive = 0;
#if DEBUG
			fprintf(stderr, "%s:%d: ping error: %s\n", 
				__FILE__, __LINE__, gerror->message);
#endif
			g_error_free(gerror);
			return FALSE;
		}
	}
	else {
		return FALSE;
	}
}



static void keepalive_start(void) 
{
	alive = 1;
	g_timeout_add(ka_interval, exe_timeout, (gpointer) NULL);
}


void keepalive_stop()
{
	alive = 0;
}


static void on_kaOKButton_clicked(GtkButton *button,
				  gpointer user_data)
{
	g_snprintf(ka_host, sizeof(ka_host), "%s", 
		   gtk_entry_get_text(GTK_ENTRY(kaHostnameEntry)) );
	ka_interval = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(kaIntervalSpin)) * 1000;

	keepalive_start( );
	gtk_widget_destroy(kaWindow);
	kaWindow = NULL;
}


static void on_kaCancelButton_clicked(GtkButton *button,
				      gpointer user_data)
{
	gtk_widget_destroy(kaWindow);
	kaWindow = NULL;
}



GtkWidget* create_ka_window (void)
{
	GtkWidget *ka_window=0;
	GtkWidget *vbox1=0;
	GtkWidget *hbox1=0;
	GtkWidget *label1=0;
	GtkWidget *hbox2=0;
	GtkWidget *label2=0;
	GtkObject *kaIntervalSpin_adj=0;
	GtkWidget *hseparator1=0;
	GtkWidget *hbox3=0;
	GtkWidget *kaOKButton=0;
	GtkWidget *kaCancelButton=0;

	ka_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (ka_window), "Send keepalive ping");
	gtk_window_set_position (GTK_WINDOW (ka_window), GTK_WIN_POS_CENTER_ON_PARENT);
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (ka_window), TRUE);
	gtk_window_set_skip_pager_hint (GTK_WINDOW (ka_window), TRUE);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (ka_window), vbox1);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

	label1 = gtk_label_new ("Hostname:");
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 2);
	gtk_label_set_width_chars (GTK_LABEL (label1), 12);

	kaHostnameEntry = gtk_entry_new ();
	gtk_widget_show (kaHostnameEntry);
	gtk_box_pack_start (GTK_BOX (hbox1), kaHostnameEntry, TRUE, TRUE, 0);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);

	label2 = gtk_label_new ("Interval:");
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (hbox2), label2, FALSE, FALSE, 2);
	gtk_label_set_width_chars (GTK_LABEL (label2), 12);

	kaIntervalSpin_adj = gtk_adjustment_new (60, 15, 6000, 15, 10, 10);
	kaIntervalSpin = gtk_spin_button_new (GTK_ADJUSTMENT (kaIntervalSpin_adj), 1, 0);
	gtk_widget_show (kaIntervalSpin);
	gtk_box_pack_start (GTK_BOX (hbox2), kaIntervalSpin, FALSE, FALSE, 0);

	hseparator1 = gtk_hseparator_new ();
	gtk_widget_show (hseparator1);
	gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, FALSE, FALSE, 2);

	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox3);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox3, FALSE, FALSE, 0);

	kaOKButton = gtk_button_new_with_mnemonic ("  OK  ");
	gtk_widget_show (kaOKButton);
	gtk_box_pack_start (GTK_BOX (hbox3), kaOKButton, TRUE, FALSE, 0);

	kaCancelButton = gtk_button_new_with_mnemonic ("Cancel");
	gtk_widget_show (kaCancelButton);
	gtk_box_pack_start (GTK_BOX (hbox3), kaCancelButton, TRUE, FALSE, 0);

	g_signal_connect ((gpointer) kaOKButton, "clicked",
			  G_CALLBACK (on_kaOKButton_clicked),
			  NULL);
	g_signal_connect ((gpointer) kaCancelButton, "clicked",
			  G_CALLBACK (on_kaCancelButton_clicked),
			  NULL);

	return ka_window;
}




void keepalive_window_show() 
{
	if (!kaWindow) {
		kaWindow = create_ka_window();
		gtk_widget_show(kaWindow);
	}
}


