#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <oci.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "orautil.h"
#include "query.h"

GString *display_attr_val(OCIEnv *envhp,
			  OCIError *errhp,
			  text *names,
			  OCITypeCode typecode,
			  dvoid *attr_value,
			  GString *val)
{
	text str_buf[200];
	text numbuf[40];
	double dnum;
	ub4 str_len;
	OCIRaw *raw = (OCIRaw *) 0;
	OCIString *vs = (OCIString *) 0;

	switch (typecode) {
		
	case OCI_TYPECODE_DATE : /* fixed length string */
		OCIDateToText(errhp, (OCIDate *) attr_value, (CONST text*)
			      "Month dd, SYYYY, HH:MI A.M.",
			      (ub1) 27, (CONST text*) "American", (ub4) 8,
			      &str_len, str_buf);
		str_buf[str_len+1] = '\0';
		val = g_string_append(val, (gchar *)str_buf);
		val = g_string_append(val, ",");
		break;

	case OCI_TYPECODE_RAW : /* RAW */
		raw = *(OCIRaw **) attr_value;
		val = g_string_append(val, (gchar *)OCIStringPtr(envhp, vs));
		val = g_string_append(val, ",");
		break;
		
	case OCI_TYPECODE_CHAR : /* fixed length string */

	case OCI_TYPECODE_VARCHAR : /* varchar */

	case OCI_TYPECODE_VARCHAR2 : /* varchar2 */
		vs = *(OCIString **) attr_value;
		val = g_string_append(val, (gchar *)OCIStringPtr(envhp, vs));
		val = g_string_append(val, ",");
		break;

	case OCI_TYPECODE_SIGNED8:
		fprintf(stderr, "attr %s = %d\n", names, *(b1 *) attr_value);
		break;

	case OCI_TYPECODE_UNSIGNED8:
		fprintf(stderr, "attr %s = %d\n", names, *(ub1 *) attr_value);
		break;

	case OCI_TYPECODE_OCTET:
		fprintf(stderr, "attr %s = %d\n", names, *(ub1 *) attr_value);
		break;

	case OCI_TYPECODE_UNSIGNED16: /* UNSIGNED SHORT */
	case OCI_TYPECODE_UNSIGNED32: /* UNSIGNED LONG */
	case OCI_TYPECODE_REAL: /* REAL */
	case OCI_TYPECODE_DOUBLE: /* DOUBLE */
	case OCI_TYPECODE_INTEGER: /* INT */
	case OCI_TYPECODE_SIGNED16: /* SHORT */
	case OCI_TYPECODE_SIGNED32: /* LONG */
	case OCI_TYPECODE_DECIMAL: /* DECIMAL */
	case OCI_TYPECODE_FLOAT: /* FLOAT */
	case OCI_TYPECODE_NUMBER: /* NUMBER */
		OCINumberToReal(errhp, (OCINumber *) attr_value, sizeof(dnum),
				(dvoid *) &dnum);
		g_snprintf((gchar *)numbuf, sizeof(numbuf), "%d", (int)dnum);
		val = g_string_append(val, (gchar *)numbuf);
		val = g_string_append(val, ",");
		break;

	default:
		val = g_string_append(val, "embedded() ");
		//fprintf(stderr, "unhandeld: %s - typecode %d\n", names, typecode);
		break;
	}
	return(val);
}



GString *type_value_string(OCIEnv *envhp,
			   OCISvcCtx *svchp,
			   OCIError *errhp,
			   colData *col)
{
	OCIInd attr_null_status;
	dvoid *attr_null_struct=0;
	dvoid *attr_value=0;
	OCIType *obj_tdo, *attr_tdo;
	ub2 numattrs=0;
	OCIDescribe *dschp = (OCIDescribe *) 0;
	OCIParam *parmh = (OCIParam *) 0;
	OCIParam *attr_iter = (OCIParam *) 0;
	OCIParam *attrhd = (OCIParam *) 0;
	guint i;
	static GString *valstring;
  

	valstring = g_string_new_len(col->typename, strlen(col->typename) );
	valstring = g_string_append(valstring, "(");

	if(*col->obj_col_ind == -1) {
		valstring = g_string_append(valstring, " )" );
		return(valstring);
	}


	if(ociErr(errhp, OCITypeByRef(envhp, errhp, col->type_ref, 
				      OCI_DURATION_SESSION, OCI_TYPEGET_ALL, 
				      &obj_tdo)))
		return(valstring);

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&dschp, (ub4)OCI_HTYPE_DESCRIBE,
		       (size_t)0, (dvoid **)0);
	if(ociErr(errhp, OCIDescribeAny(svchp, errhp, (dvoid *)obj_tdo, 
					(ub4)0, OCI_OTYPE_PTR, 0, OCI_PTYPE_TYPE,
					dschp))) 
		return(valstring);

	OCIAttrGet((dvoid *)dschp, OCI_HTYPE_DESCRIBE, (dvoid *)&parmh,
		   (ub4 *)0, OCI_ATTR_PARAM, errhp);
	OCIAttrGet((dvoid *)parmh, OCI_DTYPE_PARAM, (dvoid *)&numattrs,
		   (ub4 *)0, OCI_ATTR_NUM_TYPE_ATTRS, errhp);
	OCIAttrGet((dvoid *)parmh, OCI_DTYPE_PARAM, (dvoid *)&attr_iter,
		   (ub4 *)0, OCI_ATTR_LIST_TYPE_ATTRS, errhp);
#if DEBUG
	fprintf(stderr, "  num_attrs: %d\n", numattrs);
#endif

	for (i = 1; i <= numattrs; i++) {
		gchar *attrname;
		gchar *typename;
		gchar aname[128];
		gchar tname[128];
		guint attrnamelen;
		guint typenamelen;
		sb2 datatype;

		if(ociErr(errhp, OCIParamGet((dvoid *)attr_iter,
					     OCI_DTYPE_PARAM, errhp,
					     (dvoid **)&attrhd, i)))
			return(valstring);

		OCIAttrGet((dvoid *)attrhd, OCI_DTYPE_PARAM,
			   (dvoid *)&attrname, (ub4 *)&attrnamelen,
			   OCI_ATTR_NAME, errhp);
		OCIAttrGet((dvoid *)attrhd, OCI_DTYPE_PARAM,
			   (dvoid *)&typename, (ub4 *)&typenamelen,
			   OCI_ATTR_TYPE_NAME, errhp);
		OCIAttrGet((dvoid *)attrhd, OCI_DTYPE_PARAM,
			   (dvoid *)&datatype, (ub4 *)0,
			   OCI_ATTR_DATA_TYPE, errhp);
		snprintf(aname, attrnamelen+1, "%s", attrname);
		snprintf(tname, typenamelen+1, "%s", typename);
		/* fprintf(stderr, "   attr: %s\t%s\n", aname, typename); */

		if(datatype == SQLT_NTY) {
			valstring = g_string_append(valstring, typename);
			valstring = g_string_append(valstring, "() ");
		}
		else {
			/* now fetch the value of the attribute with that name */
			if(ociErr(errhp, OCIObjectGetAttr(envhp, 
							  errhp, 
							  col->type_data,
							  col->obj_col_ind, 
							  obj_tdo,
							  &attrname, 
							  &attrnamelen, 
							  1,
							  (ub4 *)0, 
							  0, 
							  &attr_null_status, 
							  &attr_null_struct,
							  &attr_value, 
							  &attr_tdo)))
				return(valstring);

			if (attr_null_status == OCI_IND_NOTNULL)
				display_attr_val(envhp, errhp, (text *)aname,
						 datatype, attr_value, valstring);

		}
	}
	valstring = g_string_truncate(valstring, valstring->len-1);
	valstring = g_string_append(valstring, ")");
/*   fprintf(stderr, "     %s\n", valstring->str);  */

	return(valstring);
}



void
define_oracle_object_type(OCIEnv    *envhp,
			  OCISvcCtx *svchp,
			  OCIError  *errhp,
			  OCIStmt   *st,
			  colData   *col,
			  int       pos)
{
	OCIParam *php;
	OCIComplexObject *corhp;
	ub4 typenamelen=0, indsize=sizeof(sb2);
	gchar *typename;
	sb2 dtype;
	ub4 level = UB4MAXVAL; 

	OCIHandleAlloc((dvoid *) envhp, (dvoid **) &corhp,
		       (ub4)OCI_HTYPE_COMPLEXOBJECT, 0, (dvoid **)0);
	OCIAttrSet( (dvoid *) corhp,
		    (ub4) OCI_HTYPE_COMPLEXOBJECT,
		    (dvoid *) &level, 
		    (ub4) sizeof(ub4), 
		    OCI_ATTR_COMPLEXOBJECT_LEVEL, 
		    (OCIError *) errhp);

	OCIParamGet(st, OCI_HTYPE_STMT, errhp, (dvoid **)&php, pos);
	OCIAttrGet((dvoid *)php, OCI_DTYPE_PARAM, (dvoid *)&typename,
		   (ub4 *)&typenamelen, OCI_ATTR_TYPE_NAME, errhp);
	OCIAttrGet((dvoid *)php, OCI_DTYPE_PARAM, (dvoid *)&dtype,
		   (ub4 *)0, (ub4)OCI_ATTR_DATA_TYPE, errhp);
	snprintf(col->typename, typenamelen+1, "%s", typename);
#if DEBUG
	fprintf(stderr, "  type: %s(%d)\n", col->typename, dtype);
#endif

	OCIAttrGet((dvoid *)php, OCI_DTYPE_PARAM, (dvoid *)&col->type_ref,
		   (ub4 *)0, OCI_ATTR_REF_TDO, errhp);
	OCIDefineByPos(st, &col->defnp, errhp, (ub4)pos, (dvoid *)0,
		       (sb4)0, SQLT_NTY, (dvoid *)0, (ub2 *)0, (ub2 *)0, OCI_DEFAULT);

	ociErr(errhp, OCIObjectPin(envhp ,errhp , col->type_ref,
				   (OCIComplexObject *)corhp, OCI_PIN_ANY,
				   OCI_DURATION_SESSION, OCI_LOCK_NONE, 
				   (dvoid **)&col->m_type));
#if DEBUG
	fprintf(stderr, "  OCIObjectPin\n");
#endif

	col->obj_col_ind = g_malloc(sizeof(sb2));
	*col->obj_col_ind = 0;
	ociErr(errhp, OCIDefineObject(col->defnp, errhp, col->m_type,
				      (dvoid **)&col->type_data, (ub4 *)0,
				      (dvoid **)&col->obj_col_ind, (ub4 *)&indsize));
#if DEBUG
	fprintf(stderr, "oracle_type.c: OCIDefineObject\n");
#endif

}
