/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */
/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <oci.h>

#include "orautil.h"
#include "gutil.h"


void on_connect1_activate(GtkMenuItem *, gpointer);


sword  ociErr(OCIError *ehp,
	      sword ostat)
{
	text errbuf[1024];
	char mbuf[1024];
	sword errcode = 0;
	int z = 0;

	if (OCI_SUCCESS != ostat) {

		switch (ostat) {

		case OCI_NEED_DATA:
		case OCI_NO_DATA:
		case OCI_STILL_EXECUTING:
			break;

		case OCI_SUCCESS_WITH_INFO:
			ostat=OCI_SUCCESS;
			break;

		case OCI_ERROR:

			OCIErrorGet((dvoid *)ehp, (ub4)1,
				    (text *)NULL, &errcode,
				    errbuf, (ub4) sizeof(errbuf),
				    (ub4)OCI_HTYPE_ERROR);

			if (14107 == errcode 
			    || 20000 == errcode) 
			{
				return errcode;
			}
#if DEBUG
			fprintf(stderr, "%s:%d %s", __FILE__, __LINE__,
				errbuf);
#endif
			g_strchomp(errbuf);
			gtk_label_set_text(GTK_LABEL(lw("statusLabel")),
					   errbuf);

			switch (errcode) {

			case 28:
			case 3113:
			case 3114:
			case 3135:
				on_connect1_activate(NULL, NULL);
				break;

			default:
				break;
			}
      
			break;

		case OCI_INVALID_HANDLE:
			gtk_label_set_markup(GTK_LABEL(lw("statusLabel")),
					     "<span color=\"red4\"><b>OCI_INVALID_HANDLE</b></span>");
			break;

		case OCI_CONTINUE:
			fprintf(stderr, "Error OCI_CONTINUE\n");
			break;

		default:
			break;
		}
	}
	return ostat;
}



GList *get_tns_entries(GtkWidget *stat_label)
{
	GList *ents=0;
	GtkLabel *status_label=0;
	GIOChannel *ioc=0;
	GIOStatus iocstat=0;
	GError *gerr=0;
	guint c=0, e=0, paren=0;
	const gchar *tns_path=0;
	gchar *tns_file=0;
	gchar *buffer=0;
	gsize bufflen=0;
	gchar entry[128];

	status_label = GTK_LABEL(stat_label);

	tns_path = g_getenv("TNS_ADMIN");
	if (!tns_path) {
 
		tns_path = g_getenv("ORACLE_HOME");

		if (!tns_path) {
			return 0;
		}
		
		tns_file = g_build_filename(tns_path, "network", 
					    "admin", "tnsnames.ora",
					    NULL);
  
	}
	else {
		tns_file = g_build_filename(tns_path, "tnsnames.ora",
					    NULL);
	}

#if DEBUG
	fprintf(stderr, "%s:%d tns_file='%s'\n",
		__FILE__, __LINE__, tns_file);
#endif

	if (!g_file_test(tns_file, G_FILE_TEST_IS_REGULAR)) {
		gtk_label_set_markup(status_label, "tnsnames.ora not in use");
		return 0;
	}

	if (status_label) {
		gtk_label_set_text(status_label, tns_file);
	}


	ioc = g_io_channel_new_file(tns_file, "r", &gerr);
	if (!ioc) {
		gchar err[512];
    
		if (status_label) {
			g_snprintf(err, sizeof(err),
				   "Could not open TNS file <b>%s</b>: <span color=\"red4\">%s</span>",
				   tns_file, gerr->message);
			g_error_free(gerr);
			gtk_label_set_markup(status_label, err);
		}
		g_free(tns_file);
 		return 0;
	}

	iocstat = g_io_channel_read_to_end(ioc, &buffer, &bufflen, &gerr);
	if (G_IO_STATUS_NORMAL != iocstat) {
		gchar err[512];
    
		if (status_label) {
			g_snprintf(err, sizeof(err),
				   "Reading TNS file: <b>%s</b>", gerr->message);
			g_error_free(gerr);
			gtk_label_set_markup(status_label, err);
		}
		g_free(tns_file);
 		return 0;
	}


	g_io_channel_shutdown(ioc, FALSE, (GError **)0);
	g_free(tns_file);

	for (c=0; c < strlen(buffer); c++) {
		e=0;
		entry[e]='\0';

		while (buffer[c] != '\0' && g_ascii_isspace(buffer[c])) {
			c++;
		}
		if (buffer[c] == '\0')
			break;

		if (buffer[c] == '#') {
			while(buffer[c]!='\n' && buffer[c]!='\0') {
				c++;
			}
			continue;
		}

		while (c < strlen(buffer)
		       && e < sizeof(entry)
		       && !g_ascii_isspace(buffer[c]) 
		       && buffer[c]!='=') {
			entry[e] = buffer[c];
			e++;
			c++;
		}

		entry[e] = '\0';
		if (e) {
#if DEBUG
			fprintf(stderr, "%s:%d '%s'\n", __FILE__, __LINE__, entry);
#endif
			ents = g_list_append(ents, (gpointer) g_strdup(entry) );
		}

		while (c < strlen(buffer) && buffer[c]!='(') {
			c++;
		}

		paren=1;
		while (c < strlen(buffer) && paren) {
			c++;
			if(buffer[c]=='(') 
				paren++;

			if(buffer[c]==')')
				paren--;
		}
	}
	g_free(buffer);
	return ents;
}



void oci_dtype_to_string(ub2 datatype,
			 char *buf, 
			 int bufsize,
			 int col_width)
{

	switch (datatype) {

	case SQLT_STR:
	case SQLT_VCS:
	case SQLT_CHR:
		if (col_width) {
			snprintf(buf, bufsize, "VARCHAR2(%d)", col_width);
		}
		else {
			snprintf(buf, bufsize, "VARCHAR2");
		}
		break;

	case SQLT_NUM:
	case SQLT_INT:
	case SQLT_VNU:
	case SQLT_LNG:
		if (col_width) {
			snprintf(buf, bufsize, "NUMBER(%d)", col_width);
		}
		else {
			snprintf(buf, bufsize, "NUMBER");
		}
		break;

	case SQLT_BIN:
		snprintf(buf, bufsize, "RAW");
		break;

	case SQLT_AFC:
		if (col_width) {
			snprintf(buf, bufsize, "CHAR(%d)", col_width);
		}
		else {
			snprintf(buf, bufsize, "CHAR");
		}
		break;

	case SQLT_LBI:
		snprintf(buf, bufsize, "RAW");
		break;

	case SQLT_FLT:
		snprintf(buf, bufsize, "FLOAT");
		break;

	case SQLT_RDD:
		snprintf(buf, bufsize, "ROWID");
		break;

	case SQLT_CLOB:
		snprintf(buf, bufsize, "CLOB");
		break;

	case SQLT_BLOB:
		snprintf(buf, bufsize, "BLOB");
		break;

	case SQLT_DATE:
	case SQLT_DAT:
		snprintf(buf, bufsize, "DATE");
		break;

	case SQLT_TIMESTAMP:
		snprintf(buf, bufsize, "TIMESTAMP(%d)", col_width);
		break;

	case SQLT_TIMESTAMP_TZ:
		snprintf(buf, bufsize, "TIMESTAMP(%d) WITH TIME ZONE", col_width);
		break;

	case SQLT_TIMESTAMP_LTZ:
		snprintf(buf, bufsize, "TIMESTAMP(%d) WITH LOCAL TIME ZONE", col_width);
		break;

	case SQLT_INTERVAL_YM:
		snprintf(buf, bufsize, "INTERVAL Year to Month");
		break;

	case SQLT_INTERVAL_DS:
		snprintf(buf, bufsize, "INTERVAL Day to Second");
		break;
	default:
		snprintf(buf, bufsize, "unknown");
	}
}



sword ociPrepare(OCIStmt *st, 
		 OCIError *ehp,
		 char *sql)
{
	sword ret;

	ret = ociErr(ehp, OCIStmtPrepare(st, ehp, (text *)sql,
					 (ub4) strlen(sql),
					 (ub4)OCI_NTV_SYNTAX,
					 (ub4)OCI_DEFAULT));
	return ret;
}



sword ociExecute(OCISvcCtx *svc,
		 OCIStmt *st,
		 OCIError *err,
		 ub4 iters)
{
	sword ret;
	
	ret = OCIStmtExecute(svc, st, err, (ub4)iters,
			     (ub4)0, (OCISnapshot *)0,
			     (OCISnapshot *)0,
			     (ub4)OCI_DEFAULT);
	return ret;
}



sword ociBindString(OCIStmt *st,
		    OCIError *ehp,
		    ub4 pos, 
		    char *value)
{
	sword ret;
	OCIBind *bhp = (OCIBind *)0;

	ret = OCIBindByPos((OCIStmt *)st, 
			   (OCIBind **)&bhp, 
			   (OCIError *)ehp, 
			   (ub4)pos, 
			   (dvoid *)value, (sb4)strlen(value)+1, 
			   (ub2)SQLT_STR, 
			   (dvoid *)0, (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0, 
			   (ub4)OCI_DEFAULT);

	return ret;
}



sword  ociBindNumber(OCIStmt *st,
		     OCIError *ehp,
		     ub4 pos,
		     dvoid *value,
		     sb4 valsize)
{
	sword ret;
	OCIBind *bhp = (OCIBind *)0;

	ret = ociErr(ehp, OCIBindByPos((OCIStmt *)st, 
				       (OCIBind **)&bhp, 
				       (OCIError *)ehp, 
				       (ub4)pos, 
				       (dvoid *)value, (sb4)valsize, 
				       (ub2)SQLT_INT, 
				       (dvoid *)0, (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0, 
				       (ub4)OCI_DEFAULT)
		);

	return ret;
}




sword ociDefine(OCIStmt *st,
		OCIError *ehp,
		ub4 pos,
		ub2 dtype,
		dvoid *buffer,
		ub4 size)
{
	sword ret;
	OCIDefine *defnp = (OCIDefine *)0;
	sb2 indp=0;

	ret = ociErr(ehp, OCIDefineByPos((OCIStmt *)st,
					 (OCIDefine **)&defnp,
					 (OCIError *)ehp, 
					 (ub4)pos,
					 (dvoid *)buffer, (sb4) size, 
					 (ub2)dtype, 
					 (dvoid *)&indp, (ub2 *)0, (ub2 *)0, 
					 (ub4)OCI_DEFAULT)
		);

	return ret;
}


sword ociDefineNumber(OCIStmt *st,
		      OCIError *ehp,
		      ub4 pos,
		      int *value,
		      sb4 size)
{
	sword ret;
	OCIDefine *defnp = (OCIDefine *)0;
	sb2 indp=0;

	ret = ociErr(ehp, OCIDefineByPos((OCIStmt *)st,
					 (OCIDefine **)&defnp,
					 (OCIError *)ehp, 
					 (ub4)pos,
					 (dvoid *)value, size, 
					 (ub2)SQLT_INT, 
					 (dvoid *)&indp, (ub2 *)0, (ub2 *)0, 
					 (ub4)OCI_DEFAULT)
		);

	return ret;
}

