/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <oci.h>
#include <gtk/gtk.h>

#include "gutil.h"
#include "statement.h"
#include "orautil.h"
#include "query.h"

extern OCIError       *errhp;
extern OCIEnv         *envhp;
extern OCISvcCtx      *svchp;



gchar *get_partition_keys(const gchar *type,
			  const gchar *owner,
			  const gchar *object)
{
	OCIStmt *sth;
	OCIDefine *defnp;
	gchar buffer[512];
	gchar *part_key=0;
	sword ret;
	gchar *sql = "select\
  decode(pos, 1, cname, NULL) || ' ' ||\
  lead(decode(pos, 2, cname, NULL),1) over(order by pos) || ' ' ||\
  lead(decode(pos, 3, cname, NULL),2) over (order by pos) || ' ' ||\
  lead(decode(pos, 4, cname, NULL),3) over (order by pos) || ' ' ||\
  lead(decode(pos, 5, cname, NULL),4) over (order by pos) || ' ' ||\
  lead(decode(pos, 6, cname, NULL),5) over (order by pos)\
  from (\
   select column_position pos, column_name cname from all_part_key_columns\
   where name=:p1 and object_type=:p2 and owner=:p3\
   order by column_position)";

	if( !type || !owner || !object)
		return 0;

	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT,
		       0, (dvoid **)0);
	ociPrepare(sth, errhp, sql);
	ociBindString(sth, errhp, 1, (gchar *)object);
	ociBindString(sth, errhp, 2, (gchar *)type);
	ociBindString(sth, errhp, 3, (gchar *)owner);
	OCIDefineByPos((OCIStmt *) sth,
		       (OCIDefine **) &defnp,
		       (OCIError *)errhp, 
		       (ub4) 1,
		       (dvoid *)&buffer, sizeof(buffer)+1, 
		       (ub2)SQLT_STR,
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);
	ret = ociErr(errhp, OCIStmtExecute(svchp, sth, errhp,
					   1, 0, NULL, NULL,
					   OCI_DEFAULT));
	if (OCI_SUCCESS == ret) {
		part_key = g_strdup(buffer); 
	}

	OCIHandleFree(sth, OCI_HTYPE_STMT);
	return part_key;
}


gchar *get_partition_highvalue(const gchar *type,
			       const gchar *owner,
			       const gchar *table,
			       const gchar *partition)
{
	OCIStmt *sth;
	OCIDefine *defnp;
	gchar buffer[512];
	gchar *highvalue=0;
	sword ret;
	gchar *sql = "select high_value from all_tab_partitions where table_owner=:p1 and table_name=:p2 and partition_name=:p3";

	if (!owner || !table || !partition) {
		return 0;
	}

	OCIHandleAlloc(envhp, (dvoid **)&sth, (ub4)OCI_HTYPE_STMT,
		       0, (dvoid **)0);
	ociPrepare(sth, errhp, sql);
	ociBindString(sth, errhp, 1, (gchar *)owner);
	ociBindString(sth, errhp, 2, (gchar *)table);
	ociBindString(sth, errhp, 3, (gchar *)partition);
	OCIDefineByPos((OCIStmt *) sth,
		       (OCIDefine **) &defnp,
		       (OCIError *)errhp, 
		       (ub4) 1,
		       (dvoid *)&buffer, sizeof(buffer)+1, 
		       (ub2)SQLT_STR,
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);

	ret = ociErr(errhp, OCIStmtExecute(svchp, sth, errhp,
					   1, 0, NULL, NULL,
					   OCI_DEFAULT));

	if (OCI_SUCCESS == ret) {
		highvalue = g_strdup(buffer); 
	}

	OCIHandleFree(sth, OCI_HTYPE_STMT);
	return highvalue;
}

