/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <oci.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "dbms_aq.h"
#include "plobject.h"
#include "gutil.h"
#include "orautil.h"
#include "query.h"
#include "appconfig.h"
#include "syntax.h"


extern OCIEnv      *envhp;
extern OCIError    *errhp;
extern OCISvcCtx   *svchp;


void show_oracle_type(const gchar *type_name,
		      GtkTreeView *treeview,
		      GtkTreeIter *parent_iter,
		      gint colnum)
{
	GtkTreeModel *model = gtk_tree_view_get_model(treeview);
	GtkTreeIter iter;
	OCIStmt *sth = (OCIStmt *)0;
	OCIDefine *defnp = (OCIDefine *)0;
	sb2 indp=0;
	gchar **type=0;
	gchar d_attr_name[128], d_attr_type_owner[128], d_attr_type_name[128];
	gchar *sql = "\
  select \
   attr_name, \
   attr_type_owner, \
   case \
     when precision is not null then \
       decode(scale, NULL, attr_type_name || '(' || precision || ')', attr_type_name || '(' || precision || ',' || scale || ')') \
     when scale is not null then attr_type_name || '(' || scale || ')' \
     when length is not null then attr_type_name || '(' || length || ')' \
     else attr_type_name \
   end dt\
  from all_type_attrs \
  where owner=:p1 and type_name=:p2\
  order by type_name, attr_no";


	if (!model) {
		return;
	}

	if (!type_name) {
		return;
	}

	if ((type = g_strsplit(type_name, ".", -1)) == NULL) {
		return;
	}

	if (g_strv_length(type) >= 2 ) {

		OCIHandleAlloc(envhp, (dvoid **)&sth,
			       (ub4)OCI_HTYPE_STMT, 0,
			       (dvoid **)0);
		ociPrepare(sth, errhp, sql);
		ociBindString(sth, errhp, 1, type[0]);
		ociBindString(sth, errhp, 2, type[1]);
		OCIDefineByPos((OCIStmt *) sth,
			       (OCIDefine **) &defnp,
			       (OCIError *)errhp, 
			       (ub4) 1,
			       (dvoid *)&d_attr_name,
			       sizeof(d_attr_name)+1, 
			       (ub2)SQLT_STR, 
			       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
			       (ub4)OCI_DEFAULT);
		OCIDefineByPos((OCIStmt *) sth,
			       (OCIDefine **) &defnp,
			       (OCIError *)errhp, 
			       (ub4) 2,
			       (dvoid *)&d_attr_type_owner,
			       sizeof(d_attr_type_owner)+1, 
			       (ub2)SQLT_STR, 
			       (dvoid *)&indp, (ub2 *)0, (ub2 *)0, 
			       (ub4)OCI_DEFAULT);
		OCIDefineByPos((OCIStmt *) sth,
			       (OCIDefine **) &defnp,
			       (OCIError *)errhp, 
			       (ub4) 3,
			       (dvoid *)&d_attr_type_name,
			       sizeof(d_attr_type_name)+1, 
			       (ub2)SQLT_STR, 
			       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
			       (ub4)OCI_DEFAULT);

		if (OCI_SUCCESS == ociErr(errhp, ociExecute(svchp, sth, errhp, 0))) {
      
			while (OCI_SUCCESS == OCIStmtFetch2(sth, errhp, 1, 
							   OCI_FETCH_NEXT, 
							   0, OCI_DEFAULT)) {
	
				gtk_tree_store_append(GTK_TREE_STORE(model),
						      &iter, parent_iter);
				gtk_tree_store_set(GTK_TREE_STORE(model),
						   &iter, colnum, d_attr_name,
						   colnum+1, d_attr_type_name, -1);

				if (!indp) {
					gchar inner_type[128];

					g_snprintf(inner_type, sizeof(inner_type), "%s.%s",
						   d_attr_type_owner, d_attr_type_name);
					show_oracle_type(inner_type, treeview, &iter, colnum);
				}
			}
		}

		g_strfreev(type);
	}

}





