/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h> 
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <oci.h>
#include <glib.h>

#include "query.h"
#include "gutil.h"
#include "orautil.h"
#include "oracle_type.h"
#include "table.h"
#include "index.h"
#include "dbms_output.h"
#include "plobject.h"
#include "command_tab.h"
#include "appconfig.h"

#include "icons/gtko_icon_gray_pixbuf.h"
#include "icons/gtko_icon_green_pixbuf.h"
#include "icons/heap_pixbuf.h"
#include "icons/partition_pixbuf.h"
#include "icons/iot_pixbuf.h"
#include "icons/iot_partition_pixbuf.h"
#include "icons/view_pixbuf.h"
#include "icons/bitmap_pixbuf.h"
#include "icons/bitmap_partition_pixbuf.h"
#include "icons/function_pixbuf.h"
#include "icons/lob_pixbuf.h"
#include "icons/mview_pixbuf.h"
#include "icons/mview_log_pixbuf.h"
#include "icons/temptable_pixbuf.h"
#include "icons/synonym_pixbuf.h"
#include "icons/sequence_pixbuf.h"
#include "icons/directory_pixbuf.h"
#include "icons/cluster_pixbuf.h"
#include "icons/queue_pixbuf.h"
#include "icons/recyclebin_pixbuf.h"
#include "icons/dblink_pixbuf.h"
#include "icons/plsql_pixbuf.h"
#include "icons/procedure_pixbuf.h"
#include "icons/plsqlargument_pixbuf.h"
#include "icons/plsqlargumentout_pixbuf.h"
#include "icons/plsqlargumentinout_pixbuf.h"
#include "icons/type_pixbuf.h"
#include "icons/plsqlattribute_pixbuf.h"
#include "icons/typemethod_pixbuf.h"
#include "icons/typebody_pixbuf.h"
#include "icons/package_pixbuf.h"
#include "icons/session_pixbuf.h"
#include "icons/sessionactive_pixbuf.h"
#include "icons/schedule_pixbuf.h"
#include "icons/advice_pixbuf.h"
#include "icons/instance_pixbuf.h"
#include "icons/schema_pixbuf.h"
#include "icons/schemas_pixbuf.h"
#include "icons/library_pixbuf.h"
#include "icons/indexinvalid_pixbuf.h"

#define GLIST_FETCHSIZE 32

extern OCIEnv    *envhp;
extern OCIError  *errhp;
extern OCISvcCtx *bsvchp;
extern OCISvcCtx *svchp;

static GHashTable *pixbuf_hash=0;
static gchar pagetext[128];
static GtkLabel *pagelabel=0;
static GtkWidget *interfaceDebugWindow;
static GtkWidget *interfaceDebugText;

#if GTK_CHECK_VERSION(2,10,0)
static GtkStatusIcon  *status_icon;
#endif

 
 
void query_create_pixbufs() {

  pixbuf_hash = g_hash_table_new(g_str_hash, g_str_equal);
  g_hash_table_insert(pixbuf_hash, (gpointer) "gtko_active_pixbuf", 
		      (gpointer) gdk_pixbuf_new_from_inline(-1, gtko_icon_green_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "gtko_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, gtko_icon_gray_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "heap_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, heap_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "partition_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, partition_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "iot_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, iot_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "iot_partition_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, iot_partition_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "view_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, view_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "bitmap_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, bitmap_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "bitmap_partition_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, bitmap_partition_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "function_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, function_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "lob_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, lob_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "mview_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, mview_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "temptable_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, temptable_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "mview_log_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, mview_log_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "synonym_pixbuf", 
		      (gpointer) gdk_pixbuf_new_from_inline(-1, synonym_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "sequence_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, sequence_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "directory_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, directory_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "queue_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, queue_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "cluster_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, cluster_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "recyclebin_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, recyclebin_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "dblink_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, dblink_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "plsql_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, plsql_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "procedure_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, procedure_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "plsqlargument_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, plsqlargument_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "plsqlargumentout_pixbuf", 
		      (gpointer) gdk_pixbuf_new_from_inline(-1, plsqlargumentout_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "plsqlargumentinout_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, plsqlargumentinout_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "type_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, type_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "plsqlattribute_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, plsqlattribute_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "typemethod_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, typemethod_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "typebody_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, typebody_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "package_pixbuf", 
		      (gpointer) gdk_pixbuf_new_from_inline(-1, package_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "session_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, session_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "sessionactive_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, sessionactive_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "schedule_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, schedule_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "advice_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, advice_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "instance_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, instance_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "schema_pixbuf", 
		      (gpointer) gdk_pixbuf_new_from_inline(-1, schema_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "schemas_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, schemas_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "library_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, library_pixbuf, FALSE, NULL) );
  g_hash_table_insert(pixbuf_hash, (gpointer) "indexinvalid_pixbuf",
		      (gpointer) gdk_pixbuf_new_from_inline(-1, indexinvalid_pixbuf, FALSE, NULL) );

#if GTK_CHECK_VERSION(2,10,0)
  status_icon = gtk_status_icon_new_from_pixbuf(query_get_pixbuf("gtko_pixbuf") );
#endif
  
}


GdkPixbuf *query_get_pixbuf(const gchar *key)
{
	if (!key) {
 		return 0;
	}

	return ((GdkPixbuf *)g_hash_table_lookup(pixbuf_hash, key));
}


qCommand *alloc_qCommand(gchar *treeview) {

	qCommand *qs = g_malloc(sizeof(qCommand));

	if (treeview) {
		qs->target = GTK_TREE_VIEW(lw(treeview));
	}
	else {
		qs->target = GTK_TREE_VIEW(lw("resList"));
	}

	qs->op = 0;
	qs->offset = -1;
	qs->len = 0;
	qs->post = 0;
	qs->page = 0;
	qs->sql = 0;
	qs->buffer = 0;

	OCIHandleAlloc((dvoid *)envhp, (dvoid **)&qs->sth,
		       (ub4)OCI_HTYPE_STMT, (size_t)0, (dvoid **)0);

	return qs;
}



void free_column_node(gpointer *data,
		      gpointer *user_data) 
{
	colData *col = (colData *) data;


	if (col->type_ref) {
		ociErr(errhp, OCIObjectUnpin(envhp,
					     errhp ,
					     (dvoid *)col->m_type));
		if(col->type_data) {
			ociErr(errhp, OCIObjectFree(envhp, errhp,
						    col->type_data,
						    OCI_OBJECTFREE_FORCE));
		}
	}

	g_free(col->indp);
	g_free(col->colname);
	g_free(col->value);
	g_free(col);
}


static void  delete_column(gpointer *data, gpointer *ud){
	gtk_tree_view_remove_column((GtkTreeView *)ud, (GtkTreeViewColumn *)data);
}


void on_interfaceDebugClose(gpointer user_data)
{
	if (interfaceDebugWindow) {
		gtk_widget_destroy(interfaceDebugWindow);
		interfaceDebugWindow = 0;
	}
}



GtkWidget* create_interfaceDebugWindow (void)
{

	GtkWidget *vbox44;
	GtkWidget *scrolledwindow119;
	GtkWidget *hbox106;
	GtkWidget *interfaceDebugCloseButton;

	interfaceDebugWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (interfaceDebugWindow), _("GTKO: Embedded SQL"));

	vbox44 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox44);
	gtk_container_add (GTK_CONTAINER (interfaceDebugWindow), vbox44);

	scrolledwindow119 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow119);
	gtk_box_pack_start (GTK_BOX (vbox44), scrolledwindow119, TRUE, TRUE, 0);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW (scrolledwindow119), GTK_SHADOW_IN);

	interfaceDebugText = gtk_text_view_new ();
	gtk_widget_show (interfaceDebugText);
	gtk_container_add (GTK_CONTAINER (scrolledwindow119), interfaceDebugText);
	gtk_widget_set_size_request (interfaceDebugText, 400, 400);

	hbox106 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox106);
	gtk_box_pack_start (GTK_BOX (vbox44), hbox106, FALSE, TRUE, 0);

	interfaceDebugCloseButton = gtk_button_new_with_mnemonic (_("  Close  "));
	gtk_widget_show (interfaceDebugCloseButton);
	gtk_box_pack_start (GTK_BOX (hbox106), interfaceDebugCloseButton, FALSE, FALSE, 8);

	g_signal_connect(G_OBJECT(interfaceDebugCloseButton), 
			 "clicked",
			 G_CALLBACK(on_interfaceDebugClose),
			 (gpointer) NULL);

	g_signal_connect ((gpointer) interfaceDebugWindow, "delete_event",
			  G_CALLBACK (on_interfaceDebugClose),
			  NULL);

	return interfaceDebugWindow;
}


static void execute_prepared_sql(OCIStmt *sth,
				 GtkTreeView *treeview,
				 GtkTreeIter *parent_iter)
{

	GtkCellRenderer *crend=0;
	GtkTreeViewColumn *column=0;
	GtkTreeIter iter;
	GtkTreeStore *store=0;
	GtkTreeSortable *sortable=0;
	GType *gtypes=0;
	ub2 stmt_type;
	OCIParam *param = (OCIParam *)0;
	gint i, nrows, iters=0;
	ub4 ncol, sizep = sizeof(ub4);
	colData *thisCol = 0;
	GSList *column_nodes=0, *int_nodes=0, *tmpnode=0;
	guint array_size = 48;
	sword ret = OCI_SUCCESS;
	const gchar *font = 0;


	OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT, 
		   (dvoid *)&stmt_type, (ub4 *)0, 
		   (ub4) OCI_ATTR_STMT_TYPE, errhp);

	if(OCI_STMT_SELECT != stmt_type) {
		iters=1;
	}

	ret = ociErr(errhp, ociExecute(svchp, sth, errhp, (ub4)iters));

	if (ret != OCI_SUCCESS || stmt_type != OCI_STMT_SELECT ) {

		if (OCI_SUCCESS != ret) {
			OraText *sql = 0;
			ub2   pe_offset=0, sql_len=0;

			OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT,
				   (dvoid *)&sql, (ub4 *)&sql_len,
				   (ub4)OCI_ATTR_STATEMENT, errhp);
			OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT,
				   (dvoid *)&pe_offset, (ub4 *)0,
				   (ub4)OCI_ATTR_PARSE_ERROR_OFFSET, errhp);
			fprintf(stderr, "%s:%d \n>\n%*s\n<\noffset: %d\n", 
				__FILE__, __LINE__, sql_len, sql, pe_offset);
		}

		OCIStmtRelease(sth, errhp, (OraText) NULL, 0, OCI_DEFAULT);
		return;
	}


	if (!parent_iter) {
		GList          *cols; 

		store  = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
		if(store) {
			gtk_tree_store_clear(GTK_TREE_STORE(store));
			store = 0;
		}

		cols = gtk_tree_view_get_columns(treeview);
		g_list_foreach(cols, (GFunc) delete_column, (gpointer) treeview);
		g_list_free(cols);
	}


	OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT, (dvoid *)&ncol,
		   (ub4 *)0, (ub4) OCI_ATTR_PARAM_COUNT, errhp);
	gtypes = g_new0(GType, ncol);
  
	font = config_get("listfont");

	for (i=0; i < ncol; i++) {
		ub4 colnamelen;
		text *colname;
		gint underscores, cnt;
		text *p, *p1;

		thisCol = (colData *) g_malloc(sizeof(colData) );
		thisCol->valuelen = 0;
		thisCol->dtype = 0;
		thisCol->type_data = 0;
		thisCol->type_ref = 0;
		thisCol->progress = 0;
		column_nodes = g_slist_append(column_nodes, (gpointer) thisCol);

		colname = NULL;
		colnamelen = 0;

		OCIParamGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT, (OCIError *)errhp,
			    (dvoid **)&param, (ub4)i+1);
		OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM, (dvoid *)&thisCol->dtype,
			   (ub4 *)0, (ub4)OCI_ATTR_DATA_TYPE, errhp);
		OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM, (dvoid *)&colname, &colnamelen,
			   (ub4)OCI_ATTR_NAME, errhp);
		OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM, (dvoid *)&thisCol->valuelen,
			   (ub4 *)0, (ub4)OCI_ATTR_DATA_SIZE, errhp);
    
		underscores = 0;
		for(p=colname, cnt=0; cnt < colnamelen; cnt++, p++)
			if(*p=='_')
				underscores++;
    
		switch(thisCol->dtype) {
		case SQLT_DAT:
		case SQLT_DATE:
		case SQLT_TIMESTAMP:
		case SQLT_TIMESTAMP_TZ:
		case SQLT_TIMESTAMP_LTZ:
		case SQLT_INTERVAL_YM:
		case SQLT_INTERVAL_DS:
			thisCol->valuelen = 64;
			gtypes[i] = G_TYPE_STRING; 
			break;
		case SQLT_NUM:
		case SQLT_FLT:
		case SQLT_INT:
			thisCol->valuelen = 40;
			gtypes[i] = G_TYPE_STRING;
			int_nodes = g_slist_append(int_nodes, GUINT_TO_POINTER(i));
			break;
		case SQLT_LNG:
			thisCol->valuelen = 2048;
			gtypes[i] = G_TYPE_STRING; 
			break;
		default:
			gtypes[i] = G_TYPE_STRING; 
			break;
		}
 
		thisCol->value = (char *)calloc(array_size, (size_t)thisCol->valuelen+1 );
		thisCol->indp  = (sb2 *)calloc(array_size, (size_t)sizeof(sb2) );
		OCIDefineByPos(sth,
			       &thisCol->defnp,
			       errhp, 
			       (ub4)i+1, 
			       (dvoid *)thisCol->value, (sb4)thisCol->valuelen+1, 
			       (ub2)SQLT_STR,
			       (dvoid *)&thisCol->indp[0], 
			       (ub2 *)0, 
			       (ub2 *)0, 
			       (ub4)OCI_DEFAULT);

		thisCol->colname = (char *)g_malloc(colnamelen+underscores+1);
		if(underscores) {
			for(p=colname, p1=(text *)thisCol->colname, cnt=0; cnt < colnamelen; cnt++) {
				if (*p=='_')
					*p1++ = '_';
				*p1++ = *p++;
			}
		}
		else {
			strncpy(thisCol->colname, (char *)colname, colnamelen);
		}

		thisCol->colname[colnamelen+underscores] = '\0';
     
		if(!parent_iter) {

			if(!g_ascii_strncasecmp(thisCol->colname, "foreground", 10) ||
			   !g_ascii_strncasecmp(thisCol->colname, "background", 10) ||
			   !g_ascii_strncasecmp(thisCol->colname, "family", 6)) {

				gtk_tree_view_column_add_attribute(column, crend, thisCol->colname, i);
			}
			else if(!g_ascii_strncasecmp(thisCol->colname, "style", strlen(thisCol->colname)) ||
				!g_ascii_strncasecmp(thisCol->colname, "weight", strlen(thisCol->colname))
				) {
	
				gtk_tree_view_column_add_attribute(column, crend, thisCol->colname, i);
				gtypes[i] = G_TYPE_INT;
			}
			else if(!g_ascii_strncasecmp(thisCol->colname, "GtkProgress", 11)) {

				crend = gtk_cell_renderer_progress_new();
				g_object_set(G_OBJECT(crend), "ypad", 0, "xpad", 0, NULL);
				column = gtk_tree_view_column_new_with_attributes(&thisCol->colname[11], crend, "value", i, NULL);
				gtk_tree_view_append_column(treeview, column);
				gtypes[i] = G_TYPE_INT;
				thisCol->progress = 1;
			}
			else {

				if( !g_ascii_strncasecmp(thisCol->colname, "pxb", 3) ) {

					gtypes[i] = GDK_TYPE_PIXBUF;

					crend = gtk_cell_renderer_pixbuf_new();
					g_object_set(G_OBJECT(crend), "ypad", 0, "xpad", 0, NULL);
					column = gtk_tree_view_column_new_with_attributes(&thisCol->colname[3], crend, "pixbuf", i, NULL);
					gtk_tree_view_append_column(treeview, column);
				}
				else {
	
					crend = gtk_cell_renderer_text_new();
      
					if(font)
						g_object_set(G_OBJECT(crend), "font", font, NULL);
      
					g_object_set(G_OBJECT(crend), "ypad", 0, "xpad", 2, NULL);
	
					column = gtk_tree_view_column_new_with_attributes(thisCol->colname, crend, "text", i, NULL);
					gtk_tree_view_append_column(treeview, column);
					gtk_tree_view_column_set_sort_column_id(column, i);
				}
			}
		}
	}


	if(parent_iter) {

		store = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
		if(!store) {
			fprintf(stderr, "%s:%d could not find store in GtkTreeView\n",
				__FILE__, __LINE__);
			return;
		}

		if( gtk_tree_model_iter_n_children(GTK_TREE_MODEL(store), parent_iter) ) {
			GtkTreeIter    child_iter;
    
			gtk_tree_model_iter_children(GTK_TREE_MODEL(store), &child_iter, parent_iter);
			while(gtk_tree_store_remove(store, &child_iter));
		}
	}
	else {
		store = gtk_tree_store_newv((gint)ncol, gtypes);
	}


	sortable = GTK_TREE_SORTABLE(store);
	  
	for(tmpnode=int_nodes; tmpnode; tmpnode=g_slist_next(tmpnode)) {
		gint     colnum = GPOINTER_TO_INT(tmpnode->data);

		gtk_tree_sortable_set_sort_func(sortable, colnum,
						query_sort_column_as_gint64,
						GUINT_TO_POINTER(colnum), NULL);
	}
	g_slist_free(int_nodes);
	int_nodes=0;


	while(ociErr(errhp, OCIStmtFetch2(sth, errhp, array_size, OCI_FETCH_NEXT, 0, OCI_DEFAULT)) == OCI_SUCCESS) {
		OCIAttrGet((dvoid *) sth, (ub4) OCI_HTYPE_STMT, (dvoid *) &nrows,
			   (ub4 *) &sizep, (ub4)OCI_ATTR_ROWS_FETCHED, errhp);
		output_rows(store, parent_iter, &iter, column_nodes, nrows, NULL);
	}

	OCIAttrGet((dvoid *) sth, (ub4) OCI_HTYPE_STMT, (dvoid *) &nrows, (ub4 *) &sizep,
		   (ub4)OCI_ATTR_ROWS_FETCHED, errhp);
	output_rows(store, parent_iter, &iter, column_nodes, nrows, NULL);
  
	if(!parent_iter) {
		gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store) );
		g_object_unref(G_OBJECT(store));
	}

	g_free(gtypes);

	g_slist_foreach(column_nodes, (GFunc) free_column_node, (gpointer) NULL);
	g_slist_free(column_nodes);
	OCIStmtRelease(sth, errhp, (OraText) NULL, 0, OCI_DEFAULT);
}



static void  execute_prepared_sql_pivotted(OCIStmt *sth,
					   GtkTreeView *treeview,
					   GtkTreeIter *parent_iter,
					   gint begin_col)
{

	GtkCellRenderer *rend=0;
	GtkTreeViewColumn *col=0;
	GtkTreeIter iter;
	GtkTreeStore *store=0;
	ub2 stmt_type;
	OCIParam *param=0;
	gint i, iters=0;
	ub4 ncol;
	colData *thisCol=0;
	GSList *column_nodes=0, *tmpnode=0;
	GList *cols=0; 
	guint array_size = 1;
	sword ret;
	const gchar *font;


	OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT, 
		   (dvoid *)&stmt_type, (ub4 *)0,
		   (ub4) OCI_ATTR_STMT_TYPE, errhp);

	if (stmt_type != OCI_STMT_SELECT) {
		iters=1;
	}

	ret = ociErr(errhp, OCIStmtExecute(svchp, sth, errhp,
					   (ub4)iters, (ub4)0, (OCISnapshot *)0,
					   (OCISnapshot *)0, (ub4)OCI_DEFAULT));

	if (ret != OCI_SUCCESS || stmt_type != OCI_STMT_SELECT ) {
		OCIStmtRelease(sth, errhp, (OraText *) 0, 0, OCI_DEFAULT);
		return;
	}


	if (!parent_iter) {

		store  = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
		if(store) {
			gtk_tree_store_clear(GTK_TREE_STORE(store));
			store = 0;
		}

		cols = gtk_tree_view_get_columns(treeview);
		g_list_foreach(cols, (GFunc) delete_column, (gpointer) treeview);
		g_list_free(cols);

		font = config_get("listfont");

		rend = gtk_cell_renderer_text_new();
		g_object_set(G_OBJECT(rend), "ypad", 0, "xpad", 0, NULL);
		if(font)
			g_object_set(G_OBJECT(rend), "font", font, NULL);
      
		col  = gtk_tree_view_column_new_with_attributes(" ", rend, "text", 0, NULL);
		gtk_tree_view_append_column(treeview, col);

		rend = gtk_cell_renderer_text_new();
		g_object_set(G_OBJECT(rend), "ypad", 0, "xpad", 0, NULL);
		if(font)
			g_object_set(G_OBJECT(rend), "font", font, NULL);
      
		col  = gtk_tree_view_column_new_with_attributes(" ", rend, "text", 1, NULL);
		gtk_tree_view_append_column(treeview, col);

		store = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
	}
	else {
		store = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
	}


	OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT, (dvoid *)&ncol,
		   (ub4 *)0, (ub4) OCI_ATTR_PARAM_COUNT, errhp);
	for(i=0; i < ncol; i++) {
		ub4        colnamelen;
		text       *colname;

		thisCol = (colData *) g_malloc(sizeof(colData) );
		thisCol->valuelen = 0;
		thisCol->dtype = 0;
		thisCol->type_data = 0;
		thisCol->type_ref = 0;
		column_nodes = g_slist_append(column_nodes, (gpointer) thisCol);

		colname = NULL;
		colnamelen = 0;

		OCIParamGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT, (OCIError *)errhp,
			    (dvoid **)&param, (ub4)i+1);
		OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM, (dvoid *)&thisCol->dtype,
			   (ub4 *)0, (ub4)OCI_ATTR_DATA_TYPE, errhp);
		OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM, (dvoid *)&colname,
			   &colnamelen, (ub4)OCI_ATTR_NAME, errhp);
		OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM, (dvoid *)&thisCol->valuelen,
			   (ub4 *)0, (ub4)OCI_ATTR_DATA_SIZE, errhp);
    
		switch(thisCol->dtype) {
		case SQLT_DAT:
		case SQLT_DATE:
		case SQLT_TIMESTAMP:
		case SQLT_TIMESTAMP_TZ:
		case SQLT_TIMESTAMP_LTZ:
		case SQLT_INTERVAL_YM:
		case SQLT_INTERVAL_DS:
			thisCol->valuelen = 64;
			break;
		case SQLT_NUM:
		case SQLT_FLT:
		case SQLT_INT:
			thisCol->valuelen = 40;
			break;
		case SQLT_LNG:
			thisCol->valuelen = 8192;
			break;
		default:
			break;
		}
 
		thisCol->value = (char *)calloc(array_size, (size_t)thisCol->valuelen+1 );
		thisCol->indp  = (sb2 *)calloc(array_size, (size_t)sizeof(sb2) );
		OCIDefineByPos(sth,
			       &thisCol->defnp,
			       errhp, 
			       (ub4)i+1, 
			       (dvoid *)thisCol->value, (sb4)thisCol->valuelen+1, 
			       (ub2)SQLT_STR,
			       (dvoid *)&thisCol->indp[0], 
			       (ub2 *)0, 
			       (ub2 *)0, 
			       (ub4)OCI_DEFAULT);

		thisCol->colname = (char *)g_malloc(colnamelen+1);
    
		strncpy(thisCol->colname, (char *)colname, colnamelen);
		thisCol->colname[colnamelen] = '\0';
	}


	ret = ociErr(errhp, OCIStmtFetch2(sth, errhp, 1, OCI_FETCH_NEXT,
					  0, OCI_DEFAULT));
	while (OCI_SUCCESS == ret || OCI_SUCCESS_WITH_INFO == ret) {

		for(tmpnode = column_nodes; tmpnode; tmpnode = g_slist_next(tmpnode)) {
			colData     *col = (colData *) tmpnode->data;

			gtk_tree_store_append(store, &iter, parent_iter);
			gtk_tree_store_set(store, &iter, begin_col, col->colname,
					   begin_col+1, &col->value[0], -1);
		}

		ret = ociErr(errhp, OCIStmtFetch2(sth, errhp, 1, OCI_FETCH_NEXT,
						  0, OCI_DEFAULT));
	}

	if( !parent_iter ) {
		gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store) );
		g_object_unref(G_OBJECT(store));
	}

	g_slist_foreach(column_nodes, (GFunc) free_column_node, (gpointer) NULL);
	g_slist_free(column_nodes);
	OCIStmtRelease(sth, errhp, (OraText *) 0, 0, OCI_DEFAULT);
}




GList *execute_sql_to_glist(const gchar *sql,
			    ...)
{
	OCIStmt *sth = (OCIStmt *) 0;
	OCIDefine *defnp = (OCIDefine *) 0;
	OCIBind *bhp = (OCIBind *) 0;
	GList *values = (GList *) 0;
	gchar value[GLIST_FETCHSIZE][1024];
	sb2 indp[GLIST_FETCHSIZE];
	ub2 stmt_type;
	va_list ap;
	gchar *bind_value;
	ub4 sizep=sizeof(ub4);
	gint x=1, nrows=0, iters=0;
	sword ret;

	ret = ociErr(errhp, OCIStmtPrepare2(svchp, &sth, errhp,
					    (text *)sql, strlen(sql),
					    (OraText)NULL, 0,
					    OCI_NTV_SYNTAX, OCI_DEFAULT));
	if (OCI_SUCCESS != ret) {
		return 0;
	}

	OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT, 
		   (dvoid *)&stmt_type, (ub4 *)0, 
		   (ub4) OCI_ATTR_STMT_TYPE, errhp);

	if (OCI_STMT_SELECT != stmt_type) {
		iters=1;
	}

	va_start(ap, sql);
	while ((bind_value = va_arg(ap, char *)) != NULL) {

		OCIBindByPos((OCIStmt *) sth, 
			     (OCIBind **) &bhp, 
			     (OCIError *) errhp, 
			     (ub4) x++, 
			     (dvoid *) bind_value, (sb4) strlen(bind_value)+1, 
			     (ub2) SQLT_STR, 
			     (dvoid *) 0, (ub2 *)0, (ub2 *)0, (ub4)0, (ub4 *)0, 
			     (ub4) OCI_DEFAULT);
	}
	va_end(ap);

	for (x=0; x<GLIST_FETCHSIZE; x++) {
		indp[x]=0;
	}

	OCIDefineByPos((OCIStmt *) sth,
		       (OCIDefine **) &defnp,
		       (OCIError *) errhp, 
		       (ub4) 1,
		       (dvoid *) &value, sizeof(value[0]), 
		       (ub2) SQLT_STR, 
		       (dvoid *) &indp, (ub2 *)0, (ub2 *)0, 
		       (ub4) OCI_DEFAULT);

	if (OCI_SUCCESS == ociErr(errhp, OCIStmtExecute(svchp,
							sth, errhp,
							iters, 0, NULL,
							NULL, OCI_DEFAULT))) {

		if (OCI_STMT_SELECT == stmt_type) {

			while (OCI_SUCCESS == ociErr(errhp, OCIStmtFetch(sth, errhp,
									 (ub4)GLIST_FETCHSIZE,
									 (ub2)OCI_FETCH_NEXT,
									 (ub4)OCI_DEFAULT)) ) 
			{

				OCIAttrGet((dvoid *) sth, (ub4) OCI_HTYPE_STMT,
					   (dvoid *) &nrows, (ub4 *) &sizep,
					   (ub4)OCI_ATTR_ROWS_FETCHED, errhp);

				for (x=0; x<nrows; x++) {
					values = g_list_prepend(values, g_strdup(value[x]) );
				}
			}

			OCIAttrGet((dvoid *) sth, (ub4) OCI_HTYPE_STMT,
				   (dvoid *) &nrows, (ub4 *) &sizep,
				   (ub4)OCI_ATTR_ROWS_FETCHED, errhp);

			for (x=0; x<nrows; x++) {
				values = g_list_prepend(values, g_strdup(value[x]) );
			}

			values = g_list_reverse(values);
		}
	}

	OCIStmtRelease(sth, errhp, (OraText) NULL, 0, OCI_DEFAULT);
	return values;
}



void execute_sql(const gchar *sql,
		 gchar *treeview_text,
		 ...)
{
	GtkTreeView *tree_view=0;
	va_list ap;
	OCIStmt *sth=0;
	int x=1;
	gchar *bind_value=0;
	sword ret;


	ret = ociErr(errhp, OCIStmtPrepare2(svchp, &sth,
					    errhp, (text *)sql, 
					    strlen(sql), (text *)0,
					    0, OCI_NTV_SYNTAX,
					    OCI_DEFAULT));
	if (OCI_SUCCESS != ret) {
		return;
	}

#if DEBUG
	fprintf(stderr, "%s:%d  sth: %p\n",
		__FILE__, __LINE__, sth);
#endif

	va_start(ap, treeview_text);

	while ((bind_value = va_arg(ap, char *)) != NULL) {
		ociBindString(sth, errhp, x++, bind_value);
	}

	va_end(ap);

	tree_view =  GTK_TREE_VIEW(lw(treeview_text));
	if (tree_view) {
		execute_prepared_sql(sth, tree_view, NULL);
	}
}


void execute_sql_gtktreeview(const gchar  *sql,
			     GtkTreeView  *treeview,
			     ...)
{
	GtkTextBuffer *ddl_buffer=0;
	GtkTextIter iter_start, iter_end;
	va_list ap;
	OCIStmt *sth=0;
	int x=1;
	gchar *bind_value=0;
	ub4 prefetch_rows=32;
	sword ret;

	ret = ociErr(errhp, OCIStmtPrepare2(svchp, &sth, errhp,
					    (text *)sql, strlen(sql),
					    (text *)0, 0, OCI_NTV_SYNTAX,
					    OCI_DEFAULT));
	if (OCI_SUCCESS != ret) {
		return;
	}

#if DEBUG
	fprintf(stderr, "%s:%d  sth: %p\n",
		__FILE__, __LINE__, sth);
#endif

  
	if (interfaceDebugWindow) {
		ddl_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(interfaceDebugText));
		gtk_text_buffer_get_bounds(ddl_buffer, &iter_start, &iter_end);
		gtk_text_buffer_delete(ddl_buffer, &iter_start, &iter_end);
	}


	va_start(ap, treeview);
	while ((bind_value = va_arg(ap, char *)) != NULL) {

		if (interfaceDebugWindow) {
			gchar      bind_buffer[512];

			g_snprintf(bind_buffer, sizeof(bind_buffer), 
				   "variable p%d varchar2(%d);\nexec :p%d := '%s';\n",
				   x, strlen(bind_value)+2, x, bind_value);
			gtk_text_buffer_insert(ddl_buffer, &iter_start, bind_buffer, -1);
		}

		ociBindString(sth, errhp, x++, bind_value);
	}

	va_end(ap);

	if (interfaceDebugWindow) {
		gtk_text_buffer_insert(ddl_buffer, &iter_start, "\n", -1);
		gtk_text_buffer_insert(ddl_buffer, &iter_start, sql, -1);
	}

	if (treeview) {
		execute_prepared_sql(sth, treeview, NULL);
	}
}



void  execute_sql_at_iter(const gchar  *sql,
			  GtkTreeView  *treeview,
			  GtkTreeIter  *iter,
			  ...)
{
	va_list ap;
	OCIStmt *sth=0;
	int x=1;
	gchar *bind_value=0;
	sword ret;

	if (!treeview) {
		return;
	}

	ret = ociErr(errhp, OCIStmtPrepare2(svchp, &sth, errhp,
					    (text *)sql, strlen(sql),
					    (OraText)NULL, 0, OCI_NTV_SYNTAX,
					    OCI_DEFAULT));
	if (OCI_SUCCESS != ret) {
		return;
	}

	va_start(ap, iter);
	while ((bind_value = va_arg(ap, char *)) != NULL) {
		ociBindString(sth, errhp, x++, bind_value);
	}

	va_end(ap);

	execute_prepared_sql(sth, treeview, iter);
}



void execute_sql_pivotted(const gchar *sql,
			  GtkTreeView *treeview,
			  GtkTreeIter *parent,
			  gint        begin_col,
			  ...)
{
	va_list ap;
	OCIStmt *sth = (OCIStmt *)0;
	int x=1;
	gchar *bind_value;
	sword ret;

	ret = ociErr(errhp, OCIStmtPrepare2(svchp, &sth, errhp, (text *)sql,
					    strlen(sql), (OraText)NULL, 0,
					    OCI_NTV_SYNTAX, OCI_DEFAULT));
	if (OCI_SUCCESS != ret) {
		return;
	}

	va_start(ap, begin_col);

	while ((bind_value = va_arg(ap, char *)) != NULL) {
		ociBindString(sth, errhp, x++, bind_value);
	}

	va_end(ap);

	execute_prepared_sql_pivotted(sth, treeview, parent, begin_col);
}


gint query_sort_column_as_gint64(GtkTreeModel *model,
				 GtkTreeIter *a,
				 GtkTreeIter *b,
				 gpointer userdata)
{
	gint sortcol = GPOINTER_TO_INT(userdata);
	gint ret=0;
	gchar *aval=0, *bval=0;
	gint64 aval_int=G_MININT64, bval_int=G_MININT64;

	gtk_tree_model_get(model, a, sortcol, &aval, -1);
	gtk_tree_model_get(model, b, sortcol, &bval, -1);

	if (aval) {
		aval_int = strtoll(aval, NULL, 10);
	}

	if (bval) {
		bval_int = strtoll(bval, NULL, 10);
	}

	if (aval_int != bval_int) {
		ret = (aval_int > bval_int) ? 1 : -1;
	}

	g_free(aval);
	g_free(bval);
    
	return ret;
}


void output_rows(GtkTreeStore *store,
		 GtkTreeIter *parent_iter,
		 GtkTreeIter *iter,
		 GSList *coldata,
		 guint nrows,
		 GIOChannel *fio) 
{
	GString *obj_string=0;
	GSList *tmpcol;
	guint i, z;
	gsize written=0;


	for (z=0; z<nrows; z++) {
		i=0;

		if (store) {
			gtk_tree_store_append(store, iter, parent_iter);
		}

		for (tmpcol=coldata; tmpcol; tmpcol = g_slist_next(tmpcol), i++ ) {
			colData     *col = (colData *) tmpcol->data;

			if (store) {

				if (col->dtype == SQLT_NTY) {
					if (*col->obj_col_ind == 0) {

						obj_string = type_value_string(envhp, bsvchp,
									       errhp, col);
	  
						if(obj_string) {

							gtk_tree_store_set(store, iter, i,
									   obj_string->str, -1);

							g_string_free(obj_string, TRUE);
							obj_string = NULL;
						}
					}
					continue;
				}
   

				if (!g_ascii_strncasecmp(col->colname, "style", strlen(col->colname)) ||
				    !g_ascii_strncasecmp(col->colname, "weight", strlen(col->colname))
					) 
				{
					int style = atoi( &col->value[z * col->valuelen + z] );

					gtk_tree_store_set(store, iter, i, style, -1);
					continue;
				}


				if (1 == col->progress) {
					gint   progvalue = atoi( &col->value[z * col->valuelen + z ] );

					if(progvalue > 100) {
						progvalue = 100;
					}
	  
					if(progvalue < 0) {
						progvalue = 0;
					}

					gtk_tree_store_set(store, iter, i, progvalue, -1);
					continue;
				}


				if (GDK_TYPE_PIXBUF == gtk_tree_model_get_column_type(GTK_TREE_MODEL(store), i) ) {
					const GdkPixbuf *pbuf;

					pbuf = (GdkPixbuf *)g_hash_table_lookup(pixbuf_hash, &col->value[z * col->valuelen + z]);
					if (pbuf) {
						gtk_tree_store_set(store, iter, i, pbuf, -1);
					}

					continue;
				}

				gtk_tree_store_set(store, iter, i, &col->value[z * col->valuelen + z ], -1);
			}

			/*  file output */
			if(fio) {
				g_io_channel_write_chars(fio, &col->value[z * col->valuelen + z ],
							 -1, &written, (GError **) NULL );

				if (i+1 < g_slist_length(coldata)) {
					g_io_channel_write_chars(fio, ",", -1, &written, 
								 (GError **) NULL );
				}
			}
	
		}

		/* file output */
		if (fio) {
			g_io_channel_write_chars(fio, "\n", -1, &written, (GError **) NULL );
		}
	}
}


void query_mark_error(OCIStmt *sth,
		      OCIError* errhp,
		      GtkTextBuffer *buffer,
		      guint offset,
		      guint len)
{
	GtkTextIter start, end, finish, save;
	ub2 pe_offset;


	OCIAttrGet((dvoid *)sth, (ub4)OCI_HTYPE_STMT,
		   (dvoid *)&pe_offset, (ub4 *)0,
		   (ub4)OCI_ATTR_PARSE_ERROR_OFFSET, errhp);

#if DEBUG
	fprintf(stderr, "%s:%d  offset: %d  len: %d  parse_error_offset: %d\n",
		__FILE__, __LINE__, offset, len, pe_offset);
#endif


	if (!buffer) {
		return;
	}

	gtk_text_buffer_get_iter_at_offset(buffer, &start, offset + pe_offset);
	gtk_text_buffer_get_iter_at_offset(buffer, &finish, offset + len);
	end = start;
	save = start;

	while (!gtk_text_iter_starts_line(&start)
	       && !g_ascii_isspace(gtk_text_iter_get_char(&start))
	       && '(' != gtk_text_iter_get_char(&start)
	       && '=' != gtk_text_iter_get_char(&start))
	{
		gtk_text_iter_backward_char(&start);
	}

	if (g_ascii_isspace(gtk_text_iter_get_char(&start))
	    || '(' == gtk_text_iter_get_char(&start)
	    || '=' == gtk_text_iter_get_char(&start)) {
		gtk_text_iter_forward_char(&start);
	}

	while (!gtk_text_iter_ends_line(&end)
	       && !g_ascii_isspace(gtk_text_iter_get_char(&end))
	       && gtk_text_iter_get_char(&end) != '('
	       && gtk_text_iter_compare(&end, &finish))
	{
		gtk_text_iter_forward_char(&end);
	}

	gtk_text_buffer_apply_tag_by_name(buffer, "error", &start, &end);
	gtk_text_view_scroll_to_iter(getCurrentTextView(), &start, 0.1, FALSE, 0, 0);

	start = end = save;
	gtk_text_iter_forward_char(&end);
	gtk_text_buffer_apply_tag_by_name(buffer, "red", &start, &end);
}


static void query_set_status_icon(gchar *pbf_name,
				  gchar *tip)
{

#if GTK_CHECK_VERSION(2,10,0)
	gtk_status_icon_set_from_pixbuf(status_icon, query_get_pixbuf(pbf_name));
	gtk_status_icon_set_tooltip(status_icon, tip);
#endif
}


void query_start_interface_update(const gchar *sql,
				  GtkWidget *page)
{
	GtkTextBuffer *buffer;
	GtkTextIter begin, end;
	GtkTooltips *execute_tip = gtk_tooltips_new();
	gchar tiptext[256];


	g_snprintf(tiptext, sizeof(tiptext), "GTKO Executing:\n%s", sql);
	gtk_tooltips_set_tip(execute_tip, lw("executeEvent"),
			     tiptext, NULL);
	g_object_set_data_full(G_OBJECT(lw("resList")), "last_query",
			       (gpointer) g_strdup(sql), g_free );
	gtk_image_set_from_stock(GTK_IMAGE(lw("statusImage")), "gtk-execute",
				 GTK_ICON_SIZE_SMALL_TOOLBAR);
   
	buffer = getCurrentSqlBuffer(-1);
	gtk_text_buffer_get_bounds(buffer, &begin, &end);
	gtk_text_buffer_remove_tag_by_name(buffer, "error",
					   &begin, &end);
	gtk_text_buffer_remove_tag_by_name(buffer, "red",
					   &begin, &end);

	if (page) {
		gchar     exetext[128];
  
		pagelabel = GTK_LABEL(command_tab_get_page_label());
		if(pagelabel) {
			snprintf(pagetext, sizeof(pagetext),
				 gtk_label_get_text(pagelabel) );
			snprintf(exetext, sizeof(exetext), 
				 "<span color=\"green4\"><b>%s</b></span>",
				 pagetext);
			gtk_label_set_markup(pagelabel, exetext);
		}
	}
	else {
		pagelabel = NULL;
	}

	gtk_widget_set_sensitive(lw("breakButton"), TRUE);
	gtk_label_set_text(GTK_LABEL(lw("statusLabel")), "Executing query....");
	gtk_label_set_text(GTK_LABEL(lw("elapsedLabel")), "00:00");

	query_set_status_icon("gtko_active_pixbuf", tiptext);
}


void query_error_interface_update(qCommand *qs,
				  OCIError *errhp,
				  sword retval)
{
	gchar errbuf[512];
	sword errcode = (sb4) 0;


	if (pagelabel) {
		gtk_label_set_markup(pagelabel, pagetext);
	}

	if (1013 != retval 
	    && 1008 != retval 
	    && 24344 != retval)
	{
		if (qs->offset >= 0) {
			query_mark_error(qs->sth, errhp, 
					 qs->buffer, qs->offset,
					 qs->len);
		}
	}

	OCIErrorGet((dvoid *)errhp, (ub4)1, (text *)NULL,
		    &errcode, (text *)errbuf,
		    (ub4) sizeof(errbuf), (ub4)OCI_HTYPE_ERROR);
	g_strchomp(errbuf);

	gtk_widget_set_sensitive(lw( "executeButton"), TRUE);
	gtk_widget_set_sensitive(lw( "breakButton"), FALSE);
	query_set_status_icon("gtko_pixbuf", errbuf);
}


void query_select_interface_update()
{

	if (pagelabel) {
		gtk_label_set_markup(pagelabel, pagetext);
	}

	gtk_label_set_text(GTK_LABEL(lw("elapsedLabel")), " ");
	gtk_widget_set_sensitive (lw( "breakButton"), FALSE);
	gtk_widget_set_sensitive (lw( "mainNotebook"), TRUE);
	gtk_image_set_from_stock(GTK_IMAGE(lw( "statusImage")), "nothing",
				 GTK_ICON_SIZE_SMALL_TOOLBAR);

	query_set_status_icon("gtko_pixbuf", "GTKO: Query done");
}



void query_finish_interface_update(OCIEnv *envhp,
				   OCIError *errhp,
				   OCISvcCtx *svc,
				   ub2 stmt_type,
				   gint nrows,
				   gdouble elapsed)
{
	gchar resbuf[512];
  
	gtk_image_set_from_stock(GTK_IMAGE(lw( "statusImage")), "nothing",
				 GTK_ICON_SIZE_SMALL_TOOLBAR);

	switch (stmt_type) {
	case 1:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Table Created</b>  Elapsed: %.03f seconds",
			 elapsed );
		break;
	case 3:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>%d rows inserted</b> Elapsed: %.03f seconds",
			 nrows, elapsed );
		break;
	case 5:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>%d rows updated</b>  Elapsed: %.03f seconds",
			 nrows, elapsed );
		break;
	case 7:
		snprintf(resbuf, sizeof(resbuf), "<b>View Dropped</b>");
		break;
	case 8:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Table Dropped</b>  Elapsed: %.03f seconds",
			 elapsed);
		break;
	case 9:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>%d rows deleted</b> Elapsed: %.03f seconds",
			 nrows, elapsed );
		break;
	case 10:
		snprintf(resbuf, sizeof(resbuf), "<b>View Created</b>");
		break;
	case 11:
		snprintf(resbuf, sizeof(resbuf), "<b>User Dropped</b>");
		break;
	case 12:
		snprintf(resbuf, sizeof(resbuf), "<b>Role Created</b>");
		break;
	case 13:
		snprintf(resbuf, sizeof(resbuf), "<b>Sequence Created</b>");
		break;
	case 14:
		snprintf(resbuf, sizeof(resbuf), "<b>Sequence Altered</b>");
		break;
	case 16:
		snprintf(resbuf, sizeof(resbuf), "<b>Sequence Dropped</b>");
		break;
	case 18:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Cluster Created</b>  Elapsed: %.03f seconds",
			 elapsed);
		break;
	case 19:
		snprintf(resbuf, sizeof(resbuf), "<b>User Created</b>");
		break;
	case 20:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Index Created</b>  Elapsed: %.03f seconds",
			 elapsed );
		break;
	case 21:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Index Dropped</b>  Elapsed: %.03f seconds",
			 elapsed );
		break;
	case 24:
		snprintf(resbuf, sizeof(resbuf), "<b>Procedure Created</b>");
		break;
	case 25:
		snprintf(resbuf, sizeof(resbuf), "<b>Procedure Altered</b>");
		break;
	case 26:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Table Altered</b> Elapsed: %.03f seconds",
			 elapsed);
		break;
	case 27:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Explained</b>  Elapsed: %.03f seconds", elapsed);
		break;
	case 34:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>PL/SQL Executed</b>  Elapsed: %.03f seconds",
			 elapsed);
		show_dbms_output(envhp, errhp, svc);
		break;
	case 41:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Index Altered</b>  Elapsed: %.03f seconds",
			 elapsed);
		break;
	case 52:
		snprintf(resbuf, sizeof(resbuf), "<b>Session Altered</b>");
		break;
	case 77:
		snprintf(resbuf, sizeof(resbuf), "<b>Type Created</b>" );
		break;
	case 78:
		snprintf(resbuf, sizeof(resbuf), "<b>Type Dropped</b>" );
		break;
	case 85:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Table Truncated</b>  Elapsed: %.03f seconds",
			 elapsed);
		break;
	case 86:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Cluster Truncated</b>  Elapsed: %.03f seconds",
			 elapsed);
		break;
	case 91:
		snprintf(resbuf, sizeof(resbuf), "<b>Function Created</b>" );
		break;
	case 93:
		snprintf(resbuf, sizeof(resbuf), "<b>Function Dropped</b>" );
		break;
	case 94:
		snprintf(resbuf, sizeof(resbuf), "<b>Package Created</b>" );
		break;
	case 96:
		snprintf(resbuf, sizeof(resbuf), "<b>Package Dropped</b>" );
		break;
	case 97:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Package Body Created</b>  Elapsed: %.03f seconds",
			 elapsed );
		break;
	case 99:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Package Body Dropped</b>  Elapsed: %.03f seconds",
			 elapsed);
		break;
	case 157:
		snprintf(resbuf, sizeof(resbuf), "<b>Directory Created</b>");
		break;
	case 158:
		snprintf(resbuf, sizeof(resbuf), "<b>Directory Dropped</b>");
		break;
	case 189:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Merged</b>  Elapsed: %.03f seconds", elapsed);
		break;
	  
	default:
		snprintf(resbuf, sizeof(resbuf),
			 "<b>Successful</b>  Elapsed: %.03f seconds", elapsed);
		break;
	}
	gtk_label_set_markup(GTK_LABEL(lw( "statusLabel")), resbuf);

	if(pagelabel)
		gtk_label_set_markup(pagelabel, pagetext);
 
	if(stmt_type==3 
	   || stmt_type==5 
	   || stmt_type==9 
	   || stmt_type==34 
	   || stmt_type==189)
	{
		gtk_widget_set_sensitive (lw( "commitButton"), TRUE);
		gtk_widget_set_sensitive (lw( "rollbackButton"), TRUE);
	}
	else {
		gtk_widget_set_sensitive (lw( "commitButton"), FALSE);
		gtk_widget_set_sensitive (lw( "rollbackButton"), FALSE);
	}
      
	gtk_widget_set_sensitive (lw( "executeButton"), TRUE);
	gtk_widget_set_sensitive (lw( "breakButton"), FALSE);
	gtk_widget_set_sensitive (lw( "mainNotebook"), TRUE);
	query_set_status_icon("gtko_pixbuf", resbuf);


}


GString *oci_piecewise_fetch_to_gstring(OCIStmt *sth)
{
	OCIDefine *defnp, *piecedef;
	gchar buffer[1024];
	ub1 piece=0;
	ub4 alenp = (ub4) sizeof(buffer);
	GString *gstr = NULL;
	sword ret=OCI_SUCCESS;


	OCIDefineByPos((OCIStmt *) sth, (OCIDefine **) &defnp,
		       (OCIError *)errhp, (ub4) 1, (dvoid *)0,
		       sizeof(buffer) * 100000, (ub2)SQLT_STR, 
		       (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DYNAMIC_FETCH);
 
	ret = OCIStmtFetch2(sth, errhp, 1, OCI_FETCH_NEXT,
			    0, OCI_DEFAULT);

	gstr = g_string_new_len(" ", 1);
	while (OCI_NEED_DATA == ret) {
		ub1 inout;
		sb2 indp=0;
		ub2 rcode=0;
		ub4 htype, row, index;

		if (piece) {
			gstr = g_string_append_len(gstr, buffer, alenp);
		}

		alenp = (ub4) sizeof(buffer);
		OCIStmtGetPieceInfo((OCIStmt *)sth, (OCIError *)errhp,
				    (dvoid *) &piecedef, &htype,
				    &inout, &row, &index, &piece);

		OCIStmtSetPieceInfo((dvoid *) piecedef, htype,
				    errhp, (dvoid *)&buffer, &alenp, 
				    piece, &indp, &rcode);

#if DEBUG
		fprintf(stderr, "%s:%d  piecewise: alenp=%d piece=%d\n",
			__FILE__, __LINE__, alenp, piece);
#endif

		ret = OCIStmtFetch2(sth, errhp, 1, OCI_FETCH_NEXT,
				    0, OCI_DEFAULT);

		if (OCI_SUCCESS == ret) {
			ret = OCIStmtFetch2(sth, errhp, 1,
					    OCI_FETCH_NEXT,
					    0, OCI_DEFAULT);
		}
	}

	if(piece) {
		gstr = g_string_append_len(gstr, buffer, alenp);
	}

	return(gstr);
}


void on_display_browser_sql_window_activate (GtkMenuItem *menuitem,
					     gpointer user_data)
{
	if (!interfaceDebugWindow) {
		interfaceDebugWindow = create_interfaceDebugWindow();
		gtk_widget_show(interfaceDebugWindow);
	}
}
