/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <oci.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "dbms_output.h"
#include "gutil.h"
#include "statement.h"
#include "orautil.h"
#include "query.h"
#include "trace.h"
#include "table.h"
#include "index.h"
#include "plobject.h"
#include "oracle_type.h"
#include "command_tab.h"
#include "appconfig.h"
#include "bind_variable.h"
#include "query_async.h"

extern GtkWidget       *window1;

extern OCIError        *errhp;
extern OCIEnv          *envhp;
extern OCISvcCtx       *svchp;
extern OCISvcCtx       *bsvchp;
extern OCISession      *bseshp;

static   GAsyncQueue   *request_queue=0;
static   GTimer        *timer;
static   guint         running=0;


void on_connect1_activate(GtkMenuItem *, gpointer);
   



static sword ocierr_async(OCIError   *ehp, 
			  sword      ostat)
{
	gchar mbuf[1024];
	sword errcode = 0;


	if (OCI_SUCCESS != ostat) {

		switch (ostat) {

		case OCI_SUCCESS_WITH_INFO:
		case OCI_ERROR:

			OCIErrorGet((dvoid *)ehp, (ub4)1,
				    (text *)NULL, &errcode,
				    (text *)mbuf, (ub4) sizeof(mbuf),
				    (ub4)OCI_HTYPE_ERROR);

			if(strlen(mbuf) > 2)
				mbuf[strlen(mbuf) - 1] = '\0';

			switch(errcode) {

			case 28:
			case 3113:
			case 3114:
			case 3135:
				on_connect1_activate(NULL, NULL);
				break;

			default:
				break;
			}
      
			break;

		case OCI_INVALID_HANDLE:
			snprintf(mbuf, sizeof(mbuf), "Error: OCI_INVALID_HANDLE");
			break;

		case OCI_NEED_DATA:
		case OCI_STILL_EXECUTING:
		case OCI_CONTINUE:
		case OCI_NO_DATA:
			return(ostat);

		default:
			break;
		}

		gdk_threads_enter();
		gtk_label_set_text(GTK_LABEL(lw("statusLabel")), mbuf);
		gtk_image_set_from_stock(GTK_IMAGE(lw("statusImage")),
					 "gtk-dialog-error",
					 GTK_ICON_SIZE_SMALL_TOOLBAR);
		gdk_threads_leave();
	}
	return(ostat);
}




static gboolean exe_timeout(gpointer data) 
{
	static gchar elabuf[128];
	static gdouble elapsed;
	static gulong micro;
  
	if (running) {

		elapsed = g_timer_elapsed(timer, &micro);
		snprintf(elabuf, sizeof(elabuf), 
			 "%02d:%02d", (int)elapsed/60, (int)elapsed%60);
		setLabel("elapsedLabel", elabuf);
		return(TRUE);
	}
	else {

		setLabel("elapsedLabel", " ");
		return(FALSE);
	}
}



void query_async_init()
{

	request_queue = g_async_queue_new();
	g_thread_create((GThreadFunc) query_async_thread,
			(gpointer)request_queue, TRUE, (GError **)NULL);
}



void query_async_push(qCommand *qc)
{
	g_async_queue_push(request_queue, (gpointer *) qc);
}



void query_async_execute()
{
	GSList *stlist=0;
	GSList *tmpstlist=0;

	if ((stlist = get_statements(getCurrentTextView())) == NULL) {
		return;
	}

	for (tmpstlist=stlist; tmpstlist; tmpstlist=g_slist_next(tmpstlist) ) {
		stmt *st = (stmt *) tmpstlist->data;
		qCommand *qs;

		qs = alloc_qCommand("resList");
		qs->sql = g_strdup(st->sql);
		qs->offset = st->offset;
		qs->len = strlen(st->sql);
		qs->page = command_tab_get_page();
		qs->buffer = getCurrentSqlBuffer(-1);

		OCIStmtPrepare(qs->sth, errhp, (text *)qs->sql,
			       strlen(qs->sql), OCI_NTV_SYNTAX,
			       OCI_PREP2_GET_PLSQL_WARNINGS);

		bind_variable_set(qs->sth, errhp, qs->offset);

		g_async_queue_push(request_queue, (gpointer *) qs);
		g_free(st->sql);
		g_free(st);
	}

	g_slist_free(stlist);
	return;
}



void query_async_break()
{
	ocierr_async(errhp, OCIBreak(bsvchp, errhp));
}


void plsql_compilation_error(const gchar *sql,
			     const gchar *schema)
{
	GArray *toks=0;
	gchar *upper_name=0;
	gchar *val;
	gchar **ownobj=0;
	guint y=0;
	guint objind=0;
	static gchar *plsql_error_sql =
		"select line \"Line #\", position \"Position\", \
                  text \"Error text\", 'red4' \"foreground\" \
                  from all_errors \
                  where owner=upper(:p2) and name=:p1 order by sequence";

  
	toks = g_strsplit_garray(sql, " \n\t", -1);

	val = g_array_index(toks, gchar *, 1);
	
	if (!g_ascii_strncasecmp(val, "PROCEDURE", 9) 
	    || !g_ascii_strncasecmp(val, "FUNCTION", 8)
	    || !g_ascii_strncasecmp(val, "PACKAGE", 7)
	    || !g_ascii_strncasecmp(val, "TYPE", 4)) {
		
		objind = 2;
	}

	if (!g_ascii_strncasecmp(val, "OR", 2)) {

		if (!g_ascii_strncasecmp(g_array_index(toks, gchar *, 4), "BODY", 4)) {
			objind = 5;
		}
		else {
			objind = 4;
		}
	}	
		
	upper_name = g_ascii_strup(g_array_index(toks, gchar *, objind), -1);

	for (y=0; y<strlen(upper_name); y++)
		if(upper_name[y] == '"')
			upper_name[y] = ' ';
	    
	ownobj = g_strsplit_set(upper_name, ".", 2);

	execute_sql_gtktreeview(plsql_error_sql, 
				GTK_TREE_VIEW(lw("resList")),
				g_strv_length(ownobj)==2 ? g_strstrip(ownobj[0]) : schema,
				g_strv_length(ownobj)==2 ? g_strstrip(ownobj[1]) : upper_name, 
				NULL);
	g_free(upper_name);
	g_strfreev(ownobj);
	g_strsplit_garray_free(toks);
}



void query_async_thread(gpointer data)
{

	qCommand *qs;
	GtkTreeView *list = GTK_TREE_VIEW(lw("resList"));
	GtkLabel *status_label = GTK_LABEL(lw("statusLabel"));
	GtkCellRenderer *crend=0;
	GtkTreeViewColumn *column=0;
	GtkTreeIter iter;
	GtkTreeStore *store=0;
	GtkTreeSortable *sortable=0;
	GSList *column_nodes=0;
	GSList *int_nodes=0;
	GType *treecol_types;
	GIOChannel *fio;

	OCIParam *param = (OCIParam *)0;
	colData *thisCol=0;
	sword ret = OCI_SUCCESS;
	ub2 stmt_type, retval;
	ub4 ncol, sizep=sizeof(ub4);

	guint array_size;
	gint i, nrows=0, tnrows=0, iters=0;
	gulong microseconds;
	gdouble firstrow=0, allrows=0;
	gboolean session_trace=FALSE, output_screen=TRUE, output_file=TRUE;
	gchar resbuf[64];
	gsize written=0;

	timer = g_timer_new();

	while (1) {
 
#if DEBUG
		fprintf(stderr, "query_async.c: waiting on message\n");
#endif
		qs = (qCommand *) g_async_queue_pop(request_queue);

		array_size = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw( "arraySizeSpin")));
		session_trace = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("traceCheck")));
		output_screen = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("outputScreenCheck")));
		output_file   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("outputFileCheck")));
		fio = NULL;

		gdk_threads_enter();
		query_start_interface_update(qs->sql, qs->page);
		gdk_threads_leave();
   
		OCIAttrGet((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT, 
			   (dvoid *)&stmt_type, (ub4 *)0,
			   (ub4)OCI_ATTR_STMT_TYPE, errhp);

		if (OCI_STMT_SELECT != stmt_type) {
			iters=1;
		}
		else {
			iters=0;
		}

		if (session_trace) {
			session_trace_mark(svchp, errhp); 
		}

		running=1;
		g_timeout_add(1000, exe_timeout, (gpointer) NULL);
		g_timer_start(timer);


		/*   EXECUTE  */
		ret = ocierr_async(errhp, OCIStmtExecute(bsvchp, qs->sth,
							 errhp, (ub4)iters,
							 (ub4)0, (OCISnapshot *)0,
							 (OCISnapshot *)0, (ub4)OCI_DEFAULT)
			);

		if (OCI_SUCCESS != ret)  {
			gchar errbuf[1024];
			sword errcode = 0;

			OCIErrorGet((dvoid *)errhp, (ub4)1,
				    (text *)NULL, &errcode,
				    (text *)errbuf, (ub4) sizeof(errbuf),
				    (ub4)OCI_HTYPE_ERROR);

			gdk_threads_enter();
			running=0;

			query_error_interface_update(qs, errhp, errcode);

			if (24344 == errcode) {
				gchar *schema = gtk_combo_box_get_active_text(GTK_COMBO_BOX(lw("schemaCombo")));
				plsql_compilation_error(qs->sql, schema);
			}

			gdk_threads_leave();

			OCIHandleFree((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT);
			g_free(qs->sql);
			g_free(qs);
		}
		else {
			if(OCI_STMT_SELECT != stmt_type) {
				gdouble    elapsed;

				running=0;
				firstrow = g_timer_elapsed(timer, &microseconds);
				elapsed = (float)firstrow+(1/(float)microseconds);

				OCIAttrGet((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT,
					   (dvoid *)&stmt_type, (ub4 *)0, (ub4)OCI_ATTR_SQLFNCODE, errhp);
				OCIAttrGet((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT,
					   (dvoid *)&nrows, (ub4 *)0, (ub4)OCI_ATTR_ROW_COUNT, errhp);

				gdk_threads_enter();

				query_finish_interface_update(envhp, errhp, bsvchp,
							      stmt_type, nrows, elapsed);

				if(session_trace) 
					session_trace_display(svchp, errhp, "traceList", elapsed, nrows);

				gdk_threads_leave();

				OCIHandleFree((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT);
				g_free(qs->sql);
				g_free(qs);

			}
			else {
				const gchar *font = config_get("listfont");

				gdk_threads_enter();

				if(output_screen) {
					clear_gtktreeview(GTK_TREE_VIEW(lw("resList")));
					gtk_tree_view_set_model(GTK_TREE_VIEW(lw("resList")), NULL);
				}

				if(output_file) {
					gchar   *outfile = (gchar *) g_object_get_data(G_OBJECT(lw("outputFileButton")), "filename");

					if(outfile) {
						fio = g_io_channel_new_file(outfile, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lw("outputAppendCheck"))) ? "a" : "w", (GError **)NULL);
					}
				}


				OCIAttrGet((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT, (dvoid *)&ncol,
					   (ub4 *)0, (ub4)OCI_ATTR_PARAM_COUNT, errhp);
				treecol_types = g_new0(GType, ncol);

				for(i=0; i < ncol; i++) {
					text             *colname=0;
					gint             underscores;
					ub4              colnamelen, cnt;
					text             *p, *p1;

					thisCol = (colData *)g_malloc(sizeof(colData) );
					thisCol->value = NULL;
					thisCol->valuelen = 0;
					thisCol->dtype = 0;
					thisCol->type_data = 0;
					thisCol->type_ref = 0;
					thisCol->progress = 0;

					column_nodes = g_slist_append(column_nodes,
								      (gpointer) thisCol);

					colnamelen = 0;
					OCIParamGet(qs->sth, OCI_HTYPE_STMT, errhp, (dvoid **)&param, i+1);
					OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM,
						   (dvoid *)&thisCol->dtype,
						   (ub4 *)0, (ub4)OCI_ATTR_DATA_TYPE, errhp);
					OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM, (dvoid *)&colname,
						   &colnamelen, (ub4)OCI_ATTR_NAME, errhp);
					OCIAttrGet((dvoid *)param, (ub4)OCI_DTYPE_PARAM,
						   (dvoid *)&thisCol->valuelen,
						   (ub4 *)0, (ub4)OCI_ATTR_DATA_SIZE, errhp);

					underscores = 0;
					for(p=colname, cnt=0; cnt < colnamelen; cnt++, p++)
						if(*p=='_')
							underscores++;

					switch(thisCol->dtype) {
					case SQLT_DAT:
					case SQLT_DATE:
					case SQLT_TIMESTAMP:
					case SQLT_TIMESTAMP_TZ:
					case SQLT_TIMESTAMP_LTZ:
					case SQLT_INTERVAL_YM:
					case SQLT_INTERVAL_DS:
						thisCol->valuelen = 64;
						treecol_types[i] = G_TYPE_STRING;
						break;
					case SQLT_NUM:
					case SQLT_FLT:
					case SQLT_INT:
						thisCol->valuelen = 64;
						treecol_types[i] = G_TYPE_STRING;
						int_nodes = g_slist_append(int_nodes, GUINT_TO_POINTER(i));
						break;
					case SQLT_RDD:
						thisCol->valuelen=40;
						treecol_types[i] = G_TYPE_STRING;
						break;
					case SQLT_LNG:
						thisCol->valuelen = 2048;
						treecol_types[i] = G_TYPE_STRING;
						break;
					case SQLT_BIN:
					case SQLT_LBI:
					case SQLT_LVB:
						thisCol->valuelen = thisCol->valuelen * 2;
						treecol_types[i] = G_TYPE_STRING;
						break;
					default:
						treecol_types[i] = G_TYPE_STRING;
						break;
					}


					thisCol->indp  = (sb2 *) calloc(array_size, (size_t)sizeof(sb2) );

					if (SQLT_NTY==thisCol->dtype) {
						array_size=1;
						define_oracle_object_type(envhp, bsvchp, errhp, qs->sth, thisCol, i+1);
					}
					else {

						thisCol->value = (char *) calloc(array_size, (size_t)thisCol->valuelen+1 );
						OCIDefineByPos(qs->sth, &thisCol->defnp, 
							       errhp, 
							       (ub4)i+1, 
							       (dvoid *)thisCol->value, 
							       (sb4)thisCol->valuelen+1, 
							       (ub2)SQLT_STR,
							       (dvoid *)&thisCol->indp[0], 
							       (ub2 *)0, 
							       (ub2 *)0, 
							       (ub4)OCI_DEFAULT);
					}

	  
					thisCol->colname = (char *)g_malloc(colnamelen+underscores+1);
					if (underscores) {
						for (p=colname, p1=(text *)thisCol->colname, cnt=0; 
						     cnt < colnamelen; 
						     cnt++) {
							if (*p=='_')
								*p1++ = '_';
							*p1++ = *p++;
						}
					}
					else {
						strncpy(thisCol->colname, (char *)colname, colnamelen);
					}
					thisCol->colname[colnamelen+underscores] = '\0';
#if DEBUG
					fprintf(stderr, "query_async.c:   %s\n", thisCol->colname);
#endif

					if(output_file) {
						g_io_channel_write_chars(fio, (char *)colname,
									 colnamelen, &written,
									 (GError **) NULL );

						if(i+1 < ncol)
							g_io_channel_write_chars(fio, ",", -1, 
										 &written,
										 (GError **) NULL );
					}

					if(output_screen) {

						if(!g_ascii_strncasecmp(thisCol->colname, "foreground", 10) || 
						   !g_ascii_strncasecmp(thisCol->colname, "background", 10) || 
						   !g_ascii_strncasecmp(thisCol->colname, "family", 6)) {
							gchar *lower = g_ascii_strdown(thisCol->colname, -1);

							gtk_tree_view_column_add_attribute(column, crend, lower, i);
							g_free(lower);
						}
						else if(!g_ascii_strncasecmp(thisCol->colname, "style", 5) ) {
	     
							gtk_tree_view_column_add_attribute(column, crend, "style", i);
							treecol_types[i] = G_TYPE_INT;
						}
						else {
	     
							crend = gtk_cell_renderer_text_new();
	     
							if(font)
								g_object_set(G_OBJECT(crend), "font", font, NULL);
	  
							g_object_set(G_OBJECT(crend), "ypad", 0, NULL);

							column = gtk_tree_view_column_new_with_attributes(thisCol->colname, 
													  crend, "text", i, NULL);

							gtk_tree_view_append_column(list, column);
							gtk_tree_view_column_set_sort_column_id(column, i);
							g_object_set_data_full(G_OBJECT(column), "oracle_column_name",
									       (gpointer) g_strndup((gchar *)colname,
												    colnamelen), g_free);
						}
					}

				}
				gdk_threads_leave();

				if(output_file)
					g_io_channel_write_chars(fio, "\n", -1, &written, (GError **) NULL );


				if(output_screen) {
					GSList     *tmpnode=0;

					store = gtk_tree_store_newv((gint)ncol, treecol_types);
					sortable = GTK_TREE_SORTABLE(store);
	  
					for(tmpnode=int_nodes; tmpnode; tmpnode=g_slist_next(tmpnode)) {
						gint     colnum = GPOINTER_TO_INT(tmpnode->data);

						gtk_tree_sortable_set_sort_func(sortable, colnum,
										query_sort_column_as_gint64,
										GUINT_TO_POINTER(colnum), NULL);
					}

				}
				else
					store = NULL;

				g_slist_free(int_nodes);
				int_nodes=0;

				tnrows = 0;
				retval = ocierr_async(errhp, OCIStmtFetch2(qs->sth, errhp, (ub4) array_size,
									   (ub2) OCI_FETCH_NEXT, (sb4)0,
									   (ub4)OCI_DEFAULT));
				while( retval == OCI_SUCCESS || retval==OCI_SUCCESS_WITH_INFO) {

					if( !tnrows )
						firstrow = g_timer_elapsed(timer, &microseconds);

					OCIAttrGet((dvoid *) qs->sth, (ub4) OCI_HTYPE_STMT, (dvoid *) &nrows,
						   (ub4 *) &sizep, (ub4)OCI_ATTR_ROWS_FETCHED, errhp);
	 
					output_rows(store, (GtkTreeIter *)NULL, &iter, column_nodes, nrows, fio);
   
					tnrows += nrows;
					snprintf(resbuf, sizeof(resbuf), "Fetched: %d", tnrows);

					gdk_threads_enter();
					gtk_label_set_markup(status_label, resbuf);
					gdk_threads_leave();

					retval = ocierr_async(errhp, OCIStmtFetch2(qs->sth, errhp,
										   (ub4)array_size,
										   (ub2)OCI_FETCH_NEXT,
										   (sb4)0, (ub4)OCI_DEFAULT));
				}
       

				if( !tnrows ) 
					firstrow = g_timer_elapsed(timer, &microseconds);

				allrows = g_timer_elapsed(timer, &microseconds);

				OCIAttrGet((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT,
					   (dvoid *)&nrows, (ub4 *)&sizep,
					   (ub4)OCI_ATTR_ROWS_FETCHED, errhp);
				tnrows += nrows;

				gdk_threads_enter();
				running=0;

				/* successfully fetched all rows */
				if( retval == OCI_SUCCESS || retval == 100) {

					snprintf(resbuf, sizeof(resbuf), "<b>%d rows</b> %.02f sec (First: %.02f sec)",
						 tnrows, allrows, firstrow );
					gtk_label_set_markup(status_label, resbuf);
				}
				/* some kind of error during fetch, usually user cancelled */
				else {
	  
				}

				output_rows(store, (GtkTreeIter *)NULL, &iter, column_nodes, nrows, fio);

				if(session_trace)
					session_trace_display(svchp, errhp, "traceList", allrows, tnrows);
      
				if(output_screen) {
					gtk_tree_view_set_model(list, GTK_TREE_MODEL(store) );
					g_object_unref(G_OBJECT(store));
				}
	
				if(output_file) {
					g_io_channel_shutdown(fio, TRUE, (GError **)NULL);
					g_io_channel_unref(fio);
				}

				query_select_interface_update();

				gdk_threads_leave();

				g_free(treecol_types);
				OCIHandleFree((dvoid *)qs->sth, (ub4)OCI_HTYPE_STMT);
				g_free(qs->sql);
				g_free(qs);

				g_slist_foreach(column_nodes, (GFunc) free_column_node, (gpointer) NULL);
				g_slist_free(column_nodes);
				column_nodes=0;
  	
			}
		}
	}
}




