/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#include <stdio.h>
#include <string.h>
#include <glib.h>

#include "result_clip.h"
#include "gutil.h"
#include "interface.h"
#include "command_tab.h"
#include "appconfig.h"


static GtkWidget    *clipWindow = NULL;
static gboolean     titles = TRUE;


void popdown_clip_window() 
{

	if(clipWindow) {
		gtk_widget_destroy(clipWindow);
		clipWindow = NULL;
	}
}

void set_clip_titles(gboolean set) 
{
	titles = set;
}


static void insert_clip_rows(GtkTreeView *tree,
			     GtkTextBuffer *buffer,
			     GtkTreeIter *iter,
			     GtkTextIter *cursor)
 {
	 GtkTreeModel *model = gtk_tree_view_get_model(tree);
	 GList *cols;
	 guint ncols;
	 guint x;
	 gchar *value;
	 gint64 int_val;
	 gchar char_val[64];


	 cols  = gtk_tree_view_get_columns(tree);
	 ncols = g_list_length(cols);

	 for (x=0; x < ncols; x++) {
		 GList *cl = g_list_nth(cols, x);
		 guint y, cwidth;
		 const gchar *title;

		 title = gtk_tree_view_column_get_title((GtkTreeViewColumn *)cl->data);

		 cwidth =  GPOINTER_TO_UINT(g_object_get_data(cl->data, "cwidth"));
  
		 switch (gtk_tree_model_get_column_type(model, x)) {

		 case G_TYPE_INT64:
			 gtk_tree_model_get(model, iter, x, &int_val, -1);
			 g_snprintf(char_val, sizeof(char_val), "%-20lld", int_val);
			 value = g_strdup(char_val);
			 break;

		 case G_TYPE_STRING:
			 gtk_tree_model_get(model, iter, x, &value, -1);
			 break;

		 default:
			 value = g_strdup(" ");
			 break;
		 }
     
		 gtk_text_buffer_insert(buffer, cursor, value ? value : " ", -1);
      
		 if (value) {
			 y=strlen(value);
		 }
		 else {
			 y=1;
		 }

		 if (G_TYPE_STRING == gtk_tree_model_get_column_type(model, x)) {
			 for( ; y < cwidth+1; y++) {
				 gtk_text_buffer_insert(buffer, cursor, " ", -1);
			 }
		 }
		 g_free(value);
	 }

	 gtk_text_buffer_insert(buffer, cursor, "\n", -1);
	 g_list_free(cols);
 }


static GtkTextBuffer *open_clip_window(void)
{
	GtkTextBuffer   *buffer;


	if (!clipWindow) {
		const gchar *font = config_get("textfont");
		PangoFontDescription *font_desc;


		clipWindow = create_clipWindow();
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(clipWindow, "clipText")));

		if (font) {
			font_desc = pango_font_description_from_string(font);
			gtk_widget_modify_font(GTK_WIDGET(GTK_TEXT_VIEW(lookup_widget(clipWindow, "clipText"))),
					       font_desc);
			pango_font_description_free(font_desc);
		}

		gtk_widget_show(clipWindow);
	}
	else {
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(clipWindow, "clipText")));
	}
	return(buffer);
}



void clip_results(GtkTreeView *tv,
		  guint mode) 
{
	GtkTreeModel *model = gtk_tree_view_get_model(tv);
	GtkTreeIter iter;
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
	GList *cols = NULL;
	guint ncols=0, x, ret;
	gchar *value;


	if (!model) {
		return;
	}

	cols  = gtk_tree_view_get_columns(tv);
	ncols = g_list_length(cols);

	if (!ncols) {
		return;
	}

	buffer = open_clip_window();
	gtk_text_buffer_get_end_iter(buffer, &end);

	for (x=0; x < ncols; x++) {
		GList *cl = g_list_nth(cols, x);
		guint ctwidth=0, cwidth;

		cwidth = 0;

		for(ret=gtk_tree_model_get_iter_first(model, &iter);
		    ret!=FALSE;
		    ret=gtk_tree_model_iter_next(model, &iter) ) {
			static guint      width;

			switch (gtk_tree_model_get_column_type(model, x)) {

			case G_TYPE_INT64:
				width = 20;
				break;

			case G_TYPE_STRING:
	
				gtk_tree_model_get(model, &iter, x, &value, -1);

				if(value) {
					width = strlen(value);
					g_free(value);
					value=0;
				}
				break;

			default:
				break;

			}

			if(width > cwidth)
				cwidth = width;
		}

		ctwidth = strlen(gtk_tree_view_column_get_title((GtkTreeViewColumn *)cl->data));

		if (ctwidth > cwidth) {
			cwidth = ctwidth;
		}

		g_object_set_data(G_OBJECT(cl->data), "cwidth", GUINT_TO_POINTER(cwidth) );

		if (titles) {
			gchar title_buf[256];

			g_snprintf(title_buf, sizeof(title_buf), "%-*s", 
				   cwidth, 
				   gtk_tree_view_column_get_title((GtkTreeViewColumn *)cl->data) );
			gtk_text_buffer_insert(buffer, &end, title_buf, -1);
		}
	}

	if (titles) {
		gtk_text_buffer_insert(buffer, &end, "\n", -1);
    
		for (x=0; x < ncols; x++) {
			GList *cl = g_list_nth(cols, x);
			guint cwidth, y;

			if (!gtk_tree_view_column_get_visible((GtkTreeViewColumn *) cl->data)) {
				continue;
			}

			cwidth = GPOINTER_TO_UINT(g_object_get_data(cl->data, "cwidth"));

			for (y=1; y < cwidth+1; y++) {
				gtk_text_buffer_insert(buffer, &end, "-", -1);
			}

		}
		gtk_text_buffer_insert(buffer, &end, "\n", -1);
	}
    


	if (mode) {
		GtkTreeSelection *sel = gtk_tree_view_get_selection(tv);
		GList *rowpaths = gtk_tree_selection_get_selected_rows(sel, &model);
		GList *tmp = rowpaths;

		for (tmp=rowpaths; tmp; tmp=g_list_next(tmp)) {
			gtk_tree_model_get_iter(model, &iter, (GtkTreePath *)tmp->data);
			insert_clip_rows(tv, buffer, &iter, &end);
		}
	}
	else {
		for (ret=gtk_tree_model_get_iter_first(model, &iter);
		     ret!=FALSE;
		     ret=gtk_tree_model_iter_next(model, &iter) ) {
			insert_clip_rows(tv, buffer, &iter, &end);
		}
	}

	gtk_text_buffer_insert(buffer, &end, "\n", -1);
	g_list_free(cols);

	gtk_text_buffer_get_bounds(buffer, &start, &end);
}



void clip_table_list(void) 
{
	GtkTextBuffer *buffer;
	GtkTextIter end;
	GtkTreeView *tview;
	GtkTreeModel *model;
	GtkTreeIter titer;
	guint ret;
	gchar *owner, *table, *analyzed, *tablespace, *clust;
	gchar line[512];
	gint64 rows, blocks, avglen, sampled, pctfree;


	buffer = open_clip_window();
	if (!buffer) {
		return;
	}

	gtk_text_buffer_get_end_iter(buffer, &end);
	tview = GTK_TREE_VIEW(lw("tableList"));
	if(!tview)
		return;

	model = gtk_tree_view_get_model(tview);
	if (!model) {
		return;
	}

	g_snprintf(line, sizeof(line), 
		   "\n%-24s %-12s %-12s %-12s %-12s %-16s %-16s %-3s %-24s\n",
		   "Table", "Rows", "Blocks", "AvgLen", "Sampled", "Analyzed", 
		   "Tablespace", "PctFree", "Cluster/IOT");
	gtk_text_buffer_insert(buffer, &end, line, -1);

	g_snprintf(line, sizeof(line), "----------------------------------------------------------------------------------------------------------------------------------\n");
	gtk_text_buffer_insert(buffer, &end, line, -1);

	for (ret=gtk_tree_model_get_iter_first(model, &titer);
	     ret;
	     ret=gtk_tree_model_iter_next(model, &titer)) {

		gtk_tree_model_get(model, &titer, 
				   1, &owner, 
				   2, &table, 
				   5, &rows,
				   6, &blocks, 
				   7, &avglen, 
				   8, &sampled, 
				   9, &analyzed, 
				   10, &tablespace, 
				   11, &pctfree, 
				   12, &clust,
				   -1);
 
		g_snprintf(line, sizeof(line),
			   "%-24s %12lld %12lld %12lld %12lld %-16s %-16s %3lld %-24s\n",
			   table, rows, blocks, avglen, sampled, analyzed, tablespace,
			   pctfree, clust);

		g_free(owner);
		g_free(table);
		g_free(analyzed);
		g_free(tablespace);
		g_free(clust);

		gtk_text_buffer_insert(buffer, &end, line, -1);
	}
}


void clip_index_list(void) 
{
	GtkTextBuffer *buffer;
	GtkTextIter end;
	GtkTreeView *tview;
	GtkTreeModel *model;
	GtkTreeIter titer;
	guint ret;
	gchar *owner, *index, *table, *analyzed, *type, *tablespace;
	gchar line[512];
	gint64 rows, distinct, clfact, blevel;


	buffer = open_clip_window();
	if (!buffer) {
		return;
	}

	gtk_text_buffer_get_end_iter(buffer, &end);
	tview = GTK_TREE_VIEW(lw("indexList"));
	if (!tview) {
		return;
	}

	model = gtk_tree_view_get_model(tview);
	if (!model) {
		return;
	}

	g_snprintf(line, sizeof(line),
		   "\n%-24s %-24s %-12s %-12s %-12s %-8s %-20s %-24s %-24s\n",
		   "Index", "Table", "Rows", "Distinct", "ClustFact", "Level",
		   "Analyzed", "Type", "Tablespace");
	gtk_text_buffer_insert(buffer, &end, line, -1);

	g_snprintf(line, sizeof(line), "-------------------------------------------------------------------------------------------------------------------------------------------------------\n");
	gtk_text_buffer_insert(buffer, &end, line, -1);

	for (ret=gtk_tree_model_get_iter_first(model, &titer);
	    ret;
	    ret=gtk_tree_model_iter_next(model, &titer)) {

		gtk_tree_model_get(model, &titer, 
				   1, &owner,
				   2, &index, 
				   4, &table,
				   5, &rows,
				   6, &distinct, 
				   7, &clfact, 
				   8, &blevel, 
				   9, &analyzed, 
				   10, &type,
				   11, &tablespace,
				   -1);
 
		g_snprintf(line, sizeof(line),
			   "%-24s %-24s %12lld %12lld %12lld %8lld %-20s %-24s %-24s\n",
			   index, table, rows, distinct, clfact, blevel, analyzed, 
			   type, tablespace );

		g_free(owner);
		g_free(index);
		g_free(table);
		g_free(analyzed);
		g_free(tablespace);
		g_free(type);
		gtk_text_buffer_insert(buffer, &end, line, -1);
	}

}



