/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "result_pop.h"
#include "gutil.h"
#include "support.h"
#include "interface.h"
#include "appconfig.h"


extern GtkWidget     *window1;

static GtkWidget     *popWindow=0;


void  pop_results(guint selected) 
{
	gchar *last_query = (gchar *) g_object_get_data(G_OBJECT(lw("resList")), "last_query");
	GtkTextIter start;
	GtkTextBuffer *buffer;
	GtkTreeView *oldview = GTK_TREE_VIEW(lookup_widget(window1, "resList"));
	GtkTreeView *popview;
	GtkTreeIter iter, new_iter;
	GList *cols=NULL;
	GtkTreeModel *model, *new_model;
	GtkTreeStore *store;
	GtkTreeSelection *selection;
	guint x, ncols, ret;
	GType *gtypes;
	guint poprows=0;
	gchar poplabel[128];
	const gchar *font = config_get("listfont");


	popWindow = create_popWindow();

	if (last_query) {
		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(lookup_widget(popWindow,
									      "popQueryText")));
		gtk_text_buffer_get_start_iter(buffer, &start);
		gtk_text_buffer_insert(buffer, &start, last_query, -1);

	}    
  
	popview = GTK_TREE_VIEW(lookup_widget(popWindow, "popView"));

	if ((cols = gtk_tree_view_get_columns(oldview)) == NULL) {
		return;
	}

	ncols = g_list_length(cols);
	if (!ncols) {
		return;
	}

	model = gtk_tree_view_get_model(oldview);
	new_model = gtk_tree_view_get_model(popview);
	if (!model) {
		return;
	}

	gtypes = g_new0(GType, ncols);
	for (x=0; x < ncols; x++) {
		GtkCellRenderer *crend;
		GList *cl = g_list_nth(cols, x);
		GtkTreeViewColumn *column, *new_column;
		gchar *column_title;

		gtypes[x] = gtk_tree_model_get_column_type(model, x);

		column = GTK_TREE_VIEW_COLUMN(cl->data);
		column_title = (gchar *)gtk_tree_view_column_get_title(column);
		crend = gtk_cell_renderer_text_new();

		if (font) {
			g_object_set(G_OBJECT(crend), "font", font, NULL);
		}

		g_object_set(G_OBJECT(crend), "ypad", 0, NULL);
		new_column = gtk_tree_view_column_new_with_attributes(column_title, crend, "text", x, NULL);
		gtk_tree_view_append_column(popview, new_column);
		gtk_tree_view_column_set_sort_column_id(new_column, x);
	}


	store = gtk_tree_store_newv((gint)ncols, gtypes);
	selection = gtk_tree_view_get_selection(oldview);

	for(ret=gtk_tree_model_get_iter_first(model, &iter);
	    ret!=FALSE;
	    ret=gtk_tree_model_iter_next(model, &iter) ) {

		if (!selected 
		    || (selected && gtk_tree_selection_iter_is_selected(selection, &iter))) {

			gtk_tree_store_append(store, &new_iter, NULL);
			poprows++;

			for (x=0; x < ncols; x++) {

				if (gtk_tree_model_get_column_type(model, x) == G_TYPE_INT64) {
					gint64     int_value;

					gtk_tree_model_get(model, &iter, x, &int_value, -1);
					gtk_tree_store_set(store, &new_iter, x, int_value, -1);
				}
				else {
					gchar     *value;

					gtk_tree_model_get(model, &iter, x, &value, -1);
					gtk_tree_store_set(store, &new_iter, x, value, -1);
					g_free(value);
				}
			}

		}
	}

	gtk_tree_view_set_model(popview, GTK_TREE_MODEL(store) );
	g_snprintf(poplabel, sizeof(poplabel), "Popped <b>%d</b> rows.", poprows);
	gtk_label_set_markup(GTK_LABEL(lookup_widget(popWindow, "popLabel")), poplabel);
	gtk_widget_show(popWindow);
}


