/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <gtk/gtk.h>
#include <string.h>
#include <glib.h>

#include "syntax.h"

static GtkTextTagTable *table=0;
static guint parenset=0;

enum {WORD_VALUE, WORD_TAGNAME};

static gchar *syntax_keyword[][2] = {
	/* DML / DDL */
	{"SELECT","font_red"},
	{"FROM","font_red"},
	{"WHERE","font_red"},
	{"CREATE","font_red"},
	{"INSERT","font_red"},
	{"INTO","font_red"},
	{"DELETE","font_red"},
	{"UPDATE","font_red"},
	{"ALTER","font_red"},
	{"ANALYZE","font_red"},
	{"AUDIT","font_red"},
	{"CALL","font_red"},
	{"COMMENT","font_red"},
	{"COMMIT","font_red"},
	{"DELETE","font_red"},
	{"DROP","font_red"},
	{"GRANT","font_red"},
	{"MERGE","font_red"},
	{"REVOKE","font_red"},
	{"ROLLBACK","font_red"},
	{"TRUNCATE","font_red"},
	{"VALUES","font_red"},
	{"WITH","font_red"},
	{"GROUP", "font_red"},
	{"ORDER", "font_red"},
	{"BY", "font_red"},
	{"JOIN", "font_red"},
	{"ON", "font_red"},
	/* PL/SQL */
	{"BEGIN", "orange"},
	{"DECLARE", "orange"},
	{"END", "orange"},
	{"IF", "orange"},
	{"THEN", "orange"},
	{"ELSE", "orange"},
	{"FOR", "orange"},
	{"FORALL", "orange"},
	{"LOOP", "orange"},
	{"RETURN", "orange"},
	{"CASE", "orange"},
	{"WHEN", "orange"},
	{"OPEN", "orange"},
	{"FETCH", "orange"},
	{"FORALL", "orange"},
	/* SQL FUNCTIONS */
	{"TO_CHAR","blue"},
	{"TO_TIMESTAMP","blue"},
	{"ABS","blue"},
	{"ACOS","blue"},
	{"ADD_MONTHS","blue"},
	{"ASCII","blue"},
	{"ASIN","blue"},
	{"ATAN","blue"},
	{"ATAN2","blue"},
	{"BITAND","blue"},
	{"CARDINALITY","blue"},
	{"CAST","blue"},
	{"CEIL","blue"},
	{"CHARTOROWID","blue"},
	{"CHR","blue"},
	{"COALESCE","blue"},
	{"COLLECT","blue"},
	{"COMPOSE","blue"},
	{"CONCAT","blue"},
	{"CONVERT","blue"},
	{"CORR","blue"},
	{"COS","blue"},
	{"COSH","blue"},
	{"COUNT","blue"},
	{"CURRENT_DATE","blue"},
	{"CURRENT_TIMESTAMP","blue"},
	{"CV","blue"},
	{"DBTIMEZONE","blue"},
	{"DECODE","blue"},
	{"DECOMPOSE","blue"},
	{"DELETEXML","blue"},
	{"DEPTH","blue"},
	{"DEREF","blue"},
	{"DUMP","blue"},
	{"EMPTY_BLOB","blue"},
	{"EMPTY_CLOB","blue"},
	{"EXP","blue"},
	{"EXTRACT","blue"},
	{"EXTRACTVALUE","blue"},
	{"FLOOR","blue"},
	{"GREATEST","blue"},
	{"GROUP_ID","blue"},
	{"GROUPING","blue"},
	{"GROUPING_ID","blue"},
	{"HEXTORAW","blue"},
	{"INITCAP","blue"},
	{"INSTR","blue"},
	{"LAST_DAY","blue"},
	{"LEAST","blue"},
	{"LENGTH","blue"},
	{"LN","blue"},
	{"LNNVL","blue"},
	{"LOCALTIMESTAMP","blue"},
	{"LOG","blue"},
	{"LOWER","blue"},
	{"LPAD","blue"},
	{"LTRIM","blue"},
	{"MAKE_REF","blue"},
	{"MAX","blue"},
	{"MEDIAN","blue"},
	{"MIN","blue"},
	{"MOD","blue"},
	{"MONTHS_BETWEEN","blue"},
	{"NANVL","blue"},
	{"NCHR","blue"},
	{"NEW_TIME","blue"},
	{"NEXT_DAY","blue"},
	{"NTITLE","blue"},
	{"NULLIF","blue"},
	{"NVL","blue"},
	{"NVL2","blue"},
	{"PATH","blue"},
	{"PERCENT_RANK","blue"},
	{"PERCENTILE_CONT","blue"},
	{"PERCENTILE_DISC","blue"},
	{"POWER","blue"},
	{"PREVIOUS","blue"},
	{"RAWTOHEX","blue"},
	{"RAWTONHEX","blue"},
	{"REF","blue"},
	{"REFTOHEX","blue"},
	{"REGEXP_INSTR","blue"},
	{"REGEXP_REPLACE","blue"},
	{"REGEX_SUBSTR","blue"},
	{"REGR","blue"},
	{"REMAINDER","blue"},
	{"REPLACE","blue"},
	{"ROUND","blue"},
	{"ROW_NUMBER","blue"},
	{"RPAD","blue"},
	{"RTRIM","blue"},
	{"SCN_TO_TIMESTAMP","blue"},
	{"SET","blue"},
	{"SIGN","blue"},
	{"SIN","blue"},
	{"SINH","blue"},
	{"SOUNDEX","blue"},
	{"SQRT","blue"},
	{"SUBSTR","blue"},
	{"SUM","blue"},
	{"SYS_CONNECT_BY_PATH","blue"},
	{"SYS_CONTEXT","blue"},
	{"SYSDATE","blue"},
	{"SYSTIMESTAMP","blue"},
	{"TAN","blue"},
	{"TANH","blue"},
	{"TO_CLOB","blue"},
	{"TO_DATE","blue"},
	{"TO_DSINTERVAL","blue"},
	{"TO_LOB","blue"},
	{"TO_NUMBER","blue"},
	{"TO_TIMESTAMP_TZ","blue"},
	{"TRANSLATE","blue"},
	{"TRUNC","blue"},
	{"TZ_OFFSET","blue"},
	{"UPPER","blue"},
	{"USERENV","blue"},
	{"VALUE","blue"},
	{"WIDTH_BUCKET","blue"},
	{"AVG","blue"},
	/* ANALYTICS */
	{"RANK","purple"},
	{"DENSE_RANK","purple"},
	{"ROW_NUMBER","purple"},
	{"LEAD","purple"},
	{"LAG","purple"},
	{"CUME_DIST","purple"},
	{"PERCENTILE_RANK","purple"},
	{"NTILE","purple"},
	{"FIRST_VALUE","purple"},
	{"LAST_VALUE","purple"},
	{"FIRST","purple"},
	{"LAST","purple"},
	{"RATIO_TO_REPORT", "purple"}
};

#define NUM_KEYWORDS (sizeof(syntax_keyword) / sizeof(syntax_keyword[0]))


GtkTextTagTable *get_syntax_tag_table()
{
	return(table);
}


void add_error_tags(GtkTextTagTable *table)
{
	GtkTextTag *tag=0;

	if (!table) {
		return;
	}

	tag = gtk_text_tag_new("error");
	g_object_set(G_OBJECT(tag), "background", "pink", NULL);
	gtk_text_tag_table_add(table, tag);

	tag = gtk_text_tag_new("red");
	g_object_set(G_OBJECT(tag), "background", "red3", "foreground", "white", NULL);
	gtk_text_tag_table_add(table, tag);
}


void create_syntax_tag_table(GtkTextBuffer *buffer) 
{
	GtkTextTagTable *tab=0;
	GtkTextTag *tag=0;

	if (buffer) {
		tab = gtk_text_buffer_get_tag_table(buffer);
	}
	else {
		table = gtk_text_tag_table_new();
		tab = table;
	}

	tag = gtk_text_tag_new("bold");
	g_object_set(G_OBJECT(tag), "weight", 800, NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("orange");
	g_object_set(G_OBJECT(tag), "foreground", "#b75200", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("font_red");
	g_object_set(G_OBJECT(tag), "foreground", "red4", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("blue");
	g_object_set(G_OBJECT(tag), "foreground", "#008a8c", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("purple");
	g_object_set(G_OBJECT(tag), "foreground", "purple", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("text");
	g_object_set(G_OBJECT(tag), "foreground", "#ff00ff", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("dtext");
	g_object_set(G_OBJECT(tag), "foreground", "#ff00ff", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("paren");
	g_object_set(G_OBJECT(tag), "background", "darkseagreen2", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("comment");
	g_object_set(G_OBJECT(tag), "style", PANGO_STYLE_ITALIC, 
		     "weight",  PANGO_WEIGHT_BOLD, 
		     "foreground", "blue4", NULL);
	gtk_text_tag_table_add(tab, tag);

	tag = gtk_text_tag_new("hint");
	g_object_set(G_OBJECT(tag), 
		     "style", PANGO_STYLE_ITALIC,
		     "weight", PANGO_WEIGHT_BOLD,
		     "foreground", "blue4",
		     NULL);
	gtk_text_tag_table_add(tab, tag);

	add_error_tags(tab);
}


void match_highlight(GtkTextBuffer *buffer,
		     const gchar *open, 
		     const gchar *close, 
		     const gchar *tag, 
		     GtkTextIter *cursor)
{
	GtkTextIter start, end;
	GtkTextIter match_start, match_end;


	if (!cursor) {
		gtk_text_buffer_get_bounds(buffer, &start, &end);
		gtk_text_buffer_remove_tag_by_name(buffer, tag,
						   &start, &end);
		cursor = &start;
	}


	if (gtk_text_iter_forward_search(cursor, open, GTK_TEXT_SEARCH_TEXT_ONLY,
					 &match_start, NULL, NULL)) {
		cursor = &match_start;

		gtk_text_iter_forward_char(&match_start);
		if (gtk_text_iter_forward_search(&match_start, close,
						 GTK_TEXT_SEARCH_TEXT_ONLY,
						 NULL, &match_end, NULL)) {

			gtk_text_iter_backward_char(&match_start);
			gtk_text_buffer_apply_tag_by_name(buffer, tag, cursor, &match_end);

			gtk_text_iter_forward_char(&match_end);
			match_highlight(buffer, open, close, tag, &match_end);
		}
	}

}


const gchar *get_word_tag(const gchar *word)
{
	static guint i;
  
	for (i=0; i < NUM_KEYWORDS; i++ ) {
		int keyword_len = strlen(syntax_keyword[i][WORD_VALUE]);
		int word_len = strlen(word);

		if (!g_ascii_strncasecmp(word, 
					 syntax_keyword[i][WORD_VALUE], 
					 keyword_len > word_len ? keyword_len : word_len)
			) {
			return (syntax_keyword[i][WORD_TAGNAME]);
		}
	}

  return 0;
}




void buffer_syntax_highlight(GtkTextBuffer *buffer,
			     gint nchars) 
{
	static GtkTextMark *cursor_mark=0;
	static GtkTextIter start, cursor, end;
	static gchar *word=0;
	const gchar *tag=0;


	if (-1 == nchars) {
		gtk_text_buffer_get_bounds(buffer, &cursor, &end);
	}
	else {
		cursor_mark = gtk_text_buffer_get_insert(buffer);
		gtk_text_buffer_get_iter_at_mark(buffer, &cursor, cursor_mark);

		end = cursor;
		gtk_text_iter_backward_word_starts(&cursor, 3);
	}
  
	gtk_text_buffer_remove_all_tags(buffer, &cursor, &end);

	while (gtk_text_iter_compare(&cursor, &end) <= 0) {

		while (!g_ascii_isalpha(gtk_text_iter_get_char(&cursor)) 
		       && !gtk_text_iter_is_end(&cursor)) {

			gtk_text_iter_forward_char(&cursor);
		}
    
		if (gtk_text_iter_is_end(&cursor)) {
			break;
		}

		start=cursor;

		gtk_text_iter_forward_word_end(&cursor);
		while (gtk_text_iter_get_char(&cursor)=='_' && !gtk_text_iter_is_end(&cursor)) {
			gtk_text_iter_forward_char(&cursor);
			gtk_text_iter_forward_word_end(&cursor);
		}
    
		word = gtk_text_buffer_get_text(buffer, &start, &cursor, FALSE);
  
		if ((tag = get_word_tag(word)) != NULL) {
			gtk_text_buffer_apply_tag_by_name(buffer, tag, &start, &cursor);
		}
  
		g_free(word);
	}

	match_highlight(buffer, "/*", "*/", "hint", NULL);
	match_highlight(buffer, "'", "'", "text", NULL);
	match_highlight(buffer, "\"", "\"", "dtext", NULL);
	match_highlight(buffer, "--", "\n", "comment", NULL);
}


gboolean highlight_timeout(gpointer *buffer) 
{
	buffer_syntax_highlight(GTK_TEXT_BUFFER(buffer), -1);
	return FALSE;
}


void buffer_insert_text(GtkTextBuffer *buffer,
			GtkTextIter *location,
			gchar *text, 
			gint length, 
			gpointer user_data)
{
	static GtkTextIter cursor, end;
	static gchar *word=0;
	static gchar word_new[256];
	const  gchar *tag=0;
	gchar c;


	if (length > 1) {
		g_timeout_add(50, (GSourceFunc) highlight_timeout, (gpointer) buffer);
		return;
	}


	switch (text[0]) {
	case '\'':
	case '"':
	case '/':
	case '-':
		g_timeout_add(50, (GSourceFunc) highlight_timeout, (gpointer) buffer);
		return;
	default:
		break;
	}

#if DEBUG
	fprintf(stderr, "%s:%d  text=>%s<\n", __FILE__, __LINE__, text);
#endif

	cursor = *location;
	gtk_text_iter_backward_word_start(&cursor);
	end = cursor;
	gtk_text_iter_forward_word_end(&end);

	c = gtk_text_iter_get_char(&cursor);
	while (g_ascii_isalnum(c) || c=='_' || c=='.') {

		if (!gtk_text_iter_backward_chars(&cursor, 1)) {
			break;
		}

		c = gtk_text_iter_get_char(&cursor);
	}

	if (!g_ascii_isalpha(c)) {
		gtk_text_iter_forward_char(&cursor);
	}


	word = gtk_text_buffer_get_text(buffer, &cursor, &end, FALSE);
#if DEBUG
	fprintf(stderr, "%s:%d word=>%s<\n", __FILE__, __LINE__, word);
#endif

	tag = get_word_tag(word);
	if (tag) {
		gtk_text_buffer_apply_tag_by_name(buffer, tag, &cursor, &end);
	}
	else {
		gtk_text_buffer_remove_tag_by_name(buffer, "red", &cursor, location);
		gtk_text_buffer_remove_tag_by_name(buffer, "orange", &cursor, location);
		gtk_text_buffer_remove_tag_by_name(buffer, "blue", &cursor, location);
		gtk_text_buffer_remove_tag_by_name(buffer, "purple", &cursor, location);
	}

	return;
}




void text_move_cursor_cb(GtkTextView *textview,
			 GtkMovementStep step,
			 gint count,
			 gboolean extend_selection,
			 gpointer user_data)
{
	static gchar infront, behind, c;
	static int level=0;
	static GtkTextIter iter1, iter2, begin, end;
	static GtkTextBuffer *tb=0;
	static GtkTextMark *cursor=0;


	tb = gtk_text_view_get_buffer(textview);

	if (parenset) {
		gtk_text_buffer_get_bounds(tb, &begin, &end);
		gtk_text_buffer_remove_tag_by_name(tb, "paren",
						   &begin, &end);
		parenset=0;
	}

	if (!tb) {
		return;
	}


#if DEBUG
	fprintf(stderr, "%s:%d  step=%d count=%d  char=%c\n",
		__FILE__, __LINE__, step, count, c);
#endif

  
	if (step != 1) {
		return;
	}

	cursor = gtk_text_buffer_get_insert(tb);
	gtk_text_buffer_get_iter_at_mark(tb, &iter1, cursor);  
	c = gtk_text_iter_get_char(&iter1);
	iter2 = iter1;

	infront = gtk_text_iter_get_char(&iter1);
	gtk_text_iter_backward_chars(&iter2, 1);
	behind = gtk_text_iter_get_char(&iter2);

#if DEBUG
	fprintf(stderr, "%s:%d infront=%c behind=%c\n",
		__FILE__, __LINE__, infront, behind);
#endif

	if (infront == '(' ) {
		static GtkTextIter enditer;
		enditer = iter1;
    
		level=1;
		while (!gtk_text_iter_is_end(&enditer) && level > 0) {
			gtk_text_iter_forward_char(&enditer);
			c = gtk_text_iter_get_char(&enditer);

			if(c == '(') 
				level++;

			if(c == ')') 
				level--;
			
#if DEBUG
			fprintf(stderr, "%s:%d '(' level=%d\n",
				__FILE__, __LINE__, level);
#endif
		}
		gtk_text_iter_forward_char(&enditer);
		gtk_text_buffer_apply_tag_by_name(tb, "paren",
						  &iter1, &enditer);
		parenset=1;
	}

	if (behind == ')' ) {
		GtkTextIter startiter = iter1;

		level=1;
		gtk_text_iter_backward_char(&startiter);
		while(!gtk_text_iter_is_start(&startiter) && level > 0) {

			gtk_text_iter_backward_char(&startiter);
			c = gtk_text_iter_get_char(&startiter);
			
			if(c == ')')
				level++;

			if(c == '(')
				level--;

#if DEBUG
			fprintf(stderr, "%s:%d ')' level=%d\n",
				__FILE__, __LINE__, level);
#endif
		}
		gtk_text_buffer_apply_tag_by_name(tb, "paren",
						  &iter1, &startiter);
		parenset=1;
	}
}


