/* This program is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU General Public License */
/* as published by the Free Software Foundation; either version 2 */
/* of the License, or (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <oci.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "gutil.h"
#include "orautil.h"
#include "query.h"
#include "browser.h"
#include "table.h"


extern OCIEnv      *envhp;
extern OCIError    *errhp;
extern OCISvcCtx   *svchp;


gchar 
*get_table_comments(const gchar *owner,
		    const gchar *table)
{
	static OCIStmt  *sth = (OCIStmt *)0;
	OCIDefine       *defnp = (OCIDefine *) 0;
	OCIBind         *bhp1 = (OCIBind *) 0, *bhp2 = (OCIBind *)0;
	static gchar    value[4000];
	sb2             indp=0;
	sword           ret;
	const gchar     *sql = "select comments from all_tab_comments where owner=:p1 and table_name=:p2 and table_type='TABLE'";

  
	value[0] = '\0';

	ret = OCIStmtPrepare2(svchp, &sth, errhp,
			      (text *)sql, strlen(sql),
			      (text *)0, 0, OCI_NTV_SYNTAX,
			      OCI_DEFAULT);
	if (OCI_SUCCESS != ret) {
		return 0;
	}
 
	OCIBindByPos((OCIStmt *)sth, 
		     (OCIBind **)&bhp1, 
		     (OCIError *)errhp, 
		     (ub4)1, 
		     (dvoid *) owner, (sb4) strlen(owner)+1, 
		     (ub2)SQLT_STR, 
		     (dvoid *)0, (ub2 *)0, (ub2 *)0, 
		     (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);
	OCIBindByPos((OCIStmt *)sth, 
		     (OCIBind **)&bhp2, 
		     (OCIError *)errhp, 
		     (ub4)2, 
		     (dvoid *) table, (sb4) strlen(table)+1, 
		     (ub2)SQLT_STR, 
		     (dvoid *)0, (ub2 *)0, (ub2 *)0,
		     (ub4)0, (ub4 *)0, 
		     (ub4)OCI_DEFAULT);
  
	OCIDefineByPos((OCIStmt *) sth,
		       (OCIDefine **) &defnp,
		       (OCIError *)errhp, 
		       (ub4) 1,
		       (dvoid *)&value, sizeof(value), 
		       (ub2)SQLT_STR, 
		       (dvoid *)&indp, (ub2 *)0, (ub2 *)0, 
		       (ub4)OCI_DEFAULT);

	ret = OCIStmtExecute(svchp, sth, errhp, (ub4)1, 
			     (ub4)0, (OCISnapshot *)0,
			     (OCISnapshot *)0, (ub4)OCI_DEFAULT);

	OCIStmtRelease(sth, errhp, (OraText *) 0, 0, OCI_DEFAULT);

	if (OCI_SUCCESS == ret) {
		return(g_strdup(value));
	}
	else {
		return NULL;
	}
}




void  edit_column_comments(GtkCellRendererText *cell,
			   gchar *path_string,
			   gchar *new_text,
			   gpointer user_data) 
{
	OCIStmt *sth=0;
	GtkTreeIter iter;
	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(lw("detailList")));
	gchar *column=0;
	sword ret=OCI_SUCCESS;
	gchar ddl[512];


	if (!user_data) {
		return;
	}

	gtk_tree_model_get_iter_from_string(model, &iter, path_string);
	gtk_tree_model_get(model, &iter, 2, &column, -1);

	g_snprintf(ddl, sizeof(ddl), "COMMENT ON COLUMN %s.%s IS '%s'", 
		   (gchar *) user_data, column, new_text);
	fprintf(stderr, "%s\n", ddl);

	OCIHandleAlloc(envhp, (dvoid **)&sth,
		       (ub4)OCI_HTYPE_STMT, 0, NULL);
	
	OCIStmtPrepare(sth, errhp, (text *)ddl, strlen(ddl),
		       OCI_NTV_SYNTAX, OCI_DEFAULT);

	ret = OCIStmtExecute(svchp, sth, errhp, (ub4)1, (ub4)0,
			     (OCISnapshot *)0, (OCISnapshot *)0,
			     (ub4)OCI_DEFAULT);

	if (OCI_SUCCESS == ret) {
		gtk_tree_store_set(GTK_TREE_STORE(model), &iter, 14,
				   new_text, -1);
		gtk_label_set_text(GTK_LABEL(lw("statusLabel")),
				   "Column comment saved.");
	}

	g_free(column);
	OCIHandleFree((dvoid *)sth, (ub4)OCI_HTYPE_STMT);
}





void on_tableCommentEntry_activate(GtkEntry *entry,
				   gpointer user_data)
{
	OCIStmt *sth = (OCIStmt *) 0;
	const gchar *comment = gtk_entry_get_text(GTK_ENTRY(lw("tableCommentEntry")));
	gchar *owner=0, *table=0, sql[128];
	browserSelection  bs;


	if (!browser_get_selection(GTK_TREE_VIEW(lw("objectList")), &bs) ) {
		return;
	}

	gtk_tree_model_get(bs.model, &bs.iter, BC_OWNER, &owner,
			   BC_OBJECT, &table, -1);

	g_snprintf(sql, sizeof(sql), "comment on table %s.%s is '%s'",
		   owner, table, comment);

#if DEBUG
	fprintf(stderr, "%s:%d '%s'\n", __FILE__, __LINE__, sql);
#endif

	OCIHandleAlloc(envhp, (dvoid **)&sth,
		       (ub4)OCI_HTYPE_STMT, 0, NULL);
	OCIStmtPrepare(sth, errhp, (text *)sql,
		       strlen(sql), OCI_NTV_SYNTAX, OCI_DEFAULT);
	if(ociErr(errhp, OCIStmtExecute(svchp, sth, errhp, (ub4)1,
					(ub4)0, (OCISnapshot *)0,
					(OCISnapshot *)0,
					(ub4)OCI_DEFAULT)) == OCI_SUCCESS) {

		gtk_label_set_text(GTK_LABEL(lw("statusLabel")), "Table comment saved.");
	}

	OCIHandleFree((dvoid *)sth, (ub4)OCI_HTYPE_STMT);
	g_free(owner);
	g_free(table);
	browser_free_selection(&bs);
}


void
update_table_data(const gchar *owner,
		  const gchar *table)
{
	gchar     sql[512], rmax[16], rmin[16];
	gint      pagenum  = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw("tableDataPageSpin")));
	gint      pagesize = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lw("tableDataPageSizeSpin")));


	g_snprintf(sql, sizeof(sql),
		   "select * from ( select /*+ FIRST_ROWS */ a.*, rownum rnum from (select * from %s.%s) a where rownum <= :p1) where rnum > :p2",
		   owner,
		   table);

	g_snprintf(rmin, sizeof(rmin), "%d", pagesize * (pagenum-1) ); 
	g_snprintf(rmax, sizeof(rmax), "%d", pagesize * pagenum);
 
	execute_sql(sql, "tableDataList", rmax, rmin, NULL);

}


static void on_fkCloseButton_clicked(GtkButton *button,
				     gpointer user_data)
{
	gtk_widget_destroy(GTK_WIDGET(user_data));
}


void on_fkSpin_changed(GtkEditable *editable,
		       gpointer user_data)
{
	GtkSpinButton *start=0, *stop=0;
	GtkTreeView *dataview=0;
	gint start_val=0, stop_val=0;
	gchar start_val_str[8], stop_val_str[8];
	gchar *query=0;


	query = (gchar *)g_object_get_data(G_OBJECT(user_data), "query");
	start = GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(user_data), "start_spin"));
	stop = GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(user_data), "stop_spin"));
	dataview = GTK_TREE_VIEW(g_object_get_data(G_OBJECT(user_data), "data_view"));

	start_val = gtk_spin_button_get_value_as_int(start);
	stop_val = gtk_spin_button_get_value_as_int(stop);
	g_snprintf(start_val_str, sizeof(start_val_str), "%d", start_val);
	g_snprintf(stop_val_str, sizeof(stop_val_str), "%d", stop_val);

	execute_sql_gtktreeview(query, dataview, stop_val_str, start_val_str, NULL);
}




void show_table_data_window(const gchar *query)
{
	GtkWidget *fkWindow;
	GtkWidget *vbox1;
	GtkWidget *hbox2;
	GtkWidget *label1;
	GtkObject *fkStartSpin_adj;
	GtkWidget *fkStartSpin;
	GtkWidget *label2;
	GtkObject *fkToSpin_adj;
	GtkWidget *fkToSpin;
	GtkWidget *scrolledwindow1;
	GtkWidget *fkDataView;
	GtkWidget *hbox1;
	GtkWidget *fkCloseButton;

	fkWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (fkWindow), "fkWindowTable");

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (fkWindow), vbox1);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);

	label1 = gtk_label_new ("From : ");
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (hbox2), label1, FALSE, FALSE, 0);

	fkStartSpin_adj = gtk_adjustment_new (1, 1, 10000000, 20, 100, 10);
	fkStartSpin = gtk_spin_button_new (GTK_ADJUSTMENT (fkStartSpin_adj), 1, 0);
	gtk_widget_show (fkStartSpin);
	gtk_box_pack_start (GTK_BOX (hbox2), fkStartSpin, FALSE, FALSE, 0);

	label2 = gtk_label_new ("    To :");
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (hbox2), label2, FALSE, FALSE, 0);

	fkToSpin_adj = gtk_adjustment_new (201, 2, 1000000, 20, 100, 10);
	fkToSpin = gtk_spin_button_new (GTK_ADJUSTMENT (fkToSpin_adj), 1, 0);
	gtk_widget_show (fkToSpin);
	gtk_box_pack_start (GTK_BOX (hbox2), fkToSpin, FALSE, FALSE, 0);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_SHADOW_IN);

	fkDataView = gtk_tree_view_new ();
	gtk_widget_show (fkDataView);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), fkDataView);
	gtk_widget_set_size_request (fkDataView, 500, 300);
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (fkDataView), TRUE);
	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (fkDataView), TRUE);
	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (fkDataView), FALSE);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

	fkCloseButton = gtk_button_new_with_mnemonic ("Close");
	gtk_widget_show (fkCloseButton);
	gtk_box_pack_start (GTK_BOX (hbox1), fkCloseButton, FALSE, FALSE, 8);

	g_signal_connect ((gpointer) fkStartSpin, "changed",
			  G_CALLBACK (on_fkSpin_changed),
			  (gpointer) fkWindow);
	g_signal_connect ((gpointer) fkToSpin, "changed",
			  G_CALLBACK (on_fkSpin_changed),
			  (gpointer) fkWindow);
	g_signal_connect ((gpointer) fkCloseButton, "clicked",
			  G_CALLBACK (on_fkCloseButton_clicked),
			  (gpointer) fkWindow);

	g_object_set_data_full(G_OBJECT(fkWindow), "query", (gpointer) g_strdup(query), g_free);
	g_object_set_data(G_OBJECT(fkWindow), "start_spin", (gpointer) fkStartSpin);
	g_object_set_data(G_OBJECT(fkWindow), "stop_spin", (gpointer) fkToSpin);
	g_object_set_data(G_OBJECT(fkWindow), "data_view", (gpointer) fkDataView);

	gtk_widget_show_all(fkWindow);

	execute_sql_gtktreeview(query, GTK_TREE_VIEW(fkDataView), "201", "1", NULL);
}




void table_create_infowin(const gchar *owner,
			  const gchar *table_name,
			  gint is_synonym)
{
	gchar win_title[128];
	GtkWidget *descWin=0;
	GtkWidget *vbox1=0;
	GtkWidget *scrolledwindow1=0;
	GtkWidget *descView=0;


	g_snprintf((gchar *)win_title, sizeof(win_title), "Info for %s.%s",
		   owner, table_name);
	descWin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (descWin), win_title);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (descWin), vbox1);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_widget_set_size_request (scrolledwindow1, 500, 400);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_SHADOW_IN);

	descView = gtk_tree_view_new ();
	gtk_widget_show (descView);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), descView);

	if (is_synonym) 

		execute_sql_gtktreeview("select\
    t.column_name \"Column\", \
    max(t.data_type || \
    CASE \
       WHEN t.char_used='B' THEN '(' || t.data_length || ')'\
       WHEN t.data_type like 'TIMESTAMP%%' THEN NULL\
       ELSE decode(t.data_scale, NULL ,' ', '(' || t.data_precision || ',' || t.data_scale || ')') \
    END) \"Datatype\",\
    max(t.nullable) \"Null\",\
    max(decode(c.constraint_type,'P','Y',NULL)) \"PK\",\
    max(decode(c.r_owner,NULL,NULL,c.r_owner || '.' || c.r_constraint_name)) \"Ref Constaint\",\
    max(com.comments) \"Comments\", \
    max(t.num_distinct) \"Distinct\", \
    max(t.num_nulls) \"Nulls\", \
    max(t.density) \"Density\",\
    max(t.avg_col_len) \"AvgLen\",\
    max(t.num_buckets) \"Bckts\",\
    max(t.sample_size) \"Sample\", \
    max(to_char(t.last_analyzed,'MM/DD HH24:MI')) \"Analyzed\",\
    max(decode(t.data_type_owner, NULL, NULL, t.data_type_owner || '.' || t.data_type)) \"User Datatype\"\
   from all_synonyms s join all_tab_columns t on s.table_name=t.table_name and s.table_owner=t.owner \
        join all_col_comments com on com.owner=t.owner and com.table_name=t.table_name and com.column_name=t.column_name \
        left outer join all_cons_columns cc on cc.owner=t.owner and cc.table_name=t.table_name and cc.column_name=t.column_name \
        left outer join all_constraints c on cc.table_name=c.table_name and cc.owner=c.owner and c.constraint_name=cc.constraint_name and c.constraint_type in ('R','P') \
   where s.owner=:p1 and s.synonym_name=:p2 \
   group by t.column_name \
   order by max(t.column_id)",
					GTK_TREE_VIEW(descView),
					owner,
					table_name,
					NULL);

	else

		execute_sql_gtktreeview("select\
    t.column_name \"Column\",\
    max(t.data_type || \
    CASE \
       WHEN t.char_used='B' THEN '(' || t.data_length || ')'\
       WHEN t.data_type like 'TIMESTAMP%%' THEN NULL\
       ELSE decode(t.data_scale, NULL ,' ', '(' || t.data_precision || ',' || t.data_scale || ')') \
    END) \"Datatype\",\
    max(t.nullable) \"Null\",\
    max(decode(c.constraint_type,'P','Y',NULL)) \"PK\",\
    max(decode(c.r_owner,NULL,NULL,c.r_owner || '.' || c.r_constraint_name)) \"Ref Constaint\",\
    max(com.comments) \"Comments\",\
    max(t.num_distinct) \"Distinct\", \
    max(t.num_nulls) \"Nulls\", \
    max(t.density) \"Density\",\
    max(t.avg_col_len) \"AvgLen\",\
    max(t.num_buckets) \"Bckts\",\
    max(t.sample_size) \"Sample\", \
    max(to_char(t.last_analyzed,'MM/DD HH24:MI')) \"Analyzed\",\
    max(decode(t.data_type_owner, NULL, NULL, t.data_type_owner || '.' || t.data_type)) \"User Datatype\"\
   from all_tab_columns t JOIN all_col_comments com on com.owner=t.owner and com.table_name=t.table_name and com.column_name=t.column_name \
        left outer join all_cons_columns cc on cc.owner=t.owner and cc.table_name=t.table_name and cc.column_name=t.column_name \
        left outer join all_constraints c on cc.table_name=c.table_name and cc.owner=c.owner and c.constraint_name=cc.constraint_name and c.constraint_type in ('R','P') \
   where t.owner=:p1 and t.table_name=:p2 \
   group by t.column_name \
   order by max(t.column_id)",
					GTK_TREE_VIEW(descView),
					owner,
					table_name,
					NULL);


	gtk_widget_show(descWin);
}
